bl_info = {
    'name': 'Gizmo PRO',
    'author': 'Max Derksen',
    'version': (4, 2, 0), # current 4.0.2
    'blender': (4, 0, 0),
    'location': 'VIEW 3D > Tool Bar',
    #'warning': 'This is a test version of the addon. Write in the discord channel(link below) about the errors.',
    'support': 'COMMUNITY',
    'category': '3D View',
    #'doc_url': 'https://blendermarket.com/products/gizmo-pro-addon-blender-2', # TODO
    #'tracker_url': 'https://discord.gg/fk3ewHYJcb', # TODO
}

def get_prefs():
    return bpy.context.preferences.addons[__package__].preferences
    

from .tools import GizmoProTools


import bpy





from . import (
    icons,
    preferences,
    keymaps,
    tools,
    ui,
)
# --- Utils
from .utils import (
    check_updates,
    handlers,
    ops,
    select_ops,
)


# --- Gizmos
from .gizmo.gizmos_3d import (
    gt_cone, 
    gt_box, 
    gt_arc,
    gt_malibu_items,
    gt_pivot,
    gt_preselect,
    gt_transpose,
    gt_blank,
)

# --- Gizmo Groups
from .gizmo_group import (
    gg_camera,
    gg_cursor,
    gg_curve,
    gg_mirror,
    gg_navigate,
    gg_placement,
    gg_preselect,
    gg_quick_widget,
    gg_rotate,
    gg_scale,
    gg_searcher,
    gg_shear,
    gg_transpose,
    gg_vr,
    gg,
    gg_2d_navigate,
)








from .gizmo.gizmos_2d import (
    gt_button,
    gt_cross,
    gt_searcher,
    gt_value,
)




# --- Operators
from .utils.gizmo_operators import (
    arc_op,
    blank_op,
    box_op,
    cone_op,
    dot_op,
    trackball_op,
    transpose_op,
)

from .utils.operators import (
    align,
    bevel,
    cursor_transform,
    duplicate_rotate,
    edge_slide,
    increment_rotate,
    paint_select,
    modo_transform,
    pivot_transform,
    pro_cursor_ops,
    qmesh,
    qsnap,
    resize_empty,
    transform_from_value,
    calculator,
)


""" # --- Quick Menu/Wheel
from .quick_menu import (
    qw_gizmo,
    qw_op,
    qw_widget,
) """





# --- UPDATE
import bpy
from .utils.check_updates import update_version






 





def register():
    icons.register()
    check_updates.register()
    preferences.register()
    handlers.register()
    


    # --- Operators
    arc_op.register()
    blank_op.register()
    box_op.register()
    cone_op.register()
    dot_op.register()
    trackball_op.register()
    transpose_op.register()


    align.register()
    bevel.register()
    pro_cursor_ops.register()
    cursor_transform.register()
    modo_transform.register()
    pivot_transform.register()
    qmesh.register()
    qsnap.register()
    increment_rotate.register()
    paint_select.register()
    transform_from_value.register()
    resize_empty.register()
    duplicate_rotate.register()
    edge_slide.register()
    calculator.register()
    
    
    ops.register()


    

    """ # --- Quick Widget
    qm_widgets.register()
    qw_gizmo.register()
    qw_op.register()
    qw_widget.register() """
    

    # --- RefBoard
    """ rb_gizmo.register()
    rb_panel.register()
    rb_widget.register() """




    # --- KeyMap
    keymaps.register()
    


    # --- Gizmo
    gt_cone.register()
    gt_box.register()
    gt_arc.register()
    gt_button.register()
    gt_transpose.register()
    gt_cross.register()
    gt_value.register()
    gt_preselect.register()
    gt_pivot.register()
    gt_searcher.register()
    gt_blank.register()
    gt_malibu_items.register()

    gg_scale.register()    
    gg_camera.register()
    gg_curve.register()
    gg_mirror.register()
    gg_navigate.register()
    gg_placement.register()
    gg_preselect.register()
    gg_quick_widget.register()
    gg_rotate.register()
    gg_searcher.register()
    gg_shear.register()
    gg_transpose.register()
    gg_vr.register()
    gg.register()
    gg_cursor.register()
    gg_2d_navigate.register()
    

    # --- UI
    ui.register()

    select_ops.register()
    




   
    for tool in GizmoProTools:
        bpy.utils.register_tool(tool['tool'] ,  after = tool['after'], separator=tool['separator'], group = tool['group'])
    

    # --- Update Addon
    props = get_prefs()
    try:
        if props.check_updates:
            check, version = update_version()
        else:
            check, version = False, False
            
        if check:
            props.update = True
            props.version = version
        else:
            props.update = False
    except:
        props.update = False


def unregister():
    
    for tool in GizmoProTools:
        bpy.utils.unregister_tool(tool['tool'])

    keymaps.unregister()

    # --- Key Map
    
    


    check_updates.unregister(),
    preferences.unregister()
    icons.unregister()
    handlers.unregister()
    
    gg_camera.unregister()
    gg_curve.unregister()
    gg_mirror.unregister()
    gg_navigate.unregister()
    gg_placement.unregister()
    gg_quick_widget.unregister()
    gg_preselect.unregister()
    gg_rotate.unregister()
    gg_scale.unregister()
    gg_searcher.unregister()
    gg_shear.unregister()
    gg_transpose.unregister()
    gg_vr.unregister()
    gg.unregister()
    gg_cursor.unregister()
    gg_2d_navigate.unregister()
    
    # --- UI
    ui.unregister()

    
    
    
    # --- Gizmo
    gt_cone.unregister()
    gt_box.unregister()
    gt_arc.unregister()
    gt_button.unregister()
    gt_transpose.unregister()
    gt_cross.unregister()
    gt_value.unregister()
    gt_preselect.unregister()
    gt_pivot.unregister()
    gt_searcher.unregister()
    gt_blank.unregister()
    gt_malibu_items.unregister()
    

    # --- Operators
    arc_op.unregister()
    blank_op.unregister()
    box_op.unregister()
    cone_op.unregister()
    dot_op.unregister()
    trackball_op.unregister()
    transpose_op.unregister()


    align.unregister()
    bevel.unregister()
    pro_cursor_ops.unregister()
    cursor_transform.unregister()
    modo_transform.unregister()
    pivot_transform.unregister()
    qmesh.unregister()
    qsnap.unregister()
    resize_empty.unregister()
    transform_from_value.unregister()
    increment_rotate.unregister()
    paint_select.unregister()
    duplicate_rotate.unregister()
    edge_slide.unregister()
    calculator.unregister()
    select_ops.unregister()
    
    ops.unregister()


    """ # --- Quick Widget
    qm_widgets.unregister()
    qw_widget.unregister()
    qw_gizmo.unregister()
    qw_op.unregister() """

    # --- RefBoard
    """ rb_gizmo.unregister()
    rb_panel.unregister()
    rb_widget.unregister() """


    
    # --- Sketch
    #skch_circle.unregister()


    
    