import bpy, gpu
from bpy.types import Gizmo
from math import sqrt
from ...utils.draw_utils.draw import capsule_2d, draw_texture_2d, get_circle
#from gpu_extras.presets import draw_texture_2d
from ...icons import gl_textures
from ... import var,get_prefs

class GP_GT_button(Gizmo):
    bl_idname = 'GP_GT_button'
    
    __slots__ = (
        'props',
        'settings',
        'prefs',

        'shape',
        'shape_outline',
        'icon',
        'icon_id',
        'bool_op',
        'n',
        'id',
        'div', # division

        'type',
        'custom_shape_circle',
        'custom_shape_circle_line',
        'custom_shape_circle_outline',

        'custom_shape_capsule',
        'custom_shape_capsule_line',
        'custom_shape_capsule_outline',
        
    )

    def setup(self):
        self.props = get_prefs()
        self.settings = bpy.context.scene.GP_scene_set
        self.prefs = bpy.context.preferences.system

        self.color = (0.95, 0.95, 0.95)
        self.color_highlight = (1.0, 1.0, 1.0)
        self.alpha = 0.5
        self.alpha_highlight = 0.5

        self.line_width = 2

        if not hasattr(self, 'bool_op'):
            self.bool_op = False

        #self.use_grab_cursor = False
        #self.select_bias = -0.1
        #self.use_tooltip = False # не отображать description
        self.use_select_background = True
        self.use_draw_modal = True
        #self.hide_select = False
        self.n = 1.0
        
        self.icon = None
        self.icon_id = 1


        self.id = -1
        self.div = 4


        if not hasattr(self, 'bool_op'):
            self.bool_op = False

        circle = get_circle( (0.0, 0.0), 1, segments = 28 )
        circle_line  = get_circle( (0.0, 0.0), 0.8, segments = 28 )
        if not hasattr(self, 'custom_shape_circle'):
            self.custom_shape_circle = self.new_custom_shape('TRI_FAN', circle)
        if not hasattr(self, 'custom_shape_circle_line'):
            self.custom_shape_circle_line = self.new_custom_shape('LINE_STRIP', circle_line)
        if not hasattr(self, 'custom_shape_circle_outline'):
            self.custom_shape_circle_outline = self.new_custom_shape('LINE_STRIP', circle)

        
        
        if not hasattr(self, 'custom_shape_capsule'):
            capsule = capsule_2d( size=0.5, radius=0.7, segments=24 )
            self.custom_shape_capsule = self.new_custom_shape('TRI_FAN', capsule)
        if not hasattr(self, 'custom_shape_capsule_line'):
            capsule = capsule_2d( size=0.5, radius=0.5, segments=24, closing_point=True )
            self.custom_shape_capsule_line = self.new_custom_shape('LINE_STRIP', capsule)
        if not hasattr(self, 'custom_shape_capsule_outline'):
            capsule = capsule_2d( size=0.5, radius=0.7, segments=24, closing_point=True )
            self.custom_shape_capsule_outline = self.new_custom_shape('LINE_STRIP', capsule)

        if not hasattr(self, 'type'):
            self.type = 'CIRCLE'


    def test_select(self, context, location):
        dpi = context.preferences.system.ui_scale
        pos = self.matrix_basis.translation

        if self.type == 'CIRCLE':
            out = self.scale_basis * dpi

            sqx = (location[0] - pos[0])**2
            sqy = (location[1] - pos[1])**2

            if sqrt(sqx + sqy) < out:
                return 0

        elif self.type == 'CAPSULE':
            out = self.scale_basis * dpi * 0.8

            sqx = (location[0] - pos[0])**2
            sqy = (location[1] - pos[1])**2

            if sqrt(sqx + sqy) < out:
                return 0
        return -1


    def draw(self, context):
        props = get_prefs()
        dpi = context.preferences.system.ui_scale
        pos = self.matrix_basis.translation


        gpu.state.line_width_set(self.line_width * props.qw_scale * dpi)
        


        self.alpha_highlight = 0.5 
        if self.type == 'CIRCLE':
            self.draw_custom_shape(self.custom_shape_circle)
            self.alpha_highlight = 0.75
            self.draw_custom_shape(self.custom_shape_circle_outline)
            if self.bool_op:
                self.color = props.qw_color_active
                self.color_highlight = props.qw_color_active
                self.alpha = 0.9
                self.draw_custom_shape(self.custom_shape_circle_line)
                self.color = (0.95, 0.95, 0.95)
                self.color_highlight = (1.0, 1.0, 1.0)


        elif self.type == 'CAPSULE':
            self.draw_custom_shape(self.custom_shape_capsule)
            self.alpha_highlight = 0.75
            self.draw_custom_shape(self.custom_shape_capsule_outline)
            if self.bool_op:
                self.color = props.qw_color_active
                self.color_highlight = props.qw_color_active
                self.alpha = 0.9
                self.draw_custom_shape(self.custom_shape_capsule_line)
                self.color = (0.95, 0.95, 0.95)
                self.color_highlight = (1.0, 1.0, 1.0)
            
        


        gpu.state.blend_set('ALPHA')


        # --- Icon
        if self.icon_id:
            # --- относительная озиция иконки внутри атласа
            r = (self.icon_id-1) // 10
            c = (self.icon_id-1) % 10
            i = 0.1
            f = 0.05
            xpi = c * i + f
            ypi = r * i + f

            # --- размер иконки
            if self.icon_id:
                scale_factor = 10
            else:
                scale_factor = 1
            if self.is_highlight:
                tS = self.scale_basis * scale_factor * 1.3 * dpi
            else:
                tS = self.scale_basis * scale_factor * 1.1 * dpi

            x = pos[0] - tS * xpi
            y = pos[1] - tS * ypi
            tp = (x, y)
            
            texture = gl_textures['colored_icons']
            draw_texture_2d(texture, tp, tS, tS, icon_id=self.icon_id)

        # TODO сделать иконки по названию, не только по id
            

        gpu.state.line_width_set(1)
        gpu.state.blend_set('NONE')



class GP_GT_button_backplane(Gizmo):
    bl_idname = 'GP_GT_button_backplane'
    
    __slots__ = (
        'icon_show',
        'icon_id',
        'width',
        'height',
    )

    def setup(self):
        self.icon_show = True
        self.icon_id = 1
        self.width = 10
        self.height = 10


    def test_select(self, context, location):
        pos = self.matrix_basis.translation

        """ out = self.scale_basis * dpi

        sqx = (location[0] - pos[0])**2
        sqy = (location[1] - pos[1])**2

        if sqrt(sqx + sqy) < out:
            return 0 """

        if pos[0] - self.width / 2 < location[0] < pos[0] + self.width / 2 and pos[1] - self.height / 2 < location[1] < pos[1] + self.height / 2:
            return 0

        return -1


    def draw(self, context):
        dpi = context.preferences.system.ui_scale
        pos = self.matrix_basis.translation

        gpu.state.blend_set('ALPHA')

        # --- Icon
        if self.icon_show:
            # --- относительная озиция иконки внутри атласа
            r = (self.icon_id-1) // 10
            c = (self.icon_id-1) % 10
            i = 0.1
            f = 0.05
            xpi = c * i + f
            ypi = r * i + f

            # --- размер иконки
            if self.icon_id:
                scale_factor = 10
            else:
                scale_factor = 1
            if self.is_highlight:
                tS = self.scale_basis * scale_factor * 1.3 * dpi
            else:
                tS = self.scale_basis * scale_factor * 1.1 * dpi

            x = pos[0] - tS * xpi
            y = pos[1] - tS * ypi
            tp = (x, y)
            
            texture = gl_textures['colored_icons']
            draw_texture_2d(texture, tp, tS, tS, icon_id=self.icon_id)

        # TODO сделать иконки по названию, не только по id
        

        gpu.state.blend_set('NONE')


classes = [
    GP_GT_button,
    GP_GT_button_backplane,
    ]

def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)