import bpy
from bpy.types import Gizmo
from ...utils.draw_utils.draw import cross_3d
from ...matrix.orientation.orientation_object import view_orient_matrix
from mathutils import Matrix, Vector
from gpu import state
from ... import get_prefs



class GP_GT_cross(Gizmo):
    bl_idname = 'GP_GT_cross'

  
    __slots__ = (
        'props',
        'dpi',
        'operator',
        'cross',
        'shape',
        'align_view',
    )


    def setup(self):
        self.props = get_prefs()
        self.dpi = bpy.context.preferences.system.ui_scale # Для мониторов с высоким разрешением

        self.align_view = True
        self.cross = cross_3d(l=0.5, axis='Z')

        if not hasattr(self, 'shape'):
            self.shape = self.new_custom_shape('LINES', self.cross)

        self.use_draw_offset_scale = True 
        self.use_draw_modal = True
        self.use_tooltip = False # не отображать description
        self.use_grab_cursor = False #draw grab cursor
        #self.use_select_background = True
        
        
    def draw(self, context):
        if self.align_view:
            self.matrix_basis = view_orient_matrix(self.matrix_basis.translation)[0] #@ Matrix.Translation( Vector( (0.0, 0.0, 5.0) ) )
        #self.matrix_offset = Matrix.Translation(Vector((0.0, 0.0, -10)))

        state.blend_set('ALPHA')
        line_width = 2 * self.props.size_gizmo_pro * self.dpi
        state.line_width_set(line_width)

        self.draw_custom_shape(self.shape)

        state.line_width_set(1.0)
        state.blend_set('NONE')


    def draw_select(self, context, select_id):
        self.draw_custom_shape(self.new_custom_shape('LINES', self.cross), select_id=select_id) 


    ''' def test_select(self, context, location):
        x = location[0]
        y = location[1]

        pos_x = self.matrix_basis.translation[0]
        pos_y = self.matrix_basis.translation[1]
      
        scale = self.props.size_gizmo_pro * self.scale_basis * self.dpi
        offset = scale * 1.7
        

        sqx = (x - pos_x)**2
        sqy = (y - pos_y)**2


        if sqrt(sqx + sqy) < offset:
            return 0

        return -1 '''


    def invoke(self, context, event):
        if self.operator == 'EMPTY':
            bpy.ops.gp.resize_empty('INVOKE_DEFAULT')
        return {'RUNNING_MODAL'}
       
    
    def modal(self, context, event, tweak):
        return {'FINISHED'}





classes = [
    GP_GT_cross,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls) 