import bpy
import blf
from bpy.types import Gizmo, GizmoGroup
from gpu import state
from math import degrees
from ... import var
from ...utils.draw_utils.draw import get_rounded_rectangle, draw_batch_mat, get_rectangle_with_bevels, get_triangle_sliding



font_info = {
    "font_id": 0,
    "handler": None,
}



class GP_GT_value(Gizmo):
    bl_idname = 'GP_GT_value'
    
    __slots__ = (
        'value',
        'position',
        'shape',
    )

    def setup(self):
        if not hasattr(self, 'value'):
            self.value = 0
        if not hasattr(self, 'position'):
            self.position = (0, 0)
        self.color = (0.7, 0.7, 0.7)
        self.alpha = 1.0


    def draw(self, context):
        dpi = context.preferences.system.ui_scale
        text = str(self.value) + '°' # TODO убрать градусы
        font_id = font_info["font_id"]
        dim = blf.dimensions(font_id, text)
        size = 17.0*dpi
        x = self.position[0] - dim[0] / 2 - size/4
        y = self.position[1] - dim[1] / 2 - size/6

        state.blend_set('ADDITIVE')
        state.line_width_set(3)
        rectangle = get_rounded_rectangle((0.0,0.0), dim[0]*2, dim[1]*3.5, 10)
        draw_batch_mat(self.matrix_basis.translation, rectangle, (*self.color, self.alpha), scale=1.0, type='TRI_FAN')
        draw_batch_mat(self.matrix_basis.translation, rectangle, (*self.color, 1.0), scale=1.0, type='LINE_STRIP')
        state.line_width_set(1)
        state.blend_set('NONE')

        blf.position(font_id, x, y, 0)
        blf.size(font_id, size)
        blf.color(font_id, 0.0, 0.0, 0.0, 1.0)
        blf.enable(font_id, blf.SHADOW)
        blf.shadow(font_id, 5, 0.0, 0.0, 0.0, 1.0)
        blf.draw(font_id, text)



class GP_GGT_value(GizmoGroup):
    bl_idname = 'GP_GGT_value'
    bl_label = 'Value'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'SHOW_MODAL_ALL', 'SCALE'}

    def setup(self, context):
        self.value = self.gizmos.new('GP_GT_value')
        
    def draw_prepare(self, context):
        self.value.value = var.VALUE[0]
        self.value.position = var.VALUE[1]
        self.value.matrix_basis.translation = (var.VALUE[1][0], var.VALUE[1][1], 0.0)

classes = (
    GP_GT_value,
    GP_GGT_value,
)


def register():
    for cls in classes:
        bpy.utils.register_class(cls)
          
        
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)