import bpy
from ...utils.custom_shape.arc import custom_shape_arc
from mathutils import Matrix, Vector
from ...utils.pro_types import GizmoPRO
from ...matrix.orientation.orientation_object import view_orient_matrix
from gpu import state
from ... import get_prefs

from ...utils.draw_utils.draw import get_circle, draw_3d_dashed, draw_custom_batch, draw_circle_2d

class GP_GT_arc(GizmoPRO):
    bl_idname = 'GP_GT_arc'

    __slots__ = (
        'align_view',
        'mesh',
        'mesh_normal',
        'custom_shape',
    )


    def setup(self):
        self.mesh = custom_shape_arc()[0]
        self.mesh_normal = custom_shape_arc()[1]
        self.align_view = False

        if not hasattr(self, 'custom_shape'):
            self.custom_shape = self.new_custom_shape('TRIS', self.mesh, self.mesh_normal)

        self.use_draw_offset_scale = True
        self.use_draw_modal = True
        self.use_tooltip = False
        self.use_grab_cursor = False

   
    def draw(self, context):
        props = get_prefs()
        if self.align_view:
            self.matrix_basis = view_orient_matrix(self.matrix_basis.translation)[0]
            self.matrix_offset = Matrix.Translation(Vector((props.offset_view, 0.0, 0.0)))
        else:
            self.matrix_offset = Matrix.Translation(Vector((props.offset_arc, 0.0, 0.0)))

        
        """ if props.draw_line:
            state.blend_set('ALPHA')
            state.line_width_set(props.width_line)
            if self.is_highlight:
                sca = 0.76 * props.offset_arc
                verts = get_circle((0.0, 0.0, 0.0), sca, segments=60)
                mw = self.matrix_world @ self.matrix_offset.inverted()
                draw_custom_batch(self, verts=verts, type='LINE_STRIP', matrix=mw)
            state.line_width_set(1.0)
            state.blend_set('NONE') """


        self.draw_custom_shape(self.custom_shape, normal=True)


    def draw_select(self, context, select_id):
        self.draw_custom_shape(self.new_custom_shape('TRIS', self.mesh, self.mesh_normal), select_id=select_id, normal=True)



classes = [
    GP_GT_arc,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)