import bpy
from ...utils.custom_shape.box import custom_shape_box
from mathutils import Matrix, Vector
from ...utils.pro_types import GizmoPRO
from ...matrix.orientation.orientation_object import view_orient_matrix
from gpu import state
from ...utils.draw_utils.draw import draw_custom_batch
from ... import get_prefs



class GP_GT_box(GizmoPRO):
    bl_idname = 'GP_GT_box'

    __slots__ = (
        'mesh',
        'mesh_normal',
        'shape',
        'align_view',
        'line_shape',
    )

    
    def setup(self):
        self.mesh = custom_shape_box()[0]
        self.mesh_normal = custom_shape_box()[1]
        self.align_view = False
        if not hasattr(self, 'custom_shape'):
            self.shape = self.new_custom_shape('TRIS', self.mesh, self.mesh_normal)

        self.use_draw_offset_scale = True
        self.use_draw_modal = True
        self.use_tooltip = False
        self.use_grab_cursor = False
        
        if not hasattr(self, 'line_shape'):
            props = get_prefs()
            points = [
                Vector((0.0, 0.0, 0.0)),
                Vector((0.0, 0.0, -props.offset_box + props.offset_line)),
                ]
            self.line_shape = self.new_custom_shape('LINES', points)


    def draw(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set
        if self.align_view:
            self.matrix_basis = view_orient_matrix(self.matrix_basis.translation)[1]
            self.matrix_offset = Matrix.Translation(Vector((0.0, 0.0, props.offset_view)))
        else:
            self.matrix_offset = Matrix.Translation(Vector((0.0, 0.0, props.offset_box)))


        if props.draw_line:
            state.blend_set('ALPHA')
            state.line_width_set(props.width_line)
           
            if props.draw_line_on_hover:
                if self.is_highlight:
                    self.draw_custom_shape(self.line_shape)
                    #draw_custom_batch(self, verts=points, type='LINES', matrix=self.matrix_world)
            else:
                if props.offset_box > props.offset_cone:
                    if settings.tra_gizmo is False:
                        self.draw_custom_shape(self.line_shape)
                        #draw_custom_batch(self, verts=points, type='LINES', matrix=self.matrix_world)
                else:
                    self.draw_custom_shape(self.line_shape)
                    #draw_custom_batch(self, verts=points, type='LINES', matrix=self.matrix_world)
            state.line_width_set(1.0)
            state.blend_set('NONE')


        self.draw_custom_shape(self.shape, normal=True)


    def draw_select(self, context, select_id):
        self.draw_custom_shape(self.new_custom_shape('TRIS', self.mesh, self.mesh_normal), select_id=select_id, normal=True)



classes = [
    GP_GT_box,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)