import bpy
from ...utils.custom_shape.cone import custom_shape_cone
from ...utils.draw_utils.draw import draw_custom_batch
from mathutils import Matrix, Vector
from ...utils.pro_types import GizmoPRO
from ...utils.func import convert_units
from ... import var,get_prefs
from gpu import state



class GP_GT_cone(GizmoPRO):
    bl_idname = 'GP_GT_cone'

    __slots__ = (
        'mesh',
        'mesh_normal',
        'shape',
        'init_position',
        'final_position',
        'line_shape',
    )


    def setup(self):
        self.use_draw_offset_scale = True 
        self.use_draw_modal = True
        self.use_tooltip = False # не отображать description
        #self.use_operator_tool_properties = True
        self.use_grab_cursor = False #draw grab cursor
        #self.select = True
        #self.select_bias = 0.5
        #self.use_select_background = False

        
        self.init_position = Vector()
        self.final_position = Vector()

        self.mesh = custom_shape_cone()[0]
        self.mesh_normal = custom_shape_cone()[1]

        if not hasattr(self, 'shape'):
            self.shape = self.new_custom_shape('TRIS', self.mesh, self.mesh_normal)

        

        if not hasattr(self, 'line_shape'):
            props = get_prefs()
            points = [
                Vector((0.0, 0.0, 0.0)),
                Vector((0.0, 0.0, -props.offset_cone + props.offset_line)),
                ]
            self.line_shape = self.new_custom_shape('LINES', points)
        


    def draw(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set

        self.matrix_offset = Matrix.Translation(Vector((0.0, 0.0, props.offset_cone)))



        if props.draw_line:
            state.blend_set('ALPHA')
            state.line_width_set(props.width_line)
            if props.draw_line_on_hover:
                if self.is_highlight:
                    self.draw_custom_shape(self.line_shape)
                    #draw_custom_batch(self, verts=points, type='LINES', matrix=self.matrix_world)
            else:
                if props.offset_cone > props.offset_box:
                    if settings.sca_gizmo is False:
                        self.draw_custom_shape(self.line_shape)
                        #draw_custom_batch(self, verts=points, type='LINES', matrix=self.matrix_world)
                else:
                    self.draw_custom_shape(self.line_shape)
                    #draw_custom_batch(self, verts=points, type='LINES', matrix=self.matrix_world)
           
            state.line_width_set(1)
            state.blend_set('NONE')


        # --- Get Transform Value TODO
        if self.is_modal and props.test:
            l, q , s = self.matrix_basis.decompose()
            ip = self.init_position
            dif_pos = l-ip

            dir = q @ Vector((0.0, 0.0, 1.0))
            if dif_pos.magnitude != 0.0:
                ang = dir.angle(dif_pos.normalized())
            else:
                ang = 0.0

            delta = (dif_pos).magnitude
            v, u = convert_units( context, value = delta )
            if ang <= 0.0:
                value = str(round(v, 2)) + ' ' + u
            else:
                value = '-' + str(round(v, 2)) + ' ' + u
            
            

        self.draw_custom_shape(self.shape, normal=True)


    """ def select_refresh(self):
        print(1) """

  
    def draw_select(self, context, select_id):
        props = get_prefs()
        if props.draw_line:
            self.draw_custom_shape(self.line_shape, select_id=select_id)

        self.draw_custom_shape(self.new_custom_shape('TRIS', self.mesh, self.mesh_normal), select_id=select_id, normal=True) 
        
        



classes = [
    GP_GT_cone,
    ]

def register():
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)