import bpy
from ...utils.custom_shape.cone import custom_shape_cone
from ...utils.draw_utils.draw import get_circle, get_capsule
from mathutils import Matrix, Vector
from ...utils.pro_types import Gizmo_Malibu
from ...utils.func import convert_units
from ... import var
from gpu import state, matrix
from bpy.types import Gizmo
from ...utils.custom_math.quaternion import look_rotation



class GP_GT_line(Gizmo_Malibu):
    bl_idname = 'GP_GT_line'

    __slots__ = (
        'shape',
        'line_shape',
        'align_view',
    )

    def setup(self):
        self.use_draw_offset_scale = True 
        self.use_draw_modal = True
        self.use_tooltip = False
        self.use_grab_cursor = False
        self.align_view = False

        capsule = get_capsule((0.0, 0.0, 1.0), 2, 0.06, segments=8, axis='Z')
        capsule_line = capsule
        capsule_line.append(capsule[0])
        if not hasattr(self, 'shape'):
            self.shape = self.new_custom_shape('TRI_FAN', capsule)
            self.line_shape = self.new_custom_shape('LINE_STRIP', capsule_line)

    def draw(self, context):
        region_3d = context.space_data.region_3d
        dpi = context.preferences.system.ui_scale

        # --- поворот к камере
        l, q, s = self.matrix_basis.decompose()
        if region_3d.is_perspective:
            view_loc = region_3d.view_matrix.inverted().to_translation()
            view_up = view_loc - l
        else:
            qat = context.region_data.view_rotation
            view_up = qat @ Vector((0, 0, 1))

        forward = q @ Vector((0.0, 0.0, 1.0))
        rotation = look_rotation(forward, view_up)
        self.matrix_basis = Matrix.LocRotScale(l, rotation, s)
        
        self.draw_custom_shape(self.shape)
        state.line_width_set(self.scale_basis*1.5*dpi)
        self.color = (0.0, 0.0, 0.0)
        self.draw_custom_shape(self.line_shape)
        state.line_width_set(1.0)

    def draw_select(self, context, select_id):
        self.draw_custom_shape(self.shape, select_id=select_id)



class GP_GT_point(Gizmo_Malibu):
    bl_idname = 'GP_GT_point'

    __slots__ = (
        'shape',
        'align_view',
    )

    def setup(self):
        self.use_draw_offset_scale = True 
        self.use_draw_modal = True
        self.use_tooltip = False
        self.use_grab_cursor = False
        self.align_view = True
        #self.select_bias = 0.5

        if not hasattr(self, 'shape'):
            self.shape = self.new_custom_shape('POINTS', [Vector((0.0, 0.0, 0.0))])

        """ circle = get_circle((0.0, 0.0, 0.0), 3, segments=8)
        if not hasattr(self, 'shape'):
            self.shape = self.new_custom_shape('TRI_FAN', circle)
            self.line_shape = self.new_custom_shape('LINE_STRIP', circle) """

    def draw(self, context):
        #self.select_bias = 10.0
        #dpi = context.preferences.system.ui_scale
        state.point_size_set(20 * self.scale_basis * 1.5) # * dpi
        self.draw_custom_shape(self.shape)
        state.point_size_set(1.0)

    def draw_select(self, context, select_id):
        self.draw_custom_shape(self.shape, select_id=select_id)
        
        
        

       




classes = [
    GP_GT_point,
    GP_GT_line,
    ]

def register():
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)