import bpy, bmesh, gpu
from ...utils.draw_utils.draw import draw_batch
from bpy.types import Gizmo
from ...utils.raycast import ray_cast, get_element
from gpu_extras.batch import batch_for_shader
from gpu.types import GPUShader
from gpu import state
from ...utils.shaders import GPU_SHADER_DIAG_STRIPES_UNIFORM_COLOR, GPU_SHADER_3D_POINT_UNIFORM_SIZE_OUTLINE


from bpy_extras import view3d_utils
from mathutils import Vector


def draw_vertex(coords, color):
    shader = GPU_SHADER_3D_POINT_UNIFORM_SIZE_OUTLINE()
    batch = batch_for_shader( shader, 'POINTS', {'pos': coords} )
    shader.bind()
    shader.uniform_float( 'color', color )
    shader.uniform_float( 'outlineColor', (0.1, 0.1, 0.1, 1.0) )
    shader.uniform_float( 'size', 20.0 )
    shader.uniform_float( 'outlineWidth', 3.0 )
    batch.draw( shader )


def draw_edge(coords, color):
    shader = gpu.shader.from_builtin('UNIFORM_COLOR')
    batch = batch_for_shader( shader, 'LINES', {'pos': coords} )
    shader.bind()
    shader.uniform_float( 'color', color )
    batch.draw( shader )


def draw_poly(coords, color):
    shader = GPU_SHADER_DIAG_STRIPES_UNIFORM_COLOR()
    batch = batch_for_shader( shader, 'TRI_FAN', {'pos': coords} )
    shader.bind()
    shader.uniform_float( 'color', color )
    shader.uniform_float( 'color2', (0.0, 0.0, 0.0, color[3]) )
    shader.uniform_int( 'size1', 5 )
    shader.uniform_int( 'size2', 5 )
    batch.draw( shader )














""" def get_object_under_cursor(context, mose_pos):
    x, y = mose_pos[0], mose_pos[1]
    region = context.region
    rv3d = context.region_data
    origin = view3d_utils.region_2d_to_origin_3d(region, rv3d, (x, y))
    direction = view3d_utils.region_2d_to_vector_3d(region, rv3d, (x, y))
    depsgraph = context.evaluated_depsgraph_get()
    result, location, normal, index, obj, matrix = context.scene.ray_cast(depsgraph, origin, direction)
    if result:
        return True, location, normal, index, obj, matrix
    return None, None, None, None, None, None


def get_face_under_cursor(obj, index):
    if obj:
        bm = bmesh.from_edit_mesh(obj.data)
        mw = obj.matrix_world
        vertex_positions = [mw @ v.co for v in bm.faces[index].verts]
        bm.free() # ?? освобождаем бмеш 
    else:
        vertex_positions = []
    return vertex_positions """


""" def get_vertex_under_cursor(obj, hit_location):
    bm = bmesh.from_edit_mesh(obj.data)
    mw = obj.matrix_world
    
    closest_vert = None
    closest_distance = float('inf')
    
    # Проходим по всем вершинам и ищем ближайшую к точке попадания
    for vert in bm.verts:
        distance = (mw @ vert.co - hit_location).length
        if distance < closest_distance:
            closest_distance = distance
            closest_vert = vert
    
    if closest_vert:
        return closest_vert, [mw @ closest_vert.co]
    else:
        return None, None """


""" def get_vertex_under_cursor(context, obj, mose_pos):
    bm = bmesh.from_edit_mesh(obj.data)
    mw = obj.matrix_world

    region = context.region
    rv3d = context.region_data

    closest_vert = None
    closest_distance = float('inf')

    mouse_coord = (mose_pos[0], mose_pos[1])
    view_vector = view3d_utils.region_2d_to_vector_3d(region, rv3d, mouse_coord)
    ray_origin = view3d_utils.region_2d_to_origin_3d(region, rv3d, mouse_coord)
    
    for vert in bm.verts:
        vert_coord_3d = mw @ vert.co
        vert_coord_2d = view3d_utils.location_3d_to_region_2d(region, rv3d, vert_coord_3d)

        if vert_coord_2d:
            # Проверка перекрытия
            hit, _, _, _, hit_obj, _ = context.scene.ray_cast(context.evaluated_depsgraph_get(), ray_origin, vert_coord_3d - ray_origin)
            
            if hit and hit_obj == obj:
                distance = (Vector(mouse_coord) - Vector(vert_coord_2d)).length
                if distance < closest_distance:
                    closest_distance = distance
                    closest_vert = vert

    if closest_vert:
        return closest_vert, [mw @ closest_vert.co]
    else:
        return None, None """


DIST_MAX = 7 # TODO add distance from object and angle to make in variable
DIST_MAX_SQR = DIST_MAX**2
from mathutils.geometry import intersect_point_line

def initBM(self, obj):
    bm = bmesh.new()
    bm.from_mesh(obj.to_mesh())
    bm.normal_update() # Need to update normals

    self.bm = bm
    self.backup = obj.name

def getPointSquare(coord,coords): # TODO find blender internal analog
    x = coord[0] - coords[0]
    y = coord[1] - coords[1]
    return x*x + y*y

def getClosestPoint(coord,coords):
    vectorAP = (coord[0] - coords[0][0], coord[1] - coords[0][1])
    vectorEdge = (coords[1][0] - coords[0][0], coords[1][1] - coords[0][1])
    sEdge = vectorEdge[0]**2 + vectorEdge[1]**2
    product = vectorEdge[0]*vectorAP[0] + vectorEdge[1]*vectorAP[1] 

    if sEdge == 0.0:
        return None # Edge with zero length
    else:
        distance = product / sEdge       

    if distance < 0.0:
        return coords[0]
    elif distance > 1.0:
        return coords[1]
    else:
        return coords[0][0] + vectorEdge[0]*distance, coords[0][1] + vectorEdge[1]*distance

def intersectSurface(self, context, mouse):
    scene = context.scene
    region = context.region
    rv3d = context.region_data
    coord = mouse[0], mouse[1]
    self.result = False

    if bpy.app.version >= (2, 91, 0):
        scene_data = context.evaluated_depsgraph_get()
    else:
        scene_data = context.view_layer
    
    # get the ray from the viewport and mouse
    view_vector = view3d_utils.region_2d_to_vector_3d(region, rv3d, coord) 
    ray_origin = view3d_utils.region_2d_to_origin_3d(region, rv3d, coord)
    
    """ if self.kdt: # Force Arma and Curves
        if len(self.force_2d_coords) > 0: 
            co, index, dist = self.kd.find(Vector(coord).to_3d()) 

            if dist**2 < DIST_MAX_SQR:
                comp_2d_idx = list(self.force_2d_coords.keys())[index]
                comp_3d_idx = self.force_2d_coords[comp_2d_idx]
                return self.force_list[comp_3d_idx], self.force_list[comp_3d_idx] # TODO
            else:
                return None, None
        else:
            return None, None
    
    else: # Mesh raycast """
    # Mesh raycast
    if context.space_data.local_view:
        # Object raycast requires evaluated mesh in object mode
        if context.mode == 'EDIT_MESH':
            bpy.ops.object.mode_set(mode='OBJECT')

        for objt in self.selection['data']:
            if objt.data.rna_type.name == 'Mesh':
                mwi = objt.matrix_world.inverted()

                ray_origin_mwi = mwi @ ray_origin
                view_vector_mwi = ((mwi @ view_vector) - mwi.translation).normalized()

                self.result, self.location, normal, index = objt.ray_cast(ray_origin_mwi, view_vector_mwi)

                if self.result:
                    break
            else:
                pass

        if self.selection['mode'] == 'EDIT_MESH':
            bpy.ops.object.mode_set(mode='EDIT')
    else:
        self.result, self.location, normal, index, objt, matrix = scene.ray_cast(scene_data, ray_origin, view_vector)
        
        # Filtering
        if self.result:
            # Filter(exclude) linked object from execution
            if objt.library:
                self.result = None
                return None, None
        
            # TODO remove when other types will be
            if objt.data.rna_type.name == 'Mesh':
                return index, objt
            else:
                self.result = None
                return None, None
        else:
            return None, None

def getComponents(self, mouse, context, index, c_object):
    region = context.region
    rv3d = context.region_data
    coord = mouse[0], mouse[1]

    if c_object and self.result:
        # Get editmode changes
        c_object.update_from_editmode()
        
        for o in context.selected_objects:
            o.select_set(state=False)  

        depgraph = bpy.context.evaluated_depsgraph_get()
        obj_eval = c_object.evaluated_get(depgraph)

        matrix_world = obj_eval.matrix_world # TODO self.matrix (?)  
        if obj_eval.name != self.backup: # Init bmesh
            initBM(self, obj_eval)

        if self.transformed: # Init bmesh after set origin
            initBM(self, obj_eval)
            self.transformed = False
            
        if self.kdt and not self.result: # If object.faces == 0
            self.bm.verts.ensure_lookup_table()
            element = self.bm.verts[index]

        else: # If args[0] == face
            # Get current face
            self.bm.verts.ensure_lookup_table()
            self.bm.faces.ensure_lookup_table()
            self.bm.edges.ensure_lookup_table()

            poly = self.bm.faces[index]
            verts, edges = poly.verts, poly.edges 
        
            # look for verts
            cVert = {} # TODO Make it simple in future: remove vert dict and use only edges
            for vert in verts:
                if not vert.hide:
                    point = matrix_world@vert.co
                    point_2d = view3d_utils.location_3d_to_region_2d(region, rv3d, point)
                    if point_2d == None:
                        pass
                    else:
                        cVert[vert] = getPointSquare(coord, point_2d)
         
            # look for edges
            cEdge = {}
            for edge in edges:
                if not edge.hide:
                    points = [matrix_world @ v.co for v in edge.verts]
                    points_2d = [view3d_utils.location_3d_to_region_2d(region, rv3d, point) for point in points]
                    
                    percent = intersect_point_line(self.location, points[0],points[1])[1]
                    fakepoint = points[0].lerp(points[1], percent) # fake edge close to cursor point
                    fakepoint_2d = view3d_utils.location_3d_to_region_2d(region, rv3d, fakepoint)

                    count = len([p for p in points_2d if p == None])
                    if count == 1:
                        for n,p in enumerate(points_2d):
                            if p == None:
                                points_2d[n] = fakepoint_2d
                    # TODO rewrite/simplify
                    if count == 2:
                        cEdge[edge] = DIST_MAX_SQR * 2 # exclude edge from chance to be selected for now

                    # TODO rewrite when if count == 2 will be solved
                    if None not in points_2d: 
                        point_2d = getClosestPoint(coord, points_2d)
                        if point_2d:
                            cEdge[edge] = getPointSquare(coord, point_2d)
                        else:
                            cEdge[edge] = DIST_MAX_SQR * 2 # exclude edge from chance to be selected

            minVert = min(cVert, key=cVert.get)
            minEdge = min(cEdge, key=cEdge.get)

            if cVert.get(minVert) > DIST_MAX_SQR and cEdge.get(minEdge) > DIST_MAX_SQR:
                element = poly
            else:
                if cVert.get(minVert) < DIST_MAX_SQR:
                    element = minVert
                else:
                    element = minEdge

        return element, obj_eval # TODO check if addon realy need whole object data in variable maybe replace by necessary data with low store weight?

def identify_element(element, obj):
    bm = bmesh.new()
    bm.from_mesh(obj.data)
    mw = obj.matrix_world
    if isinstance(element, bmesh.types.BMFace):
        return "FACE", [mw @ vert.co for vert in element.verts]
    elif isinstance(element, bmesh.types.BMEdge):
        return "EDGE", [mw @ vert.co for vert in element.verts]
    elif isinstance(element, bmesh.types.BMVert):
        return "VERT", [mw @ element.co]
    bm.free()








class GP_GT_preselect(Gizmo):
    bl_idname = 'GP_GT_preselect'


    __slots__ = (
        'elem',
        'pos_elem',
        'result',
        'location',
        'bm',
        'backup',
        'transformed',
        'kdt',
    )
    

    def setup(self):
        self.use_draw_offset_scale = True
        self.use_draw_modal = True
        self.use_tooltip = False
        self.use_grab_cursor = False

        self.color = (0.0, 0.5, 1.0) 
        self.alpha = 0.7
        #self.elem = [None, None, None, None] # --- obj, vertex, edge, face
        #self.pos_elem = [None, None, None, None] # --- obj, vertex, edge, face


        self.backup = None
        self.transformed = False
        self.kdt = False
    def test_select(self, context, location):
        #success, location, normal, face_index, obj, matrix = get_object_under_cursor(context, location)
        



        idx, obj = intersectSurface(self, context, location)
        self.elem = getComponents(self, location, context, idx, obj)
        #print(self.elem)


        if context.area:
            context.area.tag_redraw()
        return -1


    def draw(self, context):
        if self.result:
            el, verts = identify_element(self.elem[0], self.elem[1])
            
            state.blend_set('ALPHA')
            state.point_size_set(20.0)
            state.line_width_set(3.0)
            
            msm = context.scene.tool_settings.mesh_select_mode

           
            if el == 'VERT' and msm[0]:
                draw_vertex(verts, (*self.color, self.alpha))

            if el == 'EDGE' and msm[1]:
                draw_edge(verts, (*self.color, self.alpha))

            if el == 'FACE' and msm[2]:
                draw_poly(verts, (*self.color, self.alpha))




            state.point_size_set(1.0)                
            state.line_width_set(1.0)
            state.blend_set('NONE') 
       

            
            

    



   


    

classes = [
    GP_GT_preselect,
    ]

def register():
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)