import bpy, gpu
from bpy.types import Gizmo
from mathutils import Matrix, Vector
from ...utils.draw_utils.draw import draw_batch, draw_3d_dashed
from ...utils.raycast import raycast
from ...matrix.transpose_matrix import transpose_matrix
from gpu import state
from ... import get_prefs



class GP_GT_ruller(Gizmo):
    bl_idname = 'GP_GT_ruller'

    def draw(self, context):
        state.blend_set('ALPHA')
        state.line_width_set(self.line_width)
        settings = context.scene.GP_scene_set
        start = settings.tr_p1
        end = settings.tr_p2
        
        color = (*self.color, self.alpha)

        draw_3d_dashed([start, end], color, 'LINES', dash_width=5.0, dash_factor=0.5)

        state.line_width_set(1.0)
        state.blend_set('NONE')
    


def translate_end_dial(self, context, event, pos=1):
    mouse = event.mouse_region_x, event.mouse_region_y
    success, location, normal, face_index, obj, matrix = raycast(context, mouse)
    
    if pos == 1:
        self.settings.tr_p1 = location
    else:
        self.settings.tr_p2 = location

 
    target =  Vector(self.settings.tr_p1) - Vector(self.settings.tr_p2)
    self.settings.transpose_direction = target

    if success:
        self.settings.use_align = True
        self.settings.use_global_mat = False
    else:
        self.settings.use_global_mat = True
        self.settings.use_align = False



# --- Axis
class GP_GT_arrow(Gizmo):
    bl_idname = 'GP_GT_arrow'

    __slots__ = (
        'init_mes_pos',
        'init_end_pos',
        'props',
        'settings',
        'location',
        'normal',

        'xN',
        'xP',
        'yN',
        'yP',
        'zN',
        'zP',
    )

    def setup(self):
        self.props = get_prefs()
        self.settings = bpy.context.scene.GP_scene_set

        self.use_draw_modal = True
       

    def draw(self, context):
        state.blend_set('ALPHA')
        state.line_width_set(self.line_width)
        
        sca = 1 * self.scale_basis

        self.xN = self.matrix_world @ Vector((-sca, 0, 0))
        self.xP = self.matrix_world @ Vector((sca, 0, 0))
        self.yN = self.matrix_world @ Vector((0, -sca, 0))
        self.yP = self.matrix_world @ Vector((0, sca, 0))
        self.zN = self.matrix_world @ Vector((0, 0, -sca))
        self.zP = self.matrix_world @ Vector((0, 0, sca))

        
        xCol = (*self.props.x_axis_color, self.alpha)
        yCol = (*self.props.y_axis_color, self.alpha)
        zCol = (*self.props.z_axis_color, self.alpha)


        zero = self.matrix_world.translation
        draw_batch([zero, self.xP], xCol, 'LINES')
        draw_batch([zero, self.yP], yCol, 'LINES')
        draw_batch([zero, self.zP], zCol, 'LINES')


        state.line_width_set(1.0)
        state.blend_set('NONE')
            


classes = [
    GP_GT_ruller,
    GP_GT_arrow,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)