import bpy
from mathutils import Matrix, Vector, Quaternion
from bpy.types import GizmoGroup
from ..utils.poll import poll_gizmopro, poll_luna, poll_malibu
from ..matrix.matrix_ensure import get_orient_from_type, get_position
from .. import var
from ..utils.draw_utils.through import alpha_gizmo, alpha_arc_gizmo
from math import radians, atan2
from ..utils.custom_math.quaternion import look_rotation
from ..keymaps import keymaps_gizmo_tweak
from .. import get_prefs



# --- Standard Gizmo
standard_gizmo = (
    'GIZMO_GT_arrow_2d',
    'GIZMO_GT_arrow_3d', # draw_style: NORMAL | CROSS | BOX| CONE , transform: INVERT| CONSTRAIN props: offset
    'GIZMO_GT_blank_3d',
    'GIZMO_GT_button_2d', # props: icon | shape| show_drag , draw_style: OUTLINE | BACKDROP| HELPLINE
    'GIZMO_GT_cage_2d',  # props: dimensions , draw_style: BOX | CIRCLE
    'GIZMO_GT_cage_3d',
    'GIZMO_GT_dial_3d',
    'GIZMO_GT_move_3d',
    'GIZMO_GT_primitive_3d', # plane
    'GIZMO_GT_value_2d', # props: offset
    'VIEW3D_GT_navigate_rotate',
    'GIZMO_GT_mesh_preselect_elem_3d', # props: object_index | vert_index| edge_index | face_index
    'GIZMO_GT_mesh_preselect_edgering_3d',
    'VIEW3D_GT_ruler_item',
)


# --- общие настройки
use_grab_cursor = False
use_draw_value = True



class GP_GGT_general(GizmoGroup):
    bl_label = 'Gizmo PRO'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW' 
    bl_options = {'3D', 'PERSISTENT', 'SHOW_MODAL_ALL', 'TOOL_FALLBACK_KEYMAP'} #'DEPTH_3D' , 'TOOL_INIT', 'SELECT', , 'SCALE'  , 'SHOW_MODAL_ALL' , 'PERSISTENT' , VR_REDRAWS 

    @classmethod
    def poll(cls, context):
        if cls.bl_idname == 'GP_GGT_gizmo_pro_3d':
            return poll_gizmopro(context)
        elif cls.bl_idname == 'GP_GGT_luna':
            return poll_luna(context)
        elif cls.bl_idname == 'GP_GGT_malibu':
            return poll_malibu(context)

    @classmethod
    def setup_keymap(cls, keyconfig):
        return keymaps_gizmo_tweak(keyconfig)



class GP_GGT_gizmo_pro_3d(GP_GGT_general):
    bl_idname = 'GP_GGT_gizmo_pro_3d'


    def setup(self, context):

        self.trackball = self.gizmos.new('GIZMO_GT_move_3d')
        self.trackball.draw_options = {'ALIGN_VIEW'}
        self.trackball.use_tooltip = False
        self.trackball.use_draw_modal = True
        self.trackball_op = self.trackball.target_set_operator('gp.trackball_operator')


        self.rot_view = self.gizmos.new('GP_GT_arc')
        self.rot_view.align_view = True
        self.dvo = self.rot_view.target_set_operator('gp.arc_operator')
        self.dvo.axis = 'VIEW'
        
       
        self.sca_view = self.gizmos.new('GP_GT_box')
        self.sca_view.align_view = True
        self.svo = self.sca_view.target_set_operator('gp.box_operator')
        self.svo.axis = 'VIEW'
  

        self.blank_X = self.gizmos.new('GIZMO_GT_primitive_3d')
        #self.blank_X.draw_style = 'CIRCLE'
        #self.blank_X.use_draw_value = True
        self.blankxo = self.blank_X.target_set_operator('gp.blank_operator')
        self.blankxo.axis = 'X'

        self.blank_Y = self.gizmos.new('GIZMO_GT_primitive_3d')
        #self.blank_Y.draw_style = 'CIRCLE'
        #self.blank_Y.use_draw_value = True
        self.blankyo = self.blank_Y.target_set_operator('gp.blank_operator')
        self.blankyo.axis = 'Y'

        self.blank_Z = self.gizmos.new('GIZMO_GT_primitive_3d')
        #self.blank_Z.draw_style = 'CIRCLE'
        #self.blank_Z.use_draw_value = True
        self.blankzo = self.blank_Z.target_set_operator('gp.blank_operator')
        self.blankzo.axis = 'Z'


        self.arrow_x = self.gizmos.new('GP_GT_cone')
        self.axo = self.arrow_x.target_set_operator('gp.cone_operator')
        self.axo.axis = 'X'
      
        self.arrow_y = self.gizmos.new('GP_GT_cone')
        self.ayo = self.arrow_y.target_set_operator('gp.cone_operator')
        self.ayo.axis = 'Y'
        
        self.arrow_z = self.gizmos.new('GP_GT_cone')
        self.azo = self.arrow_z.target_set_operator('gp.cone_operator')
        self.azo.axis = 'Z'
    

        


        self.dial_x = self.gizmos.new('GP_GT_arc')
        self.dxo = self.dial_x.target_set_operator('gp.arc_operator')
        self.dxo.axis = 'X'
     
        self.dial_y = self.gizmos.new('GP_GT_arc')
        self.dyo = self.dial_y.target_set_operator('gp.arc_operator')
        self.dyo.axis = 'Y'
   
        self.dial_z = self.gizmos.new('GP_GT_arc')
        self.dzo = self.dial_z.target_set_operator('gp.arc_operator')
        self.dzo.axis = 'Z'



        self.scale_x = self.gizmos.new('GP_GT_box')
        self.sxo = self.scale_x.target_set_operator('gp.box_operator')
        self.sxo.axis = 'X'
      
        self.scale_y = self.gizmos.new('GP_GT_box')
        self.syo = self.scale_y.target_set_operator('gp.box_operator')
        self.syo.axis = 'Y'
     
        self.scale_z = self.gizmos.new('GP_GT_box')
        self.szo = self.scale_z.target_set_operator('gp.box_operator')
        self.szo.axis = 'Z'



        self.dots_view = self.gizmos.new('GIZMO_GT_move_3d')
        self.dots_view.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.dot_op = self.dots_view.target_set_operator('gp.dot_operator')


        # --- Modo Plane
        self.modo_plane = self.gizmos.new('GIZMO_GT_primitive_3d')
        self.modo_plane.hide_select = True
        self.modo_plane.color = (0.3, 0.3, 0.3)
        self.modo_plane.alpha = 0.7


        # --- Set For All
        for gizmo in self.gizmos:
            gizmo.use_draw_value = use_draw_value
            gizmo.use_tooltip = False
            gizmo.use_draw_offset_scale = True
            gizmo.use_draw_modal = True
        

    def invoke_prepare(self, context, gizmo):
        matrix = gizmo.matrix_basis
        if gizmo.bl_idname == 'GP_GT_cone':
            self.axo.matrix_basis = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
            self.ayo.matrix_basis = matrix
            self.azo.matrix_basis = matrix

        elif gizmo.bl_idname == 'GP_GT_box':
            self.svo.matrix_basis = matrix
            self.sxo.matrix_basis = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
            self.syo.matrix_basis = matrix
            self.szo.matrix_basis = matrix

        elif gizmo.bl_idname == 'GP_GT_arc':
            self.dvo.matrix_basis = matrix
            self.dxo.matrix_basis = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
            self.dyo.matrix_basis = matrix
            self.dzo.matrix_basis = matrix

        elif gizmo.bl_idname == 'GIZMO_GT_move_3d':
            self.trackball_op.matrix_basis = matrix
            self.dot_op.matrix_basis = matrix
        
        elif gizmo.bl_idname == 'GIZMO_GT_primitive_3d':
            self.blankxo.matrix_basis = self.arrow_x.matrix_basis @ Matrix.Rotation(radians(180), 4, 'Y')
            self.blankyo.matrix_basis = self.arrow_y.matrix_basis
            self.blankzo.matrix_basis = self.arrow_z.matrix_basis @ Matrix.Rotation(radians(90), 4, 'Z')


    def refresh(self, context): #refresh update_matrix
        props = get_prefs()
        mXW, mYW, mZW = get_orient_from_type() # , mXW_d, mYW_d, mZW_d 
        #view_matrix, view_matrix_scale = view_orient_matrix(mXW.translation)
        #settings = context.scene.GP_scene_set
        #set.pp = mXW.translation
        
        #print(self.name)
        # --- Transform
        self.arrow_x.matrix_basis = mXW 
        self.arrow_y.matrix_basis = mYW 
        self.arrow_z.matrix_basis = mZW 

        # --- Rotate  
        self.dial_x.matrix_basis = mXW @ Matrix.Rotation(radians(props.arc_angle[0]), 4, 'Z')
        self.dial_y.matrix_basis = mYW @ Matrix.Rotation(radians(props.arc_angle[1]), 4, 'Z')
        self.dial_z.matrix_basis = mZW @ Matrix.Rotation(radians(props.arc_angle[2]), 4, 'Z') 

        # --- Scale
        self.scale_x.matrix_basis = mXW 
        self.scale_y.matrix_basis = mYW 
        self.scale_z.matrix_basis = mZW 


        # --- DOT
        self.dots_view.matrix_basis = mZW #view_matrix
       
        # --- TRACKBALL
        self.trackball.matrix_basis = mXW #view_matrix 
       
        # --- ROTATE ALL
        self.rot_view.matrix_basis = mXW #view_matrix

        # --- SCALE ALL
        self.sca_view.matrix_basis = mXW #view_matrix_scale
 
        # --- EMPTY
        #self.res_empty.matrix_basis = mXW #view_matrix


        # --- BLANK
        self.blank_X.matrix_basis = mXW
        self.blank_Y.matrix_basis = mYW
        self.blank_Z.matrix_basis = mZW


        # --- Modo Plane
        self.modo_plane.matrix_basis.translation = mZW.translation



    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set
        
        # --- принудительное обновление матрицы
        orient_slots = context.window.scene.transform_orientation_slots[0].type
        if var.REFRESH_MAT or orient_slots=='VIEW' or settings.orientation=='VIEW' or settings.position=='VIEW':
            self.refresh(context)
        var.REFRESH_MAT = False

        # --- общие
        for gizmo in self.gizmos:
            gizmo.alpha_highlight = props.highlight_alpha_gizmo
            gizmo.color_highlight = props.select_color
            gizmo.alpha = props.alpha_gizmo



        self.lock_loc = context.object.lock_location
        self.lock_rot = context.object.lock_rotation
        self.lock_sca = context.object.lock_scale


       

        
        
        


     
        self.color_disable = (0.5, 0.5, 0.5)

        if props.hide_at_perp:
            alpha_x, hide_select_x = alpha_gizmo(self.arrow_x.matrix_basis, props)
            alpha_y, hide_select_y = alpha_gizmo(self.arrow_y.matrix_basis, props)
            alpha_z, hide_select_z = alpha_gizmo(self.arrow_z.matrix_basis, props)
            alpha_xd, hide_select_xd = alpha_arc_gizmo(self.dial_x.matrix_basis, props)
            alpha_yd, hide_select_yd = alpha_arc_gizmo(self.dial_y.matrix_basis, props)
            alpha_zd, hide_select_zd = alpha_arc_gizmo(self.dial_z.matrix_basis, props)
        else:
            alpha_x = props.alpha_gizmo
            alpha_y = props.alpha_gizmo
            alpha_z = props.alpha_gizmo
            alpha_xd = props.alpha_gizmo
            alpha_yd = props.alpha_gizmo
            alpha_zd = props.alpha_gizmo
            hide_select_x = False
            hide_select_y = False
            hide_select_z = False
            hide_select_xd = False
            hide_select_yd = False
            hide_select_zd = False

# --- MOVE
        arrow_x = self.arrow_x
        arrow_x.scale_basis = props.size_gizmo_pro * props.size_cone 
        arrow_x.color = props.x_axis_color
        arrow_x.alpha = alpha_x
        arrow_x.hide_select = hide_select_x

        arrow_y = self.arrow_y
        arrow_y.scale_basis = props.size_gizmo_pro * props.size_cone 
        arrow_y.color = props.y_axis_color
        arrow_y.alpha = alpha_y
        arrow_y.hide_select = hide_select_y

        arrow_z = self.arrow_z
        arrow_z.scale_basis = props.size_gizmo_pro * props.size_cone
        arrow_z.color = props.z_axis_color
        arrow_z.alpha = alpha_z
        arrow_z.hide_select = hide_select_z
            
        # --- ROTATE
        dial_x = self.dial_x
        dial_x.scale_basis = props.size_gizmo_pro * props.size_arc
        dial_x.color = props.x_axis_color
        dial_x.alpha = alpha_xd
        dial_x.hide_select = hide_select_xd

        dial_y = self.dial_y
        dial_y.scale_basis = props.size_gizmo_pro * props.size_arc 
        dial_y.color = props.y_axis_color
        dial_y.alpha = alpha_yd
        dial_y.hide_select = hide_select_yd

        dial_z = self.dial_z
        dial_z.scale_basis = props.size_gizmo_pro * props.size_arc 
        dial_z.color = props.z_axis_color
        dial_z.alpha = alpha_zd
        dial_z.hide_select = hide_select_zd


        # --- SCALE
        scale_x = self.scale_x
        scale_x.scale_basis = props.size_gizmo_pro * props.size_box
        scale_x.color = props.x_axis_color
        scale_x.alpha = alpha_x
        scale_x.hide_select = hide_select_x

        scale_y = self.scale_y
        scale_y.scale_basis = props.size_gizmo_pro * props.size_box 
        scale_y.color = props.y_axis_color
        scale_y.alpha = alpha_y
        scale_y.hide_select = hide_select_y

        scale_z = self.scale_z
        scale_z.scale_basis = props.size_gizmo_pro * props.size_box 
        scale_z.color = props.z_axis_color
        scale_z.alpha = alpha_z
        scale_z.hide_select = hide_select_z
            



# --- BLANK
        self.blank_X.scale_basis = props.size_gizmo_pro * props.size_blank
        self.blank_X.color = tuple(min(a + b, 1.0) for a, b in zip(props.y_axis_color, props.z_axis_color)) if props.blank_color else props.x_axis_color
        self.blank_X.matrix_offset = Matrix.Translation( Vector( (-props.offset_blank, props.offset_blank, 0.0) ) )
        self.blank_X.alpha = alpha_xd
        self.blank_X.hide_select = hide_select_xd

        self.blank_Y.scale_basis = props.size_gizmo_pro * props.size_blank
        self.blank_Y.color = tuple(min(a + b, 1.0) for a, b in zip(props.x_axis_color, props.z_axis_color)) if props.blank_color else props.y_axis_color
        self.blank_Y.matrix_offset = Matrix.Translation( Vector( (props.offset_blank, -props.offset_blank, 0.0) ) )
        self.blank_Y.alpha = alpha_yd
        self.blank_Y.hide_select = hide_select_yd

        self.blank_Z.scale_basis = props.size_gizmo_pro * props.size_blank
        self.blank_Z.color = tuple(min(a + b, 1.0) for a, b in zip(props.x_axis_color, props.y_axis_color)) if props.blank_color else props.z_axis_color
        self.blank_Z.matrix_offset = Matrix.Translation( Vector( (-props.offset_blank, props.offset_blank, 0.0) ) )
        self.blank_Z.alpha = alpha_zd
        self.blank_Z.hide_select = hide_select_zd

        if settings.blank_gizmo:
            self.blank_X.hide = False
            self.blank_Y.hide = False
            self.blank_Z.hide = False
        else:
            self.blank_X.hide = True
            self.blank_Y.hide = True
            self.blank_Z.hide = True

        if self.blank_X.is_highlight or self.blank_X.is_modal:
            arrow_y.color = props.select_color
            arrow_z.color = props.select_color
            scale_y.color = props.select_color
            scale_z.color = props.select_color
            dial_x.color =  props.select_color
            arrow_y.alpha = props.highlight_alpha_gizmo
            arrow_z.alpha = props.highlight_alpha_gizmo
            scale_y.alpha = props.highlight_alpha_gizmo
            scale_z.alpha = props.highlight_alpha_gizmo
            dial_x.alpha =  props.highlight_alpha_gizmo
        if self.blank_Y.is_highlight or self.blank_Y.is_modal:
            arrow_x.color = props.select_color
            arrow_z.color = props.select_color
            scale_x.color = props.select_color
            scale_z.color = props.select_color
            dial_y.color =  props.select_color
            arrow_x.alpha = props.highlight_alpha_gizmo
            arrow_z.alpha = props.highlight_alpha_gizmo
            scale_x.alpha = props.highlight_alpha_gizmo
            scale_z.alpha = props.highlight_alpha_gizmo
            dial_y.alpha =  props.highlight_alpha_gizmo
        if self.blank_Z.is_highlight or self.blank_Z.is_modal:
            arrow_x.color = props.select_color
            arrow_y.color = props.select_color
            scale_x.color = props.select_color
            scale_y.color = props.select_color
            dial_z.color =  props.select_color
            arrow_x.alpha = props.highlight_alpha_gizmo
            arrow_y.alpha = props.highlight_alpha_gizmo
            scale_x.alpha = props.highlight_alpha_gizmo
            scale_y.alpha = props.highlight_alpha_gizmo
            dial_z.alpha =  props.highlight_alpha_gizmo



# --- VISIBLE GIZMO
        if settings.tra_gizmo:
            arrow_x.hide = False
            arrow_y.hide = False
            arrow_z.hide = False
        else:
            arrow_x.hide = True
            arrow_y.hide = True
            arrow_z.hide = True

        if settings.rot_gizmo:
            dial_x.hide = False
            dial_y.hide = False
            dial_z.hide = False
        else:
            dial_x.hide = True
            dial_y.hide = True
            dial_z.hide = True

        if settings.sca_gizmo:
            scale_x.hide = False
            scale_y.hide = False
            scale_z.hide = False
        else:
            scale_x.hide = True
            scale_y.hide = True
            scale_z.hide = True


        # --- LOCK AXIS
        if context.mode == 'OBJECT':
            if self.lock_loc[0]:
                self.arrow_x.hide = True
                self.blank_Y.hide = True
                self.blank_Z.hide = True

            if self.lock_loc[1]:
                self.arrow_y.hide = True
                self.blank_X.hide = True
                self.blank_Z.hide = True

            if self.lock_loc[2]:
                self.arrow_z.hide = True
                self.blank_X.hide = True
                self.blank_Y.hide = True


            if self.lock_rot[0]:
                self.dial_x.hide = True
            if self.lock_rot[1]:
                self.dial_y.hide = True
            if self.lock_rot[2]:
                self.dial_z.hide = True

            if self.lock_sca[0]:
                self.scale_x.hide = True
            if self.lock_sca[1]:
                self.scale_y.hide = True
            if self.lock_sca[2]:
                self.scale_z.hide = True







        # --- DOT 
        dots_view = self.dots_view
        if settings.dot_move:
            dots_view.hide = False
            dots_view.color = props.elements_color
            dots_view.line_width = props.size_gizmo_pro * 2.0 * props.lw_dot
            if props.dot_action == 'ADVANCE' and dots_view.is_highlight:
                dots_view.scale_basis = props.size_gizmo_pro * 0.3 * props.size_dot
                dots_view.draw_options = {'FILL_SELECT'}
            else:
                dots_view.scale_basis = props.size_gizmo_pro * 0.15 * props.size_dot
                dots_view.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        else:
            dots_view.hide = True
            


        # --- TRACKBALL
        trackball = self.trackball
        if settings.rotate_trackball:
            trackball.hide = False
            trackball.scale_basis = props.size_gizmo_pro * 1.5 * props.size_trackball
            trackball.color = props.elements_color
            trackball.line_width = props.size_gizmo_pro * 2.0 * props.lw_trackball
        else:
            trackball.hide = True

        if self.trackball.is_highlight or self.trackball.is_modal:
            dial_x.color = props.select_color
            dial_y.color = props.select_color
            dial_z.color = props.select_color


# --- ROTATE ALL  
        rot_view = self.rot_view
        if settings.rotate_view:
            rot_view.hide = False
            rot_view.scale_basis = props.size_gizmo_pro
            rot_view.color = props.elements_color
            #rot_view.matrix_basis = view_matrix
            #rot_view.matrix_offset = Matrix.Translation(Vector((self.props.offset_view, 0.0, 0.0)))
        
        else:
            rot_view.hide = True

        # --- SCALE ALL
        sca_view = self.sca_view
        if settings.scale_view:
            sca_view.hide = False
            sca_view.scale_basis = props.size_gizmo_pro 
            sca_view.color = props.elements_color
        else:
            sca_view.hide = True


        # --- MODO PLANE
        if props.space_drag_lmb == 'TRANSFORM':
            self.modo_plane.hide = False
            self.modo_plane.scale_basis = props.size_gizmo_pro * props.size_cone
            view_normal = bpy.context.region_data.view_rotation @ Vector((0.0, 0.0, -1.0))

            q = self.arrow_z.matrix_basis.decompose()[1]
            vec_x = (q.to_matrix() @ Vector((1,0,0))).normalized()
            vec_y = (q.to_matrix() @ Vector((0,1,0))).normalized()
            vec_z = (q.to_matrix() @ Vector((0,0,1))).normalized()

            dot_x = abs(view_normal.dot(vec_x))
            dot_y = abs(view_normal.dot(vec_y))
            dot_z = abs(view_normal.dot(vec_z))

            max_dot = max(dot_x, dot_y, dot_z)

            if max_dot == dot_x:
                self.modo_plane.matrix_basis = self.arrow_y.matrix_basis
            elif max_dot == dot_y:
                self.modo_plane.matrix_basis = self.arrow_x.matrix_basis
            else:
                self.modo_plane.matrix_basis = self.arrow_z.matrix_basis

        else:
            self.modo_plane.hide = True




import time


start_time = None

def ease_in_out_cubic(t, b, c, d): # медленное начало и окончание
    t /= d / 2
    if t < 1:
        return c / 2 * t * t * t + b
    t -= 2
    return c / 2 * (t * t * t + 2) + b

def ease_out_cubic(t, b, c, d): # медленное окончание
    t = t / d - 1
    return c * (t * t * t + 1) + b

def ease_out_quint(t, b, c, d): # более медленное окончание
    t = t / d - 1
    return c * (t * t * t * t * t + 1) + b

def smooth_rotation(start_value, end_value, duration, is_continuing):
    global start_time
    if not is_continuing or start_time is None:
        start_time = time.time()

    current_time = time.time() - start_time
    current_time = min(current_time, duration)  # Ограничьте current_time, чтобы он не превышал duration
    change_in_value = end_value - start_value
    return ease_out_quint(current_time, start_value, change_in_value, duration)




class GP_GGT_luna(GP_GGT_general):
    bl_idname = 'GP_GGT_luna'
    bl_options = {'3D', 'PERSISTENT', 'SHOW_MODAL_ALL', 'VR_REDRAWS', 'TOOL_FALLBACK_KEYMAP'}

    def setup(self, context):
        self.apa_X_anim = False
        self.apa_Y_anim = False
        self.apa_Z_anim = False


        # --- TRACKBALL
        self.trackball = self.gizmos.new('GIZMO_GT_move_3d')
        self.trackball.draw_options = {'ALIGN_VIEW'}
        self.trackball_op = self.trackball.target_set_operator('gp.trackball_operator')

        # --- BLANK
        self.blank_X = self.gizmos.new('GP_GT_blank') # 'GIZMO_GT_primitive_3d'
        #self.blank_X.draw_style = 'CIRCLE' # 'PLANE'
        self.blankxo = self.blank_X.target_set_operator('gp.blank_operator')
        self.blankxo.axis = 'X'
        self.blank_Y = self.gizmos.new('GP_GT_blank')
        #self.blank_Y.draw_style = 'CIRCLE' # 'PLANE'
        self.blankyo = self.blank_Y.target_set_operator('gp.blank_operator')
        self.blankyo.axis = 'Y'
        self.blank_Z = self.gizmos.new('GP_GT_blank')
        #self.blank_Z.draw_style = 'CIRCLE' # 'PLANE'
        self.blankzo = self.blank_Z.target_set_operator('gp.blank_operator')
        self.blankzo.axis = 'Z'

        # --- MOVE
        self.arrow_x = self.gizmos.new('GIZMO_GT_arrow_3d')
        #self.arrow_x.draw_options = {'STEM', 'ORIGIN'}
        #self.arrow_x.aspect = (0.5, 0.5)
        self.axo = self.arrow_x.target_set_operator('gp.cone_operator')
        self.axo.axis = 'X'

        self.arrow_y = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.ayo = self.arrow_y.target_set_operator('gp.cone_operator')
        self.ayo.axis = 'Y'
        
        self.arrow_z = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.azo = self.arrow_z.target_set_operator('gp.cone_operator')
        self.azo.axis = 'Z'
    
        # --- SCALE
        self.scale_x = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.scale_x.draw_style = 'PLANE'
        self.sxo = self.scale_x.target_set_operator('gp.box_operator')
        self.sxo.axis = 'X'
      
        self.scale_y = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.scale_y.draw_style = 'PLANE'
        self.syo = self.scale_y.target_set_operator('gp.box_operator')
        self.syo.axis = 'Y'
     
        self.scale_z = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.scale_z.draw_style = 'PLANE'
        self.szo = self.scale_z.target_set_operator('gp.box_operator')
        self.szo.axis = 'Z'

        # --- ROTATE
        self.dial_x = self.gizmos.new('GIZMO_GT_dial_3d')
        self.dxo = self.dial_x.target_set_operator('gp.arc_operator')
        self.dxo.axis = 'X'
     
        self.dial_y = self.gizmos.new('GIZMO_GT_dial_3d')
        self.dyo = self.dial_y.target_set_operator('gp.arc_operator')
        self.dyo.axis = 'Y'
   
        self.dial_z = self.gizmos.new('GIZMO_GT_dial_3d')
        self.dzo = self.dial_z.target_set_operator('gp.arc_operator')
        self.dzo.axis = 'Z'
     
        # --- DOT MOVE
        self.dots_view = self.gizmos.new('GIZMO_GT_move_3d')
        self.dots_view.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.dot_op = self.dots_view.target_set_operator('gp.dot_operator')

        
        # --- Modo Plane
        self.modo_plane = self.gizmos.new('GIZMO_GT_primitive_3d')
        self.modo_plane.hide_select = True
        self.modo_plane.color = (0.3, 0.3, 0.3)
        self.modo_plane.alpha = 0.7
        

        # --- Set For All
        for gizmo in self.gizmos:
            gizmo.use_draw_value = use_draw_value
            gizmo.use_tooltip = False
            gizmo.use_draw_offset_scale = True
            gizmo.use_draw_modal = True

        
    def invoke_prepare(self, context, gizmo):
        matrix = gizmo.matrix_basis
        if gizmo.bl_idname == 'GIZMO_GT_arrow_3d':
            if self.dX>0:
                matrix_x = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
            else:
                matrix_x = matrix
            if self.dY>0:
                matrix_y = matrix
            else:
                matrix_y = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
            if self.dZ>0:
                matrix_z = matrix 
            else:
                matrix_z = matrix @ Matrix.Rotation(radians(180), 4, 'Y')

            self.axo.matrix_basis = matrix_x
            self.ayo.matrix_basis = matrix_y
            self.azo.matrix_basis = matrix_z
            self.sxo.matrix_basis = matrix_x
            self.syo.matrix_basis = matrix_y
            self.szo.matrix_basis = matrix_z

        elif gizmo.bl_idname == 'GIZMO_GT_dial_3d':
            if self.dY>0:
                if self.dX>0:
                    matrix_x = matrix
                else:
                    matrix_x = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
            else:
                if self.dX>0:
                    matrix_x = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
                else:
                    matrix_x = matrix

            if self.dZ>0:
                if self.dY>0:
                    matrix_y = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
                else:
                    matrix_y = matrix
            else:
                if self.dY>0:
                    matrix_y = matrix @ Matrix.Rotation(radians(180), 4, 'Z')
                else:
                    matrix_y = matrix @ Matrix.Rotation(radians(180), 4, 'Y')

            if self.dX>0:
                if self.dZ>0:
                    matrix_z = matrix 
                else:
                    matrix_z = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
            else:
                if self.dZ>0:
                    matrix_z = matrix @ Matrix.Rotation(radians(180), 4, 'Z')
                else:
                    matrix_z = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
            
            self.dxo.matrix_basis = matrix_x
            self.dyo.matrix_basis = matrix_y
            self.dzo.matrix_basis = matrix_z

        elif gizmo.bl_idname == 'GIZMO_GT_move_3d':
            self.trackball_op.matrix_basis = matrix
            self.dot_op.matrix_basis = matrix
            
        elif gizmo.bl_idname == 'GP_GT_blank': # GIZMO_GT_primitive_3d
            self.blankxo.matrix_basis = self.arrow_x.matrix_basis @ Matrix.Rotation(radians(180), 4, 'Y')
            self.blankyo.matrix_basis = self.arrow_y.matrix_basis
            self.blankzo.matrix_basis = self.arrow_z.matrix_basis @ Matrix.Rotation(radians(90), 4, 'Z')


    def refresh(self, context): #refresh update_matrix
        settings = context.scene.GP_scene_set

        # --- get position
        loc = get_position(context)
        #mXW, mYW, mZW = get_orient_from_type(type='LUNA')
        
        # --- Transform
        self.arrow_x.matrix_basis.translation = loc
        self.arrow_y.matrix_basis.translation = loc
        self.arrow_z.matrix_basis.translation = loc

        # --- Rotate
        self.dial_x.matrix_basis.translation = loc
        self.dial_y.matrix_basis.translation = loc 
        self.dial_z.matrix_basis.translation = loc

        # --- Scale
        self.scale_x.matrix_basis.translation = loc
        self.scale_y.matrix_basis.translation = loc
        self.scale_z.matrix_basis.translation = loc


        # --- DOT
        self.dots_view.matrix_basis.translation = loc
        self.trackball.matrix_basis.translation = loc
       
        # --- EMPTY
        #self.res_empty.matrix_basis.translation = loc # remove TODO


        

        self.blank_X.matrix_basis.translation = loc
        self.blank_Y.matrix_basis.translation = loc
        self.blank_Z.matrix_basis.translation = loc
        
        # --- Modo Plane
        self.modo_plane.matrix_basis.translation = loc


    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set
        region_3d = context.space_data.region_3d

        # --- принудительное обновление матрицы
        if var.REFRESH_MAT or settings.position=='VIEW':
            self.refresh(context)
        var.REFRESH_MAT = False

        # --- Get Orientation
        l = self.arrow_x.matrix_basis.translation
        qX, qY, qZ, self.dX, self.dY, self.dZ, qX_orig, qY_orig, qZ_orig = get_orient_from_type(type='LUNA')
        s = Vector((1.0, 1.0, 1.0))



        # --- общие
        for gizmo in self.gizmos:
            gizmo.alpha_highlight = props.highlight_alpha_gizmo
            gizmo.color_highlight = props.select_color
            gizmo.alpha = props.alpha_gizmo

        
        

        # --- поворот к камере
        if region_3d.is_perspective:
            view_loc = region_3d.view_matrix.inverted().to_translation()
            view_up = view_loc - l
        else:
            qat = context.region_data.view_rotation
            view_up = qat @ Vector((0, 0, 1))

        qX_forward = qX @ Vector((0, 0, 1))
        qY_forward = qY @ Vector((0, 0, 1))
        qZ_forward = qZ @ Vector((0, 0, 1))
        rotation_x = look_rotation(qX_forward, view_up)
        rotation_y = look_rotation(qY_forward, view_up)
        rotation_z = look_rotation(qZ_forward, view_up)




        # --- цвета
        if self.dX>0.0:
            color_x = props.x_axis_color
        else:
            color_x = props.x_axis_color / 2
        if self.dY>0.0:
            color_y = props.y_axis_color
        else:
            color_y = props.y_axis_color / 2
        if self.dZ>0.0:
            color_z = props.z_axis_color
        else:
            color_z = props.z_axis_color / 2


        


        coef = 1.2
        l_size_a = props.luna_size * coef
        l_size_d = props.luna_size * coef
        l_dot_size = props.luna_size * 0.2 * coef
        l_plane_size = props.luna_size * 0.2 * coef
        l_lw_a = props.luna_size * 2.0 * props.luna_lw
        l_lw_s = props.luna_size * 1.5 * props.luna_lw
        l_lw_r = props.luna_size * 4.0 * props.luna_lw
        bo = 4.0
        arc_partial_angle = radians(330)
        arc_partial_angle_select = radians(270)
        anim_duration = 0.3


        l_length_a = 0.8 #+ props.luna_offset
        l_length_s = 1.06
        l_mo_a = Matrix.Translation( Vector( (0.0, 0.0, 0.2) ) )
        l_mo_r = Matrix.Translation( Vector( (0.0, 0.3 + props.luna_offset/10, 0.0) ) )
        l_mo_s = Matrix.Translation( Vector( (0.0, 0.0, 0.3 + (props.luna_offset/10)*2) ) )


        lock_loc = context.object.lock_location
        lock_rot = context.object.lock_rotation
        lock_sca = context.object.lock_scale


        
        
        

        if props.hide_at_perp:
            alpha_x, hide_select_x = alpha_gizmo(self.arrow_x.matrix_basis, props)
            alpha_y, hide_select_y = alpha_gizmo(self.arrow_y.matrix_basis, props)
            alpha_z, hide_select_z = alpha_gizmo(self.arrow_z.matrix_basis, props)
            alpha_xd, hide_select_xd = alpha_arc_gizmo(self.dial_x.matrix_basis, props)
            alpha_yd, hide_select_yd = alpha_arc_gizmo(self.dial_y.matrix_basis, props)
            alpha_zd, hide_select_zd = alpha_arc_gizmo(self.dial_z.matrix_basis, props)
        else:
            alpha_x = props.alpha_gizmo
            alpha_y = props.alpha_gizmo
            alpha_z = props.alpha_gizmo
            alpha_xd = props.alpha_gizmo
            alpha_yd = props.alpha_gizmo
            alpha_zd = props.alpha_gizmo
            hide_select_x = False
            hide_select_y = False
            hide_select_z = False
            hide_select_xd = False
            hide_select_yd = False
            hide_select_zd = False




        # --- MOVE
        arrow_x = self.arrow_x
        arrow_x.length = l_length_a
        arrow_x.line_width = l_lw_a
        arrow_x.alpha = alpha_x
        arrow_x.hide_select = hide_select_x
        arrow_x.scale_basis = l_size_a
        arrow_x.color = color_x
        arrow_x.matrix_offset = l_mo_a
        arrow_x.matrix_basis = Matrix.LocRotScale(l, qX, s)

        arrow_y = self.arrow_y
        arrow_y.length = l_length_a
        arrow_y.line_width = l_lw_a
        arrow_y.alpha = alpha_y
        arrow_y.hide_select = hide_select_y
        arrow_y.scale_basis = l_size_a
        arrow_y.color = color_y
        arrow_y.matrix_offset = l_mo_a
        arrow_y.matrix_basis = Matrix.LocRotScale(l, qY, s)

        arrow_z = self.arrow_z
        arrow_z.length = l_length_a
        arrow_z.line_width = l_lw_a
        arrow_z.alpha = alpha_z
        arrow_z.hide_select = hide_select_z
        arrow_z.scale_basis = l_size_a
        arrow_z.color = color_z
        arrow_z.matrix_offset = l_mo_a
        arrow_z.matrix_basis = Matrix.LocRotScale(l, qZ, s)
        
            
        # --- ROTATE
        dial_x = self.dial_x
        dial_x.line_width = l_lw_r
        dial_x.alpha = alpha_xd
        dial_x.hide_select = hide_select_xd
        dial_x.scale_basis = l_size_d
        dial_x.color = props.elements_color # color_x32
        if dial_x.is_highlight or dial_x.is_modal:
            dial_x.arc_partial_angle = smooth_rotation(dial_x.arc_partial_angle, arc_partial_angle_select, anim_duration, self.apa_X_anim)
            self.apa_X_anim = True
        else:
            dial_x.arc_partial_angle = smooth_rotation(dial_x.arc_partial_angle, arc_partial_angle, anim_duration, not self.apa_X_anim) 
            self.apa_X_anim = False
        dial_x.matrix_offset = l_mo_r
        if self.dY>0:
            qX_invert = qX @ Quaternion((0.0, 1.0, 0.0), radians(180))
        else:
            qX_invert = qX @ Quaternion((0.0, 0.0, 1.0), radians(180))
        dial_x.matrix_basis = Matrix.LocRotScale(l, qX_invert, s)


        dial_y = self.dial_y
        dial_y.line_width = l_lw_r
        dial_y.alpha = alpha_yd
        dial_y.hide_select = hide_select_yd
        dial_y.scale_basis = l_size_d
        dial_y.color = props.elements_color # color_y
        if dial_y.is_highlight or dial_y.is_modal:
            dial_y.arc_partial_angle = smooth_rotation(dial_y.arc_partial_angle, arc_partial_angle_select, anim_duration, self.apa_Y_anim)
            self.apa_Y_anim = True
        else:
            dial_y.arc_partial_angle = smooth_rotation(dial_y.arc_partial_angle, arc_partial_angle, anim_duration, not self.apa_Y_anim) 
            self.apa_Y_anim = False
        dial_y.matrix_offset = l_mo_r
        if self.dZ>0:
            qY_invert = qY @ Quaternion((1.0, 0.0, 0.0), radians(180))
        else:
            qY_invert = qY
        dial_y.matrix_basis = Matrix.LocRotScale(l, qY_invert, s)

        dial_z = self.dial_z
        dial_z.line_width = l_lw_r
        dial_z.alpha = alpha_zd
        dial_z.hide_select = hide_select_zd
        dial_z.scale_basis = l_size_d
        dial_z.color = props.elements_color # color_z
        if dial_z.is_highlight or dial_z.is_modal:
            dial_z.arc_partial_angle = smooth_rotation(dial_z.arc_partial_angle, arc_partial_angle_select, anim_duration, self.apa_Z_anim)
            self.apa_Z_anim = True
        else:
            dial_z.arc_partial_angle = smooth_rotation(dial_z.arc_partial_angle, arc_partial_angle, anim_duration, not self.apa_Z_anim) 
            self.apa_Z_anim = False
        dial_z.matrix_offset = l_mo_r
        if self.dX>0:
            qZ_invert = qZ
        else:
            qZ_invert = qZ @ Quaternion((0.0, 0.0, 1.0), radians(180))
        dial_z.matrix_basis = Matrix.LocRotScale(l, qZ_invert, s)

        # --- SCALE
        scale_x = self.scale_x
        scale_x.line_width = l_lw_s
        scale_x.length = l_length_s
        scale_x.alpha = alpha_x
        scale_x.hide_select = hide_select_x
        scale_x.scale_basis = l_size_a
        scale_x.color = color_x
        scale_x.matrix_offset = l_mo_s
        scale_x.matrix_basis = Matrix.LocRotScale(l, rotation_x, s)

        scale_y = self.scale_y
        scale_y.line_width = l_lw_s
        scale_y.length = l_length_s
        scale_y.alpha = alpha_y
        scale_y.hide_select = hide_select_y
        scale_y.scale_basis = l_size_a
        scale_y.color = color_y
        scale_y.matrix_offset = l_mo_s
        scale_y.matrix_basis = Matrix.LocRotScale(l, rotation_y, s)

        scale_z = self.scale_z
        scale_z.line_width = l_lw_s
        scale_z.length = l_length_s
        scale_z.alpha = alpha_z
        scale_z.hide_select = hide_select_z
        scale_z.scale_basis = l_size_a
        scale_z.color = color_z
        scale_z.matrix_offset = l_mo_s
        scale_z.matrix_basis = Matrix.LocRotScale(l, rotation_z, s)


        # --- BLANK
        self.blank_X.scale_basis = l_plane_size
        self.blank_X.line_width = l_lw_s
        self.blank_X.color = tuple(min(a + b, 1.0) for a, b in zip(props.y_axis_color, props.z_axis_color)) if props.blank_color else props.x_axis_color#props.y_axis_color + props.z_axis_color if props.blank_color else props.x_axis_color
        self.blank_X.alpha = alpha_xd
        self.blank_X.hide_select = hide_select_xd

        self.blank_Y.scale_basis = l_plane_size
        self.blank_Y.line_width = l_lw_s
        self.blank_Y.color = tuple(min(a + b, 1.0) for a, b in zip(props.x_axis_color, props.z_axis_color)) if props.blank_color else  props.y_axis_color
        self.blank_Y.alpha = alpha_yd
        self.blank_Y.hide_select = hide_select_yd

        self.blank_Z.scale_basis = l_plane_size
        self.blank_Z.line_width = l_lw_s
        self.blank_Z.color = tuple(min(a + b, 1.0) for a, b in zip(props.x_axis_color, props.y_axis_color)) if props.blank_color else props.z_axis_color
        self.blank_Z.alpha = alpha_zd
        self.blank_Z.hide_select = hide_select_zd


        


        # --- BLANK X
        if self.dY>0:
            if self.dZ>0:
                self.blank_X.matrix_basis = Matrix.LocRotScale(l, qX_orig, s)
            else:
                q_invert = qX_orig @ Quaternion((0.0, 1.0, 0.0), radians(180))
                self.blank_X.matrix_basis = Matrix.LocRotScale(l, q_invert, s)
        else:
            if self.dZ>0:
                q_invert = qX_orig @ Quaternion((1.0, 0.0, 0.0), radians(180))
                self.blank_X.matrix_basis = Matrix.LocRotScale(l, q_invert, s)
            else:
                q_invert = qX_orig @ Quaternion((0.0, 0.0, 1.0), radians(180))
                self.blank_X.matrix_basis = Matrix.LocRotScale(l, q_invert, s)

        # --- BLANK Y
        if self.dX>0:
            if self.dZ>0:
                q_invert = qY_orig @ Quaternion((0.0, 0.0, 1.0), radians(180))
                self.blank_Y.matrix_basis = Matrix.LocRotScale(l, q_invert, s)
            else:
                q_invert = qY_orig @ Quaternion((0.0, 1.0, 0.0), radians(180))
                self.blank_Y.matrix_basis = Matrix.LocRotScale(l, q_invert, s)
        else:
            if self.dZ>0:
                q_invert = qY_orig @ Quaternion((1.0, 0.0, 0.0), radians(180))
                self.blank_Y.matrix_basis = Matrix.LocRotScale(l, q_invert, s)
            else:
                self.blank_Y.matrix_basis = Matrix.LocRotScale(l, qY_orig, s)

        # --- BLANK Z
        if self.dX>0:
            if self.dY>0:
                self.blank_Z.matrix_basis = Matrix.LocRotScale(l, qZ_orig, s)
            else:
                q_invert = qZ_orig @ Quaternion((0.0, 1.0, 0.0), radians(180))
                self.blank_Z.matrix_basis = Matrix.LocRotScale(l, q_invert, s)
        else:
            if self.dY>0:
                q_invert = qZ_orig @ Quaternion((1.0, 0.0, 0.0), radians(180))
                self.blank_Z.matrix_basis = Matrix.LocRotScale(l, q_invert, s)
            else:
                q_invert = qZ_orig @ Quaternion((0.0, 0.0, 1.0), radians(180))
                self.blank_Z.matrix_basis = Matrix.LocRotScale(l, q_invert, s)



        self.blank_X.matrix_offset = Matrix.Translation(Vector((-bo, bo, 0.0)))
        self.blank_Y.matrix_offset = Matrix.Translation(Vector((-bo, bo, 0.0)))
        self.blank_Z.matrix_offset = Matrix.Translation(Vector((-bo, bo, 0.0)))

        if self.blank_X.is_highlight or self.blank_X.is_modal:
            arrow_y.color = props.select_color
            arrow_z.color = props.select_color
            scale_y.color = props.select_color
            scale_z.color = props.select_color
            dial_x.color =  props.select_color
            arrow_y.alpha = props.highlight_alpha_gizmo
            arrow_z.alpha = props.highlight_alpha_gizmo
            scale_y.alpha = props.highlight_alpha_gizmo
            scale_z.alpha = props.highlight_alpha_gizmo
            dial_x.alpha =  props.highlight_alpha_gizmo
        if self.blank_Y.is_highlight or self.blank_Y.is_modal:
            arrow_x.color = props.select_color
            arrow_z.color = props.select_color
            scale_x.color = props.select_color
            scale_z.color = props.select_color
            dial_y.color =  props.select_color
            arrow_x.alpha = props.highlight_alpha_gizmo
            arrow_z.alpha = props.highlight_alpha_gizmo
            scale_x.alpha = props.highlight_alpha_gizmo
            scale_z.alpha = props.highlight_alpha_gizmo
            dial_y.alpha =  props.highlight_alpha_gizmo
        if self.blank_Z.is_highlight or self.blank_Z.is_modal:
            arrow_x.color = props.select_color
            arrow_y.color = props.select_color
            scale_x.color = props.select_color
            scale_y.color = props.select_color
            dial_z.color =  props.select_color
            arrow_x.alpha = props.highlight_alpha_gizmo
            arrow_y.alpha = props.highlight_alpha_gizmo
            scale_x.alpha = props.highlight_alpha_gizmo
            scale_y.alpha = props.highlight_alpha_gizmo
            dial_z.alpha =  props.highlight_alpha_gizmo
        




        




        # --- VISIBLE GIZMO
        if settings.tra_gizmo:
            arrow_x.hide = False
            arrow_y.hide = False
            arrow_z.hide = False
        else:
            arrow_x.hide = True
            arrow_y.hide = True
            arrow_z.hide = True

        if settings.rot_gizmo:
            dial_x.hide = False
            dial_y.hide = False
            dial_z.hide = False
        else:
            dial_x.hide = True
            dial_y.hide = True
            dial_z.hide = True

        if settings.sca_gizmo:
            scale_x.hide = False
            scale_y.hide = False
            scale_z.hide = False
        else:
            scale_x.hide = True
            scale_y.hide = True
            scale_z.hide = True

        

       
        if settings.tra_gizmo:
            self.blank_X.hide = False
            self.blank_Y.hide = False
            self.blank_Z.hide = False
        else:
            self.blank_X.hide = True
            self.blank_Y.hide = True
            self.blank_Z.hide = True
        
            

        # --- LOCK AXIS
        if context.mode=='OBJECT':
            if lock_loc[0]:
                self.arrow_x.hide = True
                self.blank_Y.hide = True
                self.blank_Z.hide = True

            if lock_loc[1]:
                self.arrow_y.hide = True
                self.blank_X.hide = True
                self.blank_Z.hide = True

            if lock_loc[2]:
                self.arrow_z.hide = True
                self.blank_X.hide = True
                self.blank_Y.hide = True


            if lock_rot[0]:
                self.dial_x.hide = True
            if lock_rot[1]:
                self.dial_y.hide = True
            if lock_rot[2]:
                self.dial_z.hide = True

            if lock_sca[0]:
                self.scale_x.hide = True
            if lock_sca[1]:
                self.scale_y.hide = True
            if lock_sca[2]:
                self.scale_z.hide = True







        # --- DOT 
        dots_view = self.dots_view
        if settings.dot_move:
            dots_view.hide = False
            dots_view.color = props.elements_color
            dots_view.line_width = props.luna_size * props.luna_lw * 2.0
            if props.dot_action == 'ADVANCE' and dots_view.is_highlight:
                dots_view.scale_basis = 1.7 * l_dot_size
                dots_view.draw_options = {'FILL_SELECT'}
                dots_view.matrix_basis = Matrix.LocRotScale(l, qZ, s)
            else:
                dots_view.scale_basis = l_dot_size
                dots_view.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}

            orient_slots = context.window.scene.transform_orientation_slots[0].type
            tra_PP = context.scene.tool_settings.transform_pivot_point
            settings = context.scene.GP_scene_set
            if orient_slots == 'CURSOR' or settings.orientation == 'CURSOR':
                dots_view.color = (1.0, 0.5, 0.0)
            if tra_PP == 'CURSOR' or settings.position == 'CURSOR':
                dots_view.color = (1.0, 0.5, 0.0)
        else:
            dots_view.hide = True



        # --- TRACKBALL
        trackball = self.trackball
        if settings.rot_gizmo and settings.rotate_trackball:
            trackball.hide = False
            trackball.scale_basis = l_size_d
            trackball.color = props.elements_color
            trackball.line_width = props.luna_size * 1.5 * props.luna_lw
        else:
            trackball.hide = True
        
        if self.trackball.is_highlight or self.trackball.is_modal:
            dial_x.color = props.select_color
            dial_y.color = props.select_color
            dial_z.color = props.select_color



        # --- MODO PLANE
        if props.space_drag_lmb == 'TRANSFORM':
            self.modo_plane.hide = False
            self.modo_plane.scale_basis = l_size_a
            view_normal = bpy.context.region_data.view_rotation @ Vector((0.0, 0.0, -1.0))

            m = qZ.to_matrix()
            vec_x = (m @ Vector((1,0,0))).normalized()
            vec_y = (m @ Vector((0,1,0))).normalized()
            vec_z = (m @ Vector((0,0,1))).normalized()


            dot_x = abs(view_normal.dot(vec_x))
            dot_y = abs(view_normal.dot(vec_y))
            dot_z = abs(view_normal.dot(vec_z))

            max_dot = max(dot_x, dot_y, dot_z)

            if max_dot == dot_x:
                self.modo_plane.matrix_basis = Matrix.LocRotScale(l, qY, s)
            elif max_dot == dot_y:
                self.modo_plane.matrix_basis = Matrix.LocRotScale(l, qX, s)
            else:
                self.modo_plane.matrix_basis = Matrix.LocRotScale(l, qZ, s)

        else:
            self.modo_plane.hide = True



class GP_GGT_malibu(GP_GGT_general):
    bl_idname = 'GP_GGT_malibu'


    def setup(self, context):
        # --- TRACKBALL
        self.trackball = self.gizmos.new('GIZMO_GT_move_3d')
        self.trackball.draw_options = {'ALIGN_VIEW'}
        self.trackball_op = self.trackball.target_set_operator('gp.trackball_operator')

        """ # --- BLANK
        self.blank_X = self.gizmos.new('GP_GT_blank') # 'GIZMO_GT_primitive_3d'
        #self.blank_X.draw_style = 'CIRCLE' # 'PLANE'
        self.blankxo = self.blank_X.target_set_operator('gp.blank_operator')
        self.blankxo.axis = 'X'
        self.blank_Y = self.gizmos.new('GP_GT_blank')
        #self.blank_Y.draw_style = 'CIRCLE' # 'PLANE'
        self.blankyo = self.blank_Y.target_set_operator('gp.blank_operator')
        self.blankyo.axis = 'Y'
        self.blank_Z = self.gizmos.new('GP_GT_blank')
        #self.blank_Z.draw_style = 'CIRCLE' # 'PLANE'
        self.blankzo = self.blank_Z.target_set_operator('gp.blank_operator')
        self.blankzo.axis = 'Z' """

        # --- MOVE
        self.arrow_x = self.gizmos.new('GP_GT_line')
        self.axo = self.arrow_x.target_set_operator('gp.cone_operator')
        self.axo.axis = 'X'

        self.arrow_y = self.gizmos.new('GP_GT_line')
        self.ayo = self.arrow_y.target_set_operator('gp.cone_operator')
        self.ayo.axis = 'Y'
        
        self.arrow_z = self.gizmos.new('GP_GT_line')
        self.azo = self.arrow_z.target_set_operator('gp.cone_operator')
        self.azo.axis = 'Z'
    
        # --- SCALE
        self.scale_x = self.gizmos.new('GP_GT_point')
        #self.scale_x.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.sxo = self.scale_x.target_set_operator('gp.box_operator')
        self.sxo.axis = 'X'
      
        self.scale_y = self.gizmos.new('GP_GT_point')
        #self.scale_y.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.syo = self.scale_y.target_set_operator('gp.box_operator')
        self.syo.axis = 'Y'
     
        self.scale_z = self.gizmos.new('GP_GT_point')
        #self.scale_z.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.szo = self.scale_z.target_set_operator('gp.box_operator')
        self.szo.axis = 'Z'

        # --- ROTATE
        self.dial_x = self.gizmos.new('GIZMO_GT_dial_3d')
        self.dial_x.draw_options = {'CLIP'}
        self.dxo = self.dial_x.target_set_operator('gp.arc_operator')
        self.dxo.axis = 'X'
     
        self.dial_y = self.gizmos.new('GIZMO_GT_dial_3d')
        self.dial_y.draw_options = {'CLIP'}
        self.dyo = self.dial_y.target_set_operator('gp.arc_operator')
        self.dyo.axis = 'Y'
   
        self.dial_z = self.gizmos.new('GIZMO_GT_dial_3d')
        self.dial_z.draw_options = {'CLIP'}
        self.dzo = self.dial_z.target_set_operator('gp.arc_operator')
        self.dzo.axis = 'Z'



        """ self.dial_x_out = self.gizmos.new('GIZMO_GT_dial_3d')
        self.dial_x_out.draw_options = {'CLIP'}
        self.dial_y_out = self.gizmos.new('GIZMO_GT_dial_3d')
        self.dial_y_out.draw_options = {'CLIP'}
        self.dial_z_out = self.gizmos.new('GIZMO_GT_dial_3d')
        self.dial_z_out.draw_options = {'CLIP'}
        self.dial_x_out.color = (0.0, 0.0, 0.0)
        self.dial_y_out.color = (0.0, 0.0, 0.0)
        self.dial_z_out.color = (0.0, 0.0, 0.0)
        self.dial_x_out.hide_select = True
        self.dial_y_out.hide_select = True
        self.dial_z_out.hide_select = True """

        """ self.scale_x_out = self.gizmos.new('GIZMO_GT_move_3d')
        self.scale_x_out.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.scale_y_out = self.gizmos.new('GIZMO_GT_move_3d')
        self.scale_y_out.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.scale_z_out = self.gizmos.new('GIZMO_GT_move_3d')
        self.scale_z_out.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.scale_x_out.color = (0.0, 0.0, 0.0)
        self.scale_y_out.color = (0.0, 0.0, 0.0)
        self.scale_z_out.color = (0.0, 0.0, 0.0)
        self.scale_x_out.hide_select = True
        self.scale_y_out.hide_select = True
        self.scale_z_out.hide_select = True """




        # --- DOT MOVE
        self.dots_view = self.gizmos.new('GP_GT_point')
        #self.dots_view.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.dot_op = self.dots_view.target_set_operator('gp.dot_operator')

        
        # --- Modo Plane
        self.modo_plane = self.gizmos.new('GIZMO_GT_primitive_3d')
        self.modo_plane.hide_select = True
        self.modo_plane.color = (0.3, 0.3, 0.3)
        self.modo_plane.alpha = 0.7
        

        # --- Set For All
        for gizmo in self.gizmos:
            gizmo.use_draw_value = use_draw_value
            gizmo.use_tooltip = False
            gizmo.use_draw_offset_scale = True
            gizmo.use_draw_modal = True
        

    def invoke_prepare(self, context, gizmo):
        matrix = gizmo.matrix_basis
        if gizmo.bl_idname == 'GP_GT_line':
            self.axo.matrix_basis = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
            self.ayo.matrix_basis = matrix
            self.azo.matrix_basis = matrix

            """     if gizmo.bl_idname == 'GP_GT_box':
                #self.svo.matrix_basis = matrix """
            

        if gizmo.bl_idname == 'GIZMO_GT_dial_3d':
            #self.dvo.matrix_basis = matrix
            self.dxo.matrix_basis = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
            self.dyo.matrix_basis = matrix
            self.dzo.matrix_basis = matrix

        if gizmo.bl_idname == 'GIZMO_GT_move_3d':
            self.trackball_op.matrix_basis = matrix
            #self.dot_op.matrix_basis = matrix
        
        if gizmo.bl_idname == 'GP_GT_point':
            self.sxo.matrix_basis = matrix @ Matrix.Rotation(radians(180), 4, 'Y')
            self.syo.matrix_basis = matrix
            self.szo.matrix_basis = matrix
            self.dot_op.matrix_basis = matrix
        """ elif gizmo.bl_idname == 'GIZMO_GT_primitive_3d':
            self.blankxo.matrix_basis = self.arrow_x.matrix_basis @ Matrix.Rotation(radians(180), 4, 'Y')
            self.blankyo.matrix_basis = self.arrow_y.matrix_basis
            self.blankzo.matrix_basis = self.arrow_z.matrix_basis @ Matrix.Rotation(radians(90), 4, 'Z') """


    def refresh(self, context): #refresh update_matrix
        #props = context.preferences.addons['GIZMO_PRO'].preferences
        mXW, mYW, mZW = get_orient_from_type() # , mXW_d, mYW_d, mZW_d 
        
        self.dots_view.matrix_basis = mZW

        self.arrow_x.matrix_basis = mXW 
        self.arrow_y.matrix_basis = mYW 
        self.arrow_z.matrix_basis = mZW 
        mat_offset = Matrix.Translation(Vector((0.0, 0.0, 0.5)))
        self.arrow_x.matrix_offset = mat_offset
        self.arrow_y.matrix_offset = mat_offset
        self.arrow_z.matrix_offset = mat_offset

        self.dial_x.matrix_basis = mXW
        self.dial_y.matrix_basis = mYW
        self.dial_z.matrix_basis = mZW

        self.scale_x.matrix_basis = mXW
        self.scale_y.matrix_basis = mYW
        self.scale_z.matrix_basis = mZW
        self.scale_x.matrix_offset = Matrix.Translation(Vector((0.0, 0.0, 4.0)))
        self.scale_y.matrix_offset = Matrix.Translation(Vector((0.0, 0.0, 4.0)))
        self.scale_z.matrix_offset = Matrix.Translation(Vector((0.0, 0.0, 4.0)))

        self.trackball.matrix_basis = mXW






        """ self.dial_x_out.matrix_basis = mXW
        self.dial_y_out.matrix_basis = mYW
        self.dial_z_out.matrix_basis = mZW

        self.scale_x_out.matrix_basis = mXW
        self.scale_y_out.matrix_basis = mYW
        self.scale_z_out.matrix_basis = mZW
        mat_offset = Matrix.Translation(Vector((0.0, 0.0, 35.0)))
        self.scale_x_out.matrix_offset = mat_offset
        self.scale_y_out.matrix_offset = mat_offset
        self.scale_z_out.matrix_offset = mat_offset """

        """
        # --- SCALE ALL
        self.sca_view.matrix_basis = mXW #view_matrix_scale
 
        # --- BLANK
        self.blank_X.matrix_basis = mXW
        self.blank_Y.matrix_basis = mYW
        self.blank_Z.matrix_basis = mZW """


        # --- Modo Plane
        self.modo_plane.matrix_basis.translation = mZW.translation



    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set

        # --- принудительное обновление матрицы
        orient_slots = context.window.scene.transform_orientation_slots[0].type
        if var.REFRESH_MAT or orient_slots=='VIEW' or settings.orientation=='VIEW' or settings.position=='VIEW':
            self.refresh(context)
        var.REFRESH_MAT = False

        lock_loc = context.object.lock_location
        lock_rot = context.object.lock_rotation
        lock_sca = context.object.lock_scale

        # --- параметры
        size = props.malibu_size * 0.5
        line_width = size * props.malibu_lw * 2
        
        # --- общие
        for gizmo in self.gizmos:
            gizmo.alpha_highlight = props.highlight_alpha_gizmo
            gizmo.color_highlight = props.select_color
            gizmo.line_width = line_width
            gizmo.alpha = props.alpha_gizmo




        
        

        
        alpha_x, hide_select_x = alpha_gizmo(self.arrow_x.matrix_basis, props)
        alpha_y, hide_select_y = alpha_gizmo(self.arrow_y.matrix_basis, props)
        alpha_z, hide_select_z = alpha_gizmo(self.arrow_z.matrix_basis, props)
        alpha_xd, hide_select_xd = alpha_arc_gizmo(self.dial_x.matrix_basis, props)
        alpha_yd, hide_select_yd = alpha_arc_gizmo(self.dial_y.matrix_basis, props)
        alpha_zd, hide_select_zd = alpha_arc_gizmo(self.dial_z.matrix_basis, props)
        




        """ self.arrow_x_out.line_width = line_width * 1.5
        self.arrow_y_out.line_width = line_width * 1.5
        self.arrow_z_out.line_width = line_width * 1.5
        self.arrow_x_out.alpha = alpha_x
        self.arrow_y_out.alpha = alpha_y
        self.arrow_z_out.alpha = alpha_z
        self.arrow_x_out.scale_basis = size
        self.arrow_y_out.scale_basis = size
        self.arrow_z_out.scale_basis = size """

        """ self.dial_x_out.line_width = line_width * 1.5
        self.dial_y_out.line_width = line_width * 1.5
        self.dial_z_out.line_width = line_width * 1.5
        self.dial_x_out.alpha = alpha_xd
        self.dial_y_out.alpha = alpha_yd
        self.dial_z_out.alpha = alpha_zd
        self.dial_x_out.scale_basis = size * 2
        self.dial_y_out.scale_basis = size * 2
        self.dial_z_out.scale_basis = size * 2

        self.scale_x_out.line_width = line_width * 1.5
        self.scale_y_out.line_width = line_width * 1.5
        self.scale_z_out.line_width = line_width * 1.5
        self.scale_x_out.alpha = alpha_xd
        self.scale_y_out.alpha = alpha_yd
        self.scale_z_out.alpha = alpha_zd
        self.scale_x_out.scale_basis = size * 0.1
        self.scale_y_out.scale_basis = size * 0.1
        self.scale_z_out.scale_basis = size * 0.1 """


        # --- MOVE
        self.arrow_x.scale_basis = size
        self.arrow_x.color = props.x_axis_color
        self.arrow_x.alpha = alpha_x
        self.arrow_x.hide_select = hide_select_x

        self.arrow_y.scale_basis = size
        self.arrow_y.color = props.y_axis_color
        self.arrow_y.alpha = alpha_y
        self.arrow_y.hide_select = hide_select_y

        self.arrow_z.scale_basis = size
        self.arrow_z.color = props.z_axis_color
        self.arrow_z.alpha = alpha_z
        self.arrow_z.hide_select = hide_select_z
            
        # --- ROTATE
        self.dial_x.scale_basis = size * 2
        self.dial_x.color = props.x_axis_color
        self.dial_x.alpha = alpha_xd
        self.dial_x.hide_select = hide_select_xd

        self.dial_y.scale_basis = size * 2
        self.dial_y.color = props.y_axis_color
        self.dial_y.alpha = alpha_yd
        self.dial_y.hide_select = hide_select_yd

        self.dial_z.scale_basis = size * 2
        self.dial_z.color = props.z_axis_color
        self.dial_z.alpha = alpha_zd
        self.dial_z.hide_select = hide_select_zd


        # --- SCALE
        self.scale_x.scale_basis = size
        self.scale_x.color = props.x_axis_color
        self.scale_x.alpha = alpha_x
        self.scale_x.hide_select = hide_select_x
        self.scale_x.line_width = line_width * 0.2

        self.scale_y.scale_basis = size
        self.scale_y.color = props.y_axis_color
        self.scale_y.alpha = alpha_y
        self.scale_y.hide_select = hide_select_y
        self.scale_y.line_width = line_width * 0.2

        self.scale_z.scale_basis = size
        self.scale_z.color = props.z_axis_color
        self.scale_z.alpha = alpha_z
        self.scale_z.hide_select = hide_select_z
        self.scale_z.line_width = line_width * 0.2

        

        """ # --- BLANK
        self.blank_X.scale_basis = props.size_gizmo_pro * props.size_blank
        self.blank_X.color = tuple(min(a + b, 1.0) for a, b in zip(props.y_axis_color, props.z_axis_color)) if props.blank_color else props.x_axis_color
        self.blank_X.color_highlight = props.select_color
        self.blank_X.alpha_highlight = props.highlight_alpha_gizmo
        self.blank_X.matrix_offset = Matrix.Translation( Vector( (-props.offset_blank, props.offset_blank, 0.0) ) )
        self.blank_X.alpha = alpha_xd
        self.blank_X.hide_select = hide_select_xd

        self.blank_Y.scale_basis = props.size_gizmo_pro * props.size_blank
        self.blank_Y.color = tuple(min(a + b, 1.0) for a, b in zip(props.x_axis_color, props.z_axis_color)) if props.blank_color else props.y_axis_color
        self.blank_Y.color_highlight = props.select_color
        self.blank_Y.alpha_highlight = props.highlight_alpha_gizmo
        self.blank_Y.matrix_offset = Matrix.Translation( Vector( (props.offset_blank, -props.offset_blank, 0.0) ) )
        self.blank_Y.alpha = alpha_yd
        self.blank_Y.hide_select = hide_select_yd

        self.blank_Z.scale_basis = props.size_gizmo_pro * props.size_blank
        self.blank_Z.color = tuple(min(a + b, 1.0) for a, b in zip(props.x_axis_color, props.y_axis_color)) if props.blank_color else props.z_axis_color
        self.blank_Z.color_highlight = props.select_color
        self.blank_Z.alpha_highlight = props.highlight_alpha_gizmo
        self.blank_Z.matrix_offset = Matrix.Translation( Vector( (-props.offset_blank, props.offset_blank, 0.0) ) )
        self.blank_Z.alpha = alpha_zd
        self.blank_Z.hide_select = hide_select_zd

        if settings.blank_gizmo:
            self.blank_X.hide = False
            self.blank_Y.hide = False
            self.blank_Z.hide = False
        else:
            self.blank_X.hide = True
            self.blank_Y.hide = True
            self.blank_Z.hide = True """

        """ if self.blank_X.is_highlight or self.blank_X.is_modal:
            arrow_y.color = props.select_color
            arrow_z.color = props.select_color
            scale_y.color = props.select_color
            scale_z.color = props.select_color
            dial_x.color =  props.select_color
            arrow_y.alpha = props.highlight_alpha_gizmo
            arrow_z.alpha = props.highlight_alpha_gizmo
            scale_y.alpha = props.highlight_alpha_gizmo
            scale_z.alpha = props.highlight_alpha_gizmo
            dial_x.alpha =  props.highlight_alpha_gizmo
        if self.blank_Y.is_highlight or self.blank_Y.is_modal:
            arrow_x.color = props.select_color
            arrow_z.color = props.select_color
            scale_x.color = props.select_color
            scale_z.color = props.select_color
            dial_y.color =  props.select_color
            arrow_x.alpha = props.highlight_alpha_gizmo
            arrow_z.alpha = props.highlight_alpha_gizmo
            scale_x.alpha = props.highlight_alpha_gizmo
            scale_z.alpha = props.highlight_alpha_gizmo
            dial_y.alpha =  props.highlight_alpha_gizmo
        if self.blank_Z.is_highlight or self.blank_Z.is_modal:
            arrow_x.color = props.select_color
            arrow_y.color = props.select_color
            scale_x.color = props.select_color
            scale_y.color = props.select_color
            dial_z.color =  props.select_color
            arrow_x.alpha = props.highlight_alpha_gizmo
            arrow_y.alpha = props.highlight_alpha_gizmo
            scale_x.alpha = props.highlight_alpha_gizmo
            scale_y.alpha = props.highlight_alpha_gizmo
            dial_z.alpha =  props.highlight_alpha_gizmo """



        # --- VISIBLE GIZMO
        if settings.tra_gizmo:
            self.arrow_x.hide = False
            self.arrow_y.hide = False
            self.arrow_z.hide = False
        else:
            self.arrow_x.hide = True
            self.arrow_y.hide = True
            self.arrow_z.hide = True

        if settings.rot_gizmo:
            self.dial_x.hide = False
            self.dial_y.hide = False
            self.dial_z.hide = False
        else:
            self.dial_x.hide = True
            self.dial_y.hide = True
            self.dial_z.hide = True

        if settings.sca_gizmo:
            self.scale_x.hide = False
            self.scale_y.hide = False
            self.scale_z.hide = False
        else:
            self.scale_x.hide = True
            self.scale_y.hide = True
            self.scale_z.hide = True


        # --- LOCK AXIS
        if context.mode == 'OBJECT':
            if lock_loc[0]:
                self.arrow_x.hide = True
                """ self.blank_Y.hide = True
                self.blank_Z.hide = True """

            if lock_loc[1]:
                self.arrow_y.hide = True
                """ self.blank_X.hide = True
                self.blank_Z.hide = True """

            if lock_loc[2]:
                self.arrow_z.hide = True
                """ self.blank_X.hide = True
                self.blank_Y.hide = True """

            if lock_rot[0]:
                self.dial_x.hide = True
            if lock_rot[1]:
                self.dial_y.hide = True
            if lock_rot[2]:
                self.dial_z.hide = True

            if lock_sca[0]:
                self.scale_x.hide = True
            if lock_sca[1]:
                self.scale_y.hide = True
            if lock_sca[2]:
                self.scale_z.hide = True







        # --- DOT 
        if settings.dot_move:
            self.dots_view.hide = False
            self.dots_view.color = props.elements_color
            self.dots_view.line_width = line_width * 0.2
            self.dots_view.scale_basis = size
            """ if props.dot_action == 'ADVANCE' and self.dots_view.is_highlight:
                self.dots_view.scale_basis = size * 0.3
                self.dots_view.draw_options = {'FILL_SELECT'}
            else:
                self.dots_view.scale_basis = size * 0.2
                self.dots_view.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'} """
        else:
            self.dots_view.hide = True
            


        # --- TRACKBALL
        if settings.rotate_trackball:
            self.trackball.hide = False
            self.trackball.scale_basis = size * 2.6
            self.trackball.color = props.elements_color
        else:
            self.trackball.hide = True

        if self.trackball.is_highlight or self.trackball.is_modal:
            self.dial_x.color = props.select_color
            self.dial_y.color = props.select_color
            self.dial_z.color = props.select_color



        # --- SCALE ALL
        """ sca_view = self.sca_view
        if settings.scale_view:
            sca_view.hide = False
            sca_view.alpha = props.alpha_gizmo
            sca_view.alpha_highlight = props.highlight_alpha_gizmo
            sca_view.scale_basis = props.size_gizmo_pro 
            sca_view.color = props.elements_color
            sca_view.color_highlight = props.select_color
        else:
            sca_view.hide = True """


        # --- MODO PLANE
        if props.space_drag_lmb == 'TRANSFORM':
            self.modo_plane.hide = False
            self.modo_plane.scale_basis = props.size_gizmo_pro * props.size_cone
            view_normal = bpy.context.region_data.view_rotation @ Vector((0.0, 0.0, -1.0))

            q = self.dial_z.matrix_basis.decompose()[1]
            vec_x = (q.to_matrix() @ Vector((1,0,0))).normalized()
            vec_y = (q.to_matrix() @ Vector((0,1,0))).normalized()
            vec_z = (q.to_matrix() @ Vector((0,0,1))).normalized()

            dot_x = abs(view_normal.dot(vec_x))
            dot_y = abs(view_normal.dot(vec_y))
            dot_z = abs(view_normal.dot(vec_z))

            max_dot = max(dot_x, dot_y, dot_z)

            if max_dot == dot_x:
                self.modo_plane.matrix_basis = self.scale_y.matrix_basis
            elif max_dot == dot_y:
                self.modo_plane.matrix_basis = self.scale_x.matrix_basis
            else:
                self.modo_plane.matrix_basis = self.scale_z.matrix_basis

        else:
            self.modo_plane.hide = True









classes = [
    GP_GGT_gizmo_pro_3d,
    GP_GGT_luna,
    GP_GGT_malibu,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)