import bpy
from bpy.types import Gizmo, GizmoGroup
from gpu import state
from ..utils.func import viewport_points
from ..utils.draw_utils.draw import get_circle
from math import sqrt, cos, sin, atan2, degrees, radians, pi
from mathutils import Vector, Matrix, Euler
from .. import get_prefs

USED = False


def border_box():
    context = bpy.context
    props = get_prefs()
    wl, wr, hb, ht, wc, hc = viewport_points(context)
    
    width_left = wl + props.bn_width
    height_top = ht - props.bn_width
    height_bot = props.bn_width
    width_right = wr - props.bn_width

    width = (width_right + width_left) / 2
    heght = (height_top + height_bot) / 2

    return height_top, height_bot, width_right, width_left, width, heght



def border_keys():
    props = get_prefs
    dpi = bpy.context.preferences.system.ui_scale

    height_top, height_bot, width_right, width_left, width, heght = border_box()

    height_top = height_top /dpi
    height_bot = height_bot /dpi
    width_right = width_right /dpi
    width_left = width_left /dpi
    width = width /dpi
    heght = heght /dpi


    WL = 60
    DOT = 4


    if props.bn_mini:
        keys = [
            (width_left, height_top, 0.0), (width_left+WL, height_top, 0.0), (width_left, height_top, 0.0), (width_left, height_top-WL, 0.0),
            (width_left, height_bot, 0.0), (width_left+WL, height_bot, 0.0), (width_left, height_bot, 0.0), (width_left, height_bot+WL, 0.0),
            (width_right, height_top, 0.0), (width_right-WL, height_top, 0.0), (width_right, height_top, 0.0), (width_right, height_top-WL, 0.0),
            (width_right, height_bot, 0.0), (width_right-WL, height_bot, 0.0), (width_right, height_bot, 0.0), (width_right, height_bot+WL, 0.0),
            (width-DOT, height_top, 0.0), (width+DOT, height_top, 0.0),
            (width-DOT, height_bot, 0.0), (width+DOT, height_bot, 0.0),
            (width_left, heght-DOT, 0.0), (width_left, heght+DOT, 0.0),
            (width_right, heght-DOT, 0.0), (width_right, heght+DOT, 0.0),
            ]
    else:
        keys = [
            (width_left, height_top, 0.0), (width_right, height_top, 0.0),
            (width_right, height_top, 0.0), (width_right, height_bot, 0.0),
            (width_right, height_bot, 0.0), (width_left, height_bot, 0.0),
            (width_left, height_bot, 0.0), (width_left, height_top, 0.0),
            ]
    return keys


RADIUS = 1.0
def calculate_small_circle_position(angle):
    radius = 1.0
    x = radius * cos(angle)
    y = radius * sin(angle)
    return (x, y)

class GP_GT_circle_rotate_2d(Gizmo):
    bl_idname = "GP_GT_circle_rotate_2d"

    __slots__ = (
        "circle", 
    )


    def setup(self):
        circle = get_circle((0.0, 0.0), 1.0, 60)
        self.circle = self.new_custom_shape('LINE_STRIP', circle)


    def draw(self, context):
        state.line_width_set(self.line_width)
        self.draw_custom_shape(self.circle)
        state.line_width_set(1)





def get_camera_roll(context):
    region_data = context.region_data
    if not region_data:
        return None

    view_matrix = region_data.view_matrix.decompose()[1].to_matrix().to_4x4()
    #cam = context.space_data.region_3d.view_rotation.to_euler()

    z_axis = Vector((0, 0, 1))
    local_z_axis = view_matrix @ z_axis
    angle = atan2(local_z_axis.y, local_z_axis.x)
    return angle





class GP_GGT_2d_navigate(GizmoGroup):
    bl_idname = 'GP_GGT_2d_navigate'
    bl_label = 'Gizmo Navigate'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'PERSISTENT', 'SHOW_MODAL_ALL', 'SCALE'} #'DEPTH_3D' , 'TOOL_INIT', 'SELECT'


    @classmethod
    def poll(cls, context):
        settings = context.scene.GP_scene_set
        return settings.gizmo_2d_navigate

    @classmethod
    def setup_keymap(cls, keyconfig):
        km = keyconfig.keymaps.new(name='Gizmo PRO: 2D Navigation Gizmo', space_type='VIEW_3D')
        km.keymap_items.new('gizmogroup.gizmo_tweak', type='LEFTMOUSE', value='CLICK_DRAG')
        km.keymap_items.new('gizmogroup.gizmo_tweak', type='LEFTMOUSE', value='CLICK_DRAG', ctrl=True)
        km.keymap_items.new('gizmogroup.gizmo_tweak', type='LEFTMOUSE', value='CLICK_DRAG', shift=True)
        km.keymap_items.new('gizmogroup.gizmo_tweak', type='LEFTMOUSE',  value='CLICK')
        return km
    
    def setup(self, context):
        self.pan = self.gizmos.new('GIZMO_GT_button_2d')
        self.pan.icon = 'SNAP_FACE'
        self.pan.draw_options = {'BACKDROP'} # 'HELPLINE' 'BACKDROP'
        self.pan.backdrop_fill_alpha = 0.8
        self.pan.use_tooltip = False
        self.pan.use_draw_modal = True
        self.pan.color = 0.3, 0.3, 0.3
        self.pan.alpha = 0.0
        self.pan.color_highlight = 0.1, 0.1, 0.1
        self.pan.alpha_highlight = 0.7
        self.pan.target_set_operator('gp.view_pan')


        self.rotate = self.gizmos.new('GIZMO_GT_button_2d')
        self.rotate.icon = 'NONE'
        self.rotate.draw_options = {'BACKDROP'} # 'HELPLINE' 'BACKDROP'
        self.rotate.backdrop_fill_alpha = 0.5
        self.rotate.line_width = 4.0
        #self.back.use_tooltip = False
        self.rotate.use_draw_modal = True
        self.rotate.color = 0.0, 0.5, 1.0
        self.rotate.alpha = 1.0
        self.rotate.color_highlight = 0.0, 0.5, 1.0
        self.rotate.alpha_highlight = 1.0
        self.rot_op = self.rotate.target_set_operator('gp.view_roll')


        self.circle = self.gizmos.new('GP_GT_circle_rotate_2d')
        self.circle.line_width = 4.0
        self.circle.color = 0.0, 0.5, 1.0
        self.circle.alpha = 0.9
        self.circle.color_highlight = 0.0, 0.5, 1.0
        self.circle.alpha_highlight = 0.9

        self.back = self.gizmos.new('GIZMO_GT_button_2d')
        self.back.icon = 'NONE'
        self.back.draw_options = {'BACKDROP'} # 'HELPLINE' 'BACKDROP'
        self.back.backdrop_fill_alpha = 0.8
        self.back.use_tooltip = False
        self.back.use_draw_modal = True
        self.back.hide_select = True
        self.back.color = 0.3, 0.3, 0.3
        self.back.alpha = 0.5
        self.back.color_highlight = 0.3, 0.3, 0.3
        self.back.alpha_highlight = 0.7

        
    def invoke_prepare(self, context, gizmo):
        pos = self.circle.matrix_basis.translation
        self.rot_op.position = pos
        

    def draw_prepare(self, context):
        global USED
        dpi = context.preferences.system.ui_scale
        height_top, height_bot, width_right, width_left, width, heght = border_box()

        preferences = bpy.context.preferences
        
        view_settings = preferences.view
        if view_settings.mini_axis_type == 'NONE':
            size_standard_nav = 40
        elif view_settings.mini_axis_type == 'MINIMAL':
            size_standard_nav = view_settings.mini_axis_size
        elif view_settings.mini_axis_type == 'GIZMO':
            size_standard_nav = view_settings.gizmo_size_navigate_v3d
        

        self.scale = (80 * 0.35) * 1.3
        self.back.scale_basis = self.scale
        self.circle.scale_basis = self.scale * 0.78
        self.rotate.scale_basis = self.scale * 0.2
        self.pan.scale_basis = self.scale * 0.5


        x = width_right - size_standard_nav - 60
        y = height_top - size_standard_nav/2 + 14
        self.back.matrix_basis[0][3] = x
        self.back.matrix_basis[1][3] = y
        self.circle.matrix_basis[0][3] = x
        self.circle.matrix_basis[1][3] = y
        self.pan.matrix_basis[0][3] = x
        self.pan.matrix_basis[1][3] = y
        
        

        angle = get_camera_roll(context)
        radius = self.scale * dpi * 0.8
        x_r = radius * cos(angle)
        y_r = radius * sin(angle)

        self.rotate.matrix_basis[0][3] = x + x_r
        self.rotate.matrix_basis[1][3] = y + y_r

        if USED:
            self.rotate.backdrop_fill_alpha = 1.0
        else:
            self.rotate.backdrop_fill_alpha = 0.0
            


classes = [
    GP_GT_circle_rotate_2d,
    GP_GGT_2d_navigate,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)