import bpy
from bpy.types import GizmoGroup
from mathutils import Matrix
from math import radians
from ..keymaps import keymaps_gizmo_tweak
from ..utils.poll import poll_cam_gizmo
from .. import get_prefs



class GP_GGT_camera(GizmoGroup):
    bl_idname = 'GP_GGT_camera'
    bl_label = 'Gizmo Camera'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'3D','PERSISTENT','SHOW_MODAL_ALL'} #'DEPTH_3D' , 'TOOL_INIT', 'SELECT', , 'SCALE'


    @classmethod
    def poll(cls, context):
        return poll_cam_gizmo(context)
    

    @classmethod
    def setup_keymap(cls, keyconfig):
        return keymaps_gizmo_tweak(keyconfig)


    def setup(self, context):

        self.tilt = self.gizmos.new('GIZMO_GT_dial_3d')
        tilt_op = self.tilt.target_set_operator('transform.rotate')
        tilt_op.constraint_axis = (True, False, False)
        tilt_op.orient_type = 'LOCAL'
        self.tilt.draw_options = {'CLIP'}
        self.tilt.line_width = 2
        self.tilt.scale_basis = 1.3
        self.tilt.use_draw_modal = True

        self.roll = self.gizmos.new('GIZMO_GT_dial_3d')
        roll_op = self.roll.target_set_operator('transform.rotate')
        roll_op.constraint_axis = (False, False, True)
        roll_op.orient_type = 'LOCAL'
        self.roll.draw_options = {'CLIP'}
        self.roll.line_width = 2
        self.roll.scale_basis = 1.0
        self.roll.use_draw_modal = True

        self.pan = self.gizmos.new('GIZMO_GT_dial_3d')
        pan_op = self.pan.target_set_operator('transform.rotate')
        pan_op.constraint_axis = (False, False, True)
        pan_op.orient_type = 'GLOBAL'
        self.pan.draw_options = {'CLIP'}
        self.pan.line_width = 2
        self.pan.scale_basis = 1.6
        self.pan.use_draw_modal = True


    def draw_prepare(self, context):
        self.props = get_prefs()

        
        # --- ORIENTATIONS
        ob = context.object.matrix_world
        orient_slots = context.window.scene.transform_orientation_slots[0].type
        orig_loc, orig_rot, orig_scale = ob.decompose() 


        # --- GLOBAL
        orig_loc_mat = Matrix.Translation(orig_loc)
            
        orig_scale_mat = Matrix.Scale(orig_scale[0], 4, (1, 0, 0)) @ Matrix.Scale(orig_scale[1], 4, (0, 1, 0)) @ Matrix.Scale(orig_scale[2], 4, (0, 0, 1))
        
        x_rot_mat = Matrix.Rotation(radians(90), 4, 'Y') 
        y_rot_mat = Matrix.Rotation(radians(-90), 4, 'X') 
        z_rot_mat =  Matrix.Rotation(radians(-90), 4, 'Z') 

        x_rot_mat_dial = Matrix.Rotation(radians(-90), 4, 'Y')  
        #y_rot_mat_dial = Matrix.Rotation(radians(90), 4, 'X')  
        z_rot_mat_dial = Matrix.Rotation(radians(0), 4, 'Z') 

        #x_matrix_world_g = orig_loc_mat @ x_rot_mat @ orig_scale_mat  
        #y_matrix_world_g = orig_loc_mat @ y_rot_mat @ orig_scale_mat  
        #z_matrix_world_g = orig_loc_mat @ z_rot_mat @ orig_scale_mat 
        
        #x_matrix_world_dial_g = orig_loc_mat @ x_rot_mat_dial @ orig_scale_mat 
        #y_matrix_world_dial_g = orig_loc_mat @ y_rot_mat_dial @ orig_scale_mat  
        z_matrix_world_dial_g = orig_loc_mat @ z_rot_mat_dial @ orig_scale_mat


        # --- LOCAL
        orig_loc_mat = Matrix.Translation(orig_loc)
        orig_scale_mat = Matrix.Scale(orig_scale[0], 4, (1, 0, 0)) @ Matrix.Scale(orig_scale[1], 4, (0, 1, 0)) @ Matrix.Scale(orig_scale[2], 4, (0, 0, 1))
        
        x_rot_mat = orig_rot.to_matrix().to_4x4() @ Matrix.Rotation(radians(90), 4, 'Y') 
        y_rot_mat = orig_rot.to_matrix().to_4x4() @ Matrix.Rotation(radians(-90), 4, 'X') 
        z_rot_mat = orig_rot.to_matrix().to_4x4() @ Matrix.Rotation(radians(-90), 4, 'Z') 

        x_rot_mat_dial = orig_rot.to_matrix().to_4x4() @ Matrix.Rotation(radians(-90), 4, 'Y')  
        y_rot_mat_dial = orig_rot.to_matrix().to_4x4() @ Matrix.Rotation(radians(90), 4, 'X')  
        z_rot_mat_dial = orig_rot.to_matrix().to_4x4() 
            

        # --- NEW MATRIX WORLD
        #x_matrix_world = orig_loc_mat @ x_rot_mat @ orig_scale_mat  
        #y_matrix_world = orig_loc_mat @ y_rot_mat @ orig_scale_mat  
        z_matrix_world = orig_loc_mat @ z_rot_mat @ orig_scale_mat 
        
        x_matrix_world_dial = orig_loc_mat @ x_rot_mat_dial @ orig_scale_mat 
        #y_matrix_world_dial = orig_loc_mat @ y_rot_mat_dial @ orig_scale_mat  
        #z_matrix_world_dial = orig_loc_mat @ z_rot_mat_dial @ orig_scale_mat 


        # --- CURSOR
        cursor = context.scene.cursor
        orig_loc, orig_rot, orig_scale = cursor.matrix.decompose()
        orig_loc_mat = Matrix.Translation(orig_loc)
        x_rot_mat = orig_rot.to_matrix().to_4x4() @ Matrix.Rotation(radians(90), 4, 'Y')
        y_rot_mat = orig_rot.to_matrix().to_4x4() @ Matrix.Rotation(radians(-90), 4, 'X') 
        x_matrix = orig_loc_mat @ x_rot_mat 
        y_matrix = orig_loc_mat @ y_rot_mat 


        # --- TRANSLATE
        self.tilt.color = self.props.x_axis_color
        self.tilt.color_highlight = self.props.select_color
        self.tilt.alpha = self.props.alpha_gizmo
        self.tilt.alpha_highlight = self.props.highlight_alpha_gizmo
        self.tilt.matrix_basis = x_matrix_world_dial.normalized()


        self.roll.color = self.props.y_axis_color
        self.roll.color_highlight = self.props.select_color
        self.roll.alpha = self.props.alpha_gizmo
        self.roll.alpha_highlight = self.props.highlight_alpha_gizmo
        self.roll.matrix_basis = z_matrix_world.normalized()

        
        self.pan.color = self.props.z_axis_color
        self.pan.color_highlight = self.props.select_color
        self.pan.alpha = self.props.alpha_gizmo
        self.pan.alpha_highlight = self.props.highlight_alpha_gizmo
        self.pan.matrix_basis = z_matrix_world_dial_g.normalized()
     
       
     









classes = [
    GP_GGT_camera,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

        
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)