import bpy
from mathutils import Matrix, Vector, Quaternion
from bpy.types import GizmoGroup, Operator
from ..keymaps import keymaps_gizmo_tweak
from math import radians
from ..utils.poll import poll_cursor, poll_cursor_navigator
from mathutils import Matrix
from bpy.props import BoolProperty
import time
from .. import get_prefs


class GP_OT_orbit_around_cursor(Operator):
    bl_idname = 'gp.orbit_around_cursor'
    bl_label = 'Orbit Rotate'
    bl_description = 'Rotate Around 3D Cursor'
    bl_options = {'REGISTER', 'UNDO'}


    def execute(self, context):
        pp = context.scene.cursor.location
        bpy.ops.transform.rotate('INVOKE_DEFAULT',constraint_axis=(False, False, True), orient_type='CURSOR', release_confirm = True, center_override=pp)
        return{'FINISHED'}



def local_translate(obj, direction, matrix):
    direction = direction.copy()
    direction.rotate(matrix)
    obj.location += direction


def local_rotate(obj, rotation):
    loc, rot, scale = obj.matrix_world.decompose()
    rotation = rotation.copy()
    rotation.rotate(rot)
    if not isinstance(rotation, Matrix):
        obj.matrix_world = rotation.to_matrix().to_4x4()
    else:
        obj.matrix_world = rotation.to_4x4()
    obj.location = loc
    obj.scale = scale


# --- X
def move_get_x(): # TODO
    context = bpy.context
    cursor = context.scene.cursor
    if context.window.scene.transform_orientation_slots[0].type == 'GLOBAL':
        return cursor.location.x
    else:
        return 0.0


def move_set_x(value):
    context = bpy.context
    cursor = context.scene.cursor
    if context.window.scene.transform_orientation_slots[0].type == 'GLOBAL':
        cursor.location.x = value
    else:
        l = cursor.location.copy()
        l.x = value
        local_translate(cursor, l, cursor.matrix)
        
    ''' for ob in context.selected_objects:
        ob.location.x = value '''


# --- Y
def move_get_y(): # TODO
    context = bpy.context
    cursor = context.scene.cursor
    if context.window.scene.transform_orientation_slots[0].type == 'GLOBAL':
        return cursor.location.y
    else:
        return 0.0


def move_set_y(value):
    context = bpy.context
    cursor = context.scene.cursor
    if context.window.scene.transform_orientation_slots[0].type == 'GLOBAL':
        cursor.location.y = value
    else:
        cursor.location.y = value
   

# --- Z
def move_get_z(): # TODO
    context = bpy.context
    cursor = context.scene.cursor
    if context.window.scene.transform_orientation_slots[0].type == 'GLOBAL':
        return cursor.location.z
    else:
        return 0.0


def move_set_z(value):
    context = bpy.context
    cursor = context.scene.cursor
    if context.window.scene.transform_orientation_slots[0].type == 'GLOBAL':
        cursor.location.z = value
    else:
        cursor.location.z = value



class Transform: # TODO

    @staticmethod
    def apply(obj):
        ''' Apply all transform for the object
        :param obj: Object
        :type obj: bpy.types.Object
        '''
        obj_world_matrix = obj.matrix_world.copy()
        for vert in obj.data.vertices:
            vert.co = obj_world_matrix @ vert.co
        obj.matrix_world.identity()


    @staticmethod
    def matrix_rotate(factor: float, axis: tuple = (1, 1, 1), size: int = 4) -> 'Matrix':
        
        rotate_matrix = Matrix()
        if axis[0]:
            rotate_matrix_x = Matrix.Rotation(factor, size, 'X')
            rotate_matrix @= rotate_matrix_x
        if axis[1]:
            rotate_matrix_y = Matrix.Rotation(factor, size, 'Y')
            rotate_matrix @= rotate_matrix_y
        if axis[2]:
            rotate_matrix_z = Matrix.Rotation(factor, size, 'Z')
            rotate_matrix @= rotate_matrix_z
        return rotate_matrix


    @staticmethod
    def matrix_scale(factor: float, axis: tuple = (1, 1, 1), size: int = 4) -> 'Matrix':
        ''' Return a scale matrix
        :param factor: Scale factor
        :type factor: float
        :param axis: XYZ-axis to apply the scale. Can be combined.
        :type axis: tuple
        :param size: the size of the resulting matrix
        :type size: int
        :return: scale matrix
        :rtype: 'Matrix'
        '''
        scale_matrix = Matrix()
        if axis[0]:
            scale_matrix_x = Matrix.Scale(factor, size, (1.0, 0.0, 0.0))
            scale_matrix @= scale_matrix_x
        if axis[1]:
            scale_matrix_y = Matrix.Scale(factor, size, (0.0, 1.0, 0.0))
            scale_matrix @= scale_matrix_y
        if axis[2]:
            scale_matrix_z = Matrix.Scale(factor, size, (0.0, 0.0, 1.0))
            scale_matrix @= scale_matrix_z
        return scale_matrix



class GP_GGT_gizmo_3d_cursor(GizmoGroup):
    bl_idname = 'GP_GGT_gizmo_3d_cursor'
    bl_label = 'Gizmo for 3d Cursor'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'3D', 'PERSISTENT', 'SHOW_MODAL_ALL'} #, 'SELECT' , 'SCALE'
    #bl_owner_id = 'GP_GGT_gizmo_pro_3d'


    @classmethod
    def poll(cls, context):
        return poll_cursor(context)


    @classmethod
    def setup_keymap(cls, keyconfig):
        return keymaps_gizmo_tweak(keyconfig)


    def setup(self, context):
        
        # --- X
        self.arrow_x = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.arrow_x.use_tooltip = False
        self.arrow_x.use_draw_offset_scale = True
        self.arrow_x.use_draw_modal = True
        self.ar_x = self.arrow_x.target_set_operator('transform.translate')
        #self.ar_x.mode = 'TRANSLATION'
        self.ar_x.constraint_axis = (True, False, False)
        self.ar_x.release_confirm = True
        self.ar_x.cursor_transform = True

        
        # --- Y
        self.arrow_y = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.arrow_y.use_tooltip = False
        self.arrow_y.use_draw_offset_scale = True
        self.arrow_y.use_draw_modal = True
        self.ar_y = self.arrow_y.target_set_operator('transform.translate')
        #self.ar_y.mode = 'TRANSLATION'
        self.ar_y.constraint_axis = (False, True, False)
        self.ar_y.release_confirm = True
        self.ar_y.cursor_transform = True
        
         
        # --- Z
        self.arrow_z = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.arrow_z.use_tooltip = False
        self.arrow_z.use_draw_offset_scale = True
        self.arrow_z.use_draw_modal = True
        self.ar_z = self.arrow_z.target_set_operator('transform.translate')
        #self.ar_z.mode = 'TRANSLATION'
        self.ar_z.constraint_axis = (False, False, True)
        self.ar_z.release_confirm = True
        self.ar_z.cursor_transform = True
        
        
        # --- DOT
        self.dot = self.gizmos.new('GIZMO_GT_move_3d')
        self.dot.use_grab_cursor = False
        self.dot.use_tooltip = False
        self.dot.use_draw_modal = True
        self.dot.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.ar_dot = self.dot.target_set_operator('gp.cursor_navigator')



    """ def invoke_prepare(self, context, gizmo):
        matrix = gizmo.matrix_basis
        pp = matrix.translation
        if gizmo.bl_idname == 'GIZMO_GT_arrow_3d':
            self.ar_x.center_override = pp
            self.ar_y.center_override = pp
            self.ar_z.center_override = pp """


    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set

        cursor = context.scene.cursor


        
        orient = 'GLOBAL' if context.window.scene.transform_orientation_slots[0].type == 'GLOBAL' else 'CURSOR'
        self.ar_x.orient_type = orient
        self.ar_x.orient_matrix_type = orient
        self.ar_y.orient_type = orient
        self.ar_y.orient_matrix_type = orient
        self.ar_z.orient_type = orient
        self.ar_z.orient_matrix_type = orient





        coef = 0.2 if props.size_cursor > 0.6 else 0.5
        # --- DOT
        if settings.cursor_navigator:
            self.dot.color = (1.0, 0.2, 0.0)
            self.dot.scale_basis = props.size_cursor * coef * 1.5
            self.dot.line_width = props.size_gizmo_pro * props.lw_dot * 3
            self.ar_dot = self.dot.target_set_operator('gp.cursor_navigator')
        else:
            self.dot.color = props.elements_color
            self.dot.scale_basis = props.size_cursor * coef
            self.dot.line_width = props.size_gizmo_pro * props.lw_dot  * 1.5
            self.ar_dot = self.dot.target_set_operator('transform.translate')
            self.ar_dot.release_confirm = True
            self.ar_dot.cursor_transform = True


        self.dot.color_highlight = props.select_color
        self.dot.alpha = props.alpha_gizmo
        self.dot.alpha_highlight = props.highlight_alpha_gizmo
        self.dot.matrix_basis = cursor.matrix.normalized()
        #self.dot.use_draw_hover = True
        if settings.cursor_gizmo_dot:
            self.dot.hide = False
        else:
            self.dot.hide = True


        # --- ARROW
        l, r, s  = cursor.matrix.decompose() 
        #s = Vector((1.0, 1.0, 1.0))
   
        


        orient_slots = context.window.scene.transform_orientation_slots[0].type
        if orient_slots == 'GLOBAL':
            xR = Quaternion( (0.0, 1.0, 0.0), radians(90) )
            yR = Quaternion( (1.0, 0.0, 0.0), radians(-90) )
            zR = Quaternion( (0.0, 0.0, 1.0), radians(0) )
            x_matrix_move = Matrix.LocRotScale(l, xR, s).normalized() 
            y_matrix_move = Matrix.LocRotScale(l, yR, s).normalized() 
            z_matrix_move = Matrix.LocRotScale(l, zR, s).normalized()

        else:
            xR = r @ Quaternion( (0.0, 1.0, 0.0), radians(90) )
            yR = r @ Quaternion( (1.0, 0.0, 0.0), radians(-90) )
            zR = r #@ Quaternion( (0.0, 0.0, 1.0), radians(0) )
            x_matrix_move = Matrix.LocRotScale(l, xR, s).normalized()
            y_matrix_move = Matrix.LocRotScale(l, yR, s).normalized() 
            z_matrix_move = Matrix.LocRotScale(l, zR, s).normalized()
            
        mo_a = Matrix.Translation( Vector( (0.0, 0.0, 0.2) ) )
            
    




        
        # --- ARROW
        arrow_x = self.arrow_x
        arrow_x.length = 0.8
        arrow_x.line_width = props.size_cursor * 2
        arrow_x.alpha_highlight = props.highlight_alpha_gizmo
        arrow_x.alpha = props.alpha_gizmo
        arrow_x.scale_basis = props.size_cursor
        arrow_x.color = props.x_axis_color
        arrow_x.color_highlight = props.select_color
        arrow_x.matrix_basis = x_matrix_move
        arrow_x.matrix_offset = mo_a

        arrow_y = self.arrow_y
        arrow_y.length = 0.8
        arrow_y.line_width = props.size_cursor * 2
        arrow_y.alpha_highlight = props.highlight_alpha_gizmo
        arrow_y.alpha = props.alpha_gizmo
        arrow_y.scale_basis = props.size_cursor
        arrow_y.color = props.y_axis_color
        arrow_y.color_highlight = props.select_color 
        arrow_y.matrix_basis = y_matrix_move
        arrow_y.matrix_offset = mo_a

        arrow_z = self.arrow_z
        arrow_z.length = 0.8
        arrow_z.line_width = props.size_cursor * 2
        arrow_z.alpha_highlight = props.highlight_alpha_gizmo
        arrow_z.alpha = props.alpha_gizmo
        arrow_z.scale_basis = props.size_cursor
        arrow_z.color = props.z_axis_color
        arrow_z.color_highlight = props.select_color 
        arrow_z.matrix_basis = z_matrix_move
        arrow_z.matrix_offset = mo_a
        


        if settings.cursor_gizmo_axes:
            self.arrow_x.hide = False
            self.arrow_y.hide = False
            self.arrow_z.hide = False
        else:
            self.arrow_x.hide = True
            self.arrow_y.hide = True
            self.arrow_z.hide = True
        









class GP_OT_cursor_navigator(Operator):
    bl_idname = 'gp.cursor_navigator'
    bl_label = '3d Cursor Navigator'
    """ bl_description = ('Gizmo to place the selected by normals\n'
                      '• Shift - Duplicate Move') """
    #bl_options = {'UNDO'}

    reset_position: BoolProperty(default=False)
    reset_rotation: BoolProperty(default=False)
    reset_all: BoolProperty(default=False)
    close_widget: BoolProperty(default=False)

    @classmethod
    def description(self, context, properties):
        if properties.reset_position:
            return 'Reset Position'
        elif properties.reset_rotation:
            return 'Reset Rotation'
        elif properties.reset_all:
            return 'Reset Position and Rotation'
        elif properties.close_widget:
            return 'Close'

    def execute(self, context):
        # Сохраняем значения опций Snap и Align Rotation
        use_snap = context.scene.tool_settings.use_snap
        use_snap_align_rotation = context.scene.tool_settings.use_snap_align_rotation
        snap_elements = context.scene.tool_settings.snap_elements
        snap_target = context.scene.tool_settings.snap_target

        # Устанавливаем нужные значения опций Snap и Align Rotation
        context.scene.tool_settings.use_snap = True
        context.scene.tool_settings.use_snap_align_rotation = True

        #if self.use_standard_options is False: # 'FACE' not in context.scene.tool_settings.snap_elements
        context.scene.tool_settings.snap_elements = {'VERTEX', 'EDGE', 'FACE', 'EDGE_MIDPOINT', 'EDGE_PERPENDICULAR'} #  'FACE_NEAREST', 'VOLUME', 
        context.scene.tool_settings.snap_target = 'CENTER'

        # Вызываем оператор bpy.ops.transform.transform
        bpy.ops.transform.translate(
                'INVOKE_DEFAULT',
                release_confirm = True,
                cursor_transform = True,
                )

        # Возвращаем опции Snap и Align Rotation к сохраненным значениям
        context.scene.tool_settings.use_snap = use_snap
        context.scene.tool_settings.use_snap_align_rotation = use_snap_align_rotation
        context.scene.tool_settings.snap_elements = snap_elements
        context.scene.tool_settings.snap_target = snap_target

        return {'FINISHED'}

    @staticmethod
    def reset(self):
        self.reset_position = False
        self.reset_rotation = False
        self.reset_all = False
        self.close_widget = False


    def invoke(self, context, event):
        if self.close_widget:
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_cursor_widget')
            self.reset(self)
            return {'FINISHED'}
        elif event.type == 'MIDDLEMOUSE':
            context.window_manager.gizmo_group_type_ensure('GP_GGT_cursor_widget')
            return {'FINISHED'}
        elif (event.shift and event.ctrl) or self.reset_all:
            context.scene.cursor.matrix = Matrix.Identity(4)
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_cursor_widget')
            self.reset(self)
            return {'FINISHED'}
        elif event.shift or self.reset_position:
            context.scene.cursor.location = Vector((0.0, 0.0, 0.0))
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_cursor_widget')
            self.reset(self)
            return {'FINISHED'}
        elif event.ctrl or self.reset_rotation:
            l = context.scene.cursor.location
            q = Quaternion((0.0, 0.0, 0.0, 1.0))
            s = Vector((1.0, 1.0, 1.0))
            context.scene.cursor.matrix = Matrix.LocRotScale(l, q, s)
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_cursor_widget')
            self.reset(self)
            return {'FINISHED'}
        else:
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_cursor_widget')
            self.reset(self)
            return self.execute(context)





from bpy_extras.view3d_utils import location_3d_to_region_2d
from ..icons import load_textures
import threading

timer_end = False
timer_run = False
def timer(seconds):
    global timer_end, timer_run
    timer_run = True
    time.sleep(seconds)
    timer_end = True
    #print("Время вышло!")

def animate_button_move(button, start_position, target_position):
    speed = 3
    # Получаем начальные координаты кнопки
    x, y = start_position
    
    # Вычисляем вектор перемещения
    dx = (target_position[0] - start_position[0]) / speed
    dy = (target_position[1] - start_position[1]) / speed
    
    # Анимация перемещения кнопки
    for _ in range(speed):
        x += dx
        y += dy
        button.matrix_basis[0][3] = x
        button.matrix_basis[1][3] = y
        
        # Задержка для создания анимации
        time.sleep(0.01)

def start_animation(button, start_position, target_position):
    # Создаем новый поток для анимации
    animation_thread = threading.Thread(target=animate_button_move, args=(button, start_position, target_position))
    animation_thread.start()

class GP_GGT_cursor_widget(GizmoGroup):
    
    bl_idname = 'GP_GGT_cursor_widget'
    bl_label = '3D Cursor Widget'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'SHOW_MODAL_ALL', 'SCALE'}


    """ @classmethod
    def poll(cls, context):
        
        return True """


    @classmethod
    def setup_keymap(cls, keyconfig):
        km = keyconfig.keymaps.new(name='Gizmo PRO: Cursor Widget', space_type='VIEW_3D')
        km.keymap_items.new('gizmogroup.gizmo_tweak', type='LEFTMOUSE', value='CLICK')
        return km
    

    def setup(self, context):
        load_textures() # --- load textures for icons

        pos2d = location_3d_to_region_2d(context.region, context.space_data.region_3d, context.scene.cursor.location)
        offset = 50


        self.bt_close = self.gizmos.new('GP_GT_button')
        self.bt_close.use_tooltip = True
        self.bt_close.use_grab_cursor = False
        self.bt_close.type = 'CIRCLE'
        self.bt_close.icon_id = 3
        self.bt_close.matrix_basis[0][3] = pos2d[0]
        self.bt_close.matrix_basis[1][3] = pos2d[1]
        op = self.bt_close.target_set_operator('gp.cursor_navigator')
        op.close_widget = True

        self.bt_all = self.gizmos.new('GP_GT_button')
        self.bt_all.use_tooltip = True
        self.bt_all.use_grab_cursor = False
        self.bt_all.type = 'CAPSULE'
        self.bt_all.icon_id = 9
        start_animation(self.bt_all, pos2d, (pos2d[0], pos2d[1]+offset))
        op = self.bt_all.target_set_operator('gp.cursor_navigator')
        op.reset_all = True
        
        self.bt_position = self.gizmos.new('GP_GT_button')
        self.bt_position.use_tooltip = True
        self.bt_position.use_grab_cursor = False
        self.bt_position.type = 'CAPSULE'
        self.bt_position.icon_id = 85
        start_animation(self.bt_position, pos2d, (pos2d[0]-offset, pos2d[1]))
        op = self.bt_position.target_set_operator('gp.cursor_navigator')
        op.reset_position = True

        self.bt_rotate = self.gizmos.new('GP_GT_button')
        self.bt_rotate.use_tooltip = True
        self.bt_rotate.use_grab_cursor = False
        self.bt_rotate.type = 'CAPSULE'
        self.bt_rotate.icon_id = 84
        start_animation(self.bt_rotate, pos2d, (pos2d[0]+offset, pos2d[1]))
        op = self.bt_rotate.target_set_operator('gp.cursor_navigator')
        op.reset_rotation = True
        
        global timer_end, timer_run
        timer_end = False
        timer_run = False

        # Создаем и запускаем таймер в отдельном потоке
        t = threading.Thread(target=timer, args=(5,))
        t.start()

    """ def invoke_prepare(self, context, gizmo):
        context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_cursor_widget') """


    """ def refresh(self, context):
        global timer_run
        if timer_run is False:
            timer(5) """


    def draw_prepare(self, context):
        global timer_run, timer_end
        """ if timer_run is False:
            timer(5) """

        props = get_prefs()
        #settings = context.scene.GP_scene_set
        #toolSet = context.scene.tool_settings
        cursor = context.scene.cursor

        dpi = context.preferences.system.ui_scale
        coef = 0.5 * dpi # TODO
        pos2d = location_3d_to_region_2d(context.region, context.space_data.region_3d, cursor.location)
        #pos_y = pos2d[1] + props.qw_offset_y * coef

        butSize = ((80 * 0.35) / 2) * coef * 1.5 * props.qw_scale
        offset = 50.0 # TODO

        self.bt_close.hide = False
        self.bt_close.scale_basis = butSize
        """ self.bt_close.matrix_basis[0][3] = pos2d[0]
        self.bt_close.matrix_basis[1][3] = pos2d[1] """
        self.bt_close.alpha = props.qw_opacity

        self.bt_all.hide = False
        self.bt_all.scale_basis = butSize
        """ self.bt_all.matrix_basis[0][3] = pos2d[0]
        self.bt_all.matrix_basis[1][3] = pos2d[1] + offset """
        self.bt_all.alpha = props.qw_opacity
        


        self.bt_position.hide = False
        self.bt_position.scale_basis = butSize
        """ self.bt_position.matrix_basis[0][3] = pos2d[0] - offset
        self.bt_position.matrix_basis[1][3] = pos2d[1]  """
        self.bt_position.alpha = props.qw_opacity

        self.bt_rotate.hide = False
        self.bt_rotate.scale_basis = butSize
        """ self.bt_rotate.matrix_basis[0][3] = pos2d[0] + offset
        self.bt_rotate.matrix_basis[1][3] = pos2d[1] """
        self.bt_rotate.alpha = props.qw_opacity

        is_highlight = [gzm.is_highlight for gzm in self.gizmos]
        if timer_end:
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_cursor_widget')



classes = [
    GP_OT_orbit_around_cursor,
    GP_GGT_gizmo_3d_cursor,
    GP_OT_cursor_navigator,
    GP_GGT_cursor_widget,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)