import bpy
from mathutils import Matrix, Vector
from bpy.types import GizmoGroup, Operator
from ..utils.poll import poll_mirror
from mathutils import Matrix
from ..matrix.matrix_ensure import get_orient_from_type
from .. import var
from bpy.props import EnumProperty, FloatVectorProperty, BoolProperty
from ..keymaps import keymaps_gizmo_tweak
from .. import get_prefs


class GP_OT_mirror_pt(Operator):
    bl_idname = 'gp.mirror_pt'
    bl_label = 'Pivot Transform'
    bl_description = 'Pivot Transform'

    axis: EnumProperty(
        items = [
            ('X', 'X', '', 0),
            ('Y', 'Y', '', 1),
            ('Z', 'Z', '', 2),
            ],
        default='X',
        )

    orient_matrix: FloatVectorProperty( size=(3, 3), subtype='MATRIX' )
    center_override: FloatVectorProperty(size=3, default=[0, 0, 0])


    def execute(self, context):
        if self.axis=='X':
            axis_set = (True, False, False)
        elif self.axis=='Y':
            axis_set = (False, True, False)
        elif self.axis=='Z':
            axis_set = (False, False, True)
        else:
            axis_set = (True, True, True)

        bpy.ops.gp.pivot_transform(
                'INVOKE_DEFAULT',
                TRANSFORM_OT_transform = {
                    'mode': 'TRANSLATION',
                    'orient_matrix': self.orient_matrix,
                    'constraint_axis': axis_set,
                    'release_confirm': True,
                    'center_override': self.center_override,
                    },
                )

        return {'FINISHED'}


    def invoke(self, context, event):
        return self.execute(context)



class GP_OT_mirror(Operator):
    bl_idname = 'gp.mirror'
    bl_label = 'Gizmo PRO: Mirror'
    bl_description = 'Mirror'
    bl_options = {'REGISTER', 'UNDO'}


    axis: bpy.props.EnumProperty(
        items = [
            ('+X', '+X', '', 0),
            ('-X', '-X', '', 1),
            ('+Y', '+Y', '', 2),
            ('-Y', '-Y', '', 3),
            ('+Z', '+Z', '', 4),
            ('-Z', '-Z', '', 5),
            ],
        default='+X',
        )

    orient_matrix: FloatVectorProperty( size=(3, 3), subtype='MATRIX' )
    center_override: FloatVectorProperty(size=3, default=[0, 0, 0])
    use_only_one = False
    use_bisect = True


    @classmethod
    def description(self, context, properties):
        settings = context.scene.GP_scene_set
        if properties.axis=='+X':
            if settings.mirror_use_modifier:
                return ('Mirror on +X axis\n'
                        '• Shift: Use Only The Selected Axis\n'
                        '• Ctrl: Bisect Off')
            else:
                return 'Mirror on +X axis'
        elif properties.axis=='-X':
            if settings.mirror_use_modifier:
                return ('Mirror on -X axis\n'
                        '• Shift: Use Only The Selected Axis\n'
                        '• Ctrl: Bisect Off')
            else:
                return 'Mirror on -X axis'
        elif properties.axis=='+Y':
            if settings.mirror_use_modifier:
                return ('Mirror on +Y axis\n'
                        '• Shift: Use Only The Selected Axis\n'
                        '• Ctrl: Bisect Off')
            else:
                return 'Mirror on +Y axis'
        elif properties.axis=='-Y':
            if settings.mirror_use_modifier:
                return ('Mirror on -Y axis\n'
                        '• Shift: Use Only The Selected Axis\n'
                        '• Ctrl: Bisect Off')
            else:
                return 'Mirror on -Y axis'
        elif properties.axis=='+Z':
            if settings.mirror_use_modifier:
                return ('Mirror on +Z axis\n'
                        '• Shift: Use Only The Selected Axis\n'
                        '• Ctrl: Bisect Off')
            else:
                return 'Mirror on +Z axis'
        elif properties.axis=='-Z':
            if settings.mirror_use_modifier:
                return ('Mirror on -Z axis\n'
                        '• Shift: Use Only The Selected Axis\n'
                        '• Ctrl: Bisect Off')
            else:
                return 'Mirror on -Z axis'
        else:
            return 'Mirror'


    def execute(self, context):
        settings = context.scene.GP_scene_set
        if settings.mirror_use_modifier:
            name = 'GP Mirror'

            for obj in context.selected_objects:

                if obj.modifiers.get(name) is None:
                    obj.modifiers.new(name, 'MIRROR')
                    obj.modifiers[name].show_expanded = False
                    obj.modifiers[name].use_axis[0] = False


                if self.use_only_one:
                    obj.modifiers[name].use_axis[0] = False
                    obj.modifiers[name].use_axis[1] = False
                    obj.modifiers[name].use_axis[2] = False
                    obj.modifiers[name].use_bisect_axis[0] = False
                    obj.modifiers[name].use_bisect_axis[1] = False
                    obj.modifiers[name].use_bisect_axis[2] = False
                    obj.modifiers[name].use_bisect_flip_axis[0] = False
                    obj.modifiers[name].use_bisect_flip_axis[1] = False
                    obj.modifiers[name].use_bisect_flip_axis[2] = False



                #obj.modifiers[name].show_on_cage = self.show_on_cage

                if self.axis == '+X':
                    obj.modifiers[name].use_axis[0] = True
                    obj.modifiers[name].use_bisect_axis[0] = self.use_bisect
                    obj.modifiers[name].use_bisect_flip_axis[0] = False
                elif self.axis == '-X':
                    obj.modifiers[name].use_axis[0] = True
                    obj.modifiers[name].use_bisect_axis[0] = self.use_bisect
                    obj.modifiers[name].use_bisect_flip_axis[0] = True

                elif self.axis == '+Y':
                    obj.modifiers[name].use_axis[1] = True
                    obj.modifiers[name].use_bisect_axis[1] = self.use_bisect
                    obj.modifiers[name].use_bisect_flip_axis[1] = False
                elif self.axis == '-Y':
                    obj.modifiers[name].use_axis[1] = True
                    obj.modifiers[name].use_bisect_axis[1] = self.use_bisect
                    obj.modifiers[name].use_bisect_flip_axis[1] = True

                elif self.axis == '+Z':
                    obj.modifiers[name].use_axis[2] = True
                    obj.modifiers[name].use_bisect_axis[2] = self.use_bisect
                    obj.modifiers[name].use_bisect_flip_axis[2] = False
                elif self.axis == '-Z':
                    obj.modifiers[name].use_axis[2] = True
                    obj.modifiers[name].use_bisect_axis[2] = self.use_bisect
                    obj.modifiers[name].use_bisect_flip_axis[2] = True

                
                obj.modifiers[name].offset_u = 1.0

                """ obj.modifiers[name].use_mirror_u = self.mirror_x
                obj.modifiers[name].use_mirror_v = self.mirror_y """
        
        else:
            bpy.ops.transform.mirror(
                orient_type='LOCAL', 
                orient_matrix=self.orient_matrix, 
                orient_matrix_type='LOCAL', 
                constraint_axis=(False, False, True),
                center_override=self.center_override,
                )

       
        return {'FINISHED'}


    def invoke(self, context, event):
        if event.ctrl:
            self.use_only_one = True
        else:
            self.use_only_one = False
        
        if event.shift:
            self.use_bisect = False
        else:
            self.use_bisect = True
        return self.execute(context)


    def draw(self, context):
        layout = self.layout
        layout.prop(self, 'axis')



class GP_OT_mirror_modifier_apply_remove(Operator):
    bl_idname = 'gp.mirror_modifier_apply_remove'
    bl_label = 'Apply Modifier(s)'
    bl_description = 'Apply Mirror Modifier'
    bl_options = {'REGISTER', 'UNDO'}

    remove: BoolProperty( name='Remove', default=False )

    @classmethod
    def description(self, context, properties):
        if properties.remove:
            return 'Remove Mirror Modifier(s)'
        else:
            return 'Apply Mirror Modifier(s)'

    def execute(self, context):
        check_count = 0
        check_edit = False
        if context.mode=='EDIT_MESH':
            bpy.ops.object.mode_set(mode='OBJECT')
            check_edit = True
        active = context.active_object
        name = 'GP Mirror'
        for obj in context.selected_objects:
            if obj.modifiers.get(name) is not None:
                context.view_layer.objects.active = obj
                if self.remove:
                    bpy.ops.object.modifier_remove(modifier=name)
                else:
                    bpy.ops.object.modifier_apply(modifier=name)
                check_count += 1
        context.view_layer.objects.active = active
        if check_edit:
            bpy.ops.object.mode_set(mode='EDIT')
        
        if self.remove:
            self.report({'INFO'}, f'{check_count} Mirror Modifier(s) Removed')
        else:
            self.report({'INFO'}, f'{check_count} Mirror Modifier(s) Applied')
        return {'FINISHED'}



class GP_OT_mirror_modifier_display(Operator):
    bl_idname = 'gp.mirror_modifier_display'
    bl_label = 'Display Modifier(s)'
    bl_description = 'Display Mirror Modifier'
    bl_options = {'REGISTER', 'UNDO'}


    def execute(self, context):
        name = 'GP Mirror'
        ckeck_on = False
        for obj in context.selected_objects:
            if obj.modifiers.get(name) is not None:
                if obj.modifiers[name].show_viewport:
                    ckeck_on = True
                    break
        check_count = 0
        active = context.active_object
        for obj in context.selected_objects:
            if obj.modifiers.get(name) is not None:
                context.view_layer.objects.active = obj
                if ckeck_on:
                    obj.modifiers[name].show_viewport = False
                else:
                    obj.modifiers[name].show_viewport = True
                check_count += 1
        context.view_layer.objects.active = active
        
        if ckeck_on:
            self.report({'INFO'}, f'{check_count} Mirror Modifier(s) Display Off')
        else:
            self.report({'INFO'}, f'{check_count} Mirror Modifier(s) Display On')
        return {'FINISHED'}



class GP_GGT_mirror(GizmoGroup):
    bl_idname = 'GP_GGT_mirror'
    bl_label = 'Gizmo Mirror'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'3D', 'PERSISTENT', 'SHOW_MODAL_ALL'}


    @classmethod
    def poll(cls, context):
        return poll_mirror(context)


    @classmethod
    def setup_keymap(cls, keyconfig):
        return keymaps_gizmo_tweak(keyconfig)


    def setup(self, context):
        # --- X Arrow
        self.x = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.x.use_draw_offset_scale = True
        self.x.use_draw_modal = True
        self.x_op = self.x.target_set_operator('gp.mirror_pt')
        self.x_op.axis = 'Z'

        # --- Y Arrow
        self.y = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.y.use_draw_offset_scale = True
        self.y.use_draw_modal = True
        self.y_op = self.y.target_set_operator('gp.mirror_pt')
        self.y_op.axis = 'Z'

        # --- Z Arrow
        self.z = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.z.use_draw_offset_scale = True
        self.z.use_draw_modal = True
        self.z_op = self.z.target_set_operator('gp.mirror_pt')
        self.z_op.axis = 'Z'


        # --- +X
        self.xp = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.xp.use_draw_offset_scale = True
        self.xp.use_draw_modal = True
        self.xp.draw_style = 'BOX'
        #self.xp.draw_options = {'STEM'}
        self.xp_op = self.xp.target_set_operator('gp.mirror')
        self.xp_op.axis = '+X'

        # --- -X
        self.xn = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.xn.use_draw_offset_scale = True
        self.xn.use_draw_modal = True
        self.xn.draw_style = 'BOX'
        #self.xn.draw_options = {'STEM'}
        self.xn_op = self.xn.target_set_operator('gp.mirror')
        self.xn_op.axis = '-X'

        # --- +Y
        self.yp = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.yp.use_draw_offset_scale = True
        self.yp.use_draw_modal = True
        self.yp.draw_style = 'BOX'
        #self.yp.draw_options = {'STEM'}
        self.yp_op = self.yp.target_set_operator('gp.mirror')
        self.yp_op.axis = '+Y'

        # --- -Y
        self.yn = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.yn.use_draw_offset_scale = True
        self.yn.use_draw_modal = True
        self.yn.draw_style = 'BOX'
        #self.yn.draw_options = {'STEM'}
        self.yn_op = self.yn.target_set_operator('gp.mirror')
        self.yn_op.axis = '-Y'

        # --- +Z
        self.zp = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.zp.use_draw_offset_scale = True
        self.zp.use_draw_modal = True
        self.zp.draw_style = 'BOX'
        #self.zp.draw_options = {'STEM'}
        self.zp_op = self.zp.target_set_operator('gp.mirror')
        self.zp_op.axis = '+Z'

        # --- -Z
        self.zn = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.zn.use_draw_offset_scale = True
        self.zn.use_draw_modal = True
        self.zn.draw_style = 'BOX'
        #self.zn.draw_options = {'STEM'}
        self.zn_op = self.zn.target_set_operator('gp.mirror')
        self.zn_op.axis = '-Z'


    def invoke_prepare(self, context, gizmo):
        matrix = gizmo.matrix_basis.copy().to_3x3()
        pp = gizmo.matrix_basis.translation.copy()
        self.x_op.orient_matrix = matrix
        self.y_op.orient_matrix = matrix
        self.z_op.orient_matrix = matrix
        self.x_op.center_override = pp
        self.y_op.center_override = pp
        self.z_op.center_override = pp

        self.xp_op.orient_matrix = matrix
        self.xn_op.orient_matrix = matrix
        self.yp_op.orient_matrix = matrix
        self.yn_op.orient_matrix = matrix
        self.zp_op.orient_matrix = matrix
        self.zn_op.orient_matrix = matrix
        self.xp_op.center_override = pp
        self.xn_op.center_override = pp
        self.yp_op.center_override = pp
        self.yn_op.center_override = pp
        self.zp_op.center_override = pp
        self.zn_op.center_override = pp


    def refresh(self, context):
        settings = context.scene.GP_scene_set

        if settings.mirror_use_modifier:
            xp_mat, yp_mat, zp_mat, xn_mat, yn_mat, zn_mat = get_orient_from_type(position='MIRROR_OBJECT', orient='LOCAL', type='MIRROR')
        else:
            xp_mat, yp_mat, zp_mat, xn_mat, yn_mat, zn_mat = get_orient_from_type(type='MIRROR')

        mo_a = Matrix.Translation(Vector((0.0, 0.0, 1.5)))
        mo_m = Matrix.Translation(Vector((0.0, 0.0, 0.2)))

        self.x.matrix_basis = xp_mat
        self.x.matrix_offset = mo_a
        self.y.matrix_basis = yp_mat
        self.y.matrix_offset = mo_a
        self.z.matrix_basis = zp_mat
        self.z.matrix_offset = mo_a

        self.xp.matrix_basis = xp_mat
        self.xp.matrix_offset = mo_m

        self.xn.matrix_basis = xn_mat
        self.xn.matrix_offset = mo_m

        self.yp.matrix_basis = yp_mat
        self.yp.matrix_offset = mo_m

        self.yn.matrix_basis = yn_mat
        self.yn.matrix_offset = mo_m

        self.zp.matrix_basis = zp_mat
        self.zp.matrix_offset = mo_m

        self.zn.matrix_basis = zn_mat
        self.zn.matrix_offset = mo_m


    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set

        # --- принудительное обновление матрицы
        orient_slots = context.window.scene.transform_orientation_slots[0].type
        if var.REFRESH_MAT or orient_slots=='VIEW' or settings.orientation=='VIEW' or settings.position=='VIEW':
            self.refresh(context)
        var.REFRESH_MAT = False
        
        sca_coef = 2.0
        color_n = props.select_color
        lw = 2.0 * props.size_gizmo_mirror

        # --- X Arrow
        self.x.length = 0.0
        self.x.line_width = lw
        self.x.alpha_highlight = props.highlight_alpha_gizmo
        self.x.alpha = props.alpha_gizmo
        self.x.scale_basis = props.size_gizmo_mirror * sca_coef
        self.x.color = props.x_axis_color
        self.x.color_highlight = props.select_color

        # --- Y Arrow
        self.y.length = 0.0
        self.y.line_width = lw
        self.y.alpha_highlight = props.highlight_alpha_gizmo
        self.y.alpha = props.alpha_gizmo
        self.y.scale_basis = props.size_gizmo_mirror * sca_coef
        self.y.color = props.y_axis_color
        self.y.color_highlight = props.select_color

        # --- Z Arrow
        self.z.length = 0.0
        self.z.line_width = lw
        self.z.alpha_highlight = props.highlight_alpha_gizmo
        self.z.alpha = props.alpha_gizmo
        self.z.scale_basis = props.size_gizmo_mirror * sca_coef
        self.z.color = props.z_axis_color
        self.z.color_highlight = props.select_color


        obj = context.object
        if obj.modifiers.get('GP Mirror'):
            if obj.modifiers['GP Mirror'].use_axis[0]:
                self.x.hide = False
            else:
                self.x.hide = True
            if obj.modifiers['GP Mirror'].use_axis[1]:
                self.y.hide = False
            else:
                self.y.hide = True
            if obj.modifiers['GP Mirror'].use_axis[2]:
                self.z.hide = False
            else:
                self.z.hide = True
        else:
            self.x.hide = True
            self.y.hide = True
            self.z.hide = True

        # --- +X
        self.xp.length = 0.8
        self.xp.line_width = lw
        self.xp.alpha_highlight = props.highlight_alpha_gizmo
        self.xp.alpha = props.alpha_gizmo
        self.xp.scale_basis = props.size_gizmo_mirror * sca_coef
        self.xp.color = props.x_axis_color
        self.xp.color_highlight = props.select_color
        
        # --- -X
        self.xn.length = 0.8
        self.xn.line_width = lw
        self.xn.alpha_highlight = props.highlight_alpha_gizmo
        self.xn.alpha = props.alpha_gizmo
        self.xn.scale_basis = props.size_gizmo_mirror * sca_coef
        self.xn.color = color_n
        self.xn.color_highlight = props.x_axis_color #props.select_color

        # --- +Y
        self.yp.length = 0.8
        self.yp.line_width = lw
        self.yp.alpha_highlight = props.highlight_alpha_gizmo
        self.yp.alpha = props.alpha_gizmo
        self.yp.scale_basis = props.size_gizmo_mirror * sca_coef
        self.yp.color = props.y_axis_color
        self.yp.color_highlight = props.select_color

        # --- -Y
        self.yn.length = 0.8
        self.yn.line_width = lw
        self.yn.alpha_highlight = props.highlight_alpha_gizmo
        self.yn.alpha = props.alpha_gizmo
        self.yn.scale_basis = props.size_gizmo_mirror * sca_coef
        self.yn.color = color_n
        self.yn.color_highlight = props.y_axis_color #props.select_color

        # --- +Z
        self.zp.length = 0.8
        self.zp.line_width = lw
        self.zp.alpha_highlight = props.highlight_alpha_gizmo
        self.zp.alpha = props.alpha_gizmo
        self.zp.scale_basis = props.size_gizmo_mirror * sca_coef
        self.zp.color = props.z_axis_color
        self.zp.color_highlight = props.select_color
        
        # --- -Z
        self.zn.length = 0.8
        self.zn.line_width = lw
        self.zn.alpha_highlight = props.highlight_alpha_gizmo
        self.zn.alpha = props.alpha_gizmo
        self.zn.scale_basis = props.size_gizmo_mirror * sca_coef
        self.zn.color = color_n
        self.zn.color_highlight = props.z_axis_color #props.select_color





classes = [
    GP_OT_mirror_pt,
    GP_OT_mirror,
    GP_OT_mirror_modifier_apply_remove,
    GP_OT_mirror_modifier_display,
    GP_GGT_mirror,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)