import bpy
from bpy.types import Gizmo, GizmoGroup
from gpu import state
from ..utils.poll import active_tool
from ..utils.func import viewport_points
from .. import get_prefs

def border_box():
    context = bpy.context
    props = get_prefs()
    wl, wr, hb, ht, wc, hc = viewport_points(context)
    
    width_left = wl + props.bn_width
    height_top = ht - props.bn_width
    height_bot = props.bn_width
    width_right = wr - props.bn_width

    width = (width_right + width_left) / 2
    heght = (height_top + height_bot) / 2

    return height_top, height_bot, width_right, width_left, width, heght



def border_keys():
    props = get_prefs()
    dpi = bpy.context.preferences.system.ui_scale

    height_top, height_bot, width_right, width_left, width, heght = border_box()

    height_top = height_top /dpi
    height_bot = height_bot /dpi
    width_right = width_right /dpi
    width_left = width_left /dpi
    width = width /dpi
    heght = heght /dpi


    WL = 60
    DOT = 4


    if props.bn_mini:
        keys = [
            (width_left, height_top, 0.0), (width_left+WL, height_top, 0.0), (width_left, height_top, 0.0), (width_left, height_top-WL, 0.0),
            (width_left, height_bot, 0.0), (width_left+WL, height_bot, 0.0), (width_left, height_bot, 0.0), (width_left, height_bot+WL, 0.0),
            (width_right, height_top, 0.0), (width_right-WL, height_top, 0.0), (width_right, height_top, 0.0), (width_right, height_top-WL, 0.0),
            (width_right, height_bot, 0.0), (width_right-WL, height_bot, 0.0), (width_right, height_bot, 0.0), (width_right, height_bot+WL, 0.0),
            (width-DOT, height_top, 0.0), (width+DOT, height_top, 0.0),
            (width-DOT, height_bot, 0.0), (width+DOT, height_bot, 0.0),
            (width_left, heght-DOT, 0.0), (width_left, heght+DOT, 0.0),
            (width_right, heght-DOT, 0.0), (width_right, heght+DOT, 0.0),
            ]
    else:
        keys = [
            (width_left, height_top, 0.0), (width_right, height_top, 0.0),
            (width_right, height_top, 0.0), (width_right, height_bot, 0.0),
            (width_right, height_bot, 0.0), (width_left, height_bot, 0.0),
            (width_left, height_bot, 0.0), (width_left, height_top, 0.0),
            ]
    return keys



class GP_GT_border_navigate(Gizmo):
    bl_idname = 'GP_GT_border_navigate'


    def setup(self):
        __pass=None


    def test_select(self, context, location):
        """ if context.area:
            context.area.tag_redraw() """

        #print(bpy.context.tool_settings.sculpt.brush)

        x = location[0]
        y = location[1]

        height_top, height_bot, width_right, width_left, width, heght = border_box()
        
        if x <  width_left or x > width_right:
            return 0
        
        elif y < height_bot or y > height_top:
            return 0

        else:
            #success, location, normal, face_index, obj, matrix = raycast(context, location)
            #if success == False:
                #return 0
            return -1


    def draw(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set
        if settings.bn_gizmo:
            self.line_width = props.bn_lineWidth
            self.color = props.bn_color
            self.color_highlight = props.bn_color
            self.alpha = props.bn_trans
            self.alpha_highlight = props.bn_trans + 0.1

        
            state.blend_set('ALPHA')
            state.line_width_set(self.line_width)

            keys = border_keys()
            self.custom_shape = self.new_custom_shape('LINES', keys)
            self.draw_custom_shape(self.custom_shape)

            state.line_width_set(1.0)
            state.blend_set('NONE')


    def modal(self, context, event, tweak):
        props = get_prefs()
        if props.bn_type == 'ORBIT':
            bpy.ops.view3d.rotate( 'INVOKE_DEFAULT', use_cursor_init = True )
        elif props.bn_type == 'PAN':
            bpy.ops.view3d.move( 'INVOKE_DEFAULT', use_cursor_init = True )
        elif props.bn_type == 'DOLLY':
            bpy.ops.view3d.zoom( 'INVOKE_DEFAULT', use_cursor_init = False ) 
        return {'FINISHED'}
        


    def invoke(self, context, event):
        if event.alt or event.ctrl or event.shift:
            return {'CANCELLED'}

        return{'RUNNING_MODAL'}



class GP_GGT_gizmo_navigate(GizmoGroup):
    
    bl_idname = 'GP_GGT_gizmo_navigate'
    bl_label = 'Gizmo Navigate'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'PERSISTENT', 'SHOW_MODAL_ALL', 'SCALE'} #'DEPTH_3D' , 'TOOL_INIT', 'SELECT'


    @classmethod
    def poll(cls, context):
        settings = context.scene.GP_scene_set
        if settings.bn_sculpt_retop:
            if context.mode in {'SCULPT'} or active_tool().idname == 'tool.gp_retopo':
                return settings.bn_gizmo
        else:
            return settings.bn_gizmo

    
    def setup(self, context):
        self.gizmos.new('GP_GT_border_navigate')

    





classes = [
    GP_GT_border_navigate,
    GP_GGT_gizmo_navigate,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)