import bpy
from bpy.types import GizmoGroup
from .. import var
from bpy_extras.view3d_utils import location_3d_to_region_2d
from ..utils.poll import poll_quick_widget, poll_mirror
from ..icons import load_textures
from ..utils.func import (
    viewport_points,
    transform_orientation,
    transform_pivot_point,
    transform_snap_to,
    transform_proportional,
)
from ..matrix.matrix_ensure import get_orient_from_type
from ..keymaps import keymaps_qw_tweak
from .. import get_prefs



class GP_GGT_quick_widget(GizmoGroup):
    bl_idname = 'GP_GGT_quick_widget'
    bl_label = 'Quick Widget'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'PERSISTENT', 'SHOW_MODAL_ALL', 'SCALE'}


    @classmethod
    def poll(cls, context):
        if context.mode in var.CONTEXT_MODE:
            settings = context.scene.GP_scene_set
            if settings.gzm_type=='MIRROR':
                return poll_mirror(context)
            else:
                return poll_quick_widget(context)

    @classmethod
    def setup_keymap(cls, keyconfig):
        return keymaps_qw_tweak(keyconfig)

    def setup(self, context):
        load_textures() # --- load textures for icons

        self.bt_orient = self.gizmos.new('GP_GT_button')
        self.bt_orient.use_tooltip = True
        self.bt_orient.type = 'CAPSULE'
        op = self.bt_orient.target_set_operator('gp.call_panel')
        op.name = 'GP_PT_transform_orientations'
        op.bool_op = 'PRO_ORIENTATION'

        self.bt_pos = self.gizmos.new('GP_GT_button')
        self.bt_pos.use_tooltip = True
        self.bt_pos.type = 'CAPSULE'
        op = self.bt_pos.target_set_operator('gp.call_panel')
        op.name = 'GP_PT_transform_positions'
        op.bool_op = 'PRO_CENTER'

        self.bt_snap = self.gizmos.new('GP_GT_button')
        self.bt_snap.use_tooltip = True
        self.bt_snap.type = 'CAPSULE'
        op = self.bt_snap.target_set_operator('gp.call_panel')
        op.name = 'VIEW3D_PT_snapping'
        op.bool_op = 'SNAP'

        self.bt_pe = self.gizmos.new('GP_GT_button')
        self.bt_pe.use_tooltip = True
        self.bt_pe.type = 'CAPSULE'
        op = self.bt_pe.target_set_operator('gp.call_panel')
        op.name = 'VIEW3D_PT_proportional_edit'
        op.bool_op = 'PROPORT_EDIT'

        self.bt_cursor = self.gizmos.new('GP_GT_button')
        self.bt_cursor.icon_id = 9
        self.bt_cursor.use_tooltip = True
        self.bt_cursor.type = 'CAPSULE'
        op = self.bt_cursor.target_set_operator('gp.call_panel')
        op.name = 'GP_PT_3d_cursor'
        op.bool_op = 'GIZMO_CURSOR'
        
        self.bt_mini = self.gizmos.new('GP_GT_button_backplane')
        self.bt_mini.icon_id = 10
        self.bt_mini.use_tooltip = False
        


        # --- Mirror
        self.mir_mod = self.gizmos.new('GP_GT_button')  # --- Modifier
        self.mir_mod.type = 'CAPSULE'
        op = self.mir_mod.target_set_operator('wm.context_toggle')
        op.data_path = 'scene.GP_scene_set.mirror_use_modifier'

        self.mir_apply = self.gizmos.new('GP_GT_button')   # --- Apply Modifier
        self.mir_apply.icon_id = 2
        self.mir_apply.type = 'CAPSULE'
        op = self.mir_apply.target_set_operator('gp.mirror_modifier_apply_remove')
        
        self.mir_remove = self.gizmos.new('GP_GT_button')   # --- Remove Modifier
        self.mir_remove.icon_id = 3
        self.mir_remove.type = 'CAPSULE'
        op = self.mir_remove.target_set_operator('gp.mirror_modifier_apply_remove')
        op.remove = True

        self.mir_display = self.gizmos.new('GP_GT_button')   # --- Display Modifier
        self.mir_display.type = 'CAPSULE'
        self.mir_display.target_set_operator('gp.mirror_modifier_display')
        

    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set
        toolSet = context.scene.tool_settings
        wl, wr, hb, ht, wc, hc = viewport_points(context)

        dpi = context.preferences.system.ui_scale
        if settings.gzm_type == 'GIZMOPRO':
            coef = props.size_gizmo_pro * dpi
        elif settings.gzm_type == 'LUNA':
            coef = props.luna_size * dpi
        elif settings.gzm_type == 'MALIBU':
            coef = props.malibu_size * dpi
        elif settings.gzm_type == 'VR':
            coef = props.size_gizmo_vr * dpi
        elif settings.gzm_type == 'MIRROR':
            coef = props.size_gizmo_mirror * dpi
        elif settings.gzm_type == 'SHEAR':
            coef = props.size_gizmo_pro * dpi

        if settings.gzm_type=='MIRROR':
            if settings.mirror_use_modifier:
                mat = get_orient_from_type(position='MIRROR_OBJECT', orient='LOCAL', type='MIRROR')[2]
            else:
                mat = get_orient_from_type(type='MIRROR')[2]
            pos2d = location_3d_to_region_2d(context.region, context.space_data.region_3d, mat.translation)
        else:
            pos2d = location_3d_to_region_2d(context.region, context.space_data.region_3d, var.LOCATION)
        
        if settings.show_gizmo is False:
            pos2d = (wc, -props.qw_offset_y)
        pos_y = pos2d[1] + props.qw_offset_y * coef

        if settings.gzm_type=='MIRROR':
            num_buttons = 5
        else:
            num_buttons = len(set(props.qw_buttons))
        butSize = ((80 * 0.35) / 2) * coef * 1.5 * props.qw_scale
        button_spacing = (props.qw_offset_x+30) * props.qw_scale * coef # расстояние между кнопками
        button_width = butSize # ширина каждой кнопки, можно изменить на соответствующее значение
        total_buttons_width = num_buttons * button_width + (num_buttons - 1) * button_spacing
        half_total_width = total_buttons_width / 2

        # --- limit position
        if (pos2d[0]-total_buttons_width/2) < wl:
            pos2d[0] = wl + total_buttons_width/2
        elif (pos2d[0]+total_buttons_width/2) > wr:
            pos2d[0] = wr - total_buttons_width/2
        if (pos_y-butSize*2) < hb:
            pos_y = hb + butSize*2
        elif (pos_y+butSize*2) > ht:
            pos_y = ht - butSize*2

        # --- QW Mini
        self.bt_mini.scale_basis = butSize / 1.2
        self.bt_mini.width = butSize
        self.bt_mini.height = butSize
        self.bt_mini.matrix_basis[0][3] = pos2d[0]
        self.bt_mini.matrix_basis[1][3] = pos_y
        if props.qw_mini:
            self.bt_mini.hide = False
            is_highlight = [gzm.is_highlight for gzm in self.gizmos]
            if True in is_highlight:
                self.bt_mini.icon_show = False
                self.bt_mini.width = total_buttons_width + 50*props.qw_scale
                self.bt_mini.height = butSize * 2
            else:
                self.bt_mini.icon_show = True
                self.bt_mini.width = butSize
                self.bt_mini.height = butSize
        else:
            self.bt_mini.hide = True
        
        # --- Getters
        to = transform_orientation(context)
        tpp = transform_pivot_point(context)
        tst = transform_snap_to(context)
        tp = transform_proportional(context)

        active_buttons_list = []
        if 'ORIENTATION' in props.qw_buttons:
            active_buttons_list.append('ORIENTATION')
        if 'POSITION' in props.qw_buttons:
            active_buttons_list.append('POSITION')
        if 'SNAPPING' in props.qw_buttons:
            active_buttons_list.append('SNAPPING')
        if 'PROP_EDIT' in props.qw_buttons:
            active_buttons_list.append('PROP_EDIT')
        if 'CURSOR' in props.qw_buttons:
            active_buttons_list.append('CURSOR')


        if 'ORIENTATION' in active_buttons_list:
            if props.qw_mini: 
                if self.bt_mini.icon_show == False:
                    self.bt_orient.hide = False
                else:
                    self.bt_orient.hide = True
            else:
                self.bt_orient.hide = False
            idx_bt_orient = active_buttons_list.index('ORIENTATION')
        else:
            self.bt_orient.hide = True
            idx_bt_orient = 0
        self.bt_orient.icon_id = to[2]
        self.bt_orient.scale_basis = butSize
        self.bt_orient.matrix_basis[0][3] = pos2d[0] - half_total_width + (idx_bt_orient * (button_width + button_spacing)) + (button_width / 2) #pos2d[0] - pos_x * 3
        self.bt_orient.matrix_basis[1][3] = pos_y
        self.bt_orient.alpha = props.qw_opacity
        

        if 'POSITION' in active_buttons_list:
            if props.qw_mini: 
                if self.bt_mini.icon_show == False:
                    self.bt_pos.hide = False
                else:
                    self.bt_pos.hide = True
            else:
                self.bt_pos.hide = False
            idx_bt_pos = active_buttons_list.index('POSITION')
        else:
            self.bt_pos.hide = True
            idx_bt_pos = 1
        self.bt_pos.icon_id = tpp[2]
        self.bt_pos.scale_basis = butSize
        self.bt_pos.matrix_basis[0][3] = pos2d[0] - half_total_width + (idx_bt_pos * (button_width + button_spacing)) + (button_width / 2) #pos2d[0] - pos_x 
        self.bt_pos.matrix_basis[1][3] = pos_y
        self.bt_pos.alpha = props.qw_opacity


        if 'SNAPPING' in active_buttons_list:
            if props.qw_mini: 
                if self.bt_mini.icon_show == False:
                    self.bt_snap.hide = False
                else:
                    self.bt_snap.hide = True
            else:
                self.bt_snap.hide = False
            idx_bt_snap = active_buttons_list.index('SNAPPING')
        else:
            self.bt_snap.hide = True
            idx_bt_snap = 2
        self.bt_snap.bool_op = toolSet.use_snap
        self.bt_snap.icon_id = tst[2]
        self.bt_snap.scale_basis = butSize
        self.bt_snap.matrix_basis[0][3] = pos2d[0] - half_total_width + (idx_bt_snap * (button_width + button_spacing)) + (button_width / 2) #pos2d[0] + pos_x 
        self.bt_snap.matrix_basis[1][3] = pos_y
        self.bt_snap.alpha = props.qw_opacity


        if 'PROP_EDIT' in active_buttons_list:
            if props.qw_mini: 
                if self.bt_mini.icon_show == False:
                    self.bt_pe.hide = False
                else:
                    self.bt_pe.hide = True
            else:
                self.bt_pe.hide = False
            idx_bt_pe = active_buttons_list.index('PROP_EDIT')
        else:
            self.bt_pe.hide = True
            idx_bt_pe = 3
        self.bt_pe.icon_id = tp[2]
        self.bt_pe.scale_basis = butSize
        self.bt_pe.matrix_basis[0][3] = pos2d[0] - half_total_width + (idx_bt_pe * (button_width + button_spacing)) + (button_width / 2) #pos2d[0] + pos_x * 3
        self.bt_pe.matrix_basis[1][3] = pos_y
        self.bt_pe.alpha = props.qw_opacity
        if context.mode=='OBJECT':
            self.bt_pe.bool_op = toolSet.use_proportional_edit_objects
        else:
            self.bt_pe.bool_op = toolSet.use_proportional_edit


        if 'CURSOR' in active_buttons_list:
            if props.qw_mini: 
                if self.bt_mini.icon_show == False:
                    self.bt_cursor.hide = False
                else:
                    self.bt_cursor.hide = True
            else:
                self.bt_cursor.hide = False
            idx_bt_cursor = active_buttons_list.index('CURSOR')
        else:
            self.bt_cursor.hide = True
            idx_bt_cursor = 4
        self.bt_cursor.bool_op = settings.cursor_gizmo
        self.bt_cursor.scale_basis = butSize
        self.bt_cursor.matrix_basis[0][3] = pos2d[0] - half_total_width + (idx_bt_cursor * (button_width + button_spacing)) + (button_width / 2)
        self.bt_cursor.matrix_basis[1][3] = pos_y
        self.bt_cursor.alpha = props.qw_opacity


        if settings.gzm_type=='MIRROR':
            self.bt_mini.hide = True
            self.bt_orient.hide = True
            self.bt_pos.hide = True
            self.bt_snap.hide = True
            self.bt_pe.hide = True
            self.bt_cursor.hide = True
            self.mir_mod.hide = False

            use_modifier = [ob.modifiers.get('GP Mirror') is not None for ob in context.selected_objects]
            display_modifier = []
            for ob in context.selected_objects:
                if ob.modifiers.get('GP Mirror') is not None:
                    display_modifier.append(ob.modifiers['GP Mirror'].show_viewport)

            self.mir_mod.scale_basis = butSize
            self.mir_mod.alpha = props.qw_opacity
            if True in use_modifier:
                self.mir_mod.matrix_basis[0][3] = pos2d[0] - half_total_width + (0 * (button_width + button_spacing)) + (button_width / 2)
            else:
                self.mir_mod.matrix_basis[0][3] = pos2d[0]
            self.mir_mod.matrix_basis[1][3] = pos_y
            if settings.mirror_use_modifier:
                self.mir_mod.icon_id = 4
            else:
                self.mir_mod.icon_id = 5
            self.mir_mod.bool_op = settings.mirror_use_modifier

            self.mir_display.scale_basis = butSize
            self.mir_display.alpha = props.qw_opacity
            self.mir_display.matrix_basis[0][3] = pos2d[0] - half_total_width + (1 * (button_width + button_spacing)) + (button_width / 2)
            self.mir_display.matrix_basis[1][3] = pos_y
            if True in display_modifier:
                self.mir_display.icon_id = 6
            else:
                self.mir_display.icon_id = 7
            if True in use_modifier:
                self.mir_display.hide = False
            else:
                self.mir_display.hide = True
            self.mir_display.bool_op = True in display_modifier

            self.mir_apply.scale_basis = butSize
            self.mir_apply.alpha = props.qw_opacity
            self.mir_apply.matrix_basis[0][3] = pos2d[0] - half_total_width + (3 * (button_width + button_spacing)) + (button_width / 2)
            self.mir_apply.matrix_basis[1][3] = pos_y
            if True in use_modifier:
                self.mir_apply.hide = False
            else:
                self.mir_apply.hide = True
            
            self.mir_remove.scale_basis = butSize
            self.mir_remove.alpha = props.qw_opacity
            self.mir_remove.matrix_basis[0][3] = pos2d[0] - half_total_width + (4 * (button_width + button_spacing)) + (button_width / 2)
            self.mir_remove.matrix_basis[1][3] = pos_y
            if True in use_modifier:
                self.mir_remove.hide = False
            else:
                self.mir_remove.hide = True

            
            
        else:
            self.mir_mod.hide = True
            self.mir_apply.hide = True
            self.mir_remove.hide = True
            self.mir_display.hide = True
            

        







classes = [
    GP_GGT_quick_widget,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)