import bpy
from mathutils import Matrix, Vector
from bpy.types import GizmoGroup
from bpy.props import EnumProperty, IntProperty, BoolProperty, FloatProperty, FloatVectorProperty, CollectionProperty, StringProperty, PointerProperty
from ..utils.poll import poll_gizmopro
from ..matrix.matrix_ensure import get_orient_from_type
from math import pi
from ..utils.operators.transform_from_value import GP_OT_transform_from_value



class GP_GGT_gizmo_rotate(GizmoGroup):
    
    bl_idname = 'GP_GGT_gizmo_rotate'
    bl_label = 'Gizmo PRO'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW' 
    bl_options = {'3D', 'EXCLUDE_MODAL'} #'DEPTH_3D' , 'TOOL_INIT', 'SELECT', , 'SCALE'  , 'SHOW_MODAL_ALL' , 'PERSISTENT' , VR_REDRAWS  EXCLUDE_MODAL
    

    # Helper functions
    @staticmethod
    def my_target_operator(context):
        wm = context.window_manager
        op = wm.operators[-1] if wm.operators else None
        if isinstance(op, GP_OT_transform_from_value):
            return op
        return None

    @staticmethod
    def my_view_orientation(context):
        rv3d = context.space_data.region_3d
        view_inv = rv3d.view_matrix.to_3x3()
        return view_inv.normalized()

    @classmethod
    def poll(cls, context):
        op = cls.my_target_operator(context)
        if op:
            check_op = True
        else:
            check_op = False
        #print(check_op)

        if check_op and op is None:
            print('off')
            wm = context.window_manager
            wm.gizmo_group_type_unlink_delayed('GP_GGT_gizmo_rotate')
            return False
        else:
            return True

    def setup(self, context):
        print('run gizmo')

        def direction_get_cb():
            op = self.my_target_operator(context)
            #op = bpy.context.active_operator
            return -op.value_angle

        def direction_set_cb(value):
            pass

        gz = self.gizmos.new("GIZMO_GT_dial_3d")
        gz.target_set_handler("offset", get=direction_get_cb, set=direction_set_cb)
        gz.draw_options = {'ANGLE_VALUE', 'ANGLE_START_Y'}
        
        
        gz.wrap_angle = False
        gz.arc_inner_factor = 0.9
        gz.click_value = pi * 2
        gz.use_draw_value = True
        
        gz.hide_select = True ###

        gz.scale_basis = 2.0
        gz.line_width = 1.5
        
        gz.color = 0.8, 0.8, 0.8
        gz.alpha = 0.5

        gz.color_highlight = 1.0, 1.0, 1.0
        gz.alpha_highlight = 1.0


    
        
       
        op = self.my_target_operator(context)
        mX, mY, mZ = get_orient_from_type(context)

        
        
        if op:
            axis = op.axis[0], op.axis[1], op.axis[2]
            if axis == (True, False, False):
                gz.matrix_basis = mX
            elif axis == (False, True, False):
                gz.matrix_basis = mY
            elif axis == (False, False, True):
                gz.matrix_basis = mZ
    

    """ def refresh(self, context):
        op = self.my_target_operator(context)
        if op is None:
            print('Off') """







classes = [
    GP_GGT_gizmo_rotate,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)