import bpy
from bpy.types import GizmoGroup
from ..utils.poll import poll_scale
from ..matrix.matrix_ensure import get_orient_from_type
from .. import var
from ..utils.draw_utils.through import alpha_gizmo_vr
from ..keymaps import keymaps_gizmo_tweak
from .. import get_prefs



class GP_GGT_scale(GizmoGroup):
    bl_idname = 'GP_GGT_scale'
    bl_label = 'Scale Gizmo'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW' 
    bl_options = {'3D', 'PERSISTENT', 'SHOW_MODAL_ALL'}
    

    @classmethod
    def poll(cls, context):
        return poll_scale(context)
    

    @classmethod
    def setup_keymap(cls, keyconfig):
        return keymaps_gizmo_tweak(keyconfig)


    def setup(self, context):
        self.scale_x_p = self.gizmos.new('GIZMO_GT_move_3d') # GP_GT_point
        self.scale_x_p.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.scale_x_p.use_draw_offset_scale = True 
        self.scale_x_p.use_tooltip = False
        self.scale_x_p.use_draw_modal = True
        self.sxop = self.scale_x_p.target_set_operator('gp.scale_from_bbox')
        self.sxop.axis = 'X'

        self.scale_x_n = self.gizmos.new('GIZMO_GT_move_3d')
        self.scale_x_n.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.scale_x_n.use_draw_offset_scale = True 
        self.scale_x_n.use_tooltip = False
        self.scale_x_n.use_draw_modal = True
        self.sxon = self.scale_x_n.target_set_operator('gp.scale_from_bbox')
        self.sxon.axis = 'X'


        self.scale_y_p = self.gizmos.new('GIZMO_GT_move_3d')
        self.scale_y_p.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.scale_y_p.use_draw_offset_scale = True 
        self.scale_y_p.use_tooltip = False
        self.scale_y_p.use_draw_modal = True
        self.syop = self.scale_y_p.target_set_operator('gp.scale_from_bbox')
        self.syop.axis = 'Y'

        self.scale_y_n = self.gizmos.new('GIZMO_GT_move_3d')
        self.scale_y_n.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.scale_y_n.use_draw_offset_scale = True 
        self.scale_y_n.use_tooltip = False
        self.scale_y_n.use_draw_modal = True
        self.syon = self.scale_y_n.target_set_operator('gp.scale_from_bbox')
        self.syon.axis = 'Y'


        self.scale_z_p = self.gizmos.new('GIZMO_GT_move_3d')
        self.scale_z_p.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.scale_z_p.use_draw_offset_scale = True 
        self.scale_z_p.use_tooltip = False
        self.scale_z_p.use_draw_modal = True
        self.szop = self.scale_z_p.target_set_operator('gp.scale_from_bbox')
        self.szop.axis = 'Z'

        self.scale_z_n = self.gizmos.new('GIZMO_GT_move_3d')
        self.scale_z_n.draw_options = {'FILL', 'FILL_SELECT', 'ALIGN_VIEW'}
        self.scale_z_n.use_draw_offset_scale = True 
        self.scale_z_n.use_tooltip = False
        self.scale_z_n.use_draw_modal = True
        self.szon = self.scale_z_n.target_set_operator('gp.scale_from_bbox')
        self.szon.axis = 'Z'


    def invoke_prepare(self, context, gizmo):
        self.sxop.matrix_basis = self.scale_x_n.matrix_basis
        self.sxop.pp = (self.scale_x_n.matrix_basis.translation + self.scale_x_p.matrix_basis.translation) / 2

        self.sxon.matrix_basis = self.scale_x_p.matrix_basis
        self.sxon.pp = (self.scale_x_n.matrix_basis.translation + self.scale_x_p.matrix_basis.translation) / 2

        self.syop.matrix_basis = self.scale_y_n.matrix_basis
        self.syop.pp = (self.scale_y_n.matrix_basis.translation + self.scale_y_p.matrix_basis.translation) / 2

        self.syon.matrix_basis = self.scale_y_p.matrix_basis
        self.syon.pp = (self.scale_y_n.matrix_basis.translation + self.scale_y_p.matrix_basis.translation) / 2

        self.szop.matrix_basis = self.scale_z_n.matrix_basis
        self.szop.pp = (self.scale_z_n.matrix_basis.translation + self.scale_z_p.matrix_basis.translation) / 2

        self.szon.matrix_basis = self.scale_z_p.matrix_basis
        self.szon.pp = (self.scale_z_n.matrix_basis.translation + self.scale_z_p.matrix_basis.translation) / 2


    def refresh(self, context):
        mXP, mYP, mZP, mXN, mYN, mZN = get_orient_from_type(type='VR')
        self.scale_x_p.matrix_basis = mXP
        self.scale_x_n.matrix_basis = mXN
        self.scale_y_p.matrix_basis = mYP
        self.scale_y_n.matrix_basis = mYN
        self.scale_z_p.matrix_basis = mZP
        self.scale_z_n.matrix_basis = mZN
        

    def draw_prepare(self, context):
        # --- принудительное обновление матрицы
        orient_slots = context.window.scene.transform_orientation_slots[0].type
        if var.REFRESH_MAT or orient_slots == 'VIEW':
            self.refresh(context)
        var.REFRESH_MAT = False

        props = get_prefs()
        settings = context.scene.GP_scene_set

        scale = 0.05
        lw = 1.5

        alpha_x, hide_select_x, scale_factor_x = alpha_gizmo_vr(self.scale_x_p.matrix_basis, props)
        alpha_y, hide_select_y, scale_factor_y = alpha_gizmo_vr(self.scale_y_p.matrix_basis, props)
        alpha_z, hide_select_z, scale_factor_z = alpha_gizmo_vr(self.scale_z_p.matrix_basis, props)
        alpha_xn, hide_select_xn, scale_factor_xn = alpha_gizmo_vr(self.scale_x_n.matrix_basis, props)
        alpha_yn, hide_select_yn, scale_factor_yn = alpha_gizmo_vr(self.scale_y_n.matrix_basis, props)
        alpha_zn, hide_select_zn, scale_factor_zn = alpha_gizmo_vr(self.scale_z_n.matrix_basis, props)

        # --- +
        self.scale_x_p.alpha = alpha_x
        self.scale_x_p.hide_select = hide_select_x
        self.scale_x_p.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_x_p.scale_basis = scale * scale_factor_x
        self.scale_x_p.color = props.secondary_color #props.x_axis_color
        self.scale_x_p.color_highlight = props.select_color
        self.scale_x_p.line_width = lw

        self.scale_y_p.alpha = alpha_y
        self.scale_y_p.hide_select = hide_select_y
        self.scale_y_p.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_y_p.scale_basis = scale * scale_factor_y
        self.scale_y_p.color = props.secondary_color #props.y_axis_color
        self.scale_y_p.color_highlight = props.select_color
        self.scale_y_p.line_width = lw
        
        self.scale_z_p.alpha = alpha_z
        self.scale_z_p.hide_select = hide_select_z
        self.scale_z_p.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_z_p.scale_basis = scale * scale_factor_z
        self.scale_z_p.color = props.secondary_color #props.z_axis_color
        self.scale_z_p.color_highlight = props.select_color
        self.scale_z_p.line_width = lw

        # --- -
        self.scale_x_n.alpha = alpha_xn
        self.scale_x_n.hide_select = hide_select_xn
        self.scale_x_n.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_x_n.scale_basis = scale * scale_factor_xn
        self.scale_x_n.color = props.secondary_color #props.color_negative_elements_vr
        self.scale_x_n.color_highlight = props.select_color #props.x_axis_color
        self.scale_x_n.line_width = lw

        self.scale_y_n.alpha = alpha_yn
        self.scale_y_n.hide_select = hide_select_yn
        self.scale_y_n.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_y_n.scale_basis = scale * scale_factor_yn
        self.scale_y_n.color = props.secondary_color #props.color_negative_elements_vr
        self.scale_y_n.color_highlight = props.select_color #props.y_axis_color
        self.scale_y_n.line_width = lw

        self.scale_z_n.alpha = alpha_zn
        self.scale_z_n.hide_select = hide_select_zn
        self.scale_z_n.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_z_n.scale_basis = scale * scale_factor_zn
        self.scale_z_n.color = props.secondary_color #props.color_negative_elements_vr
        self.scale_z_n.color_highlight = props.select_color #props.z_axis_color
        self.scale_z_n.line_width = lw
        

        # --- VISIBLE GIZMO
        """ if settings.sca_gizmo:
            self.scale_x_p.hide = False
            self.scale_y_p.hide = False
            self.scale_z_p.hide = False

            self.scale_x_n.hide = False
            self.scale_y_n.hide = False
            self.scale_z_n.hide = False
        else:
            self.scale_x_p.hide = True
            self.scale_y_p.hide = True
            self.scale_z_p.hide = True

            self.scale_x_n.hide = True
            self.scale_y_n.hide = True
            self.scale_z_n.hide = True """

        # --- LOCK AXIS
        if context.mode == 'OBJECT':
            if context.object.lock_scale[0]:
                self.scale_x_p.hide = True
                self.scale_x_n.hide = True
            if context.object.lock_scale[1]:
                self.scale_y_p.hide = True
                self.scale_y_n.hide = True
            if context.object.lock_scale[2]:
                self.scale_z_p.hide = True
                self.scale_z_n.hide = True



classes = [
    GP_GGT_scale,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)