import bpy
from mathutils import Matrix, Vector, Quaternion
from bpy.types import GizmoGroup, Operator
from ..utils.poll import poll_searcher
from bpy_extras import view3d_utils
from .. import var
from mathutils.geometry import intersect_line_line_2d, intersect_point_quad_2d
from math import atan2, radians, pi
from ..utils.func import viewport_points
from bpy.props import FloatVectorProperty, FloatProperty
#from ..utils.custom_math.quaternion import look_rotation
from .. import get_prefs




class GP_OT_camera_look_at(Operator):
    bl_idname = "gp.camera_look_at"
    bl_label = "Smooth Camera Operator"

    target_point: FloatVectorProperty(name="Target Point", default=(0.0, 0.0, 0.0))
    transition_time: FloatProperty(name="Transition Time", default=5.0)

    timer = None

    def modal(self, context, event):
        if event.type == 'TIMER':
            if self.current_time < self.transition_time:
                interp_factor = self.current_time / self.transition_time
                new_rotation = self.start_rotation.slerp(self.end_rotation, interp_factor)
                context.area.spaces.active.region_3d.view_rotation = new_rotation
                self.current_time += self.timer.time_duration
            else:
                context.window_manager.event_timer_remove(self.timer)
                return {'FINISHED'}
        return {'RUNNING_MODAL'}

    def execute(self, context):
        view3d = context.area.spaces.active.region_3d
        self.start_rotation = view3d.view_rotation.copy()

        

        # Вычисление целевого направления взгляда
        direction = (view3d.view_location - Vector(self.target_point)).normalized()
        
        #self.end_rotation = look_rotation(direction, Vector((0.0, 0.0, 1.0)))  # Используем вашу функцию
        self.end_rotation = direction.to_track_quat('Z', 'Y')  # Используем встроенную функцию



        self.current_time = 0.0
        self.timer = context.window_manager.event_timer_add(0.01, window=context.window)
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

        

        


class GP_GGT_gizmo_searcher(GizmoGroup):
    bl_idname = 'GP_GGT_gizmo_searcher'
    bl_label = 'Searcher'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'PERSISTENT', 'SCALE'}

    @classmethod
    def poll(cls, context):
        return poll_searcher(context)

    def setup(self, context):
        self.arrow = self.gizmos.new('GP_GT_searcher')
        self.arrow.alpha = 0.9
        self.arrow.alpha_highlight = 0.9
        self.op = self.arrow.target_set_operator('gp.camera_look_at')

    def invoke_prepare(self, context, gizmo):
        self.op.target_point = var.LOCATION
        
    def draw_prepare(self, context):
        props = get_prefs()

        lg = var.LOCATION
        point_2d = view3d_utils.location_3d_to_region_2d(context.region, context.region_data, lg)
        wl, wr, hb, ht, wc, hc = viewport_points(context)
        center = Vector((wc, hc))

        self.arrow.hide = (wl <= point_2d.x <= wr and hb <= point_2d.y <= ht)

        if not self.arrow.hide:
            # позиция гизмо: Вычисление пересечений с границами экрана
            positions = []
            edges_view = [(wl, wl, ht, hb), (wr, wr, ht, hb), (wl, wr, ht, ht), (wl, wr, hb, hb)]
            for edge in edges_view:
                intersection = intersect_line_line_2d(center, point_2d, Vector((edge[0], edge[2])), Vector((edge[1], edge[3])))
                if intersection:
                    positions.append(Vector(intersection))
            l = min(positions, key=lambda p: (p - center).length)
            l = Vector((l.x, l.y, 0.0))

            # Определение угла наклона
            direction = point_2d - center
            angle = atan2(direction.y, direction.x)
            angle -= pi / 2
            q = Quaternion((0.0, 0.0, 1.0), angle)

            self.arrow.matrix_basis = Matrix.LocRotScale(l, q, Vector((1.0, 1.0, 1.0)))
       
        self.arrow.scale_basis = ((80 * 0.35) / 2)
        self.arrow.color = props.primary_color
        self.arrow.color_highlight = props.select_color


classes = [
    GP_GGT_gizmo_searcher,
    GP_OT_camera_look_at,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)