import bpy
from mathutils import Matrix, Vector, Quaternion
from bpy.types import GizmoGroup
from ..utils.poll import poll_shear
from ..matrix.matrix_ensure import get_orient_from_type, get_position
from .. import var
from ..utils.draw_utils.through import alpha_gizmo, alpha_arc_gizmo
from math import radians
from ..keymaps import keymaps_gizmo_tweak
from .. import get_prefs



class GP_GGT_shear(GizmoGroup):
    
    bl_idname = 'GP_GGT_shear'
    bl_label = 'Gizmo Shear'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW' 
    bl_options = {'3D', 'PERSISTENT', 'EXCLUDE_MODAL'}
    xgzgroup = [
        [[None, None], [None, None]],
        [[None, None], [None, None]],
        [[None, None], [None, None]],
    ]

    @classmethod
    def poll(cls, context):
        return poll_shear(context)
    
    
    @classmethod
    def setup_keymap(cls, keyconfig):
        return keymaps_gizmo_tweak(keyconfig)

    
    def setup(self, context):
        

        props = get_prefs()
        axis_color = [props.x_axis_color, props.y_axis_color, props.z_axis_color]

        """ for i in range(3):
            bpy.types.Scene['axis_color'][i] = axis_color[i] """


        def interp_v3_v3v3(r, a, b, t):
            s = 1.0 - t
            r[0] = s * a[0] + t * b[0]
            r[1] = s * a[1] + t * b[1]
            r[2] = s * a[2] + t * b[2]


        for i in range(3):
            #print(i)
            for j in range(2):
                gzm = self.gizmos.new('GIZMO_GT_arrow_3d')
                gzm.draw_style = 'BOX'
                i_ortho_a = (i + j + 1) % 3
                i_ortho_b = (i + (1 - j) + 1) % 3
                interp_v3_v3v3(gzm.color, axis_color[i_ortho_a], axis_color[i_ortho_b], 0.75)
                gzm.alpha = 0.5
                op = gzm.target_set_operator('transform.transform')
                op.mode = 'SHEAR'
                op.release_confirm = True

                self.xgzgroup[i][j][0] = gzm
                self.xgzgroup[i][j][1] = op

    def refresh(self, context):
        """ def mul_v3_fl(r, f): # math vector inline
            r[0] *= f 
            r[1] *= f
            r[2] *= f """
        """ def wm_gizmo_set_matrix_rotation_from_yz_axis__internal(matrix, y_axis, z_axis):
            from ..utils.custom_math.math import normalize_v3_v3, cross_v3_v3v3
            normalize_v3_v3(matrix[1], y_axis)
            normalize_v3_v3(matrix[2], z_axis)
            cross_v3_v3v3(matrix[0], matrix[1], matrix[2])
            normalize_v3_v3(matrix[0], matrix[0]) """
        
       
        #loc = get_position(context) # --- get position
        mat = get_orient_from_type(type='SHEAR') # --- get orientation
        for i in range(3):
            for j in range(2):
                gzm = self.xgzgroup[i][j][0]
                op = self.xgzgroup[i][j][1]
                
                i_ortho_a = (i + j + 1) % 3
                i_ortho_b = (i + (1 - j) + 1) % 3
                
                gzm.matrix_basis = mat[i][j]
                #gzm.matrix_basis.translation = loc
                
                

                new_mat = mat[i][j].copy().to_3x3()
                #if i==0:
                new_mat = new_mat @ Quaternion( (0.0, 1.0, 0.0), radians(90) ).to_matrix().to_3x3()
                #new_mat = new_mat @ Quaternion( (1.0, 0.0, 0.0), radians(90) ).to_matrix().to_3x3()
                """ elif i==1:
                    new_mat = new_mat @ Quaternion( (1.0, 0.0, 0.0), radians(90) ).to_matrix().to_3x3() """
                
                op.orient_matrix = new_mat
                op.center_override = gzm.matrix_basis.translation

    def draw_prepare(self, context):
        settings = context.scene.GP_scene_set
        # --- принудительное обновление матрицы
        orient_slots = context.window.scene.transform_orientation_slots[0].type
        if var.REFRESH_MAT or settings.position=='VIEW' or orient_slots=='VIEW' or settings.orientation=='VIEW':
            self.refresh(context)
        var.REFRESH_MAT = False
                


  












classes = [
    GP_GGT_shear,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)