import bpy
from bpy.types import GizmoGroup
from ..utils.poll import poll_transpose
from ..matrix.transpose_matrix import transpose_matrix
from mathutils import Vector
from ..keymaps import keymaps_gizmo_tweak
from .. import get_prefs



class GP_GGT_transpose(GizmoGroup):
    bl_idname = 'GP_GGT_transpose'
    bl_label = 'Gizmo PRO'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'3D', 'PERSISTENT', 'SHOW_MODAL_ALL', 'VR_REDRAWS'} #'DEPTH_3D' , 'TOOL_INIT', 'SELECT', , 'SCALE'


    @classmethod
    def poll(cls, context):
        return poll_transpose(context)


    @classmethod
    def setup_keymap(cls, keyconfig):
        return keymaps_gizmo_tweak(keyconfig)


    def setup(self, context):
        self.arrow = self.gizmos.new('GP_GT_arrow')
        

        self.xDP = self.gizmos.new('GIZMO_GT_move_3d')
        self.xDP.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.xDP.use_tooltip = False
        self.xDP.use_draw_modal = True
        self.xDP_op = self.xDP.target_set_operator('gp.transpose_axis_op')
        self.xDP_op.axis = '+X'


        self.xDN = self.gizmos.new('GIZMO_GT_move_3d')
        self.xDN.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.xDN.use_tooltip = False
        self.xDN.use_draw_modal = True
        self.xDN_op = self.xDN.target_set_operator('gp.transpose_axis_op')
        self.xDN_op.axis = '-X'


        self.yDP = self.gizmos.new('GIZMO_GT_move_3d')
        self.yDP.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.yDP.use_tooltip = False
        self.yDP.use_draw_modal = True
        self.yDP_op = self.yDP.target_set_operator('gp.transpose_axis_op')
        self.yDP_op.axis = '+Y'


        self.yDN = self.gizmos.new('GIZMO_GT_move_3d')
        self.yDN.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.yDN.use_tooltip = False
        self.yDN.use_draw_modal = True
        self.yDN_op = self.yDN.target_set_operator('gp.transpose_axis_op')
        self.yDN_op.axis = '-Y'


        self.zDP = self.gizmos.new('GIZMO_GT_move_3d')
        self.zDP.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.zDP.use_tooltip = False
        self.zDP.use_draw_modal = True
        self.zDP_op = self.zDP.target_set_operator('gp.transpose_axis_op')
        self.zDP_op.axis = '+Z'

        self.zDN = self.gizmos.new('GIZMO_GT_move_3d')
        self.zDN.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.zDN.use_tooltip = False
        self.zDN.use_draw_modal = True
        self.zDN_op = self.zDN.target_set_operator('gp.transpose_axis_op')
        self.zDN_op.axis = '-Z'


        # --- Первые кольца
        self.dStart = self.gizmos.new('GIZMO_GT_move_3d')
        self.dStart.draw_options = {'ALIGN_VIEW'}
        self.dStart.use_tooltip = False
        self.dStart.use_draw_modal = True
        self.dStart.use_snap = True

        self.dTranslate = self.gizmos.new('GIZMO_GT_move_3d')
        self.dTranslate.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.dTranslate.use_tooltip = False
        self.dTranslate.use_draw_modal = True
        self.dTranslate.use_draw_hover = True
        op = self.dTranslate.target_set_operator('gp.transpose_transform_op')
        op.operator = 'TRANSLATION'
        

        # --- Последние кольца
        self.dEnd = self.gizmos.new('GIZMO_GT_move_3d')
        self.dEnd.draw_options = {'ALIGN_VIEW'}
        self.dEnd.use_tooltip = False
        self.dEnd.use_draw_modal = True
        self.dEnd.use_snap = True

        self.dRot = self.gizmos.new('GIZMO_GT_move_3d')
        self.dRot.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.dRot.use_tooltip = False
        self.dRot.use_draw_modal = True
        self.dRot.use_draw_hover = True
        op = self.dRot.target_set_operator('gp.transpose_transform_op')
        op.operator = 'RESIZE'

        

        # --- Средние кольца
        self.dMid = self.gizmos.new('GIZMO_GT_move_3d')
        self.dMid.draw_options = {'ALIGN_VIEW'}
        self.dMid.use_tooltip = False
        self.dMid.use_draw_modal = True
        self.dMid.use_snap = True

        self.dScale = self.gizmos.new('GIZMO_GT_move_3d')
        self.dScale.draw_options = {'FILL_SELECT', 'ALIGN_VIEW'}
        self.dScale.use_tooltip = False
        self.dScale.use_draw_modal = True
        self.dScale.use_draw_hover = True
        op = self.dScale.target_set_operator('gp.transpose_transform_op')
        op.operator = 'ROTATION'



        self.ruller = self.gizmos.new('GP_GT_ruller')

        #self.navigate = self.gizmos.new('VIEW3D_GT_navigate_rotate')

        #self.ruller_standard = self.gizmos.new('VIEW3D_GT_ruler_item')

    """ def invoke_prepare(self, context, gizmo):
        if gizmo.bl_idname == 'GIZMO_GT_move_3d':
            self.tra_op.matrix_basis = self.arrow.matrix_basis
            self.sca_op.matrix_basis = self.arrow.matrix_basis
            self.rot_op.matrix_basis = self.arrow.matrix_basis """
    

    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set

        mw = transpose_matrix()[2]
        #vm, vms = view_orient_matrix(mw.translation)


        color_red = props.x_axis_color #(1.0, 0.1, 0.1)
        color_green = props.y_axis_color #(0.1, 1.0, 0.1)
        color_blue = props.z_axis_color #(0.0, 0.5, 1.0)
        color_white = props.elements_color#(0.9, 0.9, 0.9)

        
        # --- Линии
        arrow = self.arrow
        arrow.alpha = props.alpha_gizmo
        arrow.scale_basis = props.size_gizmo_transpose
        arrow.matrix_basis = mw

   
        #self.navigate.scale_basis = props.size_gizmo_pro
        #self.navigate.matrix_basis = mZW 
        for gizmo in self.gizmos:
            gizmo.alpha = props.alpha_gizmo
            gizmo.alpha_highlight = props.alpha_gizmo
            gizmo.color_highlight = props.select_color

        if not hasattr(arrow, 'xP'):
            return
        else:
            xP = arrow.xP
            xN = arrow.xN
            yP = arrow.yP
            yN = arrow.yN
            zP = arrow.zP
            zN = arrow.zN
        

        # --- Маленькие кольца
        # --- +X
        self.xDP.color = props.x_axis_color
        self.xDP.scale_basis = props.size_gizmo_transpose * 0.1
        self.xDP.matrix_basis.translation = xP
        # --- -X
        self.xDN.color = props.x_axis_color
        self.xDN.scale_basis = props.size_gizmo_transpose * 0.1
        self.xDN.matrix_basis.translation = xN

        # --- +Y
        self.yDP.color = props.y_axis_color
        self.yDP.scale_basis = props.size_gizmo_transpose * 0.1
        self.yDP.matrix_basis.translation = yP
        # --- -Y
        self.yDN.color = props.y_axis_color
        self.yDN.scale_basis = props.size_gizmo_transpose * 0.1
        self.yDN.matrix_basis.translation = yN

        # --- +Z
        self.zDP.color = props.z_axis_color
        self.zDP.scale_basis = props.size_gizmo_transpose * 0.1
        self.zDP.matrix_basis.translation = zP
        # --- -Z
        self.zDN.color = props.z_axis_color
        self.zDN.scale_basis = props.size_gizmo_transpose * 0.1
        self.zDN.matrix_basis.translation = zN





        # --- Первое кольцо
        self.dStart.color = color_white
        self.dStart.line_width = props.size_gizmo_transpose * 2.5
        self.dStart.scale_basis = props.size_gizmo_transpose  * 0.3
        self.dStart.target_set_prop("offset", settings, 'tr_p1')

        self.dTranslate.color = color_red
        self.dTranslate.line_width = props.size_gizmo_transpose * 2.5
        self.dTranslate.scale_basis = props.size_gizmo_transpose  * 0.15
        self.dTranslate.matrix_basis.translation = Vector(settings.tr_p1)


        # --- Средние кольца
        self.dMid.color = color_white
        self.dMid.line_width = props.size_gizmo_transpose * 2.5
        self.dMid.scale_basis = props.size_gizmo_transpose  * 0.3
        def move_get_mid():
            settings = context.scene.GP_scene_set
            p1 = Vector(settings.tr_p1)
            p2 = Vector(settings.tr_p2)
            return (p1+p2) / 2

        def move_set_mid(value):
            settings = context.scene.GP_scene_set
            current_mid = (Vector(settings.tr_p1) + Vector(settings.tr_p2)) / 2
            diff = Vector(value) - current_mid
            settings.tr_p1 = Vector(settings.tr_p1) + diff
            settings.tr_p2 = Vector(settings.tr_p2) + diff

        self.dMid.target_set_handler("offset", get=move_get_mid, set=move_set_mid)

        self.dScale.color = color_green
        self.dScale.line_width = props.size_gizmo_transpose * 2.5
        self.dScale.scale_basis = props.size_gizmo_transpose  * 0.15
        self.dScale.matrix_basis.translation = (Vector(settings.tr_p1) + Vector(settings.tr_p2)) / 2


        # --- Последние кольца
        self.dEnd.color = color_white
        self.dEnd.line_width = props.size_gizmo_transpose * 2.5
        self.dEnd.scale_basis = props.size_gizmo_transpose  * 0.3
        self.dEnd.target_set_prop("offset", settings, 'tr_p2')

        self.dRot.color = color_blue
        self.dRot.line_width = props.size_gizmo_transpose * 2.5
        self.dRot.scale_basis = props.size_gizmo_transpose  * 0.15
        self.dRot.matrix_basis.translation = Vector(settings.tr_p2) 



        
        # --- Линейка
        self.ruller.alpha = props.alpha_gizmo
        self.ruller.color = (1.0, 0.6, 0.0)
        self.ruller.line_width = props.size_gizmo_transpose * 1.5





classes = [
    GP_GGT_transpose,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)