import bpy
from mathutils import Matrix, Vector
from bpy.types import GizmoGroup
from ..utils.poll import poll_vr
from ..matrix.matrix_ensure import get_orient_from_type
from .. import var
from ..utils.draw_utils.through import alpha_gizmo_vr
from ..keymaps import keymaps_gizmo_tweak
from .. import get_prefs



class GP_GGT_gizmo_pro_vr(GizmoGroup):
    
    bl_idname = 'GP_GGT_gizmo_pro_vr'
    bl_label = 'Gizmo PRO: VR'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW' 
    bl_options = {'3D', 'PERSISTENT', 'SHOW_MODAL_ALL', 'VR_REDRAWS'} #'DEPTH_3D' , 'TOOL_INIT', 'SELECT', , 'SCALE'  , 'SHOW_MODAL_ALL' , 'PERSISTENT' , VR_REDRAWS 
    

    @classmethod
    def poll(cls, context):
        return poll_vr(context)
    
    
    @classmethod
    def setup_keymap(cls, keyconfig):
        return keymaps_gizmo_tweak(keyconfig)

    
    def setup(self, context):
        
        # --- ROTATE
        self.dial_x_p = self.gizmos.new('GIZMO_GT_primitive_3d')
        self.dial_x_p.draw_style = 'ANNULUS'
        self.dial_x_p.arc_inner_factor = 0.8
        self.dial_x_p.use_tooltip = False
        self.dial_x_p.use_draw_modal = True
        self.dial_x_p.use_draw_offset_scale = True
        self.dxop = self.dial_x_p.target_set_operator('gp.arc_operator')
        self.dxop.axis = 'X'

        self.dial_x_n = self.gizmos.new('GIZMO_GT_primitive_3d')
        self.dial_x_n.draw_style = 'ANNULUS'
        self.dial_x_n.arc_inner_factor = 0.8
        self.dial_x_n.use_tooltip = False
        self.dial_x_n.use_draw_modal = True
        self.dial_x_n.use_draw_offset_scale = True
        self.dxon = self.dial_x_n.target_set_operator('gp.arc_operator')
        self.dxon.axis = 'X'

     
        self.dial_y_p = self.gizmos.new('GIZMO_GT_primitive_3d')
        self.dial_y_p.draw_style = 'ANNULUS'
        self.dial_y_p.arc_inner_factor = 0.8
        self.dial_y_p.use_tooltip = False
        self.dial_y_p.use_draw_modal = True
        self.dial_y_p.use_draw_offset_scale = True
        self.dyop = self.dial_y_p.target_set_operator('gp.arc_operator')
        self.dyop.axis = 'Y'

        self.dial_y_n = self.gizmos.new('GIZMO_GT_primitive_3d')
        self.dial_y_n.draw_style = 'ANNULUS'
        self.dial_y_n.arc_inner_factor = 0.8
        self.dial_y_n.use_tooltip = False
        self.dial_y_n.use_draw_modal = True
        self.dial_y_n.use_draw_offset_scale = True
        self.dyon = self.dial_y_n.target_set_operator('gp.arc_operator')
        self.dyon.axis = 'Y'

   
        self.dial_z_p = self.gizmos.new('GIZMO_GT_primitive_3d')
        self.dial_z_p.draw_style = 'ANNULUS'
        self.dial_z_p.arc_inner_factor = 0.8
        self.dial_z_p.use_tooltip = False
        self.dial_z_p.use_draw_modal = True
        self.dial_z_p.use_draw_offset_scale = True
        self.dzop = self.dial_z_p.target_set_operator('gp.arc_operator')
        self.dzop.axis = 'Z'

        self.dial_z_n = self.gizmos.new('GIZMO_GT_primitive_3d')
        self.dial_z_n.draw_style = 'ANNULUS'
        self.dial_z_n.arc_inner_factor = 0.8
        self.dial_z_n.use_tooltip = False
        self.dial_z_n.use_draw_modal = True
        self.dial_z_n.use_draw_offset_scale = True
        self.dzon = self.dial_z_n.target_set_operator('gp.arc_operator')
        self.dzon.axis = 'Z'



        # --- SCALE
        self.scale_x_p = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.scale_x_p.draw_style = 'CONE' #'BOX'
        self.scale_x_p.draw_options = {'ORIGIN'}#set()
        #self.scale_x_p.aspect = (0.5, 1.5)
        self.scale_x_p.use_draw_offset_scale = True 
        self.scale_x_p.use_tooltip = False
        self.scale_x_p.use_draw_modal = True
        self.sxop = self.scale_x_p.target_set_operator('gp.box_operator')
        self.sxop.axis = 'X'

        self.scale_x_n = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.scale_x_n.draw_style = 'CONE' #'BOX'
        self.scale_x_n.draw_options = {'ORIGIN'}
        self.scale_x_n.use_draw_offset_scale = True 
        self.scale_x_n.use_tooltip = False
        self.scale_x_n.use_draw_modal = True
        self.sxon = self.scale_x_n.target_set_operator('gp.box_operator')
        self.sxon.axis = 'X'


        self.scale_y_p = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.scale_y_p.draw_style = 'CONE' #'BOX'
        self.scale_y_p.draw_options = {'ORIGIN'}
        self.scale_y_p.use_draw_offset_scale = True 
        self.scale_y_p.use_tooltip = False
        self.scale_y_p.use_draw_modal = True
        self.syop = self.scale_y_p.target_set_operator('gp.box_operator')
        self.syop.axis = 'Y'

        self.scale_y_n = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.scale_y_n.draw_style = 'CONE' #'BOX'
        self.scale_y_n.draw_options = {'ORIGIN'}
        self.scale_y_n.use_draw_offset_scale = True 
        self.scale_y_n.use_tooltip = False
        self.scale_y_n.use_draw_modal = True
        self.syon = self.scale_y_n.target_set_operator('gp.box_operator')
        self.syon.axis = 'Y'


        self.scale_z_p = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.scale_z_p.draw_style = 'CONE' #'BOX'
        self.scale_z_p.draw_options = {'ORIGIN'}
        self.scale_z_p.use_draw_offset_scale = True 
        self.scale_z_p.use_tooltip = False
        self.scale_z_p.use_draw_modal = True
        self.szop = self.scale_z_p.target_set_operator('gp.box_operator')
        self.szop.axis = 'Z'

        self.scale_z_n = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.scale_z_n.draw_style = 'CONE' #'BOX'
        self.scale_z_n.draw_options = {'ORIGIN'}
        self.scale_z_n.use_draw_offset_scale = True 
        self.scale_z_n.use_tooltip = False
        self.scale_z_n.use_draw_modal = True
        self.szon = self.scale_z_n.target_set_operator('gp.box_operator')
        self.szon.axis = 'Z'



        # --- MOVE
        self.arrow_x_p = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.arrow_x_p.draw_options = set()
        self.arrow_x_p.use_draw_offset_scale = True 
        self.arrow_x_p.use_tooltip = False
        self.arrow_x_p.use_draw_modal = True 
        self.axop = self.arrow_x_p.target_set_operator('gp.cone_operator')
        self.axop.axis = 'X'
        
        self.arrow_y_p = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.arrow_y_p.draw_options = set()
        self.arrow_y_p.use_draw_offset_scale = True 
        self.arrow_y_p.use_tooltip = False
        self.arrow_y_p.use_draw_modal = True
        self.ayop = self.arrow_y_p.target_set_operator('gp.cone_operator')
        self.ayop.axis = 'Y'
        
        self.arrow_z_p = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.arrow_z_p.draw_options = set()
        self.arrow_z_p.use_draw_offset_scale = True 
        self.arrow_z_p.use_tooltip = False
        self.arrow_z_p.use_draw_modal = True
        self.azop = self.arrow_z_p.target_set_operator('gp.cone_operator')
        self.azop.axis = 'Z'


        self.arrow_x_n = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.arrow_x_n.draw_options = set()
        self.arrow_x_n.use_draw_offset_scale = True 
        self.arrow_x_n.use_tooltip = False
        self.arrow_x_n.use_draw_modal = True
        self.axon = self.arrow_x_n.target_set_operator('gp.cone_operator')
        self.axon.axis = 'X'
        
        self.arrow_y_n = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.arrow_y_n.draw_options = set()
        self.arrow_y_n.use_draw_offset_scale = True 
        self.arrow_y_n.use_tooltip = False
        self.arrow_y_n.use_draw_modal = True
        self.ayon = self.arrow_y_n.target_set_operator('gp.cone_operator')
        self.ayon.axis = 'Y'
        
        self.arrow_z_n = self.gizmos.new('GIZMO_GT_arrow_3d')
        self.arrow_z_n.draw_options = set()
        self.arrow_z_n.use_draw_offset_scale = True 
        self.arrow_z_n.use_tooltip = False
        self.arrow_z_n.use_draw_modal = True
        self.azon = self.arrow_z_n.target_set_operator('gp.cone_operator')
        self.azon.axis = 'Z'



        
        


        



    """ def invoke_prepare(self, context, gizmo):
        matrix = gizmo.matrix_basis """
        
        
    def invoke_prepare(self, context, gizmo):
        matrix = gizmo.matrix_basis
        if gizmo.bl_idname == 'GIZMO_GT_arrow_3d':
            self.sxop.matrix_basis = self.scale_x_n.matrix_basis
            self.sxon.matrix_basis = self.scale_x_p.matrix_basis
            self.syop.matrix_basis = self.scale_y_n.matrix_basis
            self.syon.matrix_basis = self.scale_y_p.matrix_basis
            self.szop.matrix_basis = self.scale_z_n.matrix_basis
            self.szon.matrix_basis = self.scale_z_p.matrix_basis

            self.axop.matrix_basis = matrix
            self.axon.matrix_basis = matrix
            self.ayop.matrix_basis = matrix
            self.ayon.matrix_basis = matrix
            self.azop.matrix_basis = matrix
            self.azon.matrix_basis = matrix

        elif gizmo.bl_idname == 'GIZMO_GT_primitive_3d':
            self.dxop.matrix_basis = matrix
            self.dxon.matrix_basis = matrix
            self.dyop.matrix_basis = matrix
            self.dyon.matrix_basis = matrix
            self.dzop.matrix_basis = matrix
            self.dzon.matrix_basis = matrix


    def refresh(self, context):
        mXP, mYP, mZP, mXN, mYN, mZN = get_orient_from_type(type='VR')
        
        self.arrow_x_p.matrix_basis = mXP
        self.arrow_x_n.matrix_basis = mXN
        self.arrow_y_p.matrix_basis = mYP
        self.arrow_y_n.matrix_basis = mYN
        self.arrow_z_p.matrix_basis = mZP
        self.arrow_z_n.matrix_basis = mZN

        self.scale_x_p.matrix_basis = mXP
        self.scale_x_n.matrix_basis = mXN
        self.scale_y_p.matrix_basis = mYP
        self.scale_y_n.matrix_basis = mYN
        self.scale_z_p.matrix_basis = mZP
        self.scale_z_n.matrix_basis = mZN
        
        self.dial_x_p.matrix_basis = mXP
        self.dial_x_n.matrix_basis = mXN
        self.dial_y_p.matrix_basis = mYP
        self.dial_y_n.matrix_basis = mYN
        self.dial_z_p.matrix_basis = mZP
        self.dial_z_n.matrix_basis = mZN

        # --- Offset
        #props = context.preferences.addons['GIZMO_PRO'].preferences
        #oa = 1.6
        #mat_offset_arrow = Matrix.Translation(Vector((0.0, 0.0, props.offset_cone_vr)))
        #mat_offset_box = Matrix.Translation(Vector((0.0, 0.0, props.offset_cone_vr)))
        #mat_offset_dial  = Matrix.Translation(Vector((0.0, 0.0, oa * 0.5 * props.offset_cone_vr)))
        
        """ self.arrow_x_p.matrix_offset = mat_offset_arrow
        self.arrow_y_p.matrix_offset = mat_offset_arrow
        self.arrow_z_p.matrix_offset = mat_offset_arrow
        self.arrow_x_n.matrix_offset = mat_offset_arrow
        self.arrow_y_n.matrix_offset = mat_offset_arrow
        self.arrow_z_n.matrix_offset = mat_offset_arrow

        self.scale_x_p.matrix_offset = mat_offset_box
        self.scale_y_p.matrix_offset = mat_offset_box
        self.scale_z_p.matrix_offset = mat_offset_box
        self.scale_x_n.matrix_offset = mat_offset_box
        self.scale_y_n.matrix_offset = mat_offset_box
        self.scale_z_n.matrix_offset = mat_offset_box """

        """ self.dial_x_p.matrix_offset = mat_offset_dial
        self.dial_y_p.matrix_offset = mat_offset_dial
        self.dial_z_p.matrix_offset = mat_offset_dial
        self.dial_x_n.matrix_offset = mat_offset_dial
        self.dial_y_n.matrix_offset = mat_offset_dial
        self.dial_z_n.matrix_offset = mat_offset_dial """


    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set

        self.lock_loc = context.object.lock_location
        self.lock_rot = context.object.lock_rotation
        self.lock_sca = context.object.lock_scale

        # --- принудительное обновление матрицы
        orient_slots = context.window.scene.transform_orientation_slots[0].type
        
        if var.REFRESH_MAT or orient_slots == 'VIEW':
            self.refresh(context)
        var.REFRESH_MAT = False

        len_arrow = 0.0 #0.8 * props.offset_cone_vr
        len_box = 0.0 #0.3 * props.offset_cone_vr
        scale_arrow = 1.4
        scale_box = 1.8
        scale_dial = 0.7
        color_negative = props.color_negative_elements_vr
        mat_offset_cone  = Matrix.Translation(Vector((0.0, 0.0, props.offset_cone_vr)))
        mat_offset_dial  = Matrix.Translation(Vector((0.0, 0.0, props.offset_dial_vr*2)))


        
        alpha_x, hide_select_x, scale_factor_x = alpha_gizmo_vr(self.arrow_x_p.matrix_basis, props)
        alpha_y, hide_select_y, scale_factor_y = alpha_gizmo_vr(self.arrow_y_p.matrix_basis, props)
        alpha_z, hide_select_z, scale_factor_z = alpha_gizmo_vr(self.arrow_z_p.matrix_basis, props)
        alpha_xn, hide_select_xn, scale_factor_xn = alpha_gizmo_vr(self.arrow_x_n.matrix_basis, props)
        alpha_yn, hide_select_yn, scale_factor_yn = alpha_gizmo_vr(self.arrow_y_n.matrix_basis, props)
        alpha_zn, hide_select_zn, scale_factor_zn = alpha_gizmo_vr(self.arrow_z_n.matrix_basis, props)

        

# --- MOVE
        # --- +
        self.arrow_x_p.alpha = alpha_x
        self.arrow_x_p.hide_select = hide_select_x
        self.arrow_x_p.alpha_highlight = props.highlight_alpha_gizmo
        self.arrow_x_p.scale_basis = props.size_gizmo_vr * scale_arrow * scale_factor_x
        self.arrow_x_p.color = props.x_axis_color
        self.arrow_x_p.color_highlight = props.select_color
        self.arrow_x_p.length = len_arrow

        self.arrow_y_p.alpha = alpha_y
        self.arrow_y_p.hide_select = hide_select_y
        self.arrow_y_p.alpha_highlight = props.highlight_alpha_gizmo
        self.arrow_y_p.scale_basis = props.size_gizmo_vr * scale_arrow * scale_factor_y
        self.arrow_y_p.color = props.y_axis_color
        self.arrow_y_p.color_highlight = props.select_color
        self.arrow_y_p.length = len_arrow
        
        self.arrow_z_p.alpha = alpha_z
        self.arrow_z_p.hide_select = hide_select_z
        self.arrow_z_p.alpha_highlight = props.highlight_alpha_gizmo
        self.arrow_z_p.scale_basis = props.size_gizmo_vr * scale_arrow * scale_factor_z
        self.arrow_z_p.color = props.z_axis_color
        self.arrow_z_p.color_highlight = props.select_color
        self.arrow_z_p.length = len_arrow
        

        # --- -
        self.arrow_x_n.alpha = alpha_xn
        self.arrow_x_n.hide_select = hide_select_xn
        self.arrow_x_n.alpha_highlight = props.highlight_alpha_gizmo
        self.arrow_x_n.scale_basis = props.size_gizmo_vr * scale_arrow * scale_factor_xn
        self.arrow_x_n.color = color_negative
        self.arrow_x_n.color_highlight = props.x_axis_color #props.select_color
        self.arrow_x_n.length = len_arrow
        
        self.arrow_y_n.alpha = alpha_yn
        self.arrow_y_n.hide_select = hide_select_yn
        self.arrow_y_n.alpha_highlight = props.highlight_alpha_gizmo
        self.arrow_y_n.scale_basis = props.size_gizmo_vr * scale_arrow * scale_factor_yn
        self.arrow_y_n.color = color_negative
        self.arrow_y_n.color_highlight = props.y_axis_color #props.select_color
        self.arrow_y_n.length = len_arrow
        
        self.arrow_z_n.alpha = alpha_zn
        self.arrow_z_n.hide_select = hide_select_zn
        self.arrow_z_n.alpha_highlight = props.highlight_alpha_gizmo
        self.arrow_z_n.scale_basis = props.size_gizmo_vr * scale_arrow * scale_factor_zn
        self.arrow_z_n.color = color_negative
        self.arrow_z_n.color_highlight = props.z_axis_color #props.select_color
        self.arrow_z_n.length = len_arrow
        

        # --- SCALE
        # --- +
        self.scale_x_p.alpha = alpha_x
        self.scale_x_p.hide_select = hide_select_x
        self.scale_x_p.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_x_p.scale_basis = props.size_gizmo_vr * scale_box * scale_factor_x
        self.scale_x_p.color = props.x_axis_color
        self.scale_x_p.color_highlight = props.select_color
        self.scale_x_p.length = len_box

        self.scale_y_p.alpha = alpha_y
        self.scale_y_p.hide_select = hide_select_y
        self.scale_y_p.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_y_p.scale_basis = props.size_gizmo_vr * scale_box * scale_factor_y
        self.scale_y_p.color = props.y_axis_color
        self.scale_y_p.color_highlight = props.select_color
        self.scale_y_p.length = len_box
        
        self.scale_z_p.alpha = alpha_z
        self.scale_z_p.hide_select = hide_select_z
        self.scale_z_p.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_z_p.scale_basis = props.size_gizmo_vr * scale_box * scale_factor_z
        self.scale_z_p.color = props.z_axis_color
        self.scale_z_p.color_highlight = props.select_color
        self.scale_z_p.length = len_box
        

        # --- -
        self.scale_x_n.alpha = alpha_xn
        self.scale_x_n.hide_select = hide_select_xn
        self.scale_x_n.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_x_n.scale_basis = props.size_gizmo_vr * scale_box * scale_factor_xn
        self.scale_x_n.color = color_negative
        self.scale_x_n.color_highlight = props.x_axis_color #props.select_color
        self.scale_x_n.length = len_box

        self.scale_y_n.alpha = alpha_yn
        self.scale_y_n.hide_select = hide_select_yn
        self.scale_y_n.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_y_n.scale_basis = props.size_gizmo_vr * scale_box * scale_factor_yn
        self.scale_y_n.color = color_negative
        self.scale_y_n.color_highlight = props.y_axis_color #props.select_color
        self.scale_y_n.length = len_box

        self.scale_z_n.alpha = alpha_zn
        self.scale_z_n.hide_select = hide_select_zn
        self.scale_z_n.alpha_highlight = props.highlight_alpha_gizmo
        self.scale_z_n.scale_basis = props.size_gizmo_vr * scale_box * scale_factor_zn
        self.scale_z_n.color = color_negative
        self.scale_z_n.color_highlight = props.z_axis_color #props.select_color
        self.scale_z_n.length = len_box
        

        # --- ROTATE
        self.dial_x_p.alpha = props.alpha_gizmo
        self.dial_x_p.alpha_highlight = props.highlight_alpha_gizmo
        self.dial_x_p.scale_basis = props.size_gizmo_vr * scale_dial * scale_factor_x
        self.dial_x_p.line_width = props.size_gizmo_vr * 3.0
        self.dial_x_p.color = props.x_axis_color
        self.dial_x_p.color_highlight = props.select_color
        
        self.dial_y_p.alpha = props.alpha_gizmo
        self.dial_y_p.alpha_highlight = props.highlight_alpha_gizmo
        self.dial_y_p.scale_basis = props.size_gizmo_vr * scale_dial * scale_factor_y
        self.dial_y_p.line_width = props.size_gizmo_vr * 3.0
        self.dial_y_p.color = props.y_axis_color
        self.dial_y_p.color_highlight = props.select_color
        
        self.dial_z_p.alpha = props.alpha_gizmo
        self.dial_z_p.alpha_highlight = props.highlight_alpha_gizmo
        self.dial_z_p.scale_basis = props.size_gizmo_vr * scale_dial * scale_factor_z
        self.dial_z_p.line_width = props.size_gizmo_vr * 3.0
        self.dial_z_p.color = props.z_axis_color
        self.dial_z_p.color_highlight = props.select_color
        

        # --- -
        self.dial_x_n.alpha = props.alpha_gizmo
        self.dial_x_n.alpha_highlight = props.highlight_alpha_gizmo
        self.dial_x_n.scale_basis = props.size_gizmo_vr * scale_dial * scale_factor_xn
        self.dial_x_n.line_width = props.size_gizmo_vr * 3.0
        self.dial_x_n.color = color_negative
        self.dial_x_n.color_highlight = props.x_axis_color #props.select_color

        self.dial_y_n.alpha = props.alpha_gizmo
        self.dial_y_n.alpha_highlight = props.highlight_alpha_gizmo
        self.dial_y_n.scale_basis = props.size_gizmo_vr * scale_dial * scale_factor_yn
        self.dial_y_n.line_width = props.size_gizmo_vr * 3.0
        self.dial_y_n.color = color_negative
        self.dial_y_n.color_highlight = props.y_axis_color #props.select_color
        
        self.dial_z_n.alpha = props.alpha_gizmo
        self.dial_z_n.alpha_highlight = props.highlight_alpha_gizmo
        self.dial_z_n.scale_basis = props.size_gizmo_vr * scale_dial * scale_factor_zn
        self.dial_z_n.line_width = props.size_gizmo_vr * 3.0
        self.dial_z_n.color = color_negative
        self.dial_z_n.color_highlight = props.z_axis_color #props.select_color

        self.arrow_x_p.matrix_offset = mat_offset_cone
        self.arrow_y_p.matrix_offset = mat_offset_cone
        self.arrow_z_p.matrix_offset = mat_offset_cone
        self.arrow_x_n.matrix_offset = mat_offset_cone
        self.arrow_y_n.matrix_offset = mat_offset_cone
        self.arrow_z_n.matrix_offset = mat_offset_cone

        self.dial_x_p.matrix_offset = mat_offset_dial
        self.dial_y_p.matrix_offset = mat_offset_dial
        self.dial_z_p.matrix_offset = mat_offset_dial
        self.dial_x_n.matrix_offset = mat_offset_dial
        self.dial_y_n.matrix_offset = mat_offset_dial
        self.dial_z_n.matrix_offset = mat_offset_dial

        



        # --- VISIBLE GIZMO
        if settings.tra_gizmo:
            self.arrow_x_p.hide = False
            self.arrow_y_p.hide = False
            self.arrow_z_p.hide = False

            self.arrow_x_n.hide = False
            self.arrow_y_n.hide = False
            self.arrow_z_n.hide = False
        else:
            self.arrow_x_p.hide = True
            self.arrow_y_p.hide = True
            self.arrow_z_p.hide = True

            self.arrow_x_n.hide = True
            self.arrow_y_n.hide = True
            self.arrow_z_n.hide = True

        if settings.sca_gizmo:
            self.scale_x_p.hide = False
            self.scale_y_p.hide = False
            self.scale_z_p.hide = False

            self.scale_x_n.hide = False
            self.scale_y_n.hide = False
            self.scale_z_n.hide = False
        else:
            self.scale_x_p.hide = True
            self.scale_y_p.hide = True
            self.scale_z_p.hide = True

            self.scale_x_n.hide = True
            self.scale_y_n.hide = True
            self.scale_z_n.hide = True

        if settings.rot_gizmo:
            self.dial_x_p.hide = False
            self.dial_y_p.hide = False
            self.dial_z_p.hide = False

            self.dial_x_n.hide = False
            self.dial_y_n.hide = False
            self.dial_z_n.hide = False
        else:
            self.dial_x_p.hide = True
            self.dial_y_p.hide = True
            self.dial_z_p.hide = True

            self.dial_x_n.hide = True
            self.dial_y_n.hide = True
            self.dial_z_n.hide = True



        # --- LOCK AXIS
        if context.mode == 'OBJECT':
            if self.lock_loc[0]:
                self.arrow_x_p.hide = True
                self.arrow_x_n.hide = True
            if self.lock_loc[1]:
                self.arrow_y_p.hide = True
                self.arrow_y_n.hide = True
            if self.lock_loc[2]:
                self.arrow_z_p.hide = True
                self.arrow_z_n.hide = True

            if self.lock_sca[0]:
                self.scale_x_p.hide = True
                self.scale_x_n.hide = True
            if self.lock_sca[1]:
                self.scale_y_p.hide = True
                self.scale_y_n.hide = True
            if self.lock_sca[2]:
                self.scale_z_p.hide = True
                self.scale_z_n.hide = True

            if self.lock_rot[0]:
                self.dial_x_p.hide = True
                self.dial_x_n.hide = True
            if self.lock_rot[1]:
                self.dial_y_p.hide = True
                self.dial_y_n.hide = True
            if self.lock_rot[2]:
                self.dial_z_p.hide = True
                self.dial_z_n.hide = True



classes = [
    GP_GGT_gizmo_pro_vr,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)