import bpy, gpu, os
from bpy.utils import previews
from bpy.app.handlers import persistent


icons_collections = {}
gl_textures = {}

texture_name = [
        'colored_icons',
    ]


def load_texture(name):
    icons_dir = os.path.join(os.path.dirname(__file__), './icons/qw')
    icon_path = os.path.join(icons_dir, name + '.png')
    icon = bpy.data.images.load(icon_path, check_existing=True)
    icon_id = gpu.texture.from_image(icon)
    bpy.data.images.remove(icon)
    return icon_id

def load_textures():
    for txtr in texture_name:
        if txtr not in gl_textures:
            id = load_texture(txtr)
            gl_textures[txtr] = id

@persistent
def load_textures_after_startup(dummy):
    load_textures()


def register():
    global icons_collections
    global gl_textures
    global texture_name

    
    
    icoll = previews.new()
    
    # --- GENERAL
    my_icons_dir = os.path.join(os.path.dirname(__file__), 'icons') # если нужна директория './toolkit/icons'
    icoll.load('update_icon', os.path.join(my_icons_dir, 'update.png'), 'IMAGE')
    icoll.load('market_icon', os.path.join(my_icons_dir, 'market.png'), 'IMAGE')
    icoll.load('gumroad_icon', os.path.join(my_icons_dir, 'gumroad.png'), 'IMAGE')
    icoll.load('artstation_icon', os.path.join(my_icons_dir, 'artstation.png'), 'IMAGE')
    icoll.load('discord_icon', os.path.join(my_icons_dir, 'discord.png'), 'IMAGE')

    icoll.load('settings', os.path.join(my_icons_dir, 'settings.png'), 'IMAGE')
    icoll.load('info', os.path.join(my_icons_dir, 'info.png'), 'IMAGE')
    icoll.load('PCO_CENTER', os.path.join(my_icons_dir, 'PCO_CENTER.png'), 'IMAGE')
    icoll.load('PCO_ORIENT', os.path.join(my_icons_dir, 'PCO_ORIENT.png'), 'IMAGE')
    

    icons_collections['main'] = icoll

    bpy.app.handlers.load_post.append(load_textures_after_startup)
    

def unregister():
    global icons_collections
    global gl_textures
    for icoll in icons_collections.values():
        bpy.utils.previews.remove(icoll)
    icons_collections.clear()
    gl_textures.clear()