import bpy
from . import get_prefs

def keymaps_gizmo_tweak(keyconfig):
    props = get_prefs()

    if not keyconfig.keymaps.find( name='Gizmo PRO Tweak', space_type='VIEW_3D' ):
        km = keyconfig.keymaps.new( name='Gizmo PRO Tweak', space_type='VIEW_3D' )
        kmi = km.keymap_items.new( 'gizmogroup.gizmo_tweak', type=props.map_type, value='CLICK_DRAG', any=True )
        kmi.alt,  kmi.oskey = False, False
        if props.use_mmb_drag:
            kmi = km.keymap_items.new( 'gizmogroup.gizmo_tweak', type='MIDDLEMOUSE',  value='CLICK_DRAG', any=True )
            kmi.alt,  kmi.oskey = False, False
        if props.use_mmb_click:
            kmi = km.keymap_items.new( 'gizmogroup.gizmo_tweak', type='MIDDLEMOUSE', value='CLICK', any=True )
            kmi.alt,  kmi.oskey = False, False

    else:
        km = keyconfig.keymaps.find( name='Gizmo PRO Tweak', space_type='VIEW_3D' ) #km.restore_to_default()

        for kmi in km.keymap_items:
            km.keymap_items.remove(kmi)

        if len(km.keymap_items) < 3:
            kmi = km.keymap_items.new( 'gizmogroup.gizmo_tweak', type=props.map_type,  value='CLICK_DRAG', any=True )
            kmi.alt,  kmi.oskey = False, False
            if props.use_mmb_drag:
                kmi = km.keymap_items.new( 'gizmogroup.gizmo_tweak', type='MIDDLEMOUSE',  value='CLICK_DRAG', any=True )
                kmi.alt,  kmi.oskey = False, False
            if props.use_mmb_click:
                kmi = km.keymap_items.new( 'gizmogroup.gizmo_tweak', type='MIDDLEMOUSE', value='CLICK', any=True )
                kmi.alt,  kmi.oskey = False, False
            
    return km


def keymaps_qw_tweak(keyconfig):
    km = keyconfig.keymaps.new(name='QW Tweak', space_type='VIEW_3D')
    km.keymap_items.new('gizmogroup.gizmo_tweak', type='LEFTMOUSE', value='CLICK_DRAG')
    #km.keymap_items.new('gizmogroup.gizmo_tweak', type='LEFTMOUSE', value='CLICK')
    km.keymap_items.new('gizmogroup.gizmo_tweak', type='RIGHTMOUSE', value='CLICK')
    return km


def get_keymap_tools():
    keymaps = (
        ('gp.space_drag_lmb',
        {'type': 'LEFTMOUSE', 'value': 'CLICK_DRAG', 'any': True, 'head': True, 'alt': False, 'oskey': False},
        {"properties": []},
        ), 
        ('gp.space_drag_rmb',
        {'type': 'RIGHTMOUSE', 'value': 'CLICK_DRAG', 'any': True, 'head': True, 'alt': False, 'oskey': False},
        {"properties": []},
        ),

        ('gp.switch_gizmo',
        {'type': 'W', 'value': 'PRESS', 'shift': True},
        {'properties': [('type', 'TRANSLATE')], 'active': False},
        ),
        ('gp.switch_gizmo',
        {'type': 'R', 'value': 'PRESS', 'shift': True},
        {'properties': [('type', 'ROTATE')], 'active': False},
        ),
        ('gp.switch_gizmo',
        {'type': 'E', 'value': 'PRESS', 'shift': True},
        {'properties': [('type', 'SCALE')], 'active': False},
        ),


        ('gp.transform',
        {'type': 'G', 'value': 'PRESS', 'repeat': False, 'head': True},
        {'properties': [('operator', 'TRANSLATION')]},
        ),
        ('gp.transform',
        {'type': 'R', 'value': 'PRESS', 'repeat': False, 'head': True},
        {'properties': [('operator', 'ROTATION')]},
        ),
        ('gp.transform',
        {'type': 'S', 'value': 'PRESS', 'repeat': False, 'head': True},
        {'properties': [('operator', 'RESIZE')]},
        ),
    )
    return keymaps


def get_hotkey_entry_item(km, kmi_name, kmi_value, properties):
    for i, km_item in enumerate(km.keymap_items):
        if km.keymap_items.keys()[i] == kmi_name:
            if properties == 'name':
                if km.keymap_items[i].properties.name == kmi_value:
                    return km_item
            elif properties == 'operator':
                if km.keymap_items[i].properties.operator == kmi_value:
                    return km_item
            elif properties == 'type':
                if km.keymap_items[i].properties.type == kmi_value:
                    return km_item
            elif properties == 'none':
                return km_item
    return None


addon_keymaps = []


def register():
    global addon_keymaps

    wm = bpy.context.window_manager
    actK = wm.keyconfigs.active
    addonK = wm.keyconfigs.addon
   
    if not addonK:
        return

    # --- Gizmo Pro Tweak
    for kc in (actK, addonK):
        km = keymaps_gizmo_tweak(kc)
    


    """ # --- Gizmo PRO Tool Keymaps
    for kc in (actK, addonK):
        km = kc.keymaps.new( name='3D View Tool: Gizmo PRO', space_type='VIEW_3D', tool=True )


        # --- Tweak Gizmo
        lmb = km.keymap_items.new( 'gp.space_drag_lmb', 'LEFTMOUSE', 'CLICK_DRAG', any=True, head=True )
        lmb.alt = False
        lmb.oskey = False
        rmb = km.keymap_items.new( 'gp.space_drag_rmb', 'RIGHTMOUSE', 'CLICK_DRAG', any=True, head=True )
        rmb.alt = False
        rmb.oskey = False

        # --- Switch Gizmo
        kmi = km.keymap_items.new( 'gp.switch_gizmo', 'W', 'PRESS', shift=True, head=True )
        kmi.properties.type = 'TRANSLATE'
        kmi.active = False
        kmi = km.keymap_items.new( 'gp.switch_gizmo', 'R', 'PRESS', shift=True, head=True )
        kmi.properties.type = 'ROTATE'
        kmi.active = False
        kmi = km.keymap_items.new( 'gp.switch_gizmo', 'E', 'PRESS', shift=True, head=True )
        kmi.properties.type = 'SCALE'
        kmi.active = False


        # --- Transform Operator
        kmi = km.keymap_items.new( 'gp.transform', 'G', 'PRESS', repeat=False, head=True )
        kmi.properties.operator = 'TRANSLATION'
        kmi = km.keymap_items.new( 'gp.transform', 'R', 'PRESS', repeat=False, head=True )
        kmi.properties.operator = 'ROTATION'
        kmi = km.keymap_items.new( 'gp.transform', 'S', 'PRESS', repeat=False, head=True )
        kmi.properties.operator = 'RESIZE' """
        
        
        
      
       




    km = addonK.keymaps.new(name='3D View', space_type='VIEW_3D')
    kmi = km.keymap_items.new('wm.call_menu_pie', type='SPACE', value='PRESS', ctrl=False, alt=False, shift=False)
    kmi.properties.name = 'GP_MT_pie'
    addon_keymaps.append((km, kmi))


def unregister():
    global addon_keymaps

    """ for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi) """
    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        for km, kmi in addon_keymaps:
            km.keymap_items.remove(kmi)

    addon_keymaps.clear()