import bpy
import bmesh
from mathutils import Matrix, Vector
from ..utils.custom_math.bcurve import get_curve_points, get_selected_curve_points
from ..utils.custom_math.bbone import get_bone_points, get_selected_bone_points


def get_bbox(mat):
    context = bpy.context
    sOb = context.selected_objects

    verts = []
    for ob in sOb:
        if context.mode == 'OBJECT':
            if ob.type == 'MESH':
                vs = ob.data.vertices
                verts.extend([ob.matrix_world @ v.co for v in vs])

            elif ob.type in {'CURVE', 'SURFACE'}:
                verts.extend(get_curve_points())

            elif ob.type == 'ARMATURE':
                verts.extend(get_bone_points())

            else:
                verts.extend([ob.matrix_world @ Vector(corner) for corner in ob.bound_box])

        elif context.mode == 'EDIT_MESH':
            bm = bmesh.from_edit_mesh(ob.data)
            vs = bm.verts
            verts.extend([ob.matrix_world @ v.co for v in vs if v.select])
        
        elif context.mode in {'EDIT_CURVE', 'EDIT_SURFACE'}:
            verts.extend(get_selected_curve_points())
        
        elif context.mode in {'EDIT_ARMATURE', 'POSE'}:
            verts.extend(get_selected_bone_points())

        else:
            verts.extend([ob.matrix_world @ Vector(corner) for corner in ob.bound_box])


    verts_rot = []
    verts_rot.extend([mat.inverted() @ v for v in verts])
    
    x_max = max([v.x for v in verts_rot])
    x_min = min([v.x for v in verts_rot])
    y_max = max([v.y for v in verts_rot])
    y_min = min([v.y for v in verts_rot])
    z_max = max([v.z for v in verts_rot])
    z_min = min([v.z for v in verts_rot])

    xP = mat @ Vector((x_max, (y_max+y_min)/2, (z_max+z_min)/2))
    xN = mat @ Vector((x_min, (y_max+y_min)/2, (z_max+z_min)/2))

    yP = mat @ Vector(((x_max+x_min)/2, y_max, (z_max+z_min)/2))
    yN = mat @ Vector(((x_max+x_min)/2, y_min, (z_max+z_min)/2))

    zP = mat @ Vector(((x_max+x_min)/2, (y_max+y_min)/2, z_max))
    zN = mat @ Vector(((x_max+x_min)/2, (y_max+y_min)/2, z_min))

    return xP, xN, yP, yN, zP, zN