import bpy
from mathutils import Matrix, Vector, Quaternion
from math import radians, copysign

from .orientation.orientation_object import (
    get_global_orient,
    get_local_orient,
    get_gimbal_orient,
    get_view_orient,
    get_custom_orient,
    get_cursor_orient,
    get_parent_orient,
    )
from .orientation.orientation_mesh import get_normal_mesh_qat
from .orientation.orientation_curve import get_normal_curve_qat
from .orientation.orientation_bone import get_normal_bone_qat

from .position.position_object import median_point_object, cursor_point, bb_point_ob
from .position.position_mesh import get_active_mesh_pos, get_median_mesh_pos, get_bb_mesh_pos
from .position.position_curve import get_active_curve_pos, get_median_curve_pos, get_bb_curve_pos
from .position.position_action_center import selection_border, mesh_boundary
from .position.position_bone import get_active_bone_pos, get_median_bone_pos, get_bb_bone_pos
from .bbox import get_bbox
from .. import var
from ..utils.draw_utils.through import dot
from .. import get_prefs



# --- Standard Slots
def object_get_position(context):
    tra_PP = context.scene.tool_settings.transform_pivot_point
    if tra_PP == 'ACTIVE_ELEMENT':
        loc = context.object.matrix_world.translation
    elif tra_PP in {'MEDIAN_POINT', 'INDIVIDUAL_ORIGINS'}:
        loc = median_point_object()
    elif tra_PP == 'BOUNDING_BOX_CENTER':
        loc = bb_point_ob()
    else:
        loc = cursor_point()
    return loc


def mesh_get_position(context):
    tra_PP = context.scene.tool_settings.transform_pivot_point
    if tra_PP == 'ACTIVE_ELEMENT':
        loc = get_active_mesh_pos()
    elif tra_PP in {'MEDIAN_POINT', 'INDIVIDUAL_ORIGINS'}:
        loc = get_median_mesh_pos()
    elif tra_PP == 'BOUNDING_BOX_CENTER':
        loc = get_bb_mesh_pos()
    else:
        loc = cursor_point()
    return loc


def get_curve_position(context):
    tra_PP = context.scene.tool_settings.transform_pivot_point
    if tra_PP == 'ACTIVE_ELEMENT':
        loc = get_active_curve_pos()
    elif tra_PP in {'MEDIAN_POINT', 'INDIVIDUAL_ORIGINS'}:
        loc = get_median_curve_pos()
    elif tra_PP == 'BOUNDING_BOX_CENTER':
        loc = get_bb_curve_pos()
    else:
        loc = cursor_point()
    return loc


def get_bone_position(context):
    tra_PP = context.scene.tool_settings.transform_pivot_point
    if tra_PP == 'ACTIVE_ELEMENT':
        loc = get_active_bone_pos()
    elif tra_PP in {'MEDIAN_POINT', 'INDIVIDUAL_ORIGINS'}:
        loc = get_median_bone_pos()
    elif tra_PP == 'BOUNDING_BOX_CENTER':
        loc = get_bb_bone_pos()
    else:
        loc = cursor_point()
    return loc










def get_position(context, position=None):
    settings = context.scene.GP_scene_set
    loc = Vector()

    if position is not None: # --- position argument
        if position == 'ACTIVE_ELEMENT':
            if context.mode == 'OBJECT': 
                loc = context.object.matrix_world.translation
            elif context.mode == 'EDIT_MESH':
                loc = get_active_mesh_pos()
            elif context.mode in {'EDIT_CURVE', 'EDIT_SURFACE'}:
                loc =  get_active_curve_pos()

        elif position in {'MEDIAN_POINT', 'INDIVIDUAL_ORIGINS'}:
            if context.mode == 'OBJECT': 
                loc = median_point_object()
            elif context.mode == 'EDIT_MESH':
                loc = get_median_mesh_pos()
            elif context.mode in {'EDIT_CURVE', 'EDIT_SURFACE'}:
                loc = get_median_curve_pos()
            elif context.mode in {'EDIT_ARMATURE', 'POSE'}:
                loc = get_median_bone_pos()

        elif position == 'BOUNDING_BOX_CENTER':
            if context.mode == 'OBJECT': 
                loc = bb_point_ob()
            elif context.mode == 'EDIT_MESH':
                loc = get_bb_mesh_pos()
            elif context.mode in {'EDIT_CURVE', 'EDIT_SURFACE'}:
                loc = get_bb_curve_pos()

        elif position == 'CURSOR':
            loc = cursor_point()

        elif position == 'BOR_MESH':
            if context.mode == 'OBJECT': 
                fallback = object_get_position(context)
                loc = mesh_boundary(fallback, mode='OBJECT')
            elif context.mode == 'EDIT_MESH':
                fallback = mesh_get_position(context)
                loc = mesh_boundary(fallback)
            elif context.mode in {'EDIT_CURVE', 'EDIT_SURFACE'}: # TODO
                loc = get_median_curve_pos()
            elif context.mode in {'EDIT_ARMATURE', 'POSE'}: # TODO
                loc = get_median_bone_pos()

        elif position == 'SEL_BOR':
            if context.mode == 'OBJECT': 
                loc = median_point_object()
            elif context.mode == 'EDIT_MESH':
                fallback = mesh_get_position(context)
                loc = selection_border(fallback)
            elif context.mode in {'EDIT_CURVE', 'EDIT_SURFACE'}: # TODO
                loc = get_median_curve_pos()
            elif context.mode in {'EDIT_ARMATURE', 'POSE'}: # TODO
                loc = get_median_bone_pos()

        elif position == 'VIEW':
            loc = context.region_data.view_location

        elif position == 'OBJECT': # TODO curve
            loc = object_get_position(context)

        elif position == 'MIRROR_OBJECT': # Only For Mirror Gizmo (Use Modifier)
            loc = context.object.matrix_world.translation

        else:
            loc = Vector()

    elif settings.position == 'NONE': # --- Standard Slots
        if context.mode == 'OBJECT':
            loc = object_get_position(context)
        elif context.mode == 'EDIT_MESH':
            loc = mesh_get_position(context)
        elif context.mode in {'EDIT_CURVE', 'EDIT_SURFACE'}:
            loc = get_curve_position(context)
        elif context.mode in {'EDIT_ARMATURE', 'POSE'}:
            loc = get_bone_position(context)

    else: # --- Custom Slots
        if context.mode == 'OBJECT': 
            if settings.position == 'ACTIVE_ELEMENT':
                loc = context.object.matrix_world.translation
            elif settings.position in {'MEDIAN_POINT', 'INDIVIDUAL_ORIGINS', 'OBJECT', 'SEL_BOR'}:
                loc = median_point_object()
            elif settings.position == 'BOUNDING_BOX_CENTER':
                loc = bb_point_ob()
            elif settings.position == 'CURSOR':
                loc = cursor_point()
            elif settings.position == 'BOR_MESH':
                fallback = object_get_position(context)
                loc = mesh_boundary(fallback, mode='OBJECT')
            elif settings.position == 'VIEW':
                loc = context.region_data.view_location
            else: # position WORLD
                loc = Vector((0, 0, 0))

        elif context.mode == 'EDIT_MESH':
            if settings.position == 'ACTIVE_ELEMENT':
                loc = get_active_mesh_pos()
            elif settings.position in {'MEDIAN_POINT', 'INDIVIDUAL_ORIGINS'}:
                loc = get_median_mesh_pos()
            elif settings.position == 'BOUNDING_BOX_CENTER':
                loc = get_bb_mesh_pos()
            elif settings.position == 'CURSOR':
                loc = cursor_point()
            elif settings.position == 'OBJECT':
                loc = object_get_position(context)
            elif settings.position == 'SEL_BOR':
                fallback = mesh_get_position(context)
                loc = selection_border(fallback)
            elif settings.position == 'BOR_MESH':
                fallback = mesh_get_position(context)
                loc = mesh_boundary(fallback)
            elif settings.position == 'VIEW':
                loc = context.region_data.view_location
            else: # position WORLD
                loc = Vector()
        
        elif context.mode in {'EDIT_CURVE', 'EDIT_SURFACE'}:
            if settings.position == 'ACTIVE_ELEMENT':
                loc = get_active_curve_pos()
            elif settings.position in {'MEDIAN_POINT', 'INDIVIDUAL_ORIGINS'}:
                loc = get_median_curve_pos()
            elif settings.position == 'BOUNDING_BOX_CENTER':
                loc = get_bb_curve_pos()
            elif settings.position == 'CURSOR':
                loc = cursor_point()
            elif settings.position == 'OBJECT':
                loc = object_get_position(context)
            elif settings.position == 'SEL_BOR':
                loc = get_median_curve_pos() # TODO
            elif settings.position == 'BOR_MESH':
                loc = get_median_curve_pos() # TODO
            elif settings.position == 'VIEW':
                loc = context.region_data.view_location
            else: # position WORLD
                loc = Vector()

        elif context.mode in {'EDIT_ARMATURE', 'POSE'}:
            if settings.position == 'ACTIVE_ELEMENT':
                loc = get_active_bone_pos()
            elif settings.position in {'MEDIAN_POINT', 'INDIVIDUAL_ORIGINS'}:
                loc = get_median_bone_pos()
            elif settings.position == 'BOUNDING_BOX_CENTER':
                loc = get_bb_bone_pos()
            elif settings.position == 'CURSOR':
                loc = cursor_point()
            elif settings.position == 'OBJECT':
                loc = object_get_position(context)
            elif settings.position == 'SEL_BOR':  # TODO
                loc = get_median_bone_pos()
            elif settings.position == 'BOR_MESH':  # TODO
                loc = get_median_bone_pos()
            elif settings.position == 'VIEW':
                loc = context.region_data.view_location
            else:
                loc = Vector()

    var.LOCATION = loc
    return loc


def get_orientation(context, position=None, orient=None, context_mode=None):
    settings = context.scene.GP_scene_set
    
    if orient is None:
        orient_slots = context.window.scene.transform_orientation_slots[0].type
    else:
        orient_slots = orient

    if context_mode is None:
        mode = context.mode
    else:
        mode = context_mode


    if settings.orientation == 'NONE' or orient != None: # --- Standart Slots
        if orient_slots =='GLOBAL':
            x, y, z = get_global_orient() 
        elif orient_slots == 'LOCAL':
            x, y, z = get_local_orient()
        elif orient_slots == 'GIMBAL':
            x, y, z = get_gimbal_orient()
        elif orient_slots == 'NORMAL': 
            if mode == 'OBJECT': 
                x, y, z = get_local_orient()
            elif mode == 'EDIT_MESH':
                x, y, z = get_normal_mesh_qat(position)
            elif mode in {'EDIT_CURVE', 'EDIT_SURFACE'}:
                x, y, z = get_normal_curve_qat()
            elif context.mode in {'EDIT_ARMATURE', 'POSE'}:
                x, y, z = get_normal_bone_qat()
        elif orient_slots == 'VIEW':
            x, y, z = get_view_orient()
        elif orient_slots == 'CURSOR':
            x, y, z = get_cursor_orient()
        elif orient_slots == 'PARENT':
            x, y, z = get_parent_orient()
        else:
            x, y, z = get_custom_orient()

    else: # --- Custom Slots
        if settings.orientation == 'GLOBAL':
            x, y, z = get_global_orient() 
        elif settings.orientation == 'LOCAL':
            x, y, z = get_local_orient()
        elif settings.orientation == 'GIMBAL':
            x, y, z = get_gimbal_orient()
        elif settings.orientation == 'NORMAL': 
            if mode == 'OBJECT': 
                x, y, z = get_local_orient()
            elif mode == 'EDIT_MESH':
                x, y, z = get_normal_mesh_qat(position)
            elif mode in {'EDIT_CURVE', 'EDIT_SURFACE'}:
                x, y, z = get_normal_curve_qat()
            elif context.mode in {'EDIT_ARMATURE', 'POSE'}:
                x, y, z = get_normal_bone_qat()
        elif settings.orientation == 'VIEW':
            x, y, z = get_view_orient()
        elif orient_slots == 'CURSOR':
            x, y, z = get_cursor_orient()
        elif orient_slots == 'PARENT':
            x, y, z = get_parent_orient()
        else:
            x, y, z = get_custom_orient()

    return x, y, z


def get_orient_from_type( position=None, orient=None, context_mode=None, type='STANDARD' ):
    context = bpy.context
    props = get_prefs()
    s = Vector((1.0, 1.0, 1.0))

    if type=='TRANSPOSE':
        qX, qY, qZ = get_orientation(context, orient=orient, context_mode=context_mode)
        return qX, qY, qZ

    elif type=='STANDARD':
        qX, qY, qZ = get_orientation(context, orient=orient, context_mode=context_mode)
        l = get_position(context)
        mX = Matrix.LocRotScale(l, qX, s).normalized()
        mY = Matrix.LocRotScale(l, qY, s).normalized()
        mZ = Matrix.LocRotScale(l, qZ, s).normalized()
        return mX, mY, mZ

    elif type=='LUNA':
        qX, qY, qZ = get_orientation(context, orient=orient, context_mode=context_mode)
        l = get_position(context)

        if props.use_orientation_to_camera:
            dot_x = dot(qX, use_negative=True, loc=l)
            dot_y = dot(qY, use_negative=True, loc=l)
            dot_z = dot(qZ, use_negative=True, loc=l)

            if dot_x < 0:
                qX_invert = qX @ Quaternion((0.0, 1.0, 0.0), radians(180))
            else:
                qX_invert = qX
            if dot_y < 0:
                qY_invert = qY @ Quaternion((0.0, 1.0, 0.0), radians(180))
            else:
                qY_invert = qY
            if dot_z < 0:
                qZ_invert = qZ @ Quaternion((0.0, 1.0, 0.0), radians(180))
            else:
                qZ_invert = qZ

            if context.region_data.is_orthographic_side_view:
                return qX, qY, qZ, 1, 1, 1, qX, qY, qZ
            else:
                return qX_invert, qY_invert, qZ_invert, copysign(1, dot_x), copysign(1, dot_y), copysign(1, dot_z), qX, qY, qZ
        else:
            return qX, qY, qZ, 1, 1, 1, qX, qY, qZ

    elif type=='VR':
        qX, qY, qZ = get_orientation(context, orient=orient, context_mode=context_mode)
        l = get_position(context)
        mat_rot = Matrix.LocRotScale(l, qZ, s)

        xP, xN, yP, yN, zP, zN = get_bbox(mat_rot)

        mXP = Matrix.LocRotScale(yP, qY @ Quaternion( (0.0, 1.0, 0.0), radians(90) ), s)
        mYP = Matrix.LocRotScale(xN, qX @ Quaternion( (1.0, 0.0, 0.0), radians(-90) ), s)
        mZP = Matrix.LocRotScale(zP, qZ, s)

        mXN = Matrix.LocRotScale(yN, qY @ Quaternion( (0.0, 1.0, 0.0), radians(-90) ), s)
        mYN = Matrix.LocRotScale(xP, qX @ Quaternion( (1.0, 0.0, 0.0), radians(90) ), s)
        mZN = Matrix.LocRotScale(zN, qZ @ Quaternion( (1.0, 0.0, 0.0), radians(180) ), s)

        # --- Normalized
        mXP.normalized()
        mYP.normalized()
        mZP.normalized()
        mXN.normalized()
        mYN.normalized()
        mZN.normalized()

        return mXP, mYP, mZP, mXN, mYN, mZN
    
    elif type=='PLACEMENT':
        qX, qY, qZ = get_orientation(context, position=position, orient=orient, context_mode=context_mode)
        l = get_position(context, position=position)
        s = Vector((1.0, 1.0, 1.0))
        mm = Matrix.LocRotScale(l, qZ @ Quaternion( (1.0, 0.0, 0.0), radians(180)), s).normalized()
        ms = Matrix.LocRotScale(l, qZ, s).normalized()
        return mm, ms
    
    elif type=='MIRROR':
        qX, qY, qZ = get_orientation(context, position=position, orient=orient, context_mode=context_mode)
        l = get_position(context, position=position)

        xp = Matrix.LocRotScale(l, qX, s).normalized()
        yp = Matrix.LocRotScale(l, qY, s).normalized()
        zp = Matrix.LocRotScale(l, qZ, s).normalized()

        xn = Matrix.LocRotScale(l, qX @ Quaternion((1.0, 0.0, 0.0), radians(180)), s).normalized()
        yn = Matrix.LocRotScale(l, qY @ Quaternion((1.0, 0.0, 0.0), radians(180)), s).normalized()
        zn = Matrix.LocRotScale(l, qZ @ Quaternion((1.0, 0.0, 0.0), radians(180)), s).normalized()

        return xp, yp, zp, xn, yn, zn

    elif type=='SHEAR':
        qX, qY, qZ = get_orientation(context, position=position, orient=orient, context_mode=context_mode)
        l = get_position(context)
        s = Vector((0.5, 6.5, 1.0))

        x_n = qX @ Quaternion((0.0, 0.0, 1.0), radians(90))
        y_n = qY @ Quaternion((0.0, 0.0, 1.0), radians(90))
        z_n = qZ @ Quaternion((0.0, 0.0, 1.0), radians(90))

        mX = Matrix.LocRotScale(l, qX, s)
        mY = Matrix.LocRotScale(l, qY, s)
        mZ = Matrix.LocRotScale(l, qZ, s)
        mX_n = Matrix.LocRotScale(l, x_n, s)
        mY_n = Matrix.LocRotScale(l, y_n, s)
        mZ_n = Matrix.LocRotScale(l, z_n, s)

        mX.normalized()
        mY.normalized()
        mZ.normalized()
        mX_n.normalized()
        mY_n.normalized()
        mZ_n.normalized()

        return [
            [mX, mX_n],
            [mY, mY_n],
            [mZ, mZ_n],
        ]
