import bpy
from mathutils import Matrix, Vector, Euler, Quaternion
from math import radians



def get_normal_curve_qat(): # FIXME TODO сделать как у стандартной гизмо

    obj = bpy.context.object
    l, q, s = obj.matrix_world.decompose()
    active_spline = obj.data.splines.active
    normal = Vector()
    
    if active_spline:
        direction = Vector()
        if active_spline.type == 'BEZIER':
            for point in active_spline.bezier_points:
                if point.select_control_point or point.select_right_handle or point.select_left_handle:
                    direction = point.co - point.handle_left # Направление от левой ручки к контрольной точке
                    
        elif active_spline.type in {'POLY', 'NURBS'}:
            points = active_spline.points
            for i, point in enumerate(points):
                if point.select:
                    # Предыдущая точка
                    if i > 0:
                        prev_point = points[i - 1].co
                    else:
                        prev_point = point.co

                    # Следующая точка
                    if i < len(points) - 1:
                        next_point = points[i + 1].co
                    else:
                        next_point = point.co

                    # Вычисляем направление
                    direction = (Vector((next_point[0], next_point[1], next_point[2])) - Vector((prev_point[0], prev_point[1], prev_point[2]))).normalized()
                    
                    

        normal = direction.normalized()
        q = normal.to_track_quat('Z', 'Y')
    
    x = q @ Quaternion((0.0, 1.0, 0.0), radians(90)) 
    y = q @ Quaternion((1.0, 0.0, 0.0), radians(-90))
    z = q

    return x, y, z