import bpy
from mathutils import Matrix, Vector, Quaternion
from math import radians
import bmesh


from ...utils.custom_math.math import (
    is_zero_v3,
    normalize_v3,
    add_v3_v3,
    normal_tri_v3,
    )

from ...utils.custom_math.bmesh import (
    editselection_normal,
    editselection_plane,
    face_calc_tangent_auto,
    vert_tri_calc_tangent_edge,
    select_history_active_get,
    )

from ...utils.custom_math.calculate import (
    createSpaceNormal,
    createSpaceNormalTangent,
)




def get_normal_mesh_qat(orient=None): #FIXME TODO
    context = bpy.context
    
    
    ob = context.object
        
    obmat = ob.matrix_world.copy()
    imat = obmat.inverted()
    mat = imat.transposed()
    

    #mat = ob.matrix_world.copy()
    vec = Vector()
    normal = Vector()
    plane = Vector()

    if orient is None:
        tpp = orient
    else:
        tpp = context.scene.tool_settings.transform_pivot_point

    
    
    em = bmesh.from_edit_mesh(ob.data)

    #em.verts.ensure_lookup_table() # ?
    #em.edges.ensure_lookup_table()
    #em.faces.ensure_lookup_table()

    
    result = 'ORIENTATION_VERT'


    el = select_history_active_get(em)
    



    if tpp == 'ACTIVE_ELEMENT' and el: # transform_orientation.c 770

        normal = editselection_normal(el)
        plane = editselection_plane(el)

        if isinstance(el, bmesh.types.BMVert):
            result = 'ORIENTATION_VERT'
        elif isinstance(el, bmesh.types.BMEdge):
            result = 'ORIENTATION_EDGE'
        elif isinstance(el, bmesh.types.BMFace):
            result = 'ORIENTATION_FACE'

                
    else:
        v = [v for v in em.verts if v.select]
        e = [e for e in em.edges if e.select]
        f = [f for f in em.faces if f.select]


        if len(f) >= 1:
            for fs in f:
                vec = face_calc_tangent_auto(fs)
                add_v3_v3(normal, fs.normal)
                add_v3_v3(plane, vec)
            result = 'ORIENTATION_FACE'
            
        elif len(v) == 3: # TODO
            e = None
            no_test = Vector()

            normal_tri_v3(normal, v[0].co, v[1].co, v[2].co)

            no_test[0] = v[0].normal[0] + v[1].normal[0] + v[2].normal[0]
            no_test[1] = v[0].normal[1] + v[1].normal[1] + v[2].normal[1]
            no_test[2] = v[0].normal[2] + v[1].normal[2] + v[2].normal[2]

            if no_test.dot(normal) < 0:
                normal.negate()

            


            #if e :
            plane = vert_tri_calc_tangent_edge(v)
            



            result = 'ORIENTATION_FACE'

        elif len(e) == 1 or len(v) == 2:
            result = 'ORIENTATION_EDGE'
        
        elif len(v) == 1:
            normal = editselection_normal(v[0])
            plane = editselection_plane(v[0])
            result = 'ORIENTATION_VERT' #  is_zero_v3(plane) ? ORIENTATION_VERT : ORIENTATION_EDGE;
        
        elif len(v) > 3:
            normal.zero()
            
            for vs in v:
                add_v3_v3(normal, vs.normal)
            
            #normalize_v3(normal)
            normal.normalize()
            result = 'ORIENTATION_VERT'


    plane.negate() # not needed but this matches 2.68 and older behavior




    if result == 'ORIENTATION_EDGE': # TODO
        ''' normal = obmat @ normal
        plane = obmat @ plane

        tvec = normal.project(plane)
        sub_v3_v3(normal, tvec) '''
        
        if isinstance(el, bmesh.types.BMEdge):
            v1, v2 = [v for v in el.verts]
        else:
            v1 = el
            v2 = [ v for v in em.verts if v.select and v!=v1 ][0]
                
        avrNormal = ((v1.normal + v2.normal) * 0.5)
        plane = obmat @ Vector(v1.co) - obmat @ Vector(v2.co)
        avrNormal = mat @ avrNormal
        perpVec = plane.cross(avrNormal).normalized()
        normal = plane.cross(perpVec).normalized()
        normal.negate()


    else:
        normal = mat @ normal
        plane = mat @ plane

   
   
    if is_zero_v3(plane): # Fallback, when the plane can't be calculated.
        result = 'ORIENTATION_VERT'

    r_orientation_mat = Matrix()
    if result == 'ORIENTATION_VERT':
        createSpaceNormal(r_orientation_mat, normal)
    else:
        createSpaceNormalTangent(r_orientation_mat, normal, plane)
        

    r_orientation_mat.invert_safe()
    rot = r_orientation_mat.decompose()[1]
     
    
    
    x = rot @ Quaternion((0.0, 1.0, 0.0), radians(90)) 
    y = rot @ Quaternion((1.0, 0.0, 0.0), radians(-90))
    z = rot @ Quaternion((0.0, 0.0, 1.0), radians(-90))
    

    return x, y, z


def normal_qat_from_two_point(p1, p2, pn1, pn2):
    obmat = Matrix()
    imat = obmat.inverted()
    mat = imat.transposed()

    normal = Vector()
    plane = Vector()
    plane.negate()

    avrNormal = ((pn1 + pn2) * 0.5)
    plane = obmat @ Vector(p1) - obmat @ Vector(p2)
    avrNormal = mat @ avrNormal
    perpVec = plane.cross(avrNormal).normalized()
    normal = plane.cross(perpVec).normalized()
    normal.negate()

    r_orientation_mat = Matrix()
    createSpaceNormalTangent(r_orientation_mat, normal, plane)

    r_orientation_mat.invert_safe()
    rot = r_orientation_mat.decompose()[1]

    return rot @ Quaternion( (0.0, 0.0, 1.0), radians(-90) ).normalized()





""" def get_normal_mesh_qat(props, position=None): #FIXME TODO
    context = bpy.context
    ob = context.object
    
    result = 'ORIENTATION_NONE'
    if position is None:
        tpp = position
    else:
        tpp = context.scene.tool_settings.transform_pivot_point
    activeOnly = tpp=='ACTIVE_ELEMENT'

    
    normal = plane = vec = Vector()


    
    obmat = ob.matrix_world.copy()
    imat = obmat.inverted()
    mat = imat.transposed()

    em = bmesh.from_edit_mesh(ob.data)
    el = select_history_active_get(em)

    
    #if context.mode == 'EDIT_MESH':
    if tpp == 'ACTIVE_ELEMENT' and el: # transform_orientation.c 770
        normal = editselection_normal(el)
        plane = editselection_plane(el)
        if isinstance(el, bmesh.types.BMVert):
            result = 'ORIENTATION_VERT'
        elif isinstance(el, bmesh.types.BMEdge):
            result = 'ORIENTATION_EDGE'
        elif isinstance(el, bmesh.types.BMFace):
            result = 'ORIENTATION_FACE'
    else:
        sel_v = [v for v in em.verts if v.select]
        sel_e = [e for e in em.edges if e.select]
        sel_f = [f for f in em.faces if f.select]

        if len(sel_f) >= 1:
            for fs in sel_f:
                vec = face_calc_tangent_auto(fs)
                add_v3_v3(normal, fs.normal)
                add_v3_v3(plane, vec)
            result = 'ORIENTATION_FACE'
        
        elif len(sel_v) == 3:
            v_tri = [sel_v[0], sel_v[1], sel_v[2]]
            if em.bm.verts_select_get_n(v_tri, 3) == 3: # TODO
                e = None
                no_test = Vector()
                
                normal_tri_v3(normal, v_tri[0].co, v_tri[1].co, v_tri[2].co)

                # check if the normal is pointing opposite to vert normals
                no_test[0] = v_tri[0].normal[0] + v_tri[1].normal[0] + v_tri[2].normal[0]
                no_test[1] = v_tri[0].normal[1] + v_tri[1].normal[1] + v_tri[2].normal[1]
                no_test[2] = v_tri[0].normal[2] + v_tri[1].normal[2] + v_tri[2].normal[2]
                if no_test.dot(normal) < 0.0:
                    normal.negate()

                if sel_e >= 1:
                    # find an edge that's a part of v_tri (no need to search all edges)
                    e_length = 0.0
                    j = 0
                    while j < 3:
                        e_test = BM_edge_exists(v_tri[j], v_tri[(j + 1) % 3])
                        if e_test and BM_elem_flag_test(e_test, BM_ELEM_SELECT):
                            e_test_length = BM_edge_calc_length_squared(e_test)
                            if e is None or e_length < e_test_length:
                                e = e_test
                                e_length = e_test_length
                        j += 1

                if e:
                    v_pair = [None, None]
                    if BM_edge_is_boundary(e):
                        BM_edge_ordered_verts(e, v_pair)
                    else:
                        v_pair[0] = e.v1
                        v_pair[1] = e.v2
                    sub_v3_v3v3(plane, v_pair[0].co, v_pair[1].co)
                else:
                    plane = vert_tri_calc_tangent_edge(v)

                result = 'ORIENTATION_FACE'
            else:
                assert False """