import bpy
import bmesh
from mathutils import Vector



def selection_border(fallback=None):
    uniques = bpy.context.objects_in_mode_unique_data
            
    me = bmesh.new()

    bms = {}
    for ob in uniques:
        bms[ob] = bmesh.from_edit_mesh(ob.data)

    for ob in uniques:
        onm = {}
        verts = [v for v in bms[ob].verts if v.select]
        for v in verts:
            nv = me.verts.new(ob.matrix_world @ v.co)
            onm[v.index] = nv

        faces = [f for f in bms[ob].faces if f.select]
        for f in faces:
            nfverts = [onm[v.index] for v in f.verts]
            me.faces.new(nfverts)


    verts = [v.co  for v in me.verts if v.is_boundary]
    
    if len(verts) > 0:
        l = sum(verts, Vector()) / len(verts)

    else: # --- Fallback
        l = fallback
   
    return l



def mesh_boundary(fallback=None, mode='EDIT'):
    if mode == 'EDIT':
        uniques = bpy.context.objects_in_mode_unique_data
    else:
        uniques = bpy.context.selected_objects

    bms = {}
    for obj in uniques:
        if mode == 'EDIT':
            bms[obj] = bmesh.from_edit_mesh(obj.data)
        else:
            bms[obj] = bmesh.new()
            bms[obj].from_mesh(obj.data)
            

    verts = []
    for ob in bms:
        if mode == 'EDIT':
            verts.extend([ob.matrix_world @ v.co  for v in bms[ob].verts if v.select and v.is_boundary]) 
        else:
            verts.extend([ob.matrix_world @ v.co  for v in bms[ob].verts if v.is_boundary])


    if len(verts) > 0:
        l = sum(verts, Vector()) / len(verts)

    else: # --- Fallback
        l = fallback

    return l

