import bpy
import bmesh
from mathutils import Vector
from ...utils.custom_math.bmesh import select_history_active_get



def get_slected_verts():
    sMe = [ob for ob in bpy.context.objects_in_mode_unique_data if ob.type == 'MESH']
    verts = []
    for ob in sMe:
        bm = bmesh.from_edit_mesh(ob.data)
        verts.extend([ob.matrix_world @ v.co  for v in bm.verts if v.select]) 
    return verts


def fallback():
    obj = bpy.context.object #active_object
    bm = bmesh.from_edit_mesh(obj.data)
    el = select_history_active_get(bm)
    if isinstance(el, bmesh.types.BMVert):
        l = obj.matrix_world @ el.co
        
    elif isinstance(el, bmesh.types.BMEdge):
        l = obj.matrix_world @ ((el.verts[0].co.copy() + el.verts[1].co.copy()) / 2).copy()
        
    elif isinstance(el, bmesh.types.BMFace):
        l = obj.matrix_world @ el.calc_center_median().copy()
    else:
        l = Vector()

    return l


def get_bb_mesh_pos():
    verts = get_slected_verts()
    if len(verts) > 0 :
        x = ( min([v.x for v in verts]) + max([v.x for v in verts]) ) / 2
        y = ( min([v.y for v in verts]) + max([v.y for v in verts]) ) / 2
        z = ( min([v.z for v in verts]) + max([v.z for v in verts]) ) / 2
        l = Vector((x, y, z))
    else:
        l = fallback()
    return l


def get_median_mesh_pos():
    verts = get_slected_verts()
    if len(verts) > 0 :
        l = sum(verts, Vector()) / len(verts)
    else:
        l = fallback()
    return l
 

def get_active_mesh_pos(): 
    obj = bpy.context.object #active_object
    bm = bmesh.from_edit_mesh(obj.data)
    el = select_history_active_get(bm)

    if el:
        if isinstance(el, bmesh.types.BMVert):
            l = obj.matrix_world @ el.co
            
        elif isinstance(el, bmesh.types.BMEdge):
            l = obj.matrix_world @ ((el.verts[0].co.copy() + el.verts[1].co.copy()) / 2).copy()
            
        elif isinstance(el, bmesh.types.BMFace):
            l = obj.matrix_world @ el.calc_center_median().copy()
    else:
        l = get_bb_mesh_pos()

    return l