import bpy
from mathutils import Matrix, Quaternion, Vector
from math import radians
from ..utils.custom_math.quaternion import look_rotation
from .matrix_ensure import get_orient_from_type
from .. import get_prefs



def transpose_matrix():
    context = bpy.context

    props = get_prefs()
    settings = context.scene.GP_scene_set

    
    if settings.use_align:
    
        up = Vector(settings.mes_rot)
        target =  Vector(settings.tr_p2) - Vector(settings.tr_p1)
        look = look_rotation(target, up)

        rot = look @ Quaternion( (1.0, 0.0, 0.0), radians(90) ) @ Quaternion( (0.0, 1.0, 0.0), radians(180) )
        

        # --- Final
        x = rot @ Quaternion( (1.0, 0.0, 0.0), radians(-90) )
        y = rot @ Quaternion( (0.0, 1.0, 0.0), radians(-90) )
        z = rot
    
        xD = rot @ Quaternion( (1.0, 0.0, 0.0), radians(90) ) @ Quaternion( (0.0, 1.0, 0.0), radians(180) ) @ Quaternion( (0.0, 0.0, 1.0), radians(props.arc_angle[0]) )
        yD = rot @ Quaternion( (0.0, 1.0, 0.0), radians(-90) ) @ Quaternion( (0.0, 0.0, 1.0), radians(props.arc_angle[1]) )
        zD = rot @ Quaternion( (0.0, 0.0, 1.0), radians(90) ) @ Quaternion( (0.0, 0.0, 1.0), radians(props.arc_angle[2]) )


    elif settings.use_global_mat:
        x, y, z = get_orient_from_type(context_mode='OBJECT', type='TRANSPOSE')
        xD = x
        yD = y
        zD = z
        z = z @ Quaternion( (0.0, 0.0, 1.0), radians(90) )

    else:
        rot = settings.mes_rot_quat

        x = rot @ Quaternion( (1.0, 0.0, 0.0), radians(-90) )
        y = rot @ Quaternion( (0.0, 1.0, 0.0), radians(-90) )
        z = rot

        xD = rot @ Quaternion( (1.0, 0.0, 0.0), radians(90) ) @ Quaternion( (0.0, 1.0, 0.0), radians(180) ) @ Quaternion( (0.0, 0.0, 1.0), radians(props.arc_angle[0]) )
        yD = rot @ Quaternion( (0.0, 1.0, 0.0), radians(-90) ) @ Quaternion( (0.0, 0.0, 1.0), radians(props.arc_angle[1]) )
        zD = rot @ Quaternion( (0.0, 0.0, 1.0), radians(90) ) @ Quaternion( (0.0, 0.0, 1.0), radians(props.arc_angle[2]) )


    
    l = settings.tr_p1
    s = Vector((1.0, 1.0, 1.0))
    
    
    mX = Matrix.LocRotScale(l, x, s)
    mY = Matrix.LocRotScale(l, y, s)
    mZ = Matrix.LocRotScale(l, z, s)

    mX_d = Matrix.LocRotScale(l, xD, s)
    mY_d = Matrix.LocRotScale(l, yD, s)
    mZ_d = Matrix.LocRotScale(l, zD, s)


    # --- Normalized
    mX.normalized()
    mY.normalized()
    mZ.normalized()
    mX_d.normalized()
    mY_d.normalized()
    mZ_d.normalized()


    return mX, mY, mZ, mX_d, mY_d, mZ_d