import bpy
from bpy.props import EnumProperty, IntProperty, BoolProperty, FloatProperty, FloatVectorProperty, CollectionProperty, StringProperty, PointerProperty
from bpy.types import AddonPreferences, PropertyGroup
from .icons import icons_collections
from . import var
import rna_keymap_ui
from .keymaps import get_hotkey_entry_item
from . import get_prefs



class GP_settings(PropertyGroup):  # --- Параметры Сохраненные в сцене


    def update_mat(self, context):
        from . import var
        var.REFRESH_MAT = True




    gizmopro_global: BoolProperty(name='Gizmo PRO', description='Show Gizmo PRO', default=False)
    scale_gizmo: BoolProperty(name='Scale Gizmo', description='Show Scale Gizmo', default=False)
    curve_gizmo: BoolProperty(name='Curve Gizmo', description='Show Curve Gizmo', default=True)


    # --- Increment Rotate
    def set_ia(self, context):
        if self.preset_ia == '5':
            self.ia = 5
        elif self.preset_ia == '10':
            self.ia = 10
        elif self.preset_ia == '15':
            self.ia = 15
        elif self.preset_ia == '30':
            self.ia = 30
        elif self.preset_ia == '45':
            self.ia = 45
        elif self.preset_ia == '60':
            self.ia = 60
        elif self.preset_ia == '90':
            self.ia = 90
        elif self.preset_ia == '120':
            self.ia = 120
    use_ia: BoolProperty(name='Use Incremental Rotate', description='Use Incremental Rotate operator instead of the standard ', default=False)
    ia: IntProperty(
        name='Incremental Angle',
        description='Rotation angle when snapping. (Increment Rotate modifier operator)',
        default=45,
        options=set(),
        subtype='ANGLE')

    preset_ia: EnumProperty(
            name = 'Set Angle',
            description = '',
            items = [
                    ('5', '5°', '', '', 0),
                    ('10', '10°', '', '', 1),
                    ('15', '15°', '', '', 2),
                    ('30', '30°', '', '', 3),
                    ('45', '45°', '', '', 4),
                    ('60', '60°', '', '', 5),
                    ('90', '90°', '', '', 6),
                    ('120', '120°', '', '', 7),
                    ],
            default = '45',
            update = set_ia,
            )
    show_gizmo: BoolProperty(name='Show Gizmos', description='Visibility Of Gizmo PRO', default=True, update=update_mat)
    gzm_type: EnumProperty(
        name='Gizmo Type',
        description = 'Choosing the type of gizmo',
        items = [
            ('GIZMOPRO', 'Gizmo PRO', 'A common gizmo for transformations\n'
                                      '• Middle Mouse Button - Transform Pivot Point' , 'GIZMO', 0),
            ('LUNA', 'Luna', 'A common gizmo for transformation\n'
                             '• Middle Mouse Button - Transform Pivot Point', 'ORIENTATION_VIEW', 1),
            ('MALIBU', 'Malibu', 'A common gizmo for transformation\n'
                                '• Middle Mouse Button - Transform Pivot Point', 'SNAP_NORMAL', 2),
            ('TRANSPOSE', 'Transpose', 'A common gizmo for transformation', 'SNAP_INCREMENT', 3),
            ('VR', 'VR', 'A common gizmo for transformation\n'
                         '• Middle Mouse Button - Transform Pivot Point', 'CON_LOCLIKE', 4),
            None,
            ('PLACEMENT', 'Placement', 'Gizmo to place the selected by normals\n'
                                       '• Shift - Duplicate Move', 'NORMALS_FACE', 5),
            ('MIRROR', 'Mirror', 'Gizmo to mirror the selected\n'
                                 '• Shift: Use Only The Selected Axis (Modifier)\n'
                                 '• Ctrl: Bisect Off (Modifier)', 'MOD_MIRROR', 6),
            ('SHEAR', 'Shear', 'Shear selected items along the horizontal screen axis.', 'MOD_LATTICE', 7),
            ],
        default='GIZMOPRO',
        update=update_mat,
        )


    # --- Transpose
    use_global_mat: BoolProperty(default=True)
    use_align: BoolProperty(default=False)

    tr_p1: FloatVectorProperty(default=(0,0,0))
    tr_p2: FloatVectorProperty(default=(2,0,0))

    mes_rot: FloatVectorProperty(default=(0,0,1))
    mes_rot_quat: FloatVectorProperty(size=4, subtype='QUATERNION')

    transpose_direction: FloatVectorProperty(default=(1,0,0))
    transpose_axis: StringProperty(default='+Y')
    

    position: EnumProperty(
            name = 'Position',
            description = 'The position of the gizmo in 3D space',
            items = [
                    ('NONE', 'Default', 'Use the standard position settings', 'BLENDER', 0),
                    None,
                    ('BOUNDING_BOX_CENTER', 'Bounding Вох Center', 'Pivot around bounding box center of selected object(s)', 'PIVOT_BOUNDBOX', 1),
                    ('CURSOR', '3D Cursor', 'Pivot around the 3D cursor', 'PIVOT_CURSOR', 2),
                    ('INDIVIDUAL_ORIGINS', 'Individual Origins', "Pivot around each object's own origin", 'PIVOT_INDIVIDUAL', 3),
                    ('MEDIAN_POINT', 'Median Point', 'Pivot around the median point of selected objects', 'PIVOT_MEDIAN', 4),
                    ('ACTIVE_ELEMENT', 'Active Element', 'Pivot around active object', 'PIVOT_ACTIVE', 5),
                    None,

                    ('OBJECT', 'Object', 'Takes the position from the object mode', 'FILE_3D', 6),
                    ('SEL_BOR', 'Selection Border', 'If it is not possible to calculate, the standard position is used', 'SELECT_SET', 7),
                    ('BOR_MESH', 'Boundary Mesh', 'If it is not possible to calculate, the standard position is used', 'SPHERECURVE', 8),
                    ('VIEW', 'Screen', 'Center Of The View', 'RESTRICT_VIEW_ON', 9),
                    ('WORLD', 'World Origin', '', 'WORLD', 10),
                    ],
            default = 'NONE',
            update = update_mat,
            )

    orientation: EnumProperty(
            name = 'Orientation',
            description = 'Orientation of Gizmo in 3D space',
            items = [
                    ('NONE', 'Default', 'Use the standard orientation settings', 'BLENDER', 0),
                    None,
                    ('GLOBAL', 'Global', 'Align the transformation axes to world space', 'ORIENTATION_GLOBAL', 1),
                    ('LOCAL', 'Local', 'Align the transformation axes to local space', 'ORIENTATION_LOCAL', 2),
                    ('NORMAL', 'Normal', 'Align the transformation axes to average normal of selected elements (bone Y axis for pose mode)', 'ORIENTATION_NORMAL', 3),
                    ('GIMBAL', 'Gimbal', 'Align each axis to the Euler rotation axis as used for input', 'ORIENTATION_GIMBAL', 4),
                    ('VIEW', 'View', 'Align the transformation axes to the window', 'ORIENTATION_VIEW', 5),
                    ('CURSOR', '3D Cursor', 'Align the transformation axes to the 3D cursor', 'ORIENTATION_CURSOR', 6),
                    ('PARENT', 'Parent', "Align the transformation axes to the object's parent space", 'FILE_PARENT', 7),
                    ],
            default = 'NONE',
            update = update_mat,
            )


# --- Store
    axis: StringProperty( default = 'X' )
    angle: FloatProperty( default = 0.0 )
    #matrix: FloatVectorProperty(size=(4, 4), subtype='MATRIX') # работает
    #pp: FloatVectorProperty(subtype='TRANSLATION') # работает

    lock_orbit: BoolProperty( name = 'Lock', description = 'Lock Orbit(Switch to Pan)', default = False )
    value: StringProperty( default = 'NONE' ) # Transform Value


    # --- Select Tool
    select_settings_tab: EnumProperty(
        name = 'Tabs',
        items = [
            ('LMB', 'Left Mouse Button', '', 'MOUSE_LMB_DRAG', 0),
            #('MMB', 'Middle Mouse Button', '', 'MOUSE_MMB_DRAG', 1),
            ('RMB', 'Right Mouse Button', '', 'MOUSE_RMB_DRAG', 1),
            ],
        default = 'LMB',
        )

    """ dot_retop: BoolProperty( 
        name = 'Dot Retopology',
        description = 'Actions For Gizmo Dot To Help With Retopology \n• Shift - Vertex Slide. \n• Ctrl - Merge At Center',
        default = False,
        ) """

# --- Gizmo PRO
    tra_gizmo: BoolProperty(name='Translate Visible', description = 'Gizmo To Adjust Location', default=True)
    rot_gizmo: BoolProperty(name='Rotate Visible', description = 'Gizmo To Adjust Rotation', default=True)
    sca_gizmo: BoolProperty(name='Scale Visible', description = 'Gizmo To Adjust Scale', default=True)
    blank_gizmo: BoolProperty(name='Translate Two Axes Visible', description = 'Gizmo To Adjust Location', default=True)
    dot_move: BoolProperty(name='Translate', description = 'Circle For Free Movement', default=True)
    rotate_view: BoolProperty(name='Rotate View', description = 'Gizmo To Adjust Rotate Relative To The View', default=False)
    scale_view: BoolProperty(name='Scale View', description = 'Gizmo To Adjust Scale Relative To The View', default=False)
    rotate_trackball: BoolProperty(name='Trackball Rotate', description = 'Gizmo Trackball Rotate', default=False)


# --- 3D Cursor
    cursor_navigator: BoolProperty( name='3D Cursor Navigator', description=('Advanced gizmo for the 3d cursor in the form of a regular circle.\n'
                                                                             'Serves for fast positioning of the 3d cursor in space with the pre-included feature of snapping on elements.\n'
                                                                             'MMB Click - calls the quick menu'), default=True )
    cursor_gizmo: BoolProperty( name='3D Cursor', description='A Special Gizmo For Transforming The 3d Cursor', default=False )
    cursor_gizmo_dot: BoolProperty(
        name='Show Dot',
        description='Show The Move Element Relative To The View',
        options=set(),
        default=True)
    cursor_gizmo_axes: BoolProperty( 
        name='Show Axis',
        description='Show The Movement Element Relative To The Axes',
        options=set(),
        default=True)
    
    camera_gizmo: BoolProperty( name = 'Camera', description = 'Gizmo For Special Camera Transformations', default = False )
    #empty_gizmo: BoolProperty( name = 'Empty', description = 'Gizmo To Change The Size Parameter In Empty Object', default = False )
    bn_gizmo: BoolProperty(name='Border Navigate', description='Show Border Navigation', default=False)
    bn_sculpt_retop: BoolProperty( 
        name='Use Only In Sculpt Mode and Retopo Tool',
        description='Turns off Border Navigate in all modes except Sculpt and Retopo Tool',
        options=set(),
        default=False )
    gizmo_2d_navigate: BoolProperty(name='2D Navigate', description='Gizmo for 2d navigation in 3d View', default=False)





# --- Quick Widget
    qw: BoolProperty( name='Quick Widget', description='Widget With Quick Access To Some Functions', default=False )
    qw_pos: FloatVectorProperty( size=2, default=(150.0, 150.0) )
    qw_draw_pos: FloatVectorProperty( size=2, default=(150.0, 150.0) )  
    qw_alignment: EnumProperty( name='Alignment',
                                description = '',
                                items = [
                                        ('CIRCLE', 'Circle', '', '', 0),
                                        ('HORIZONTAL', 'Horizontal', '', '', 1),
                                        ('VERTICAL', 'Vertical', ' ', '', 2),# TODO Description
                                        ],
                                options=set(),
                                default='VERTICAL')

# --- RefBoard
    refboard: BoolProperty( name='RefBoard', description='', default=False )
    rb_edit: BoolProperty( name='Edit', description='', default=True )

    searcher_gizmo: BoolProperty( name='Searcher', description='Shows the location of the gizmo when it is outside the 3D View', default=True )
    preselect_gizmo: BoolProperty( name='Preselect', description='Preselect Gizmo', default=False )
    
# --- Mirror
    mirror_use_modifier: BoolProperty( name='Use Modifier', description='Use Modifier', default=True, update=update_mat ) 



class GP_collections(PropertyGroup): # --- Параметры Сохраненные в объекте
    custom_position: FloatVectorProperty( default=(0,0,1) )
    custom_orient_matrix: FloatVectorProperty( name='Custom Orient Matrix', size=16, subtype='MATRIX' )
    


class GP_preferences(AddonPreferences):
    bl_idname = __package__

# --- Вспомогательные функции
    def update_gizmo(self, context):
        from .gizmo_group.gg import GP_GGT_gizmo_pro_3d
        from .gizmo.gizmos_3d.gt_cone import GP_GT_cone
        from .gizmo.gizmos_3d.gt_box import GP_GT_box
        from .gizmo.gizmos_3d.gt_arc import GP_GT_arc
        gizChange = [
            GP_GGT_gizmo_pro_3d,
            GP_GT_cone,
            GP_GT_box,
            GP_GT_arc,
            ]

        try:
            for gizmo in gizChange:
                if 'bl_rna' in gizmo.__dict__:
                    bpy.utils.unregister_class(gizmo)

            for gizmo in gizChange:
         
                bpy.utils.register_class(gizmo)

        except Exception as e:
            message = 'Gizmo PRO: Updating gizmos failed'
            print('\n[{}]\n{}\n\nError:\n{}'.format(__name__, message, e))
            pass


    def update_keymaps(self, context):
        wm = bpy.context.window_manager
        actK = wm.keyconfigs.active
        addonK = wm.keyconfigs.addon
    
        if not addonK:
            return
        for kc in (actK, addonK):
            km = kc.keymaps.find(name='Gizmo PRO Tweak', space_type='VIEW_3D')
            for kmi in km.keymap_items:
                km.keymap_items.remove(kmi)

            props = get_prefs()
            km = kc.keymaps.new( name='Gizmo PRO Tweak', space_type='VIEW_3D' )
            kmi = km.keymap_items.new( 'gizmogroup.gizmo_tweak', type=props.map_type, value='CLICK_DRAG', any=True )
            kmi.alt, kmi.oskey = False, False
            if props.use_mmb_drag:
                kmi = km.keymap_items.new( 'gizmogroup.gizmo_tweak', type='MIDDLEMOUSE', value='CLICK_DRAG', any=True )
                kmi.alt, kmi.oskey = False, False
            if props.use_mmb_click:
                kmi = km.keymap_items.new( 'gizmogroup.gizmo_tweak', type='MIDDLEMOUSE', value='CLICK', any=True )
                kmi.alt, kmi.oskey = False, False

        self.update_gizmo(context)


    def update_tool(self, context):
        from .tools import GizmoProTools
       
        try:
            for tool in GizmoProTools:
                bpy.utils.register_tool(tool['tool'] ,  after = tool['after'], separator=tool['separator'], group = tool['group'])

        except Exception as e:
            message = 'Gizmo PRO: Updating gizmos failed'
            print('\n[{}]\n{}\n\nError:\n{}'.format(__name__, message, e))
            pass
    
    def update_color_preset(self, context):
        if self.color_preset_type == 'BLENDER':
            theme = bpy.context.preferences.themes['Default']
            self.x_axis_color = theme.user_interface.axis_x
            self.y_axis_color = theme.user_interface.axis_y
            self.z_axis_color = theme.user_interface.axis_z
        elif self.color_preset_type == 'GIZMOPRO':
            self.x_axis_color = (1.0, 0.0, 0.0)
            self.y_axis_color = (0.2, 1.0, 0.0)
            self.z_axis_color = (0.0, 0.35, 1.0)
        elif self.color_preset_type == 'MALIBU':
            self.x_axis_color = (1.0, 0.3, 0.3)
            self.y_axis_color = (0.3, 1.0, 0.3)
            self.z_axis_color = (0.3, 0.3, 1.0)
        elif self.color_preset_type == 'SATURATED':
            self.x_axis_color = (1.0, 0.0, 0.0)
            self.y_axis_color = (0.0, 1.0, 0.0)
            self.z_axis_color = (0.0, 0.0, 1.0)
    
    def preset(self, context):
        if self.gizmo_preset == 'ZBRUSH':
            self.gt_cone = 'ZBRUSH'
            self.gt_box = 'ZBRUSH'
            self.gt_arc = 'ZBRUSH'
            
            self.offset_cone = 0.95
            self.offset_box = 0.7
            self.offset_arc = 0.58
            self.arc_angle = (135.0, 315.0, 135.0)

            self.size_dot = 1.0


        elif self.gizmo_preset == 'BLENDER':
            self.gt_cone = 'BLENDER'
            self.gt_box = 'BLENDER'
            self.gt_arc = 'BLENDER'
            
            self.offset_cone = 1.8
            self.offset_box = 1.2
            self.offset_arc = 1.56
            self.arc_angle = (180.0, 0.0, 180.0)

            self.size_dot = 1.0


        elif self.gizmo_preset == 'ASTRO':
            self.gt_cone = 'ASTRO'
            self.gt_box = 'ASTRO'
            self.gt_arc = 'ASTRO'
            
            self.offset_cone = 1.5
            self.offset_box = 1.0
            self.offset_arc = 1.0
            self.arc_angle = (135.0, 315.0, 135.0)

            self.size_dot = 1.0


        elif self.gizmo_preset == 'PEMBAL':
            self.gt_cone = 'PEMBAL'
            self.gt_box = 'PEMBAL'
            self.gt_arc = 'PEMBAL'
            
            self.offset_cone = 1.5
            self.offset_box = -1.5
            self.offset_arc = 1.0
            self.arc_angle = (0.0, 180.0, 0.0)

            self.size_dot = 1.0


        self.update_gizmo(context)


    def update_mat(self, context):
        var.REFRESH_MAT = True


# --- Переменные
    test: BoolProperty( name='Test Functions', description='To Enable The Test Functions', default=False )

    # --- Для обновления аддона
    check_updates: BoolProperty( name='Check Update When Starting The Blender', description='', default=False )
    update: BoolProperty( name='Update', description='', default=True )
    version: StringProperty( name='Version', description='', default='1.0.0' )
    
    # --- General
    use_with_standard_gizmos: BoolProperty( 
        name='Use With Standard Gizmos', 
        description='If disabled, the main gizmos will be hidden in the standard transformation tools or with Move, Rotate, Scale Gizmos enabled', 
        default=True,
        )



    gizmo_preset: EnumProperty(name='Preset', 
                            description = '', 
                            items = [('ZBRUSH', 'ZBrush', '', 'EVENT_Z', 0), 
                                    ('BLENDER', 'Blender', '', 'BLENDER', 1),
                                    ('ASTRO', 'Astro', '', 'WORLD', 2),
                                    ('PEMBAL', "Pembal'", '', 'GIZMO', 3),
                                    ], 
                            default = 'ASTRO',
                            update = preset,
                            )

    gt_cone: EnumProperty(name='Cone', 
                            description = '', 
                            items = [
                                        ('ZBRUSH', 'ZBrush', '', 'EVENT_Z', 0), 
                                        ('BLENDER', 'Blender', '', 'BLENDER', 1),
                                        ('ASTRO', 'Astro', '', 'WORLD', 2),
                                        ('PEMBAL', "Pembal'", '', 'GIZMO', 3),
                                        ('SPHERE', 'Sphere', '', 'SHADING_RENDERED', 4),
                                    ], 
                            default = 'ASTRO',
                            update = update_gizmo,
                            )

    gt_box: EnumProperty(name='Box', 
                            description = '', 
                            items = [
                                        ('ZBRUSH', 'ZBrush', '', 'EVENT_Z', 0), 
                                        ('BLENDER', 'Blender', '', 'BLENDER', 1),
                                        ('ASTRO', 'Astro', '', 'WORLD', 2),
                                        ('PEMBAL', "Pembal'", '', 'GIZMO', 3),
                                        ('SPHERE', 'Sphere', '', 'SHADING_RENDERED', 4),
                                    ], 
                            default = 'ASTRO',
                            update = update_gizmo,
                            )

    gt_arc: EnumProperty(name='Arc', 
                            description = '', 
                            items = [
                                        ('ZBRUSH', 'ZBrush', '', 'EVENT_Z', 0), 
                                        ('BLENDER', 'Blender', '', 'BLENDER', 1),
                                        ('ASTRO', 'Astro', '', 'WORLD', 2),
                                        ('PEMBAL', "Pembal'", '', 'GIZMO', 3),
                                        ('SPHERE', 'Sphere', '', 'SHADING_RENDERED', 4),
                                    ], 
                            default = 'ASTRO',
                            update = update_gizmo,
                            )


    offset_cone: FloatProperty(name='Offset', description = 'Changing The Offset Of Transform Gizmo', soft_min=-10.0, soft_max=10.0, default=1.5, subtype='FACTOR')
    offset_arc: FloatProperty(name='Offset', description = 'Changing The Offset Of Rotation Gizmo', soft_min=-10.0, soft_max=10.0, default=1.0, subtype='FACTOR')
    offset_box: FloatProperty(name='Offset', description = 'Changing The Offset Of Scale Gizmo', soft_min=-10.0, soft_max=10.0, default=1.0, subtype='FACTOR')
    offset_view: FloatProperty(name='Offset Rotate/Scale Of View Gizmos', description = 'Changing The Offset Of Rotate/Scale View Gizmos', soft_min=0.0, soft_max=10.0, default=1.2, subtype='FACTOR')
    offset_blank: FloatProperty( name = 'Offset', description = 'Changing The Offset Of Transform Two Axes Gizmo', soft_min = -20.0, soft_max = 20.0, default = 15.0, subtype = 'FACTOR' )

    size_gizmo_pro: FloatProperty(
        name='Size Gizmo',
        description='Changing The Size Of Gizmo',
        min=0.5,
        soft_max=3.0,
        default=0.8)
    size_cone: FloatProperty(name='Size', description = 'Changing The Size Of Cone Gizmo', min=0.1, soft_max=3.0, default=1.0)
    size_arc: FloatProperty(name='Size', description = 'Changing The Size Of Arc Gizmo', min=0.1, soft_max=3.0, default=1.0)
    size_box: FloatProperty(name='Size', description = 'Changing The Size Of Box Gizmo', min=0.1, soft_max=3.0, default=1.0)

    alpha_gizmo: FloatProperty(name='Gizmo Transparency', description = 'Changing The Transparency Of Gizmo', min=0.1, max=1.0, default=0.9, subtype='FACTOR')
    highlight_alpha_gizmo: FloatProperty(name='Gizmo Alpha Highlight', description = 'Changing The Highlight Transparency Of Gizmo', min=0.01, max=1.0, default=0.9, subtype='FACTOR')

    arc_angle: FloatVectorProperty( name = 'Rotation Angle Of The Gizmo Arc', description = '', subtype = 'XYZ', default = (135.0, 315.0, 135.0), min = 0.0, max = 360.0, update = update_gizmo)

    # --- line cone
    draw_line: BoolProperty(name='Draw Line For Cone', description='Add a line drawing for the cone', default=True)
    draw_line_on_hover: BoolProperty(name='On Hover', description='It will only be drawn when you hover', default=True)
    width_line: FloatProperty(name='Width Line', description='Changing The Size Of Dot Gizmos', min=0.1, soft_max=5.0, default=2.0, subtype='FACTOR')
    offset_line: FloatProperty(name='Offset Line', description='', soft_min=0.0, soft_max=5.0, default=0.3)

    
    


    size_dot: FloatProperty(name='Size Dot', description = 'Changing The Size Of Dot Gizmos', min=0.25, soft_max=5.0, default=1.0)
    size_trackball: FloatProperty(name='Size Trackball', description = 'Changing The Size Of Trackball Gizmos', min=0.01, soft_max=5.0, default=1.25)
    size_blank: FloatProperty( name = 'Size Move Plane', description = 'Changing The Size Of Trackball Gizmos', min = 0.01, soft_max = 1.0, default = 0.08)
    size_cursor: FloatProperty(name='Size 3D Cursor Gizmo', description = 'Changing The Size Of 3D Cursor Gizmo', soft_min=0.3, soft_max=1.0, default=0.6)
  

    lw_dot: FloatProperty(name='Line Width', description='Changing The Line Width Of Dot Gizmos', min=0.1, soft_max=3.0, default=1.0, subtype='FACTOR')
    lw_trackball: FloatProperty(name='Line Width', description='Changing The Line Width Of Trackball Gizmos', min=1.0, soft_max=7.0, default=2.0, subtype='FACTOR')

    


    # --- COLOR
    outline_thickness: FloatProperty( name='Outline Thickness', description = 'Changing The Outline Width', min=0.01, max=0.1, default=0.02, subtype='FACTOR')
    gizmo_thickness: FloatProperty( name='Thickness Of The Gizmo', description = '', min=-0.03, max=0.03, default=0.0, subtype='FACTOR')

    x_axis_color:  FloatVectorProperty( name='X Axis Color', description='', subtype='COLOR_GAMMA', default=(1.0, 0.0, 0.0), min=0.0, max=1.0 )
    y_axis_color:  FloatVectorProperty( name='Y Axis Color', description='', subtype='COLOR_GAMMA', default=(0.2, 1.0, 0.0), min=0.0, max=1.0 )
    z_axis_color:  FloatVectorProperty( name='Z Axis Color', description='', subtype='COLOR_GAMMA', default=(0.0, 0.35, 1.0), min=0.0, max=1.0 )
    elements_color:  FloatVectorProperty( name='Elements Color', description='Gizmos Dot, Trackball, Rotate View & Scale View Color', subtype='COLOR_GAMMA', default=(1.0, 1.0, 1.0), min=0.0, max=1.0)
    select_color:  FloatVectorProperty( name='Select Color', description='', subtype='COLOR_GAMMA', default=(0.2, 0.2, 0.2), min=0.0, max=1.0)
    primary_color:  FloatVectorProperty( name='Primary Gizmo', description='', subtype='COLOR_GAMMA', default=(1.0, 1.0, 0.0), min=0.0, max=1.0)
    secondary_color:  FloatVectorProperty(name='Secondary Gizmo', description='', subtype='COLOR_GAMMA', default=(1.0, 0.7, 0.0), min=0.0, max=1.0)
    blank_color: BoolProperty(name='Separate Move Plane Elements With Color', description='If enabled, there will be an average value between adjacent axes', default=True)

    use_shader: EnumProperty(
            name='Shader',
            description = '',
            items = [
                    ('STANDARD', 'Standard', '', 0),
                    ('OUTLINE', 'Outline', '', 1),
                    ('BASIC_LIGHT', 'Basic Light', '', 2),
                    ('DIAG_STRIPES', 'Diagonal Stripes', '', 3),
                    ],
            default = 'OUTLINE',
            update = update_gizmo
            )

    color_preset_type: EnumProperty(
            name = 'Color Preset',
            description = '',
            items = [
                ('BLENDER', 'Blender', 'Colors taken from the active blender theme'),
                ('GIZMOPRO', 'GizmoPRO', 'Standard colors for GizmoPRO'),
                ('MALIBU', 'Malibu', 'Softer colors, in a style all their own'),
                ('SATURATED', 'Saturated', 'Saturated colors'),
                ],
            default = 'GIZMOPRO',
            update = update_color_preset,
            )


    # TODO
    cursor_tool: EnumProperty(name='Cursor Tool', 
                            description = '', 
                            items = [
                                    ('DEFAULT', 'Default', '', 0),
                                    ('PAINT_BRUSH', 'Paint Brush', '', 1),
                                    ('PAINT_CROSS', 'Paint Cross', '', 2),
                                    ('DOT', 'Dot', '', 3),
                                    ('CROSSHAIR', 'Crosshair', '', 4),
                                    ],
                            default='CROSSHAIR',
                            update = update_tool,
                            )

    

    
    # --- Border Navigate
    bn_width: IntProperty( name = 'Border Width', description = 'Number Of Pixels', default = 50, min = 10, soft_max = 70 )
    bn_color:  FloatVectorProperty(name = 'Color', subtype = 'COLOR', default = (0.0, 0.5, 1.0), min = 0.0, max = 1.0 )
    bn_trans: FloatProperty( name = 'Transparency', description = 'The Transparency Of The Border', min = 0.01, max = 1.0, default = 0.2, subtype = 'FACTOR' )
    bn_lineWidth: FloatProperty( name = 'Line Width', min = 1.0, max = 10.0, default = 2.0, subtype = 'FACTOR' )
    
    bn_mini: BoolProperty(name='Minimal Design', description='If off, the boundary is drawn continuously', default=True)
    bn_type: EnumProperty(
                            name = 'Operator',
                            description = 'Navigation operator',  
                            items = [
                                    ('ORBIT', 'Orbit', 'Rotation in 3D View', '', 0),
                                    ('PAN', 'Pan', 'Panning in 3D View', '', 1),
                                    ('DOLLY', 'Dolly', 'Zooming in 3D View', '', 2),
                                    ],
                            default = 'ORBIT',
                            )


    






    # --- Select Tools
    sel_xray: BoolProperty( name = 'Show X-Ray', description = 'Enable X-Ray For Select Through', default = True )
  






    # --- Keymaps
    map_type: EnumProperty(
                            name = 'Tweak Gizmo With Mouse Button',
                            description = '',  
                            items = [
                                    ('LEFTMOUSE', 'Left Mouse', '', 0),
                                    ('RIGHTMOUSE', 'Right Mouse', '', 1), #('MIDDLEMOUSE', 'Middle Mouse', '', 1), 
                                    ], 
                            default = 'LEFTMOUSE',
                            update = update_keymaps,
                            )
    use_mmb_drag: BoolProperty( name='Use MMB Drag (Pivot Transform)', description='Use MMB Drag For Pivot Transform', default=False, update=update_keymaps )
    use_mmb_click: BoolProperty( name='Use MMB Click (Transform From Value)', description='Use MMB Click For Transform From Value', default=True, update=update_keymaps )
    use_pco_actions: BoolProperty(name='Use Pro Cursor Actions', description='If it is off: using Ctrl+MMB on Dot Gizmo will not activate the Pro Cursor function', default=True)


    # --- Space Drag
    space_drag_lmb: EnumProperty(
                            name='Space Drag Operation LMB',
                            description = '',
                            items = [
                                    ('NONE', 'None', ''),
                                    ('ORBIT', 'Orbit', 'Rotation in 3D View'),
                                    ('PAN', 'Pan', 'Panning in 3D View'),
                                    ('DOLLY', 'Dolly', 'Zooming in 3D View'),
                                    ('TWEAK', 'Tweak', 'Pressing selects an item that can be moved'),
                                    ('TRANSFORM', 'Modo Transform', 'MODO-type transformation (in development)'),
                                    ('TRANSLATE', 'Translate', 'Moving the selected'),
                                    ('ROTATE', 'Rotate', 'Rotation of the selected'),
                                    ('RESIZE', 'Resize', 'Changing the scale of the selected'),
                                    ('BOX_SEL', 'Select Box', 'Select items usicng box selection'),
                                    ('CIRCLE_SEL', 'Select Circle', 'Select items usicng circle selection'),
                                    ('LASSO_SEL', 'Select Lasso', 'Select items usicng lasso selection'),
                                    ('PAINT_SEL', 'Paint Select', ' ONLY EDIT MESH. Select items with paint selection method (Operator author - kaio)'),
                                    ],
                            default = 'ORBIT',
                            )


    space_drag_rmb: EnumProperty(
                            name='Space Drag Operation RMB',
                            description = '',  
                            items = [
                                    ('NONE', 'None', ''),
                                    ('ORBIT', 'Orbit', 'Rotation in 3D View'),
                                    ('PAN', 'Pan', 'Panning in 3D View'),
                                    ('DOLLY', 'Dolly', 'Zooming in 3D View'),
                                    ('TWEAK', 'Tweak', 'Pressing selects an item that can be moved'),
                                    ('TRANSFORM', 'Modo Transform', 'MODO-type transformation (in development)'),
                                    ('TRANSLATE', 'Translate', 'Moving the selected'),
                                    ('ROTATE', 'Rotate', 'Rotation of the selected'),
                                    ('RESIZE', 'Resize', 'Changing the scale of the selected'),
                                    ('BOX_SEL', 'Select Box', 'Select items usicng box selection'),
                                    ('CIRCLE_SEL', 'Select Circle', 'Select items usicng circle selection'),
                                    ('LASSO_SEL', 'Select Lasso', 'Select items usicng lasso selection'),
                                    ('PAINT_SEL', 'Paint Select', ' ONLY EDIT MESH. Select items with paint selection method (Operator author - kaio)'),
                                    ],
                            default = 'BOX_SEL',
                            )
    








    

    # --- Gizmo Visible
    hide_at_perp: BoolProperty( name='Perpendicular Hide', description='Hide Gizmo Elements If The Axis Is Perpendicular To The Camera', default=True )



    

    # --- Luna
    luna_size: FloatProperty(
        name='Size Gizmo',
        description='Changing The Size Of Gizmo',
        min=0.5,
        soft_max=3.0,
        default=0.9)
    luna_offset: FloatProperty(
        name='Offset Items',
        description='Distance between elements',
        min=0.0,
        soft_max=2.0,
        default=0.5)
    luna_lw: FloatProperty(name='Line Width', description='', min=0.0, soft_max=5.0, default=2.0)
    use_orientation_to_camera: BoolProperty(
        name='Orientation To The Camera',
        description='If enabled, the axes of the gizmo will be oriented in the direction of the camera location',
        default=False)

    # --- Malibu
    malibu_size: FloatProperty(
        name='Size Gizmo',
        description='Changing The Size Of Gizmo',
        min=0.5,
        soft_max=3.0,
        default=1.0)
    malibu_lw: FloatProperty(name='Line Width', description='', min=0.1, soft_max=5.0, default=4.0)
    malibu_arc_partial_angle: FloatProperty( name = 'Arc Partial Angle', description = 'Angle removed from the full arc', min = 0.0, max = 360, default = 330, subtype = 'FACTOR' )


    # --- Transpose
    size_gizmo_transpose: FloatProperty(
        name='Size Transpose',
        description='Changing The Size Of Gizmo',
        min=0.5,
        soft_max=3.0,
        default=0.9,
        subtype='FACTOR')

    # --- VR
    size_gizmo_vr: FloatProperty(
        name='Size Gizmo',
        description='Changing The Size Of Gizmo', 
        min=0.3,
        soft_max=2.0,
        default=0.6,
        subtype='FACTOR')
    offset_cone_vr: FloatProperty(
        name='Offset Cones',
        description='Offset of the Cone element from the edge of the bbox',
        min=0.0,
        soft_max=2.0,
        default=1.0)
    offset_dial_vr: FloatProperty(
        name='Offset Dials',
        description='Offset of the Dials element from the edge of the bbox',
        min=0.0,
        soft_max=3.0,
        default=0.5)
    color_negative_elements_vr: FloatVectorProperty(
        name='Color Negative Elements',
        subtype='COLOR',
        default=(0.0, 0.0, 0.0),
        min=0.0,
        max=1.0)



    # --- Placement
    size_gizmo_placement: FloatProperty(
        name='Size Placement Gizmo',
        description='Changing The Size Of Placement Gizmo',
        min=0.5,
        soft_max=3.0,
        default=0.6,
        subtype='FACTOR')
    

    # --- Mirror
    size_gizmo_mirror: FloatProperty(
        name='Size Mirror Gizmo',
        description='Changing The Size Of Mirror Gizmo',
        min=0.5,
        soft_max=3.0,
        default=0.6,
        subtype='FACTOR')

    # --- Parameter
    dot_action: EnumProperty(name='Dot Action',
                            description = '',  
                            items = [
                                    ('STANDARD', 'Standard', 'Normal movement in all axes', '', 0), 
                                    ('ADVANCE', 'Advance', 'Transform -> X, Y. Shift -> Z', '', 1),
                                    ],
                            default='STANDARD') # ('RETOP', 'Retopology', ' ', '', 2),
    

    # --- Quick Widget
    qw_scale: FloatProperty(name='Scale', description='', min=0.4, max=2.0, default=0.8)
    qw_offset_x: IntProperty(name='Offset Horizontal', description='', min=0, soft_max=50, default=15)
    qw_offset_y: IntProperty(name='Offset Vertical', description='', soft_min=-300, soft_max=300, default=220)
    qw_opacity: FloatProperty(name='Opacity', description='', min=0.0, max=1.0, default=0.75, subtype='FACTOR')
    qw_color_button: FloatVectorProperty(
        name='Color Button',
        description='',
        subtype='COLOR_GAMMA',
        default=(0.15, 0.34, 0.5),
        min=0.0,
        max=1.0)
    qw_color_active: FloatVectorProperty(
        name='Color Active',
        description='',
        subtype='COLOR_GAMMA',
        default=(0.0, 0.5, 1.0),
        min=0.0,
        max=1.0)
    
    def qw_items(self, context):
        items = [
            ('ORIENTATION', '', 'Transform Orientation', 'ORIENTATION_GLOBAL', 2), 
            ('POSITION', '', 'Transform Pivot Point', 'PIVOT_ACTIVE', 4),
            ('SNAPPING', '', 'Snapping', 'SNAP_ON', 2**3),
            ('PROP_EDIT', '', 'Proportional Editing', 'SMOOTHCURVE', 2**4), 
            #('VIEW_DISPLAY', '', 'Viewport Display', 'RESTRICT_VIEW_OFF', 2**5), 
            #('TRANSFORM', '', 'Transform Options', 'OPTIONS', 2**6), 
            ('CURSOR', '', '3D Cursor', 'PIVOT_CURSOR', 2**5),
            ]
        return items

    qw_buttons: EnumProperty(
        name = 'Buttons',
        items = qw_items,
        description = 'Snapping Options',
        options = {'ENUM_FLAG'},
        ) 
    qw_mini: BoolProperty(
        name='Minimal Design',
        description='Turns on the display as a normal point, but when you put the cursor over it, buttons appear',
        default=False)

    
    
    
    # --- Pro 3d Cursor
    #cursor_by_normal: BoolProperty(name='Align 3D Cursor By Normals', description='Alignment Of The 3D Cursor To The Normals Of The Selected', default=True)
    tdc_advance_align: BoolProperty(name='Advanced Alignment Method', description='This method uses a more advanced alignment algorithm, based on element normals, not only on direction', default=True)
    tdc_temporary: BoolProperty(name='Turn Off Pro 3D Cursor After Transform', description='If enabled, Pro 3D Cursor will turn off after the first transformation, and reset the gizmo position and orientation options to their original state', default=False)


    # --- MODIFIERS ---
    # --- CONE
    obCone_shift:  EnumProperty(
                                name = 'Modifiers Shift (Gizmo Move)', 
                                description = '', 
                                items = [
                                    ('NONE', 'None', '', 'CANCEL', 0),
                                    #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                    ('DUPLICATE', 'Duplicate', '', 'DUPLICATE', 2),
                                    ('DUPLICATE_LINKED', 'Duplicate Linked', '', 'DUPLICATE', 3),
                                    ],
                                default = 'DUPLICATE',
                                )

    obCone_ctrl:   EnumProperty(
                                name = 'Modifiers Ctrl (Gizmo Move)', 
                                description = '', 
                                items = [
                                    ('NONE', 'None', '', 'CANCEL', 0),
                                    #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                    ('DUPLICATE', 'Duplicate', '', 'DUPLICATE', 2),
                                    ('DUPLICATE_LINKED', 'Duplicate Linked', '', 'DUPLICATE', 3),
                                    ],
                                default = 'NONE',
                                )

    obCone_shift_cust: StringProperty(name='Operator', description='Custom Operator')
    obCone_ctrl_cust: StringProperty(name='Operator', description='Custom Operator')

    meCone_shift:  EnumProperty(
                                name = 'Modifiers Shift (Gizmo Move)', 
                                description = '', 
                                items = [
                                    ('NONE', 'None', '', 'CANCEL', 0),
                                    #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                    ('INSET', 'Inset', '', 'OBJECT_DATAMODE', 2),
                                    ('EXTRUDE', 'Extrude', '', 'ORIENTATION_NORMAL', 3),
                                    ('EXTRUDE_MANIFLOD', 'Extrude Manifold', '', 'ORIENTATION_NORMAL', 4),
                                    ('BEVEL', 'Bevel Fast', '', 'MOD_BEVEL', 5),
                                    ], 
                                default = 'EXTRUDE',
                                )

    meCone_ctrl:   EnumProperty(
                                name = 'Modifiers Ctrl (Gizmo Move)', 
                                description = '', 
                                items = [
                                    ('NONE', 'None', '', 'CANCEL', 0),
                                    #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                    ('INSET', 'Inset', '', 'OBJECT_DATAMODE', 2),
                                    ('EXTRUDE', 'Extrude', '', 'ORIENTATION_NORMAL', 3),
                                    ('EXTRUDE_MANIFLOD', 'Extrude Manifold', '', 'ORIENTATION_NORMAL', 4),
                                    ('BEVEL', 'Bevel Fast', '', 'MOD_BEVEL', 5),
                                    ],
                                default = 'INSET',
                                )

    # --- ARC
    obArc_shift: EnumProperty(
                            name = 'Modifiers Shift (Gizmo Rotate)', 
                            description = '', 
                            items = [
                                ('NONE', 'None', '', 'CANCEL', 0),
                                #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                ('DUPLICATE', 'Duplicate', '', 'DUPLICATE', 2),
                                ('MOVE_TWO', 'Moving Along Two Axes', '', 'ORIENTATION_VIEW', 3),
                                ('INCROT', 'Increment Rotate', '', 'DRIVER_ROTATIONAL_DIFFERENCE', 4),
                                ],
                            default = 'DUPLICATE',
                            )

    obArc_ctrl: EnumProperty(
                            name = 'Modifiers Ctrl (Gizmo Rotate)', 
                            description = '',
                            items = [
                                ('NONE', 'None', '', 'CANCEL', 0),
                                #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                ('DUPLICATE', 'Duplicate', '', 'DUPLICATE', 2),
                                ('MOVE_TWO', 'Moving Along Two Axes', '', 'ORIENTATION_VIEW', 3),
                                ('INCROT', 'Increment Rotate', '', 'DRIVER_ROTATIONAL_DIFFERENCE', 4),
                                ],
                            default = 'INCROT',
                            )

    #ia: IntProperty(name='Incremental Angle', description='Rotation angle when snapping. (Increment Rotate modifier operator)', default=45, subtype='ANGLE' )
    meArc_shift: EnumProperty(
                            name = 'Modifiers Shift (Gizmo Rotate)', 
                            items = [
                                ('NONE', 'None', '', 'CANCEL', 0),
                                #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                ('DUPLICATE', 'Duplicate', '', 'DUPLICATE', 2),
                                ('MOVE_TWO', 'Moving Along Two Axes', '', 'ORIENTATION_VIEW', 3),
                                ('BEVEL', 'Bevel Fast', '', 'MOD_BEVEL', 4),
                                ('INCROT', 'Increment Rotate', '', 'DRIVER_ROTATIONAL_DIFFERENCE', 5),
                                ],
                            default = 'DUPLICATE',
                            )

    meArc_ctrl: EnumProperty(
                            name = 'Modifiers Ctrl (Gizmo Rotate)', 
                            items = [
                                ('NONE', 'None', '', 'CANCEL', 0),
                                #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                ('DUPLICATE', 'Duplicate', '', 'DUPLICATE', 2),
                                ('MOVE_TWO', 'Moving Along Two Axes', '', 'ORIENTATION_VIEW', 3),
                                ('BEVEL', 'Bevel Fast', '', 'MOD_BEVEL', 4),
                                ('INCROT', 'Increment Rotate', '', 'DRIVER_ROTATIONAL_DIFFERENCE', 5),
                                ],
                            default = 'INCROT',
                            )

    # --- BOX
    obBox_shift:   EnumProperty(
                                name = 'Modifiers Shift (Gizmo Scale)', 
                                description = '', 
                                items = [
                                    ('NONE', 'None', '', 'CANCEL', 0),
                                    #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                    ('SCALE_TWO', 'Scale Two Axis', '', 'ORIENTATION_VIEW', 2),
                                    ('ALIGN', 'Align', '', 'TRIA_DOWN_BAR', 3),
                                    ],
                                default = 'SCALE_TWO',
                                )

    obBox_ctrl:    EnumProperty(
                                name = 'Modifiers Ctrl (Gizmo Scale)', 
                                description = '', 
                                items = [
                                    ('NONE', 'None', '', 'CANCEL', 0),
                                    #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                    ('SCALE_TWO', 'Scale Two Axis', '', 'ORIENTATION_VIEW', 2),
                                    ('ALIGN', 'Align', '', 'TRIA_DOWN_BAR', 3),
                                    ],
                                default = 'ALIGN',
                                )

    meBox_shift:   EnumProperty(
                                name = 'Modifiers Shift (Gizmo Scale)', 
                                description = '', 
                                items = [
                                    ('NONE', 'None', '', 'CANCEL', 0),
                                    #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                    ('SCALE_TWO', 'Scale Two Axis', '', 'ORIENTATION_VIEW', 2),
                                    ('ALIGN', 'Align', '', 'TRIA_DOWN_BAR', 3),
                                    ('BEVEL', 'Bevel Fast', '', 'MOD_BEVEL', 4),
                                    ],
                                default = 'SCALE_TWO',
                                )

    meBox_ctrl:    EnumProperty(
                                name = 'Modifiers Ctrl (Gizmo Scale)', 
                                description = '', 
                                items = [
                                    ('NONE', 'None', '', 'CANCEL', 0),
                                    #('CUST', 'Custom', '', 'TOOL_SETTINGS', 1),
                                    ('SCALE_TWO', 'Scale Two Axis', '', 'ORIENTATION_VIEW', 2),
                                    ('ALIGN', 'Align', '', 'TRIA_DOWN_BAR', 3),
                                    ('BEVEL', 'Bevel Fast', '', 'MOD_BEVEL', 4),
                                    ],
                                default = 'ALIGN',
                                )

    # --- UI
    pie_advanced_options: BoolProperty(
        name='Advanced Options', 
        description='Enables dropdown menu visibility in the Pie Menu with advanced settings for a number of options\n'
                    '• Proportional Editing\n'
                    '• Snap\n'
                    '• Transform Pivot Point\n'
                    '• Transform Orientations',
        default=True,
    )

    # --- Panel Preferences
    tab: EnumProperty(
        name = 'Tabs',
        description = '',
        items = [
            ('GENERAL', 'General', '', '', 0),
            ('GIZMOS', 'Gizmos', '', '', 1),
            ('QW', 'Quick Widget', '', '', 2),
            ('MODIFIERS', 'Modifiers', '', '', 3),
            ('KEYMAPS', 'Keymaps', '', '', 4),
            ],
            default='GIZMOS',
            )

    type_key: EnumProperty(
        name='Key',
        description = '',
        items = [
            ('SHIFT', 'Shift', '', '', 0),
            ('CTRL', 'Ctrl', '', '', 1),
            ], 
            default='SHIFT',
            )

    type_gizmo: EnumProperty(
        name='Key',
        description = '',
        items = [
            ('GENERAL', 'General', '', '', 0),
            ('GIZMO_PRO', 'Gizmo PRO', '', '', 1),
            ('LUNA', 'Luna', '', '', 2),
            ('MALIBU', 'Malibu', '', '', 3),
            ('TRANSPOSE', 'Transpose', '', '', 4),
            ('VR', 'VR', '', '', 5),
            ('OTHERS', 'Others', '', '', 6),
            ],
            default='GENERAL',
            )
    
    type_mode: EnumProperty(
        name='Context Mode',
        description = '',
        items = [
            ('OBJECT', 'Object', 'Object Mode'),
            ('MESH', 'Mesh', 'Edit Mesh'),
            ('CURVE', 'Curve', 'Edit Curve'),
            ('SURFACE', 'Surface', 'Edit Surface'),
            ('ARMATURE', 'Armature', 'Edit Armature'),
            ('POSE', 'Pose', 'Pose Mode'),
            ],
        default='OBJECT',
    )

    def draw(self, context):
        icoll = icons_collections['main']
        market_icon = icoll['market_icon']
        gumroad_icon = icoll['gumroad_icon']
        artstation_icon = icoll['artstation_icon']
        discord_icon = icoll['discord_icon']
        update_icon = icoll['update_icon']


        self.layout.row().prop(self, 'tab', expand=True)
        box_main = self.layout.box()

        

        # --- General
        if self.tab == 'GENERAL':
            col = box_main.column()
            
            # --- UI
            col.label(text='UI')
            col.prop(self, 'pie_advanced_options')

            # --- Pro 3D Cursor
            col.separator()
            col.label(text='Pro 3D Cursor')
            col.prop(self, 'tdc_advance_align')
            col.prop(self, 'tdc_temporary')



            # --- Update & Maintenance
            col.separator()
            col.label(text='Update & Maintenance')
            row = col.row()
            row.prop(self, 'check_updates')
            row.operator('gp.check_updates', text='Check For Updates')
            if self.update:
                box_main.label(text='New Version Is Available ' + str(self.version) +'!!!', icon_value=update_icon.icon_id)
    
            col.separator(factor=0.5)

            row = col.row()
            row.operator('gp.save_preferences', text='Save Preferences To File', icon='EXPORT').type = 'SAVE'
            row.operator('gp.save_preferences', text='Load Preferences From File', icon='IMPORT').type = 'LOAD'

            col.prop(self, 'test')

            # --- LINKS
            col.separator()
            col = box_main.column()
            col.label(text='Links')
            col = box_main.column()
            row = col.row(align=True)
            row.operator('wm.url_open', text='Blender Market', icon_value=market_icon.icon_id).url = 'https://blendermarket.com/creators/derksen'
            row.separator(factor=0.3)
            row.operator('wm.url_open', text='Gumroad', icon_value=gumroad_icon.icon_id).url = 'https://derksen.gumroad.com'
            row.separator(factor=0.3)
            row.operator('wm.url_open', text='Artstation', icon_value=artstation_icon.icon_id).url = 'https://www.artstation.com/derksen'

        
        elif self.tab == 'GIZMOS': # --- General Gizmo
            row = box_main.row()
            row.prop(self, 'type_gizmo', expand=True)
            #gizmo_settings(self, context, box, self.type_gizmo)
            
            box_main.use_property_split = True
            
            # GIZMO
            col = box_main.column()
            col.scale_y = 1.0

            if self.type_gizmo == 'GENERAL':
                col.prop(self, 'use_with_standard_gizmos')
                col.prop(self, 'hide_at_perp')
                col.separator()
                col.prop(self, 'alpha_gizmo')
                col.prop(self, 'highlight_alpha_gizmo')
                col.separator()
                col.prop(self, 'x_axis_color', text='X')
                col.prop(self, 'y_axis_color', text='Y')
                col.prop(self, 'z_axis_color', text='Z')
                col.separator()
                col.prop(self, 'elements_color')
                col.prop(self, 'select_color')
                col.prop(self, 'primary_color')
                col.prop(self, 'secondary_color')
                col.prop(self, 'color_preset_type')
                col.prop(self, 'blank_color')
                col.separator()
                col.prop(self, 'dot_action')
                
            elif self.type_gizmo == 'GIZMO_PRO':
                row = col.row(align=True)
                row.prop(self, 'gizmo_preset', expand=True)
                
                col.separator()
                col.prop(self, 'size_gizmo_pro')
                col.prop(self, 'gizmo_thickness')
                col.separator()
                col.prop(self, 'draw_line')
                if self.draw_line:
                    col.prop(self, 'draw_line_on_hover')
                    col.prop(self, 'width_line')
                    col.prop(self, 'offset_line')
                col.separator()
                
                row = col.row()
                row.prop(self, 'gt_cone', expand=False)
                row.prop(self, 'size_cone')
                row.prop(self, 'offset_cone')
                
                row = col.row()
                row.prop(self, 'gt_box', expand=False)
                row.prop(self, 'size_box')
                row.prop(self, 'offset_box')
                
                row = col.row()
                row.prop(self, 'gt_arc', expand=False)
                row.prop(self, 'size_arc')
                row.prop(self, 'offset_arc')
                

                col.prop(self, 'arc_angle')

                # --- 
                col.prop(self, 'offset_view')

                col.separator()
                row = col.row()
                row.prop(self, 'size_dot')
                row.prop(self, 'lw_dot')
                
                row = col.row()
                row.prop(self, 'size_trackball')
                row.prop(self, 'lw_trackball')

                row = col.row()
                row.prop(self, 'size_blank')
                row.prop(self, 'offset_blank')

                col.separator()
                row = col.row()
                row.prop(self, 'use_shader')
                if self.use_shader == 'OUTLINE':
                    row.prop(self, 'outline_thickness')

            elif self.type_gizmo == 'LUNA':
                col.prop(self, 'luna_size')
                col.prop(self, 'luna_lw')
                col.prop(self, 'luna_offset')
                col.separator()
                col.prop(self, 'use_orientation_to_camera')
            
            elif self.type_gizmo == 'MALIBU':
                col.prop(self, 'malibu_size')
                col.prop(self, 'malibu_lw')
            
            elif self.type_gizmo == 'TRANSPOSE':
                col.prop(self, 'size_gizmo_transpose')

            elif self.type_gizmo == 'VR':
                col.prop(self, 'size_gizmo_vr')
                col.prop(self, 'offset_cone_vr')
                col.prop(self, 'offset_dial_vr')
                col.prop(self, 'color_negative_elements_vr')

            elif self.type_gizmo == 'OTHERS':
                col.prop(self, 'size_cursor')
                col.prop(self, 'size_gizmo_placement')
                col.prop(self, 'size_gizmo_mirror')

                col.separator()
                col.label(text='Border Navigate:')
                col.prop(self, 'bn_mini')
                col.prop(self, 'bn_width')
                col.prop(self, 'bn_lineWidth')
                col.prop(self, 'bn_type')
                col.prop(self, 'bn_color')
                col.prop(self, 'bn_trans')


        elif self.tab == 'QW': # --- Quick Widget
            box_main.use_property_split = True
            
            row = box_main.row()
            #row.alignment = 'CENTER'
            #row.scale_x = 2.0
            row.prop(self, 'qw_buttons', expand=True) # icon_only=True
            
            box_main.prop(self, 'qw_scale')
            box_main.prop(self, 'qw_offset_x')
            box_main.prop(self, 'qw_offset_y')
            box_main.separator()
            box_main.prop(self, 'qw_opacity')
            box_main.prop(self, 'qw_mini')
            #box_main.prop(context.scene.GP_scene_set, 'qw_alignment')
            #box_main.prop(self, 'qw_color_button')
            #box_main.prop(self, 'qw_color_active')


        elif self.tab == 'MODIFIERS': # --- Modifiers
            settings = context.scene.GP_scene_set
            row = box_main.row()
            row.prop(self, 'type_mode', expand=True)
            box_main.use_property_split = True
            



            


            if self.type_mode == 'OBJECT':
                row = box_main.row()
                row.prop(self, 'type_key', expand=True)

                box_cone = box_main.box()
                col_cone = box_cone.column()
                col_cone.label(text='Cone Gizmo', icon='CONE')

                box_arc = box_main.box()
                col_arc = box_arc.column()
                col_arc.label(text='Arc Gizmo', icon='SPHERE')

                box_box = box_main.box()
                col_box = box_box.column()
                col_box.label(text='Box Gizmo', icon='CUBE')



                row = col_cone.row()
                if self.type_key == 'SHIFT':
                    row.prop(self, 'obCone_shift', text='Shift')
                elif self.type_key == 'CTRL':
                    row.prop(self, 'obCone_ctrl', text='Ctrl')

                row = col_arc.row()
                if self.type_key == 'SHIFT':
                    row.prop(self, 'obArc_shift', text='Shift')
                    if self.obArc_shift == 'INCROT' or self.meArc_shift == 'INCROT':
                        col_arc.separator()
                        col_arc.prop(settings, 'ia')
                elif self.type_key == 'CTRL':
                    row.prop(self, 'obArc_ctrl', text='Ctrl')
                    if self.obArc_ctrl == 'INCROT' or self.meArc_ctrl == 'INCROT':
                        col_arc.separator()
                        col_arc.prop(settings, 'ia')
                    
                row = col_box.row()
                if self.type_key == 'SHIFT':
                    row.prop(self, 'obBox_shift', text='Shift')
                elif self.type_key == 'CTRL':
                    row.prop(self, 'obBox_ctrl', text='Ctrl')

            elif self.type_mode == 'MESH':
                row = box_main.row()
                row.prop(self, 'type_key', expand=True)

                box_cone = box_main.box()
                col_cone = box_cone.column()
                col_cone.label(text='Cone Gizmo', icon='CONE')

                box_arc = box_main.box()
                col_arc = box_arc.column()
                col_arc.label(text='Arc Gizmo', icon='SPHERE')

                box_box = box_main.box()
                col_box = box_box.column()
                col_box.label(text='Box Gizmo', icon='CUBE')



                row = col_cone.row()
                if self.type_key == 'SHIFT':
                    row.prop(self, 'meCone_shift', text='Shift')
                elif self.type_key == 'CTRL':
                    row.prop(self, 'meCone_ctrl', text='Ctrl')

                row = col_arc.row()
                if self.type_key == 'SHIFT':
                    row.prop(self, 'meArc_shift', text='Shift')
                    if self.obArc_shift == 'INCROT' or self.meArc_shift == 'INCROT':
                        col_arc.separator()
                        col_arc.prop(settings, 'ia')
                elif self.type_key == 'CTRL':
                    row.prop(self, 'meArc_ctrl', text='Ctrl')
                    if self.obArc_ctrl == 'INCROT' or self.meArc_ctrl == 'INCROT':
                        col_arc.separator()
                        col_arc.prop(settings, 'ia')
                    
                row = col_box.row()
                if self.type_key == 'SHIFT':
                    row.prop(self, 'meBox_shift', text='Shift')
                elif self.type_key == 'CTRL':
                    row.prop(self, 'meBox_ctrl', text='Ctrl')



            elif self.type_mode in {'CURVE', 'SURFACE', 'ARMATURE', 'POSE'}:
                col = box_main.column()
                col.label(text="This is in development.")
                col.label(text="You can write to me on blendermarket")
                col.label(text="with suggestions for implementing necessary operators.")
                

            # --- Preset
            row = box_main.row()
            row.operator('gp.preset_modifiers', text='Turn Off All').type = 'NONE'
            row.operator('gp.preset_modifiers', text='Default').type = 'DEFAULT'
        

        elif self.tab == 'KEYMAPS': # --- Keymaps
            box_main.use_property_split = True

            row = box_main.row()
            row.prop(self, 'map_type', expand=True)
            box_main.prop(self, 'use_mmb_drag')
            box_main.prop(self, 'use_mmb_click')
            box_main.prop(self, 'use_pco_actions')

            row = box_main.row()
            row.prop(self, 'space_drag_lmb')
            row = box_main.row()
            row.prop(self, 'space_drag_rmb')


            wm = context.window_manager
            kc = wm.keyconfigs.user
            km = kc.keymaps['3D View']
            box = box_main.box()
            row = box.row()
            row.label(text='Pie Menu:')
            kmi = get_hotkey_entry_item(km, 'wm.call_menu_pie', 'GP_MT_pie', 'name')
            if kmi:
                row.context_pointer_set('keymap', km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, row, 0)
            else:
                row.label(text='No hotkey entry found')


            #box_main.separator()

            """ box = box_main.box()
            box.label(text='Tool Keymaps:')
            km = kc.keymaps['3D View Tool: Gizmo PRO']
            box.label(text='Switch Gizmo:')

            row = box.row()
            row.label(text='Move:')
            kmi = get_hotkey_entry_item(km, 'gp.switch_gizmo', 'TRANSLATE', 'type')
            if kmi:
                row.context_pointer_set('keymap', km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, row, 0)
            else:
                row.label(text='No hotkey entry found')
            
            row = box.row()
            row.label(text='Rotate:')
            kmi = get_hotkey_entry_item(km, 'gp.switch_gizmo', 'ROTATE', 'type')
            if kmi:
                row.context_pointer_set('keymap', km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, row, 0)
            else:
                row.label(text='No hotkey entry found')
            
            row = box.row()
            row.label(text='Scale:')
            kmi = get_hotkey_entry_item(km, 'gp.switch_gizmo', 'SCALE', 'type')
            if kmi:
                row.context_pointer_set('keymap', km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, row, 0)
            else:
                row.label(text='No hotkey entry found')


            box.label(text='Transform Operators:')
            row = box.row()
            row.label(text='Move:')
            kmi = get_hotkey_entry_item(km, 'gp.transform', 'TRANSLATION', 'operator')
            if kmi:
                row.context_pointer_set('keymap', km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, row, 0)
            else:
                row.label(text='No hotkey entry found')

            row = box.row()
            row.label(text='Rotate:')
            kmi = get_hotkey_entry_item(km, 'gp.transform', 'ROTATION', 'operator')
            if kmi:
                row.context_pointer_set('keymap', km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, row, 0)
            else:
                row.label(text='No hotkey entry found')

            row = box.row()
            row.label(text='Scale:')
            kmi = get_hotkey_entry_item(km, 'gp.transform', 'RESIZE', 'operator')
            if kmi:
                row.context_pointer_set('keymap', km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, row, 0)
            else:
                row.label(text='No hotkey entry found') """


            box_main.separator()
            box_main.label(text='*some hotkeys may not work because of the use of other addons')













classes = [
    GP_settings,
    GP_collections,
    GP_preferences,
]

def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    
    bpy.types.Scene.GP_scene_set = PointerProperty(type=GP_settings)
    bpy.types.Object.GP_scene_Col = PointerProperty(type=GP_collections)

    # --- For QW Buttons Items
    from . import get_prefs
    props = get_prefs()
    if len(props.qw_buttons) == 0:
        props.qw_buttons = {'ORIENTATION', 'POSITION', 'SNAPPING'} # , 'PROP_EDIT'
    

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)