from operator import index
import bpy, blf, gpu
from bpy.types import Gizmo
from ..utils.draw_utils.draw import (
    draw_circle_2d,
    draw_batch,
    get_rounded_rectangle,
    get_circle,
    capsule_2d,
    draw_texture_2d,
    draw_batch,
)
from math import sqrt, radians, atan2, degrees, cos, sin, pi
from bl_math import smoothstep, clamp, lerp
from ..icons import gl_textures
from gpu import state
from . import qw_widget
from mathutils import Matrix, Vector
from .. import var
from ..utils.shaders import GPU_SHADER_DIAG_STRIPES_UNIFORM_COLOR, GPU_SHADER_2D_POINT_UNIFORM_SIZE_OUTLINE
from ..utils.func import viewport_points
from .. import get_prefs



O_RADIUS = 1.0
I_RADIUS = 0.6






# --- Основной контейнер
class GP_GT_qw_frame(Gizmo):
    bl_idname = 'GP_GT_qw_frame'


    __slots__ = (
        'props',
        'settings',
        'dpi',
        'segments',
        'init_offset',

        'shape_circle',
        'shape_box_horizontal',
        'shape_box_verical',
    )


    def setup(self):
        self.use_grab_cursor = False
        self.use_select_background = False
        self.use_draw_modal = True
        self.use_tooltip = False
        #self.use_draw_hover = True
        

        self.color = (0.176, 0.204, 0.212)
        self.color_highlight = (0.423, 0.361, 0.906)
        self.alpha = 0.5
        self.alpha_highlight = 0.99

        self.segments = 70
        
        # --- For Drag Option
        self.init_offset = (0.0, 0.0)

        
        if not hasattr(self, 'shape_circle'):
            circle = get_circle((0,0), 1.05, segments=self.segments)
            self.shape_circle = self.new_custom_shape('LINE_STRIP', circle)

        if not hasattr(self, 'shape_box_horizontal'):
            box = get_rounded_rectangle((0.0, 0.0), 4.6, 0.5, 0.25)
            self.shape_box_horizontal = self.new_custom_shape('LINE_STRIP', box)

        if not hasattr(self, 'shape_box_verical'):
            box = get_rounded_rectangle((0.0, 0.0), 0.6, 3.7, 0.25)
            self.shape_box_verical = self.new_custom_shape('LINE_STRIP', box)


    def test_select(self, context, location):
        props = get_prefs()
        settings = bpy.context.scene.GP_scene_set

        dpi = context.preferences.system.ui_scale

        pos = self.matrix_basis.translation
        offset = self.scale_basis * dpi * 1.05

        if settings.qw_alignment == 'CIRCLE':
            sqx = (location[0] - pos[0])**2
            sqy = (location[1] - pos[1])**2

            if sqrt(sqx + sqy) > offset:
                """ if props.qw_pin:
                    if var.QW_USED_OP: # --- переместить QW в изначальную позицию
                        settings.qw_alignment = var.QW_ALIGN_STORE
                        settings.qw_pos = var.QW_STORE_POS
                        var.QW_USED_OP = False
                        var.QW_STORE_POS = Vector()
                        
                    return -1
                else:
                    var.QW_USED_OP = False
                    var.QW_STORE_POS = Vector()
                    settings.qw = False """
                return -1
            else:
                return 0

        elif settings.qw_alignment == 'HORIZONTAL':
            sqx = offset * 2.2
            sqy = offset * 0.26
            
            x = abs(pos[0] - location[0])
            y = abs(pos[1] - location[1])
            
            if x < sqx and y < sqy:
                return 0
            else:
                """ if props.qw_pin:
                    if var.QW_USED_OP: # --- переместить QW в изначальную позицию
                        settings.qw_alignment = var.QW_ALIGN_STORE
                        settings.qw_pos = var.QW_STORE_POS
                        var.QW_USED_OP = False
                        var.QW_STORE_POS = Vector()
                    
                    return -1
                else:
                    var.QW_USED_OP = False
                    var.QW_STORE_POS = Vector()
                    settings.qw = False """
                return -1

        elif settings.qw_alignment == 'VERTICAL':
            sqx = offset * 0.3
            sqy = offset * 1.78
            
            x = abs(pos[0] - location[0])
            y = abs(pos[1] - location[1])
            
            if x < sqx and y < sqy:
                return 0
            else:
                """ if props.qw_pin:
                    if var.QW_USED_OP: # --- переместить QW в изначальную позицию
                        settings.qw_alignment = var.QW_ALIGN_STORE
                        settings.qw_pos = var.QW_STORE_POS
                        var.QW_USED_OP = False
                        var.QW_STORE_POS = Vector()
                else:
                    var.QW_USED_OP = False
                    var.QW_STORE_POS = Vector()
                    settings.qw = False """
                return -1

        return 0


    def draw(self, context):
        props = get_prefs()
        settings = bpy.context.scene.GP_scene_set

        self.color_highlight = props.qw_color_active
        dpi = context.preferences.system.ui_scale


        gpu.state.blend_set('ALPHA')
        state.line_width_set( 2 * props.qw_scale )
        gpu.state.point_size_set( 15 * dpi )

        if self.is_modal:
            wl, wr, hb, ht, wc, hc = viewport_points(context)
            o = 30.0 * dpi # --- offset
            
            points = [
                (wc, ht-o),
                (wc, hb+o),
                (wl+o, hc),
                (wr+7-o, hc),
                (wc, hc),
                ]

            color = (0.2, 0.2, 0.2, 0.9)
            shader = GPU_SHADER_2D_POINT_UNIFORM_SIZE_OUTLINE()
            shader.bind()
            #shader.uniform_float("color", (0.2, 0.2, 0.2, 0.9) )
            shader.uniform_float("outlineColor", (0.05, 0.05, 0.05, 0.9) )
            shader.uniform_float("size", 500.0  )
            shader.uniform_float("outlineWidth", 80.0  )
            
            draw_batch( points, color, 'POINTS', custom_shader = shader )




        if settings.qw_alignment == 'CIRCLE':
            self.draw_custom_shape(self.shape_circle)
        
        elif settings.qw_alignment == 'HORIZONTAL':
            self.draw_custom_shape(self.shape_box_horizontal)

        elif settings.qw_alignment == 'VERTICAL':
            self.draw_custom_shape(self.shape_box_verical)



        

        gpu.state.point_size_set( 1 )
        state.line_width_set( 1 )
        gpu.state.blend_set('NONE')


    def modal(self, context, event, tweak):
        props = get_prefs()
        settings = context.scene.GP_scene_set
        if context.area:
            context.area.tag_redraw()
        mouse_x = event.mouse_region_x
        mouse_y = event.mouse_region_y

        



        wl, wr, hb, ht, wc, hc = viewport_points(context)
        dpi = context.preferences.system.ui_scale
        o = 30.0 * dpi # --- offset
        point_size = 15 * dpi

        bar_top_x = (wc - mouse_x)**2
        bar_top_y = (ht-o - mouse_y)**2

        bar_bot_x = (wc - mouse_x)**2
        bar_bot_y = (hb+o - mouse_y)**2

        bar_left_x = (wl+o - mouse_x)**2
        bar_left_y = (hc - mouse_y)**2

        bar_right_x = (wr+7-o - mouse_x)**2
        bar_right_y = (hc - mouse_y)**2

        bar_center_x = (wc - mouse_x)**2
        bar_center_y = (hc - mouse_y)**2

        if sqrt(bar_top_x + bar_top_y) < point_size:
            settings.qw_alignment = 'HORIZONTAL'
            settings.qw_pos = (wc, ht-o)
            self.init_offset = (settings.qw_pos[0] - mouse_x, settings.qw_pos[1] - mouse_y)

        elif sqrt(bar_bot_x + bar_bot_y) < point_size:
            settings.qw_alignment = 'HORIZONTAL'
            settings.qw_pos = (wc, hb+o)
            self.init_offset = (settings.qw_pos[0] - mouse_x, settings.qw_pos[1] - mouse_y)
        
        elif sqrt(bar_left_x + bar_left_y) < point_size:
            settings.qw_alignment = 'VERTICAL'
            settings.qw_pos = (wl+o, hc)
            self.init_offset = (settings.qw_pos[0] - mouse_x, settings.qw_pos[1] - mouse_y)
        
        elif sqrt(bar_right_x + bar_right_y) < point_size:
            settings.qw_alignment = 'VERTICAL'
            settings.qw_pos = (wr-o, hc)
            self.init_offset = (settings.qw_pos[0] - mouse_x, settings.qw_pos[1] - mouse_y)
        
        elif sqrt(bar_center_x + bar_center_y) < point_size:
            settings.qw_alignment = 'CIRCLE'
            settings.qw_pos = (wc, hc)
            self.init_offset = (settings.qw_pos[0] - mouse_x, settings.qw_pos[1] - mouse_y)

        else:
            settings.qw_pos = (mouse_x + self.init_offset[0], mouse_y + self.init_offset[1])
        return {'RUNNING_MODAL'}


    def invoke(self, context, event):
        if event.value == 'CLICK':
            return {'CANCELLED'}
        else:
            settings = context.scene.GP_scene_set
            init_x = event.mouse_region_x
            init_y = event.mouse_region_y
            self.init_offset = (settings.qw_pos[0] - init_x, settings.qw_pos[1] - init_y)
            return {'RUNNING_MODAL'}
    





# --- Основной контейнер
class GP_GT_qw_draw_layout(Gizmo):
    bl_idname = 'GP_GT_qw_draw_layout'


    __slots__ = (
        'init_offset',
        'shape_capsule_verical',
    )


    def setup(self):
        self.use_grab_cursor = False
        #self.use_select_background = False
        self.use_draw_modal = True
        self.use_tooltip = False
        #self.use_draw_hover = True
        

        self.color = (0.1, 0.1, 0.1)
        self.color_highlight = (0.2, 0.2, 0.2)
        self.alpha = 0.9
        self.alpha_highlight = 0.9

        
        
        # --- For Drag Option
        self.init_offset = (0.0, 0.0)

        
        

        

        if not hasattr(self, 'shape_capsule_verical'):
            capsule = capsule_2d( size=2.3, radius=1.3, segments=24, align='VERTICAL' )
            self.shape_capsule_verical = self.new_custom_shape( 'TRI_FAN', capsule )


    def test_select(self, context, location):
        props = get_prefs()
        settings = context.scene.GP_scene_set

        dpi = context.preferences.system.ui_scale

        pos = self.matrix_basis.translation
        offset = self.scale_basis * dpi #* 1.0

        
        sqx = offset * 1.3
        sqy = offset * 2.3
        
        x = abs(pos[0] - location[0])
        y = abs(pos[1] - location[1])
        
        if x < sqx and y < sqy:
            return 0
        else:
            """ if props.qw_pin:
                if var.QW_USED_OP: # --- переместить QW в изначальную позицию
                    settings.qw_alignment = var.QW_ALIGN_STORE
                    settings.qw_pos = var.QW_STORE_POS
                    var.QW_USED_OP = False
                    var.QW_STORE_POS = Vector()
                
                return -1
            else: """
            var.QW_USED_OP = False
            var.QW_STORE_POS = Vector()
            settings.qw = False
            return -1



    def draw(self, context):
        props = get_prefs()
        #self.color_highlight = props.qw_color_active
        dpi = context.preferences.system.ui_scale


        gpu.state.blend_set('ALPHA')
        state.line_width_set( 2 * props.qw_scale )
        #gpu.state.point_size_set( 15 * dpi )

        """ if self.is_modal:
            wl, wr, hb, ht, wc, hc = align_points(context)
            o = 30.0 * dpi # --- offset
            
            points = [
                (wc, ht-o),
                (wc, hb+o),
                (wl+o, hc),
                (wr+7-o, hc),
                (wc, hc),
                ]

            color = (0.2, 0.2, 0.2, 0.9)
            shader = GPU_SHADER_2D_POINT_UNIFORM_SIZE_OUTLINE()
            shader.bind()
            #shader.uniform_float("color", (0.2, 0.2, 0.2, 0.9) )
            shader.uniform_float("outlineColor", (0.05, 0.05, 0.05, 0.9) )
            shader.uniform_float("size", 500.0  )
            shader.uniform_float("outlineWidth", 80.0  )
            
            draw_batch( points, color, 'POINTS', custom_shader = shader ) """




        
        self.draw_custom_shape(self.shape_capsule_verical)



        

        #gpu.state.point_size_set( 1 )
        state.line_width_set( 1 )
        gpu.state.blend_set('NONE')


    def modal(self, context, event, tweak):
        props = get_prefs()
        settings = context.scene.GP_scene_set

        if context.area:
            context.area.tag_redraw()
        mouse_x = event.mouse_region_x
        mouse_y = event.mouse_region_y


        

        settings.qw_draw_pos = (mouse_x + self.init_offset[0], mouse_y + self.init_offset[1])
        return {'RUNNING_MODAL'}


    def invoke(self, context, event):
        if event.value == 'CLICK':
            return {'CANCELLED'}
        else:
            settings = context.scene.GP_scene_set
            init_x = event.mouse_region_x
            init_y = event.mouse_region_y
            self.init_offset = (settings.qw_draw_pos[0] - init_x, settings.qw_draw_pos[1] - init_y)
            return {'RUNNING_MODAL'}
        

# --- Гизмо для отображения информации
class GP_GT_qw_info(Gizmo):
    bl_idname = 'GP_GT_qw_info'


    __slots__ = (
        'props',
        'settings',

        'segments',
        'name',
        'text_state',
        'index',
        'icon',
        'icon_id',
    )


    def setup(self):
        self.props = get_prefs()
        self.settings = bpy.context.scene.GP_scene_set

        #ui_scale = bpy.context.preferences.system.ui_scale
        #prefs = bpy.context.preferences.system
        #self.dpi = (prefs.dpi * prefs.pixel_size / 72)
        
        # --- Gizmo Option
        self.use_grab_cursor = False
        self.use_select_background = False
        self.use_draw_modal = True
        self.use_tooltip = False

        self.color = (0.1294, 0.1411, 0.1725)#(0.1, 0.1, 0.1)
        self.color_highlight = self.props.qw_color_active
        self.alpha = 0.9
        self.alpha_highlight = 0.2

        self.segments = 70

        self.name = ''
        self.text_state = ''
        self.index = 0

        self.icon = 'GRADIENT'


    def test_select(self, context, location):
        return -1


    @staticmethod
    def draw_text(self, position, text, scale, color=(0.95, 0.95, 0.95, 1.0), offset=1.0):
        font_info = {'font_id': 0, 'handler': None}
        font_id_name = font_info['font_id']
        
        blf.enable(font_id_name, 4)
        blf.size(font_id_name, 0.2 * scale, 72)

        dim = blf.dimensions(font_id_name, text)
        x = dim[0] / 2
        y = dim[1] / 2 * offset
        blf.position(font_id_name, position[0]-x, position[1]-y, 0)

        blf.color(font_id_name, *color)

        blf.enable(font_id_name, blf.SHADOW)
        blf.shadow(font_id_name, 3, 0.0, 0.0, 0.0, 1.0)
        
        blf.draw(font_id_name, text)


    def draw(self, context):
        self.color_highlight = self.props.qw_color_active
        pos = self.matrix_basis.translation

        dpi = context.preferences.system.ui_scale
        scale = self.scale_basis * dpi

        gpu.state.blend_set('ALPHA')
        gpu.state.line_width_set(3.0 * self.props.qw_scale)


        # --- Icon
        tS = scale * 2.4 # --- размер иконки
        x = pos[0]-tS/2
        y = pos[1]-tS/2
        tp = (x, y)
        if self.icon:
            self.icon_id = gl_textures[self.icon]
        else:
            self.icon_id = 0
        draw_texture_2d(self.icon_id, tp, tS, tS)

        # --- Circle
        draw_circle_2d(pos, (*self.color_highlight, self.alpha), scale, segments=self.segments)


        gpu.state.line_width_set(1.0)
        gpu.state.blend_set('NONE')



        # --- Name Text
        if self.text_state == '':
            offset=1.0
        else:
            offset=0.0
        self.draw_text(self, pos, self.name, scale, offset=offset)


        # --- State Text
        if self.text_state != '': 
            if self.index == 1:
                color = (1.0, 0.3, 0.0, 1.0)
            elif self.index == 2:
                color = (0, 0.823, 0.415, 1.0)
            else:
                color = (*self.props.qw_color_active, 1.0)
            self.draw_text(self, pos, self.text_state, scale, color=color, offset=4.0)

        


classes = [
    #GP_GT_qw_frame,
    #GP_GT_qw_draw_layout,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

   
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)