import bpy
from bpy.types import GizmoGroup
from math import radians, cos, sin
from mathutils import Matrix, Quaternion, Vector
from bl_math import smoothstep, clamp, lerp
from .. import var
from ..utils.func import (
    get_tool,
    get_brush_strength,
    get_brush_color,
    transform_orientation,
    transform_pivot_point,
    transform_snap_to,
    transform_proportional,
)
from ..keymaps import keymaps_qw_tweak
from .. import get_prefs


QW_GIZMOS = []
QW_ITEMS_01 = {
    'but_PE': [
        'Proportional Editing',                                                             # --- Name
        'SMOOTH',                                                                           # --- icon
        ['gp.call_panel', 'VIEW3D_PT_proportional_edit', 'PROPORT_EDIT', None, None, None],     # --- operator[0], op name[1], bool op[2], a_angle[3], b_angle[4], count_button[5]
        [0, 3, 3],                                                                          # --- id position for multiple alignment CIRCLE, HORIZONTAL, VERTICAL
        ],
    'but_SNAP': [
        'Snap',
        'SNAP_INC',
        ['gp.call_panel', 'VIEW3D_PT_snapping', 'SNAP', None, None, None],
        [1, 2, 2],
        ],
    'but_POS': [
        'Position',
        'POS_MEDIAN',
        ['gp.call_panel', 'GP_PT_transform_positions', 'PRO_CENTER', None, None, None],
        [2, 1, 1],
        ],
    'but_ORIENT': [
        'Orientation',
        'ORIENT_GLOBAL',
        ['gp.call_panel', 'GP_PT_transform_orientations', 'PRO_ORIENTATION', None, None, None],
        [3, 0, 0],
        ],
    'but_CURSOR': [
        '3D Cursor',
        'CURSOR',
        ['gp.call_panel', 'GP_PT_3d_cursor', 'GIZMO_CURSOR', None, None, None],
        [4, 6, 6],
        ],
    'but_VD': [
        'Viewport Display',
        'VIEW_DISPLAY',
        ['gp.call_panel', 'OBJECT_PT_display', 'NONE', None, None, None], 
        [5, 4, 4],
        ],
    'but_TO': [
        'Transform Options',
        'TRANSFORM',
        ['gp.call_panel', 'VIEW3D_PT_tools_object_options', 'NONE', None, None, None],
        [6, 5, 5],
        ],
    'but_PT': [
        'Pivot Transform',
        'PT',
        ['gp.call_panel', 'GP_PT_pivot_transform', 'PIVOT', None, None, None],
        [7, 7, 7], 
        ],
    
}

SA = 360 / len(QW_ITEMS_01) # --- радиус сегмента


class GP_GGT_qw_standard(GizmoGroup):
    bl_idname = 'GP_GGT_qw_standard'
    bl_label = 'Quick Widget'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = { 'PERSISTENT', 'SHOW_MODAL_ALL', 'SCALE' }


    @classmethod
    def poll(cls, context):
        #props = context.preferences.addons['GIZMO_PRO'].preferences
        if context.mode in {'OBJECT', 'EDIT_MESH', 'EDIT_CURVE'}:
            settings = context.scene.GP_scene_set
            return settings.qw


      

    @classmethod
    def setup_keymap(cls, keyconfig):
        return keymaps_qw_tweak(keyconfig)


    def setup(self, context):
        #props = context.preferences.addons['GIZMO_PRO'].preferences
        #settings = context.scene.GP_scene_set



       


        """ plus_button = (
            0x5f, 0xfb, 0x40, 0xee, 0x25, 0xda, 0x11, 0xbf, 0x4,  0xa0, 0x0,  0x80, 0x4,  0x5f, 0x11, 0x40,
            0x25, 0x25, 0x40, 0x11, 0x5f, 0x4,  0x7f, 0x0,  0xa0, 0x4,  0xbf, 0x11, 0xda, 0x25, 0xee, 0x40,
            0xfb, 0x5f, 0xff, 0x7f, 0xfb, 0xa0, 0xee, 0xbf, 0xda, 0xda, 0xbf, 0xee, 0xa0, 0xfb, 0x80, 0xff,
            0x6e, 0xd7, 0x92, 0xd7, 0x92, 0x90, 0xd8, 0x90, 0xd8, 0x6d, 0x92, 0x6d, 0x92, 0x27, 0x6e, 0x27,
            0x6e, 0x6d, 0x28, 0x6d, 0x28, 0x90, 0x6e, 0x90, 0x6e, 0xd7, 0x80, 0xff, 0x5f, 0xfb, 0x5f, 0xfb,
        )
        csv_b = bytes(plus_button)

        self.close = self.gizmos.new('GIZMO_GT_button_2d')
        self.close.icon = 'CANCEL'
        self.close.draw_options = {'HELPLINE', 'OUTLINE'} # 'HELPLINE' 'BACKDROP', 
        #but.show_drag = True
        #but.shape = #Passed to 'GPU_batch_tris_from_poly_2d_encoded PROP_STRING, PROP_BYTESTRING
        self.close.backdrop_fill_alpha = 0.8


        icons_dir = os.path.join(os.path.dirname(__file__), '../icons/dat/gp.general.dat')
        #icon_path = os.path.join(icons_dir, 'gp.general' + '.dat')
        print(icons_dir)

        #self.close.shape = os.path.join(os.path.dirname(__file__), '../icons/dat', 'gp.general') #csv_b #circle_2d((0,0),1.05,segments = 3,)

        self.close.use_tooltip = False
        self.close.use_draw_modal = True
        #but.use_draw_value = True
        self.close.color = 0.0, 0.5, 1.0
        self.close.alpha = 0.5
        self.close.color_highlight = 0.0, 0.5, 1.0
        self.close.alpha_highlight = 1.0
        self.close.scale_basis = (80 * 0.35) / 1 """


        





        angle = 360 / len(QW_ITEMS_01)
        #angle = 30.0 #22.5

        #self.handler = self.gizmos.new( 'GP_GT_qw_ring_handler' )
        #self.handler.gizmo_group_del = 'GP_GGT_buttons_cursor'





        #self.info = self.gizmos.new('GP_GT_qw_info')


        self.buttons = {}
        for i, item in enumerate(QW_ITEMS_01):
            but = self.gizmos.new('GP_GT_button') # GP_GT_qw_button
            but.icon = QW_ITEMS_01[item][1]
            but.id = i
            but.use_tooltip = True

            #but.angle = angle * i
            #but.matrix_basis = Matrix.Rotation( radians(but.angle), 4, 'Z' )
            op = but.target_set_operator( QW_ITEMS_01[item][2][0] )
            op.name = QW_ITEMS_01[item][2][1]
            op.bool_op = QW_ITEMS_01[item][2][2]
            
            if QW_ITEMS_01[item][1] == 'NONE':
                but.hide_select = True
            else:
                but.hide_select = False

            self.buttons[item] = but
            

        

        



        self.frame = self.gizmos.new('GP_GT_qw_frame')
        self.frame.use_select_background = True


        global QW_GIZMOS
        QW_GIZMOS = self.gizmos
        #self.valueG = self.gizmos.new('GIZMO_GT_value_2d')
        
    
    def invoke_prepare(self, context, gizmo):
        if gizmo.bl_idname == 'GP_GT_button':

            if self.buttons['but_TO'].is_highlight and self.buttons['but_TO'].is_modal == False:
                op = self.buttons['but_TO'].target_set_operator('gp.call_panel')
                op.bool_op = 'NONE'
                if context.mode == 'EDIT_MESH':
                    op.name = 'VIEW3D_PT_tools_meshedit_options'
                elif context.mode == 'EDIT_CURVE':
                    op.name = 'VIEW3D_PT_tools_object_options' # TODO сделать панель для режима редактирования кривых
                else:
                    op.name = 'VIEW3D_PT_tools_object_options'


            if self.buttons['but_PT'].is_highlight and self.buttons['but_PT'].is_modal == False:
                op = self.buttons['but_PT'].target_set_operator( 'gp.call_panel' )
                if hasattr(bpy.types, bpy.ops.pivot.to_active.idname()):
                    op.name = 'PT_PT_transform_panel'
                else:
                    op.name = 'GP_PT_pivot_transform'
                op.bool_op = 'PIVOT'

       



    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set
        toolSet = context.scene.tool_settings


        is_highlight = [ g.is_highlight for g in self.gizmos ]

        # --- Pos Setup
        if settings.qw_pos:
            self.pos2d = Vector( (settings.qw_pos[0], settings.qw_pos[1]) )
        else:
            self.pos2d = Vector( (150, 150) )
            

        # --- Scale
        dpi = context.preferences.system.ui_scale
        #print(dpi)

       
        

        scaleDial = 80 * props.qw_scale # dpi * 
        
        

        butSize_01 = ((80 * 0.35) / 2) * props.qw_scale * 1.3 # --- размер главных кнопок
        butSize_02 = ((80 * 0.35) / 2) * props.qw_scale #* dpi  # --- размер второстепенных кнопок
        


        # --- Limit
        use_tool_header = context.space_data.show_region_tool_header
        if use_tool_header:
            header_height = context.area.regions[0].height * 2
        else:
            header_height = context.area.regions[0].height
        offset = 2 * dpi
        wl = context.area.regions[2].width + offset
        ht = context.area.height - header_height - offset
        hb = offset
        wr = context.area.width - context.area.regions[3].width - offset
        #print(wl)
        #aW = context.area.width - context.area.regions[3].width
        #aH = context.area.height - context.area.regions[0].height - context.area.regions[1].height

        if settings.qw_alignment == 'CIRCLE':
            xl = scaleDial
            yl = scaleDial

            x0 = self.pos2d[0] - xl * dpi * 1.15
            x1 = self.pos2d[0] + xl * dpi * 1.15
            if x0 < wl:
                self.pos2d[0] = wl + xl * dpi
            elif x1 > wr:
                self.pos2d[0] = wr - xl * dpi

            y0 = self.pos2d[1] - yl * dpi * 1.15
            y1 = self.pos2d[1] + yl * dpi * 1.15
            if y0 < hb:
                self.pos2d[1] = hb + yl * dpi
            elif y1 > ht:
                self.pos2d[1] = ht - yl * dpi
        else:
            oBorder = 20.0 * dpi
            if settings.qw_alignment == 'HORIZONTAL':
                l = len(QW_ITEMS_01)
                xl = butSize_01 * l * 1.2
                yl = butSize_01 * 1.2

            elif settings.qw_alignment == 'VERTICAL':
                l = len(QW_ITEMS_01)
                xl = butSize_01 * 1.0
                yl = butSize_01 * l * 1.02

            x0 = self.pos2d[0] - xl - oBorder * dpi
            x1 = self.pos2d[0] + xl + oBorder * dpi
            if x0 < wl:
                self.pos2d[0] = wl + xl * dpi
            elif x1 > wr:
                self.pos2d[0] = wr - xl * dpi

            y0 = self.pos2d[1] - yl - oBorder * dpi
            y1 = self.pos2d[1] + yl + oBorder * dpi
            if y0 < hb:
                self.pos2d[1] = hb + yl * dpi
            elif y1 > ht:
                self.pos2d[1] = ht - yl * dpi

        









# --- Dial
        if True in is_highlight:
            self.frame.color = props.qw_color_active
            self.frame.alpha = 0.99
            self.frame.use_draw_hover = False
        else:
            self.frame.color = (0.176, 0.204, 0.212)
            self.frame.alpha = 0.5
            self.frame.use_draw_hover = True
        self.frame.scale_basis = scaleDial
        self.frame.matrix_basis[0][3] = self.pos2d[0]
        self.frame.matrix_basis[1][3] = self.pos2d[1]
        

        


# --- Getter
        to = transform_orientation(context)
        tpp = transform_pivot_point(context)
        tst = transform_snap_to(context)
        tp = transform_proportional(context)


        
# --- BUTTONS
        for i, item in enumerate(self.buttons):

            if settings.qw_alignment == 'CIRCLE':
                angle = radians( (SA * self.buttons[item].id + SA/2) )      # --- угол поворота позиции иконки
                rad = scaleDial * 0.7  * dpi                                # --- радиус отступа иконки
                rx = ( rad * cos(angle) + self.pos2d[0] )                   # --- x
                ry = ( rad * sin(angle) + self.pos2d[1] )                   # --- y

            elif settings.qw_alignment == 'HORIZONTAL':
                k = 2.5 * dpi
                l = len(QW_ITEMS_01)
                sp = (((butSize_01 * l)/2) - butSize_01/2) * k
                i = QW_ITEMS_01[ item ][3][1]
                rx = (self.pos2d[0] - sp) + (i*butSize_01) * k
                ry = self.pos2d[1]

            elif settings.qw_alignment == 'VERTICAL':
                k = 2.0 * dpi
                l = len(QW_ITEMS_01)
                sp = (((butSize_01 * l)/2) - butSize_01/2) * k
                i = QW_ITEMS_01[ item ][3][2]
                rx = self.pos2d[0] 
                ry = (self.pos2d[1] + sp) - (i*butSize_01) * k

            self.buttons[item].scale_basis = butSize_01
            self.buttons[item].matrix_basis[0][3] = rx
            self.buttons[item].matrix_basis[1][3] = ry
            self.buttons[item].alpha = props.qw_opacity



            if settings.qw_alignment == 'CIRCLE':
                self.buttons[item].type = 'CIRCLE'
            else:
                self.buttons[item].type = 'CAPSULE'



            if item == 'but_VD':
                if context.mode == 'EDIT_MESH':
                    self.buttons[item].div = 8 # TODO
                else:
                    self.buttons[item].div = 3

            elif item == 'but_PE':
                self.buttons[item].icon = tp[2]
                if context.mode == 'OBJECT':
                    self.buttons[item].bool_op = toolSet.use_proportional_edit_objects
                else:
                    self.buttons[item].bool_op = toolSet.use_proportional_edit

            elif item == 'but_SNAP':
                self.buttons[item].bool_op = toolSet.use_snap
                self.buttons[item].icon = tst[2]

            elif item == 'but_POS':
                self.buttons[item].icon = tpp[2]

            elif item == 'but_ORIENT':
                self.buttons[item].icon = to[2]

            elif item == 'but_CURSOR':
                self.buttons[item].bool_op = settings.cursor_gizmo
                #self.buttons[item].type = 'CAPSULE'

            elif item == 'but_PT':
                self.buttons[item].bool_op = context.scene.tool_settings.use_transform_data_origin
       



        


        


class GP_GGT_qw_draw(GizmoGroup):
    
    bl_idname = 'GP_GGT_qw_draw'
    bl_label = 'Quick Widget'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = { 'PERSISTENT', 'SHOW_MODAL_ALL', 'SCALE' }


    @classmethod
    def poll(cls, context):
        #props = context.preferences.addons['GIZMO_PRO'].preferences
        if context.mode in { 'PAINT_TEXTURE' }: # , 'PAINT_VERTEX', 'PAINT_GPENCIL', 'VERTEX_GPENCIL' 
            settings = context.scene.GP_scene_set
            return settings.qw


    @classmethod
    def setup_keymap(cls, keyconfig):
        return keymaps_qw_tweak(keyconfig)


    def setup(self, context):
        #props = context.preferences.addons['GIZMO_PRO'].preferences
        settings = context.scene.GP_scene_set


        self.b_size = self.gizmos.new('GIZMO_GT_button_2d')
        self.b_size.icon = 'BRUSH_DATA'
        self.b_size.draw_options = {'BACKDROP', 'OUTLINE'} # 'HELPLINE'
        self.b_size.use_tooltip = False
        self.b_size.color = 0.0, 0.0, 0.0
        self.b_size.alpha = 1.0
        self.b_size.alpha_highlight = 1.0
        self.b_size.use_draw_modal = True
        op = self.b_size.target_set_operator('gp.qw_brush')
        op.type = 'BRUSH'




        self.b_color = self.gizmos.new('GIZMO_GT_button_2d')
        self.b_color.icon = 'NONE'
        self.b_color.draw_options = {'BACKDROP', 'OUTLINE'} # 'HELPLINE'
        #b_color.show_drag = True
        #but.shape = #Passed to 'GPU_batch_tris_from_poly_2d_encoded PROP_STRING, PROP_BYTESTRING
        #self.b_color.backdrop_fill_alpha = 0.9
        self.b_color.use_tooltip = False
        self.b_color.color = 0.0, 0.0, 0.0
        self.b_color.alpha = 1.0
        self.b_color.alpha_highlight = 1.0
        self.b_color.use_draw_modal = True
        op = self.b_color.target_set_operator('gp.qw_brush')
        op.type = 'COLOR'


        self.layout = self.gizmos.new('GP_GT_qw_draw_layout')




    


    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set

        # --- Scale
        dpi = context.preferences.system.ui_scale

        #ups = context.tool_settings.unified_paint_settings

       
        
        brush_strength = get_brush_strength( self, context )
        color = get_brush_color( self, context )


        button_size = ((80 * 0.35) / 2) * 2
        button_offset = button_size * 1.2 * dpi

        pos_x = settings.qw_draw_pos[0]
        pos_y = settings.qw_draw_pos[1]
        #print('pos_x', pos_x, 'pos_y', pos_y)
        #wc = context.area.width / 2
        #hc = context.area.height / 2


        self.b_size.color = 0.5, 0.5, 0.5
        self.b_size.color_highlight = 0.6, 0.6, 0.6
        self.b_size.scale_basis = button_size
        self.b_size.matrix_basis.translation = (pos_x, pos_y + button_offset, 0.0)
        self.b_size.backdrop_fill_alpha = brush_strength


        self.b_color.color = color
        self.b_color.color_highlight = color
        self.b_color.scale_basis = button_size
        self.b_color.matrix_basis.translation = (pos_x, pos_y - button_offset, 0.0)


        self.layout.scale_basis = button_size
        self.layout.matrix_basis.translation = (pos_x, pos_y, 0.0)
        


classes = [
    #GP_GGT_qw_standard,
    #GP_GGT_qw_draw,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

   
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)