import bpy, gpu
from bpy.types import Gizmo
#from gpu_extras.presets import draw_texture_2d
from ..utils.draw_utils.draw import draw_texture_2d
from ..utils.func import viewport_points



class GPRB_GT_image(Gizmo):
    bl_idname = 'GPRB_GT_image'


    __slots__ = (
        'id',
        'texture',
    )


    def setup(self):
        # --- Gizmo Option
        self.use_grab_cursor = False
        self.use_select_background = False
        self.use_draw_modal = True
        self.use_tooltip = False
        self.use_draw_offset_scale = True
        
        self.scale_basis = 1.0
        # --- 
        if not hasattr(self, 'id'):
            self.id = -1


    def test_select(self, context, location):
        return -1


    def draw(self, context):
        #id = context.scene.refboard_active_id_
        collection = context.scene.refboard_collection_
        dpi = context.preferences.system.ui_scale


        if self.id != -1:
            item = collection[self.id]
            image = item.image
            if image:
                if item.visible:
                    gpu.state.blend_set(item.blend_mode)

                    #gpu.state.color_mask_set(0, 0, 0, 0.5)
                    res = image.size
                    

                    
                    wl, wr, hb, ht, wc, hc = viewport_points(context)

                    oBorder = 20 * dpi
                    x0 = item.position[0] - oBorder
                    x1 = item.position[0] + oBorder
                    if x0 < wl:
                        item.position[0] = wl + oBorder
                    elif x1 > wr:
                        item.position[0] = wr - oBorder

                    y0 = item.position[1] - oBorder
                    y1 = item.position[1] + oBorder
                    if y0 < hb:
                        item.position[1] = hb + oBorder
                    elif y1 > ht:
                        item.position[1] = ht - oBorder


                    self.matrix_basis.translation = item.position


                    
                    # --- Icon
                    scaX = self.scale_basis * item.scale * res[0]/15 #* dpi
                    scaY = self.scale_basis * item.scale * res[1]/15 #* dpi
                    posX = self.matrix_basis.translation[0] - scaX/2
                    psoY = self.matrix_basis.translation[1] - scaY/2

                    if not hasattr(self, 'texture'):
                        #if self.texture is None:
                        self.texture = gpu.texture.from_image(image)
                    
                    draw_texture_2d( self.texture, (posX, psoY), scaX, scaY )

                    


                    
                    gpu.state.blend_set('NONE')








classes = [
    GPRB_GT_image,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

   
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)