import bpy
from bpy.types import UIList, Operator, PropertyGroup, Panel
from bpy.props import (
    FloatVectorProperty,
    BoolProperty,
    IntProperty,
    CollectionProperty,
    EnumProperty,
    FloatProperty,
    PointerProperty,
)
from ..icons import icons_collections
from ..utils.func import viewport_points as vp
from mathutils import Vector
from .. import get_prefs



def update_gizmo():
        from .rb_gizmo import GPRB_GT_image
        from .rb_widget import GP_GGT_refboard
        gizChange = [
            GPRB_GT_image,
            GP_GGT_refboard,
            ]

        try:
            for gizmo in gizChange:
                if 'bl_rna' in gizmo.__dict__:
                    bpy.utils.unregister_class(gizmo)

            for gizmo in gizChange:
         
                bpy.utils.register_class(gizmo)

        except Exception as e:
            message = 'Gizmo PRO: Updating gizmos failed'
            print('\n[{}]\n{}\n\nError:\n{}'.format(__name__, message, e))
            pass


class GPRB_UL_items(UIList):
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):

        #col = layout.column_flow(columns=2, align=False)


        row = layout.row( align = True )
        
        row.label( icon = 'GRIP' )
        row.template_ID( item, 'image', open = 'image.open', live_icon = True )
        
        row = layout.row( align = True )
        bm = row.row()
        bm.scale_x = 0.5
        bm.prop( item, 'blend_mode', text = '' )
        if item.visible:
            row.prop( item, 'visible', text = '', icon = 'HIDE_OFF', toggle = True )
        else:
            row.prop( item, 'visible', text = '', icon = 'HIDE_ON', toggle = True )


""" class GPRB_OT_image_open(Operator):
    bl_idname = 'gprb.image_open'
    bl_label = 'Image Open'
    bl_description = ' '
    bl_options= {'REGISTER'}
    
    def execute(self, context):
        bpy.ops.image.open('INVOKE_DEFAULT')
        update_gizmo(self, context)
        return {'FINISHED'} """
        

        
class GPRB_OT_move(Operator):
    bl_idname = 'gprb.move'
    bl_label = 'Move'
    bl_description = ' '
    bl_options= {'REGISTER'}
    
    isUp: BoolProperty()
    
    def execute(self, context):
        collection = context.scene.refboard_collection_
        id = context.scene.refboard_active_id_
        
        if self.isUp and id >= 1:
            collection.move(id, id-1)
            id -= 1

        elif self.isUp==False and id < len(collection) - 1:
            collection.move(id, id+1)
            id += 1

        update_gizmo()
        return {'FINISHED'}


        
class GPRB_OT_add(Operator):
    bl_idname = 'gprb.add'
    bl_label = 'Add'
    bl_description = 'Save A New Transformation 3D Cursor In The List'
    bl_options= {'REGISTER'}
    

    def execute(self, context):
        collection = context.scene.refboard_collection_
        point = collection.add()
        """ for img in bpy.data.images:
            print("name=%s, filepath=%s" % (img.name, img.filepath) ) """
        point.position = Vector( (vp(context)[4], vp(context)[5], 0.0) )  # --- position on screen (center)
        #print( point.position[:] )
        context.scene.refboard_active_id_ = len(collection)-1

        update_gizmo()
        return {'FINISHED'}
    
      
    
class GPRB_OT_remove(Operator):
    bl_idname = 'gprb.remove'
    bl_label = 'Remove'
    bl_description = 'Remove Reference'
    bl_options= {'REGISTER', 'UNDO'}
    
    def execute(self, context):
        collection = context.scene.refboard_collection_
        if len(collection) > 0:
            collection.remove(context.scene.refboard_active_id_)
            if context.scene.refboard_active_id_ == 0:
                context.scene.refboard_active_id_ += 1
            if context.scene.refboard_active_id_ > -1:
                context.scene.refboard_active_id_ = 0

        update_gizmo()
        return {'FINISHED'}



class GPRB_collection(PropertyGroup):
    image: PointerProperty( name = 'Image', type = bpy.types.Image )
    visible: BoolProperty( name = 'Visible', description = '', default = True )
    position: FloatVectorProperty()
    rotation: FloatProperty( default = 0.0 )
    scale: FloatProperty( default = 1.0)
    opacity: FloatProperty( default = 1.0)

    blend_mode: EnumProperty(
        name = 'Blend Mode',
        items = [
            ('ALPHA', 'Standard', 'The original color channels are interpolated according to the alpha value', '', 0),
            ('ADDITIVE', 'Additive', 'The original color channels are added by the corresponding ones', '', 1),
            ('ADDITIVE_PREMULT', 'Additive Pre-Multiplied', 'The original color channels are added by the corresponding ones that are pre-multiplied by the alpha value', '', 2),
            ('MULTIPLY', 'Multiply', 'The original color channels are multiplied by the corresponding ones', '', 3),
            ('NONE', 'None', 'No blending', '', 4),
            ],
        default = 'ALPHA',
        )


    matrix: FloatVectorProperty(
        size = (4, 4),
        subtype = 'MATRIX',
        default = [
            (1.0, 0.0, 0.0, 0.0),
            (0.0, 1.0, 0.0, 0.0),
            (0.0, 0.0, 1.0, 0.0),
            (0.0, 0.0, 0.0, 1.0),
        ],
        )


class GPRB_PT_collection(Panel):
    bl_label = 'RefBoard Collection'
    bl_idname = 'GPRB_PT_collection'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    #bl_options = {'DEFAULT_CLOSED'}
    bl_ui_units_x = 20

    @classmethod
    def poll(self, context):
        props = get_prefs()
        return props.test

    def draw(self, context):
        collection = context.scene.refboard_collection_
        #id = context.scene.refboard_active_id_
        layout = self.layout
        #layout.template_ID( context.scene, 'refboard_images_', open = 'image.open' )
        if len(collection) > 0:
            row = layout.row()
            row.template_list( 'GPRB_UL_items', '', context.scene, 'refboard_collection_', context.scene, 'refboard_active_id_' )

            col = row.column(align=True)
            col.operator( 'gprb.add', text = '', icon = 'ADD' )
            col.operator( 'gprb.remove', text = '', icon = 'REMOVE' )
            col.separator()
            col.operator( 'gprb.move', text = '', icon = 'TRIA_UP' ).isUp = True
            col.operator( 'gprb.move', text = '', icon = 'TRIA_DOWN' ).isUp = False

        else:
            layout.operator( 'gprb.add', text = 'Add Reference', icon = 'ADD' )



def refboard_header( self, context ):
    props = get_prefs()
    if props.test:
        settings = context.scene.GP_scene_set
        row = self.layout.row( align = True)
        row.alignment = 'CENTER'
        if settings.refboard:
            row.prop( settings, 'rb_edit', text = '', icon = 'EDITMODE_HLT' )
            row.popover( panel = 'GPRB_PT_collection', text = '', icon = 'RENDERLAYERS' )



# --- Проверка изменения
subscription_owner = object()

def subscribe_change_type_position(subscription_owner):
    bpy.msgbus.subscribe_rna(
        key=(bpy.types.Scene, 'refboard_collection_'),
        owner=bpy,
        args=(),
        notify = update_gizmo,
        options={'PERSISTENT'},
    )

    # --- Register the persistent handler, ensures that the subscription will happen 
    # --- when a new file is loaded.
    if load_handler not in bpy.app.handlers.load_post:
        bpy.app.handlers.load_post.append(load_handler)


def unsubscribe_change_type_position(subscription_owner):
    # --- Clear all subscribers by this owner
    if subscription_owner is not None:
        bpy.msgbus.clear_by_owner(subscription_owner)

    # --- Unregister the persistent handler.
    if load_handler in bpy.app.handlers.load_post:
        bpy.app.handlers.load_post.remove(load_handler)


from bpy.app.handlers import persistent
@persistent
def load_handler(dummy):
    subscribe_change_type_position(subscription_owner)





classes = [
    GPRB_UL_items,
    GPRB_OT_move,
    GPRB_OT_add,
    GPRB_OT_remove,
    GPRB_collection,
    GPRB_PT_collection,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

    bpy.types.Scene.refboard_collection_ = CollectionProperty( type = GPRB_collection )
    bpy.types.Scene.refboard_active_id_ = IntProperty()

    """ # --- Subscribe and remove handle
    if load_handler not in bpy.app.handlers.load_post:
        bpy.app.handlers.load_post.append(load_handler) """

    bpy.types.VIEW3D_HT_header.append(refboard_header)

   
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.refboard_collection_
    del bpy.types.Scene.refboard_active_id_
    """ # --- Unsubscribe and remove handle
    unsubscribe_change_type_position(subscription_owner) """

    bpy.types.VIEW3D_HT_header.remove(refboard_header)