import bpy
from bpy.types import GizmoGroup
from .. import var,get_prefs



class GP_GGT_refboard(GizmoGroup):
    
    bl_idname = 'GP_GGT_refboard'
    bl_label = 'RefBoard'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = { 'PERSISTENT', 'SHOW_MODAL_ALL', 'SCALE'} # , 'DEPTH_3D'  


    @classmethod
    def poll(cls, context):
        props = get_prefs()
        if props.test:
            settings = context.scene.GP_scene_set
            return settings.refboard


    """ @classmethod
    def setup_keymap(cls, keyconfig):
        km = keyconfig.keymaps.new( name = 'QW Tweak', space_type = 'VIEW_3D' )
        km.keymap_items.new( 'gizmogroup.gizmo_tweak', type = 'LEFTMOUSE', value = 'CLICK_DRAG' )
        km.keymap_items.new( 'gizmogroup.gizmo_tweak', type = 'LEFTMOUSE',  value = 'CLICK' )
        km.keymap_items.new( 'gizmogroup.gizmo_tweak', type = 'RIGHTMOUSE', value = 'CLICK' )

        return km #keyconfig.keymaps.find( name = 'QW Tweak', space_type = 'VIEW_3D' ) """
    

    def setup(self, context):
        #props = context.preferences.addons['GIZMO_PRO'].preferences
        #settings = context.scene.GP_scene_set
        id = context.scene.refboard_active_id_
        collection = context.scene.refboard_collection_
        

        
                    


        # --- Cage
        self.cage = self.gizmos.new('GIZMO_GT_cage_2d')
      
        self.cage.draw_style = 'CIRCLE' # BOX
        self.cage.transform = {'TRANSLATE', 'SCALE', 'SCALE_UNIFORM'} # 'ROTATE', 
        self.cage.draw_options = { 'XFORM_CENTER_HANDLE' }
 
        self.cage.use_tooltip = False
        self.cage.color = 0.0, 0.5, 1.0
        self.cage.alpha = 1.0
        self.cage.alpha_highlight = 1.0
        self.cage.use_draw_modal = True
        """ op = self.b_size.target_set_operator('gp.qw_brush')
        op.type = 'BRUSH' """

        self.cage.dimensions = [ 200.0, 100.0 ]
        self.cage.matrix_basis.translation = (200, 200, 0.0)

        """ def transform_getter():
            from ..utils.custom_math.math import pMat_to_cMat
            collection = context.scene.refboard_collection_
            id = context.scene.refboard_active_id_
            item = collection[id]
            return pMat_to_cMat( item.matrix.inverted() )

        def transform_setter(value):
            from ..utils.custom_math.math import cMat_to_pMat
            collection = context.scene.refboard_collection_
            id = context.scene.refboard_active_id_
            item = collection[id]
            
            m = cMat_to_pMat( value )
            l, q, s = m.decompose()

            item.position[0] += l[0]
            item.position[1] += l[1]

            item.scale += s[0] - 1
            if item.scale < 0.1:
                item.scale = 0.1

        self.cage.target_set_handler( 'matrix', get = transform_getter, set = transform_setter ) """


        
        #self.ref.use_draw_hover = True


        self.images = {}
        if len(collection) > 0:
            for i, item in enumerate(collection):
                image = item.image
                #print(image)
                if image:
                    ref = self.gizmos.new('GPRB_GT_image') # GP_GT_qw_button
                    ref.id = i
                    self.images[i] = ref
    



    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set
        id = context.scene.refboard_active_id_
        collection = context.scene.refboard_collection_


        # --- Scale
        #dpi = context.preferences.system.ui_scale


        if len(collection) > 0:
            item = collection[id]
            image = item.image
            if image:
                self.cage.line_width = 2
                self.cage.scale_basis = 50
                #self.cage.matrix_basis.translation = (200, 200, 0.0)

                res = image.size
                scaX = item.scale * res[0]/15 #* dpi
                scaY = item.scale * res[1]/15 #* dpi

                self.cage.dimensions = [scaX, scaY]
                self.cage.matrix_basis.translation = item.position





                def transform_getter():
                    from ..utils.custom_math.math import flatten_matrix
                    item = collection[id]
                    return flatten_matrix( item.matrix )

                def transform_setter(value):
                    from ..utils.custom_math.math import cMat_to_pMat, reshape_matrix
                    item = collection[id]
                    
                    #print( reshape_matrix(value, 4, 4))
                    m = cMat_to_pMat( value )
                    l, q, s = m.decompose()
                    #print(s)
                    #item.matrix = m
                    item.position[0] += l[0]
                    item.position[1] += l[1]

                    item.scale += s[0] - 1
                    if item.scale < 0.1:
                        item.scale = 0.1

                self.cage.target_set_handler( 'matrix', get = transform_getter, set = transform_setter )


                if item.visible:
                    self.cage.hide = False
                else:
                    self.cage.hide = True
            else:
                self.cage.hide = True
        
        else:
            self.cage.hide = True


        
        
        if settings.rb_edit is False:
            self.cage.hide = True



classes = [
    GP_GGT_refboard,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

   
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)