import bpy
from bpy.types import Operator, GizmoGroup
from bpy.props import FloatProperty, EnumProperty
from bpy_extras import view3d_utils
from .. import var


def raycast(context, coord):
    scene = context.scene
    region = context.region
    rv3d = context.region_data
    dg = context.evaluated_depsgraph_get()
    view_vector = view3d_utils.region_2d_to_vector_3d( region, rv3d, coord ) 
    ray_origin = view3d_utils.region_2d_to_origin_3d( region, rv3d, coord )
    return scene.ray_cast(dg, ray_origin, view_vector) # success, location, normal, face_index, obj, matrix



class GP_GGT_placement(GizmoGroup):
    
    bl_idname = 'GP_GGT_placement'
    bl_label = 'Placement'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'3D', 'SHOW_MODAL_ALL', 'VR_REDRAWS'} # , 'TOOL_INIT', 'SELECT', 'SCALE', 'DEPTH_3D', 'PERSISTENT', 


    @classmethod
    def poll(cls, context):
        if context.area:
            return context.area.type == 'VIEW_3D'
   
    
    def setup(self, context):
     
        self.ray = self.gizmos.new('GIZMO_GT_snap_3d')
        self.ray.snap_elements_force = {'VERTEX', 'EDGE_MIDPOINT', 'EDGE', 'FACE'} # {'EDGE_PERPENDICULAR', 'VERTEX', 'EDGE_MIDPOINT', 'EDGE', 'FACE'}
        self.ray.hide_select = True
        
        #self.ray.use_draw_value = True
        #self.ray.use_event_handle_all = True # включает возможность постаянного использования
        #self.ray.hide_keymap = True # снимает замерание, включает поворот
        #self.ray.use_operator_tool_properties = True
    

    def draw_prepare(self, context):
        
        idx = self.ray.snap_elem_index[:]
        if idx[0] == -1 and idx[1] == -1 and idx[2] == -1:
            success, l, n, face_index, obj, matrix = raycast(context, var.MOUSE_POSITION)
            if success:
                print('location', l)
            else:
                print('location', 'None')
                
        else:
            location = self.ray.location
            normal = self.ray.normal
            print('location', location)
            print('normal', normal)



class GP_OT_skch_circle(Operator):
    bl_idname = 'gp.skch_circle'
    bl_label = 'Circle'
    bl_options = {'REGISTER', 'UNDO'}
    
    
    


    def modal(self, context, event):
        if event.type == 'RIGHTMOUSE':
            context.window_manager.gizmo_group_type_unlink_delayed( 'GP_GGT_placement' )
            return {'CANCELLED'}

        return {'PASS_THROUGH'}


    def invoke(self, context, event):
        context.window_manager.gizmo_group_type_ensure( 'GP_GGT_placement' )
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}




    




















classes = [
    #GP_GGT_placement,
    GP_OT_skch_circle,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)