import bpy
import os
from .icons import icons_collections
from mathutils import Vector
from .utils.func import convert_units
from . import var
from .utils.func import (
    rotate_method_ui,
    action_center_ui,
    increment_rotate_ui,
)
from bpy.types import WorkSpaceTool
from .keymaps import get_keymap_tools
from . import get_prefs




def panel_tool(context, layout, tool):
    icoll = icons_collections['main']
    settings_icon = icoll['settings']
    #select_icon = icoll['select']
    #select_box_icon = icoll['select_box']
    #select_circle_icon = icoll['select_circle']
    #select_lasso_icon = icoll['select_lasso']
    info_icon = icoll['info']

    props = get_prefs()
    settings = context.scene.GP_scene_set

    if layout.direction == 'HORIZONTAL':
        # --- Settings
        row = layout.row(align=True)
        row.scale_x = 1.3
        #row.operator( 'gp.documentation', text = '', icon_value = info_icon.icon_id )
        row.operator( 'gp.panel_settings', text = '', icon_value = settings_icon.icon_id )
        
        
        # --- Space Drag / Select Tools
        row = layout.row(align=True)

        if settings.lock_orbit and props.space_drag_lmb == 'ORBIT':
            row.prop(settings, 'lock_orbit', text='', icon='LOCKED')
        elif settings.lock_orbit == False and props.space_drag_lmb == 'ORBIT':
            row.prop(settings, 'lock_orbit', text='', icon='UNLOCKED')
            
        #row.prop(props, 'space_drag_lmb', text='')


        # --- Select Tools
        #icon = select_box_icon.icon_id 
        row.popover( panel = 'GP_PT_settings_space_drag', text = '', icon = 'MOUSE_LMB_DRAG' )

    else:
        # --- Settings
        row = layout.row(align=True)
        row.operator('gp.panel_settings', text='Settings', icon_value=settings_icon.icon_id)
        #row.operator('gp.documentation', text='', icon_value=info_icon.icon_id)
        

        # --- Space Drag / Select Tools
        row = layout.row()

        if settings.lock_orbit and props.space_drag_lmb == 'ORBIT':
            row.prop(settings, 'lock_orbit', text='', icon='LOCKED')
        elif settings.lock_orbit == False and props.space_drag_lmb == 'ORBIT':
            row.prop(settings, 'lock_orbit', text='', icon='UNLOCKED')

        #row.prop(props, 'space_drag_lmb', text='')


        # --- Select Tools
        #icon = select_box_icon.icon_id 
        row.popover( panel = 'GP_PT_settings_space_drag', text = '', icon = 'MOUSE_LMB_DRAG' )
        
def panel_general(context, layout, tool):
    icoll = icons_collections['main']
    settings_icon = icoll['settings'].icon_id
    info_icon = icoll['info'].icon_id

    props = get_prefs()
    settings = context.scene.GP_scene_set

    scaleX = 1.3
    #row_all = layout.row()
    

    if layout.direction == 'HORIZONTAL':
        row = layout.row(align=True)
        row.scale_x = scaleX
        #row.operator( 'gp.documentation', text = '', icon_value = info_icon )
        row.operator( 'gp.panel_settings', text = '', icon_value = settings_icon)
        
        
        # --- Space Drag / Select Tools
        row = layout.row(align=True)

        if settings.lock_orbit and props.space_drag_lmb == 'ORBIT':
            row.prop(settings, 'lock_orbit', text='', icon='LOCKED')
        elif settings.lock_orbit == False and props.space_drag_lmb == 'ORBIT':
            row.prop(settings, 'lock_orbit', text='', icon='UNLOCKED')
            
        #row.prop(props, 'space_drag_lmb', text='')


        # --- Select Tools
        #icon = select_box_icon.icon_id 
        row.popover( panel = 'GP_PT_settings_space_drag', text = '', icon = 'MOUSE_LMB_DRAG' )


        # --- Visible
        #if layout.direction=='HORIZONTAL':
        row = layout.row(align=True)
        row.scale_x = scaleX
        row.prop(settings, 'show_gizmo', text='', icon='HIDE_OFF' if settings.show_gizmo else 'HIDE_ON')
        row.separator()
        row.prop(settings, 'gzm_type', text='', expand=True, icon_only=True)

        sub = row.row(align=True)
        sub.scale_x = scaleX
        sub.scale_x = 1.0
        sub.popover(panel='GP_PT_visible', text='')
        row.scale_x = 1.0

        
    


        if context.mode=='SCULPT':
            #orient_slots = context.window.scene.transform_orientation_slots[0].type
            scene = context.window.scene
            orient_slot = scene.transform_orientation_slots[0]
            layout.prop_with_popover( orient_slot, 'type', text = '', panel='VIEW3D_PT_transform_orientations' )


        action_center_ui(context, layout) # --- Action Center
        increment_rotate_ui(context, layout) # --- Increment Rotate
        #rotate_method_ui(context, layout) # --- Rotate Method

        




        # --- Длина Линейки
        if settings.gzm_type=='TRANSPOSE':
            box = layout.box()
            dim = Vector(settings.tr_p1) - Vector(settings.tr_p2) 
            len = dim.length
            value, unit = convert_units(context, len)
            len_str = str( round(value, 3)  )
            box.label( text = len_str + unit, icon='SNAP_INCREMENT' )
            

        # --- Error
        orient_slots = context.window.scene.transform_orientation_slots[0].type
        if orient_slots=='NORMAL' and context.mode=='EDIT_MESH':
            layout.separator()
            box = layout.box()
            box.alert = True
            row = box.row(align=True)
            row.label(text='Critical Errors!!!', icon='ERROR')









class GizmoPRO_Object(WorkSpaceTool):
    bl_space_type = 'VIEW_3D'
    bl_context_mode = 'OBJECT'

    bl_idname = "tool.gizmo_pro"
    bl_label = "Gizmo PRO"
    bl_description = ""
    bl_icon = os.path.join(os.path.dirname(__file__), './icons/dat', 'gp.general')
    bl_widget = None
    bl_keymap = get_keymap_tools()
    bl_cursor = 'CROSSHAIR'

    def draw_settings(context, layout, tool):
        panel_general(context, layout, tool)

    def draw_cursor(_context, tool, xy):
        x = _context.area.x
        y = _context.area.y
        var.MOUSE_POSITION = [xy[0]-x, xy[1]-y]


class GizmoPRO_Mesh(WorkSpaceTool):
    bl_space_type = 'VIEW_3D'
    bl_context_mode = 'EDIT_MESH'

    bl_idname = "tool.gizmo_pro"
    bl_label = "Gizmo PRO"
    bl_description = ""
    bl_icon = os.path.join(os.path.dirname(__file__), './icons/dat', 'gp.general')
    bl_widget = None
    bl_keymap = get_keymap_tools()
    bl_cursor = 'PAINT_CROSS'

    def draw_settings(context, layout, tool):
        panel_general(context, layout, tool)

    def draw_cursor(_context, tool, xy):
        x = _context.area.x
        y = _context.area.y
        var.MOUSE_POSITION = [xy[0]-x, xy[1]-y]


class GizmoPRO_Curve(WorkSpaceTool):
    bl_space_type = 'VIEW_3D'
    bl_context_mode = 'EDIT_CURVE'

    bl_idname = "tool.gizmo_pro"
    bl_label = "Gizmo PRO"
    bl_description = ""
    bl_icon = os.path.join(os.path.dirname(__file__), './icons/dat', 'gp.general')
    bl_widget = None
    bl_keymap = get_keymap_tools()
    bl_cursor = 'PAINT_CROSS'

    def draw_settings(context, layout, tool):
        panel_general(context, layout, tool)

    def draw_cursor(_context, tool, xy):
        x = _context.area.x
        y = _context.area.y
        var.MOUSE_POSITION = [xy[0]-x, xy[1]-y]


class GizmoPRO_Surface(WorkSpaceTool):
    bl_space_type = 'VIEW_3D'
    bl_context_mode = 'EDIT_SURFACE'

    bl_idname = "tool.gizmo_pro"
    bl_label = "Gizmo PRO"
    bl_description = ""
    bl_icon = os.path.join(os.path.dirname(__file__), './icons/dat', 'gp.general')
    bl_widget = None
    bl_keymap = get_keymap_tools()
    bl_cursor = 'PAINT_CROSS'

    def draw_settings(context, layout, tool):
        panel_general(context, layout, tool)

    def draw_cursor(_context, tool, xy):
        x = _context.area.x
        y = _context.area.y
        var.MOUSE_POSITION = [xy[0]-x, xy[1]-y]


class GizmoPRO_EditArmature(WorkSpaceTool):
    bl_space_type = 'VIEW_3D'
    bl_context_mode = 'EDIT_ARMATURE'

    bl_idname = "tool.gizmo_pro"
    bl_label = "Gizmo PRO"
    bl_description = ""
    bl_icon = os.path.join(os.path.dirname(__file__), './icons/dat', 'gp.general')
    bl_widget = None
    bl_keymap = get_keymap_tools()
    bl_cursor = 'CROSSHAIR'

    def draw_settings(context, layout, tool):
        panel_general(context, layout, tool)

    def draw_cursor(_context, tool, xy):
        x = _context.area.x
        y = _context.area.y
        var.MOUSE_POSITION = [xy[0]-x, xy[1]-y]


class GizmoPRO_Pose(WorkSpaceTool):
    bl_space_type = 'VIEW_3D'
    bl_context_mode = 'POSE'

    bl_idname = "tool.gizmo_pro"
    bl_label = "Gizmo PRO"
    bl_description = ""
    bl_icon = os.path.join(os.path.dirname(__file__), './icons/dat', 'gp.general')
    bl_widget = None
    bl_keymap = get_keymap_tools()
    bl_cursor = 'CROSSHAIR'

    def draw_settings(context, layout, tool):
        panel_general(context, layout, tool)

    def draw_cursor(_context, tool, xy):
        x = _context.area.x
        y = _context.area.y
        var.MOUSE_POSITION = [xy[0]-x, xy[1]-y]


class GizmoPRO_Sculpt(WorkSpaceTool):
    bl_space_type = 'VIEW_3D'
    bl_context_mode = 'SCULPT'

    bl_idname = "tool.gizmo_pro"
    bl_label = "Gizmo PRO"
    bl_description = ""
    bl_icon = os.path.join(os.path.dirname(__file__), './icons/dat', 'gp.general')
    bl_widget = None
    bl_keymap = get_keymap_tools()
    bl_cursor = 'CROSSHAIR'

    def draw_settings(context, layout, tool):
        panel_general(context, layout, tool)

    def draw_cursor(_context, tool, xy):
        x = _context.area.x
        y = _context.area.y
        var.MOUSE_POSITION = [xy[0]-x, xy[1]-y]


class Transpose(WorkSpaceTool):
    bl_space_type = 'VIEW_3D'
    bl_context_mode = 'SCULPT'

    bl_idname = "tool.transpose_pro"
    bl_label = "Transpose"
    bl_description = ""
    bl_icon = os.path.join(os.path.dirname(__file__), './icons/dat', 'gp.measure')
    bl_widget = None
    bl_keymap = get_keymap_tools()
    bl_cursor = 'PAINT_CROSS'

    def draw_settings(context, layout, tool):
        icoll = icons_collections['main']
        settings_icon = icoll['settings'].icon_id
        info_icon = icoll['info'].icon_id

        props = get_prefs()
        settings = context.scene.GP_scene_set

        scaleX = 1.3
        #row_all = layout.row()
        

        if layout.direction == 'HORIZONTAL':
            row = layout.row(align=True)
            row.scale_x = scaleX
            #row.operator( 'gp.documentation', text = '', icon_value = info_icon )
            row.operator( 'gp.panel_settings', text = '', icon_value = settings_icon)
            
            
            # --- Space Drag / Select Tools
            row = layout.row(align=True)
            if props.space_drag_lmb=='ORBIT':
                row.prop(settings, 'lock_orbit', text='', icon='LOCKED' if settings.lock_orbit else 'UNLOCKED')
      


            # --- Select Tools
            #icon = select_box_icon.icon_id 
            row.popover( panel = 'GP_PT_settings_space_drag', text = '', icon = 'MOUSE_LMB_DRAG' )


            # --- Visible
            row = layout.row(align=True)
            row.scale_x = scaleX
            row.prop(settings, 'show_gizmo', text='', icon='HIDE_OFF' if settings.show_gizmo else 'HIDE_ON')
            row.separator()
            row.prop(settings, 'gzm_type', text='', expand=True, icon_only=True)

            sub = row.row(align=True)
            sub.scale_x = scaleX
            sub.scale_x = 1.0
            sub.popover(panel='GP_PT_visible', text='')
            row.scale_x = 1.0


            scene = context.window.scene
            orient_slot = scene.transform_orientation_slots[0]
            layout.prop_with_popover( orient_slot, 'type', text = '', panel='VIEW3D_PT_transform_orientations' )



            # --- Длина Линейки
            box = layout.box()
            dim = Vector(settings.tr_p1) - Vector(settings.tr_p2) 
            len = dim.length
            value, unit = convert_units(context, len)
            len_str = str( round(value, 3)  )
            box.label( text = len_str + unit, icon='SNAP_INCREMENT' )

    def draw_cursor(_context, tool, xy):
        x = _context.area.x
        y = _context.area.y
        var.MOUSE_POSITION = [xy[0]-x, xy[1]-y]


class XForm(WorkSpaceTool):
    bl_space_type = 'VIEW_3D'
    bl_context_mode = 'OBJECT'

    bl_idname = "tool.xform_pro"
    bl_label = "Transform"
    bl_description = ""
    bl_icon = os.path.join(os.path.dirname(__file__), './icons/dat', 'gp.transform_xform')
    bl_widget = 'VIEW3D_GGT_xform_gizmo'
    bl_keymap = get_keymap_tools()
    bl_cursor = 'CROSSHAIR'

    def draw_settings(context, layout, tool):
        panel_tool(context, layout, tool)

        row = layout.row(align=True)
        row.prop(context.space_data, 'show_gizmo_object_translate', text='Move', toggle=True)
        row.prop(context.space_data, 'show_gizmo_object_rotate', text='Rotate', toggle=True)
        row.prop(context.space_data, 'show_gizmo_object_scale', text='Scale', toggle=True)

    def draw_cursor(_context, tool, xy):
        x = _context.area.x
        y = _context.area.y
        var.MOUSE_POSITION = [xy[0]-x, xy[1]-y]


class ScaleCage(WorkSpaceTool):
    bl_space_type = 'VIEW_3D'
    bl_context_mode = 'OBJECT'

    bl_idname = "tool.xform_cage_pro"
    bl_label = "Scale Cage"
    bl_description = ""
    bl_icon = os.path.join(os.path.dirname(__file__), './icons/dat', 'gp.cage_xform')
    bl_widget = 'VIEW3D_GGT_xform_cage'
    bl_keymap = get_keymap_tools()
    bl_cursor = 'CROSSHAIR'

    def draw_settings(context, layout, tool):
        panel_tool(context, layout, tool)

        row = layout.row(align=True)
        row.prop(context.space_data, 'show_gizmo_object_translate', text='Move', toggle=True)
        row.prop(context.space_data, 'show_gizmo_object_rotate', text='Rotate', toggle=True)
        row.prop(context.space_data, 'show_gizmo_object_scale', text='Scale', toggle=True)

    def draw_cursor(_context, tool, xy):
        x = _context.area.x
        y = _context.area.y
        var.MOUSE_POSITION = [xy[0]-x, xy[1]-y]


gizmoPro_tool = ['OBJECT', 'EDIT_MESH', 'EDIT_CURVE', 'EDIT_SURFACE', 'EDIT_ARMATURE', 'POSE', 'SCULPT']
xform_tool = ['OBJECT', 'EDIT_MESH', 'EDIT_CURVE', 'EDIT_SURFACE', 'EDIT_METABALL', 'EDIT_LATTICE', 'EDIT_GPENCIL', 'EDIT_ARMATURE', 'POSE']


GizmoProTools = (
    {'tool': GizmoPRO_Object, 'after': {"builtin.select"} , 'separator': True, 'group': True},
    {'tool': GizmoPRO_Mesh, 'after': {"builtin.select"}, 'separator': True, 'group': True},
    {'tool': GizmoPRO_Curve, 'after': {"builtin.select"} , 'separator': True, 'group': True},
    {'tool': GizmoPRO_Surface, 'after': {"builtin.select"} , 'separator': True, 'group': True},
    {'tool': GizmoPRO_EditArmature  , 'after': {"builtin.select"} , 'separator': True, 'group': True},
    {'tool': GizmoPRO_Pose, 'after': {"builtin.select"} , 'separator': True, 'group': True},
    {'tool': Transpose, 'after': {"builtin.transform"} , 'separator': True, 'group': True},
)
#{'tool': GizmoPRO_Sculpt, 'after': {"builtin.transform"} , 'separator': True, 'group': True},