import bpy
from bpy.types import Panel, Menu
from .utils.poll import active_tool
from .icons import icons_collections
from bpy.props import EnumProperty
from .utils.func import (
    rotate_method_ui,
    increment_rotate_ui,
    transform_orientation,
    transform_pivot_point,
)
from bl_ui.properties_paint_common import UnifiedPaintPanel
from . import get_prefs


# --- Панель настроек инструмента выбора
class GP_PT_settings_space_drag(Panel):
    bl_label = 'Space Drag Operation'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_description = 'Setting operators for the mouse buttons active during the tweak'
    
    
    def draw(self, context):
        tool = context.workspace.tools.from_space_view3d_mode( context.mode, create = False )
        props = get_prefs()
        settings = context.scene.GP_scene_set
        layout = self.layout



        row = layout.row()
        row.scale_x = 2.5
        row.prop( settings, 'select_settings_tab', expand = True, icon_only = True )
        row.prop( props, 'sel_xray', text = '', icon = 'XRAY' )
 


        if settings.select_settings_tab == 'LMB':
            sd = props.space_drag_lmb
            layout.prop( props, 'space_drag_lmb')

        elif settings.select_settings_tab == 'RMB':
            sd = props.space_drag_rmb
            layout.prop( props, 'space_drag_rmb')
        
        
        if sd == 'BOX_SEL':
            BST = tool.operator_properties('view3d.select_box')
            box = layout.box()
            box.label(text='Select Box', icon='SELECT_SET')
            row = box.row()
            row.scale_x = 2
            row.scale_y = 1.5
            row.prop(BST, 'mode', expand=True, icon_only=True)


        elif sd == 'CIRCLE_SEL':
            CST = tool.operator_properties('view3d.select_circle')
            box = layout.box()
            box.label(text='Select Circle', icon='PROP_OFF')
            box.prop(CST, 'radius')
            row = box.row()
            row.scale_x = 2
            row.scale_y = 1.5
            row.prop(CST, 'mode', expand=True, icon_only=True)
        

        elif sd == 'LASSO_SEL':
            LST = tool.operator_properties('view3d.select_lasso')
            box = layout.box()
            box.label(text='Select Lasso', icon='MOD_CAST')
            row = box.row()
            row.scale_x = 2
            row.scale_y = 1.5
            row.prop(LST, 'mode', expand=True, icon_only=True)

        rotate_method_ui(context, layout)


# --- Панель для выбора видимости элементов гизмо
class GP_PT_visible(Panel):
    bl_label = 'Visible Elements'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_description = 'Selection of visible elements of the gizmo'


    def draw(self, context):

        settings = context.scene.GP_scene_set

        layout = self.layout


        # --- All
        if settings.gzm_type=='GIZMOPRO':
            row = layout.row()
            row.prop( settings, 'dot_move', text = 'Dot Move' )
            """ if context.mode == 'EDIT_MESH':
                row.prop( settings, 'dot_retop', text = '', icon = 'LATTICE_DATA' ) """

            layout.separator()
            layout.prop(settings, 'tra_gizmo', text='Move') # , toggle=True
            layout.prop(settings, 'rot_gizmo', text='Rotate')
            layout.prop(settings, 'sca_gizmo', text='Scale')

            layout.separator()
            layout.prop(settings, 'rotate_trackball',text='Trackball')
            layout.prop(settings, 'rotate_view',text='Rotate View')
            layout.prop(settings, 'scale_view',text='Scale View')
            layout.prop(settings, 'blank_gizmo',text='Move Plane')


        elif settings.gzm_type=='LUNA':
            row = layout.row()
            layout.prop(settings, 'tra_gizmo')
            layout.prop(settings, 'rot_gizmo')
            layout.prop(settings, 'sca_gizmo')
            if settings.rot_gizmo:
                layout.prop(settings, 'rotate_trackball')
        
        elif settings.gzm_type=='MALIBU':
            row = layout.row()
            layout.prop(settings, 'tra_gizmo')
            layout.prop(settings, 'rot_gizmo')
            layout.prop(settings, 'sca_gizmo')
            if settings.rot_gizmo:
                layout.prop(settings, 'rotate_trackball')

        elif settings.gzm_type=='MIRROR':
            layout.prop( settings, 'mirror_use_modifier', icon='MODIFIER' )
            layout.operator('gp.mirror_modifier_apply_remove', icon='CHECKMARK')
            layout.operator('gp.mirror_modifier_apply_remove', text='Remove Modifier(s)', icon='PANEL_CLOSE').remove = True



class GP_PT_other_visible(Panel):
    bl_label = 'Visible Elements'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_description = 'Selection of visible elements of the gizmo'
    #bl_ui_units_x = 13

    def draw(self, context):
        settings = context.scene.GP_scene_set
        layout = self.layout
        #layout.use_property_split = True
        # --- 3D Cursor
        if settings.cursor_gizmo:
            col = layout.column(heading='3D Cursor', align=True)
            col.prop(settings, 'cursor_gizmo_dot')
            if settings.cursor_gizmo_dot:
                col.prop(settings, 'cursor_navigator')
            col.prop(settings, 'cursor_gizmo_axes')
            layout.separator()

        # --- Border Navigation
        if settings.bn_gizmo:
            col = layout.column(heading='Border Navigate', align=True)
            col.prop(settings, 'bn_sculpt_retop', text='Sculpt/Retopology Mode')
            layout.separator()

        increment_rotate_ui(context, layout)



# --- Increment Angle
class GP_PT_angle_ia(Panel):
    bl_label = 'Angle'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_description = ''

    def draw(self, context):
        settings = context.scene.GP_scene_set
        layout = self.layout
        layout.column().prop( settings, 'preset_ia', expand=True )
        layout.prop( settings, 'ia', text = 'Angle' )
        


# --- Pie Menu
class GP_MT_pie(Menu):
    bl_idname = 'GP_MT_pie'
    bl_label = 'Gizmo PRO Pie Menu'

    @classmethod
    def poll(cls, context):
        modes = ['OBJECT', 'EDIT_MESH', 'EDIT_CURVE', 'EDIT_SURFACE', 'EDIT_ARMATURE']
        return context.mode in modes
        
    def draw(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set
        obj = context.active_object
        tool_settings = context.tool_settings
        object_mode = 'OBJECT' if obj is None else obj.mode
        has_pose_mode = (
            (object_mode == 'POSE') or
            (object_mode == 'WEIGHT_PAINT' and context.pose_object is not None)
        )
        
        layout = self.layout
        pie = layout.menu_pie()
        pie.scale_x = 1.5
        pie.scale_y = 1.5
        


        # --- 1 --- Snap
        show_snap = False
        if obj is None:
            show_snap = True
        else:
            if (object_mode not in {
                    'SCULPT', 'SCULPT_CURVES', 'VERTEX_PAINT', 'WEIGHT_PAINT', 'TEXTURE_PAINT',
                    'PAINT_GPENCIL', 'SCULPT_GPENCIL', 'WEIGHT_GPENCIL', 'VERTEX_GPENCIL',
            }) or has_pose_mode:
                show_snap = True
            else:
                paint_settings = UnifiedPaintPanel.paint_settings(context)
                if paint_settings:
                    brush = paint_settings.brush
                    if brush and hasattr(brush, "stroke_method") and brush.stroke_method == 'CURVE':
                        show_snap = True

        if show_snap:
            row = pie.row(align=True)
            if props.pie_advanced_options:
                row.popover(panel='VIEW3D_PT_snapping', text='')
                row.separator(factor=0.2)
            row.prop(context.scene.tool_settings, 'snap_elements', text='')
            """ row.prop_enum(context.scene.tool_settings, 'snap_elements', 'FACE_NEAREST', text='')
            row.prop_enum(context.scene.tool_settings, 'snap_elements', 'FACE_PROJECT', text='')
            row.separator(factor=0.2)
            row.prop_enum(context.scene.tool_settings, 'snap_elements', 'EDGE_PERPENDICULAR', text='')
            row.prop_enum(context.scene.tool_settings, 'snap_elements', 'EDGE_MIDPOINT', text='')
            row.prop_enum(context.scene.tool_settings, 'snap_elements', 'VOLUME', text='')
            row.separator(factor=0.2)
            row.prop_enum(context.scene.tool_settings, 'snap_elements', 'FACE', text='')
            row.prop_enum(context.scene.tool_settings, 'snap_elements', 'EDGE', text='')
            row.prop_enum(context.scene.tool_settings, 'snap_elements', 'VERTEX', text='')
            row.separator(factor=0.2)
            row.prop_enum(context.scene.tool_settings, 'snap_elements', 'INCREMENT', text='') """
            row.separator(factor=0.2)
            row.prop(tool_settings, 'use_snap', text='')
            
        else:
            pie.separator()

        
        # --- 2 --- Transform Pivot Point
        row = pie.row(align=True)
        row.prop_enum(context.tool_settings, 'transform_pivot_point', 'CURSOR', text='')
        row.separator(factor=0.2)
        row.prop_enum(context.tool_settings, 'transform_pivot_point', 'ACTIVE_ELEMENT', text='')
        row.prop_enum(context.tool_settings, 'transform_pivot_point', 'MEDIAN_POINT', text='')
        row.prop_enum(context.tool_settings, 'transform_pivot_point', 'INDIVIDUAL_ORIGINS', text='')
        row.prop_enum(context.tool_settings, 'transform_pivot_point', 'BOUNDING_BOX_CENTER', text='')
        if props.pie_advanced_options:
            row.separator(factor=0.2)
            row.popover(panel='GP_PT_transform_positions', text='')


        # --- 3 --- Other Panels
        if 'Poly_Source' in context.preferences.addons.keys():
            row = pie.row()
            row.popover(panel='PS_PT_tool_kit', icon='TOOL_SETTINGS')
        else:
            pie.separator()
            

        # --- 4 --- Visible Gizmos
        col = pie.column(align=True)

        row = col.row(align=True)
        row.alignment = 'RIGHT'
        row.prop(context.space_data, 'show_gizmo_object_translate', text='', icon='OBJECT_ORIGIN', toggle=True)
        row.prop(context.space_data, 'show_gizmo_object_rotate', text='', icon='ORIENTATION_GIMBAL', toggle=True)
        row.prop(context.space_data, 'show_gizmo_object_scale', text='', icon='RESTRICT_INSTANCED_ON', toggle=True)
        row.separator(factor=0.2)
        row.prop(settings, 'scale_gizmo', text='', icon='FULLSCREEN_ENTER')
        row.separator(factor=0.2)
        row.prop(settings, 'cursor_gizmo', text='', icon='CURSOR')
        row.prop(settings, 'camera_gizmo', text='', icon='CAMERA_DATA')
        row.prop(settings, 'searcher_gizmo', text='', icon='PMARKER_SEL')
        row.prop(settings, 'bn_gizmo', text='', icon='SHADING_BBOX')
        row.prop(settings, 'qw', text='', icon='PROP_CON')
        row.separator(factor=0.2)
        row.popover(panel='GP_PT_other_visible', text='')

        col.separator()

        row = col.row(align=True)
        row.alignment = 'RIGHT'
        row.prop(settings, 'show_gizmo', text='', icon='HIDE_OFF' if settings.show_gizmo else 'HIDE_ON')
        row.separator(factor=0.2)
        #row.prop(settings, 'gzm_type', text='', expand=True, icon_only=True)
        row.prop_enum(settings, 'gzm_type', 'TRANSPOSE', text='')
        row.prop_enum(settings, 'gzm_type', 'VR', text='')
        row.prop_enum(settings, 'gzm_type', 'MALIBU', text='')
        row.prop_enum(settings, 'gzm_type', 'LUNA', text='')
        row.prop_enum(settings, 'gzm_type', 'GIZMOPRO', text='')
        
        
        
        
        row.separator(factor=0.2)
        row.prop_enum(settings, 'gzm_type', 'PLACEMENT', text='')
        row.prop_enum(settings, 'gzm_type', 'MIRROR', text='')
        row.prop_enum(settings, 'gzm_type', 'SHEAR', text='')
        row.separator(factor=0.2)
        row.popover(panel='GP_PT_visible', text='')


        # --- 5 --- Proportional editing
        if object_mode in {'EDIT', 'PARTICLE_EDIT', 'SCULPT_GPENCIL', 'EDIT_GPENCIL', 'OBJECT'}:
            row = pie.row(align=True)
            kw = {}
            if object_mode == 'OBJECT':
                attr = "use_proportional_edit_objects"
            else:
                attr = "use_proportional_edit"

                if tool_settings.use_proportional_edit:
                    if tool_settings.use_proportional_connected:
                        kw["icon"] = 'PROP_CON'
                    elif tool_settings.use_proportional_projected:
                        kw["icon"] = 'PROP_PROJECTED'
                    else:
                        kw["icon"] = 'PROP_ON'
                else:
                    kw["icon"] = 'PROP_OFF'

            sub = row.row(align=True)
            if props.pie_advanced_options:
                pop = sub.row()
                pop.scale_x = 1.0
                pop.popover(panel='VIEW3D_PT_proportional_edit', text='')
            sub.active = getattr(tool_settings, attr)
            sub.separator(factor=0.2)
            sub.prop(tool_settings, 'proportional_edit_falloff', text='', expand=True, icon_only=True)
            sub.separator(factor=0.2)
            sub.prop(tool_settings, attr, icon_only=True, **kw)
        else:
            pie.separator()


        # --- 6 --- Transform Orientation
        row = pie.row(align=True)
        #row.prop(context.scene.transform_orientation_slots[0], 'type', text='', expand=True)
        row.prop_enum(context.scene.transform_orientation_slots[0], 'type', 'CURSOR', text='')
        row.separator(factor=0.2)
        row.prop_enum(context.scene.transform_orientation_slots[0], 'type', 'GLOBAL', text='')
        row.prop_enum(context.scene.transform_orientation_slots[0], 'type', 'LOCAL', text='')
        row.prop_enum(context.scene.transform_orientation_slots[0], 'type', 'NORMAL', text='')
        row.prop_enum(context.scene.transform_orientation_slots[0], 'type', 'GIMBAL', text='')
        row.prop_enum(context.scene.transform_orientation_slots[0], 'type', 'VIEW', text='')
        row.prop_enum(context.scene.transform_orientation_slots[0], 'type', 'PARENT', text='')
        if props.pie_advanced_options:
            row.separator(factor=0.2)
            row.popover(panel='GP_PT_transform_orientations', text='')
        

        # --- 7 --- View
        row = pie.row(align=True)
        row.popover(panel='OBJECT_PT_display', text='', icon='RESTRICT_VIEW_OFF')
        row.separator(factor=0.2)
        if context.mode == 'EDIT_MESH':
            row.popover(panel='VIEW3D_PT_tools_meshedit_options', text='', icon='OPTIONS')
            row.separator(factor=0.2)
            row.prop(context.object, 'use_mesh_mirror_x', text='', icon='EVENT_X', toggle=True)
            row.prop(context.object, 'use_mesh_mirror_y', text='', icon='EVENT_Y', toggle=True)
            row.prop(context.object, 'use_mesh_mirror_z', text='', icon='EVENT_Z', toggle=True)
            row.separator(factor=0.2)
            row.prop(context.scene.tool_settings, 'use_mesh_automerge', text='')
        elif context.mode == 'OBJECT':
            #row.popover(panel='VIEW3D_PT_tools_object_options', text='', icon='OPTIONS')
            #row.separator(factor=0.2)
            row.prop(tool_settings, 'use_transform_data_origin', text='', icon='OBJECT_ORIGIN', toggle=True)
            row.separator(factor=0.2)
            row.prop(tool_settings, 'use_transform_pivot_point_align', text='', icon='PIVOT_INDIVIDUAL', toggle=True)
            row.separator(factor=0.2)
            row.prop(tool_settings, 'use_transform_skip_children', text='', icon='ORIENTATION_PARENT', toggle=True)

        # --- 8 --- 3D Cursor
        row = pie.row(align=True)
        row.operator( 'gp.cursor_to_active', text='', icon='RESTRICT_SELECT_OFF' )
        row.separator(factor=0.2)
        op = row.operator('gp.reset_cursor', text='Reset')
        op.loc = True
        op.rot = True
        row.separator(factor=0.2)
        pop = row.row()
        pop.popover(panel='GP_PT_3d_cursor', text='')



# --- QW
sca_x = 9
sca_y = 1.2


class GP_PT_transform_orientations(Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'HEADER'
    bl_label = 'Transform Orientations'
    bl_ui_units_x = sca_x

    def draw(self, context):
        settings = context.scene.GP_scene_set

        layout = self.layout
        layout.scale_y = sca_y

        scene = context.scene
        orient_slot = scene.transform_orientation_slots[0]
        #ori = orient_slot.custom_orientation

        row = layout.row(align=True)
        row.scale_x = 20
        row.active = settings.orientation == 'NONE'
        row.prop_enum(orient_slot, 'type', 'GLOBAL', text='')
        row.prop_enum(orient_slot, 'type', 'LOCAL', text='')
        row.prop_enum(orient_slot, 'type', 'NORMAL', text='')
        row.prop_enum(orient_slot, 'type', 'GIMBAL', text='')
        row.prop_enum(orient_slot, 'type', 'VIEW', text='')
        row.prop_enum(orient_slot, 'type', 'CURSOR', text='')
        row.prop_enum(orient_slot, 'type', 'PARENT', text='')
        
        col = layout.column()
        col.active = settings.orientation != 'NONE'
        col.prop(settings, 'orientation', expand=True) # settings.orientation != 'NONE'



class GP_PT_transform_positions(Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'HEADER'
    bl_label = 'Transform Positions'
    bl_ui_units_x = sca_x

    def draw(self, context):
        settings = context.scene.GP_scene_set

        layout = self.layout
        layout.scale_y = sca_y
        
        tool_settings = context.tool_settings
        row = layout.row(align=True)
        row.scale_x = 20
        row.active = settings.position == 'NONE'
        row.prop_enum(tool_settings, 'transform_pivot_point', 'ACTIVE_ELEMENT', text='')
        row.prop_enum(tool_settings, 'transform_pivot_point', 'MEDIAN_POINT', text='')
        row.prop_enum(tool_settings, 'transform_pivot_point', 'INDIVIDUAL_ORIGINS', text='')
        row.prop_enum(tool_settings, 'transform_pivot_point', 'CURSOR', text='')
        row.prop_enum(tool_settings, 'transform_pivot_point', 'BOUNDING_BOX_CENTER', text='')
        #row.prop( tool_settings, 'transform_pivot_point', expand = True, icon_only = True, emboss = settings.position == 'NONE')

        col = layout.column()
        col.active = settings.position != 'NONE'
        col.prop(settings, 'position', expand=True) # , emboss = settings.position != 'NONE'









class GP_PT_3d_cursor(Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'HEADER'
    bl_label = '3D Cursor'
    bl_ui_units_x = sca_x

    def draw(self, context):
        layout = self.layout
        layout.scale_y = sca_y
        col = layout.column()

        row = col.row( align = True )
        row.label( text = 'Reset' )
        op = row.operator( 'gp.reset_cursor', text = 'Location' )
        op.loc = True
        op.rot = False
        op = row.operator( 'gp.reset_cursor', text = 'Rotation' )
        op.loc = False
        op.rot = True
        row = row.row()
        op = row.operator( 'gp.reset_cursor', text = 'All' )
        op.loc = True
        op.rot = True

        col.separator()
        col.operator( 'gp.cursor_to_active', text = 'To Active', icon = 'RESTRICT_SELECT_OFF' )
        col.operator( 'gp.align_from_view', text = 'Align From View', icon = 'RESTRICT_VIEW_OFF' )

        col.separator()
        col.operator( 'view3d.snap_selected_to_cursor', text = 'Select To Cursor' )
        col.operator( 'view3d.snap_selected_to_cursor', text = 'Select To Cursor(Keep Offset)' ).use_offset = True



class GP_PT_pivot_transform(Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'HEADER'
    bl_label = 'Pivot Transform'
    bl_ui_units_x = sca_x

    def draw(self, context):
        layout = self.layout
        layout.scale_y = sca_y
        col = layout.column()

        layout.label( text = 'Standard Panel' )



# --- Paint
class GP_PT_color_picker(Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'HEADER'
    bl_label = 'Color Picker'
    bl_ui_units_x = 8


    @classmethod
    def poll(cls, context):
        settings = context.tool_settings.image_paint
        brush = settings.brush
        capabilities = brush.image_paint_capabilities

        return capabilities.has_color

    def draw(self, context):
        from bl_ui.properties_paint_common import draw_color_settings

        layout = self.layout
        settings = context.tool_settings.image_paint
        brush = settings.brush

        draw_color_settings(context, layout, brush, color_type=True)





# --- Это для отображения параметров в панели видимости гизмо 3d View
def gizmo_pro_global(self, context):
    props = get_prefs()
    settings = context.scene.GP_scene_set
    #self.layout.prop(props, 'gzm_visible') # gzm_global
    layout = self.layout
    
    box = layout.box()
    row = box.row()
    row.label( text='Gizmo PRO' )
    row.prop(settings, 'show_gizmo', text='', icon='HIDE_OFF' if settings.show_gizmo else 'HIDE_ON')
    row.operator( 'gp.panel_settings', text='', icon='PREFERENCES' )
    
    if settings.show_gizmo:
        row = box.row()
        row.prop( settings, 'gzm_type', text='' )
        row.popover( panel='GP_PT_visible', text='', icon='PRESET' )
       

        row = box.row()
        row.prop( settings, 'qw' )
        """ if props.qw_pin:
            row.prop( props, 'qw_pin', text = '', icon = 'PINNED' )
        else:
            row.prop( props, 'qw_pin', text = '', icon = 'UNPINNED' ) """

        box.prop(settings, 'scale_gizmo')

        row = box.row(align=True)
        row.prop(settings, 'cursor_gizmo')
        if settings.cursor_gizmo:
            row.prop( settings, 'cursor_gizmo_dot', text='', icon='ANTIALIASED' )
            row.prop( settings, 'cursor_gizmo_axes', text='', icon='OBJECT_ORIGIN' )
  

        
        row = box.row(align=True)
        row.prop(settings, 'bn_gizmo')
        if settings.bn_gizmo:
            row.prop( settings, 'bn_sculpt_retop', text='', icon='SCULPTMODE_HLT' )


        box.prop(settings, 'camera_gizmo')
        box.prop(settings, 'searcher_gizmo')
        #box.prop(settings, 'empty_gizmo')
        if context.mode == 'EDIT_MESH':
            box.prop(settings, 'preselect_gizmo', text='Preselect (TEST)' )

        box.prop(settings, 'gizmo_2d_navigate')
        #if props.test:
        """ row = box.row()
        row.prop( settings, 'refboard' )
        if settings.refboard:
            row.prop( settings, 'rb_edit', text='', icon='EDITMODE_HLT' ) """





classes = [
    GP_PT_settings_space_drag,
    GP_PT_visible,
    GP_PT_other_visible,
    GP_PT_angle_ia,
    GP_MT_pie,

    GP_PT_transform_orientations,
    GP_PT_transform_positions,
    GP_PT_3d_cursor,
    GP_PT_pivot_transform,
    GP_PT_color_picker,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    bpy.types.VIEW3D_PT_gizmo_display.prepend(gizmo_pro_global)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
    bpy.types.VIEW3D_PT_gizmo_display.remove(gizmo_pro_global)