import bpy
from mathutils import Vector
import bmesh

from .math import (
    sub_v3_v3v3,
    add_v3_v3v3,
    cross_v3_v3v3,
    mid_v3_v3v3,
    sub_v3_v3,
    len_squared_v3v3,
    len_v3v3,
    axis_sort_v3,
)



# --- bmesh_polygon
def vert_tri_find_unique_edge(verts):
    difs = Vector()

    i_next = 0
    while i_next < 3:
        if i_next == 0:
            i_prev = 1
            i_curr = 2

        elif i_next == 1:
            i_prev = 2
            i_curr = 0

        elif i_next == 2:
            i_prev = 0
            i_curr = 1


        co = verts[i_curr].co
        co_other = [ verts[i_prev].co, verts[i_next].co ]

         
       
        proj_dir = mid_v3_v3v3(co_other[0], co_other[1])
        sub_v3_v3(proj_dir, co)

        proj_pair = [ Vector(), Vector() ]
        proj_pair[0] = co_other[0].project(proj_dir)
        proj_pair[1] = co_other[1].project(proj_dir)
        
        difs[i_next] = len_squared_v3v3(proj_pair[0], proj_pair[1])

        i_next += 1


    ''' lens = Vector(
        (
            len_v3v3(verts[0].co, verts[1].co),
            len_v3v3(verts[1].co, verts[2].co),
            len_v3v3(verts[2].co, verts[0].co),
        )
    )

    difs = Vector(
        (
            abs(lens[1] - lens[2]),
            abs(lens[2] - lens[0]),
            abs(lens[0] - lens[1]),
        )
    ) '''

    order = Vector((0,1,2))
    axis_sort_v3(difs, order)
     
    return order[0]


def vert_tri_calc_tangent_edge(verts): # BM
    i = vert_tri_find_unique_edge(verts)
    i = int(i)

    r_tangent = Vector()
    sub_v3_v3v3(r_tangent, verts[i].co, verts[(i + 1) % 3].co)

    r_tangent.normalize()
    return r_tangent


def face_as_array_vert_tri(f):
    r_verts = []
    l = f.loops[0]
    
    r_verts.append(l.vert)

    l = l.link_loop_next
    r_verts.append(l.vert)
    
    l = l.link_loop_next
    r_verts.append(l.vert)
    
    return r_verts


def face_calc_tangent_auto(f): # BM
    r_tangent = Vector()
    if len(f.verts) == 3:
        verts = face_as_array_vert_tri(f)
        r_tangent = vert_tri_calc_tangent_edge(verts)
        
    elif len(f.verts) == 4:
        r_tangent = f.calc_tangent_edge_pair()
    
    else:
        r_tangent = f.calc_tangent_edge()

    return r_tangent


def editselection_normal(sha):
    select_mode = bpy.context.scene.tool_settings.mesh_select_mode
    r_normal = Vector()
    
    if select_mode[0] or select_mode[2]:
        r_normal = sha.normal
   
    elif select_mode[1]:
        v1 = sha.verts[0]
        v2 = sha.verts[1]
        normal = add_v3_v3v3(v1.normal, v2.normal)

        plane = Vector()
        sub_v3_v3v3(plane, v2.co, v1.co)

        vec = Vector()
        cross_v3_v3v3(vec, normal, plane)
        cross_v3_v3v3(r_normal, plane, vec)
        r_normal.normalize()

    return r_normal


def editselection_plane(ese):
    #select_mode = bpy.context.scene.tool_settings.mesh_select_mode
    r_plane = Vector()
    if isinstance(ese, bmesh.types.BMVert):
        vec = Vector()

        ''' if (ese->prev)
        BM_editselection_center(ese->prev, vec) # TODO BM_editselection_plane bmesh_marking.c 886
        sub_v3_v3v3(r_plane, vec, eve->co) '''
        
        if ese.normal[0] < 0.5:
            vec[0] = 1.0
        
        elif ese.normal[1] < 0.5:
            vec[1] = 1.0
    
        else:
            vec[2] = 1.0
        
        cross_v3_v3v3(r_plane, ese.normal, vec)
    
        r_plane.normalize()


    elif isinstance(ese, bmesh.types.BMEdge):
        v1 = ese.verts[0]
        v2 = ese.verts[1]

        if ese.is_boundary:
            sub_v3_v3v3(r_plane, v1, v2)

        else:
            if v2.co[1] > v1.co[1]:
                sub_v3_v3v3(r_plane, v2.co, v1.co)
            else:
                sub_v3_v3v3(r_plane, v1.co, v2.co)

        r_plane.normalize()


    elif isinstance(ese, bmesh.types.BMFace):
        r_plane = face_calc_tangent_auto(ese)
        #r_plane.orthogonal()

    return r_plane



# --- bmesh_marking
# --- Get the active mesh element (with active-face fallback)
def select_history_active_get(bm):
    ese_last = bm.select_history.active
    bm.faces.ensure_lookup_table()
    efa = bm.faces.active


    
    #print(ese_last)
    #print(efa)
    if ese_last:
        # --- If there is an active face, use it over the last selected face
        if isinstance(ese_last, bmesh.types.BMFace):
            if efa:
                return efa
            else:
                return ese_last
        else:
            return ese_last

    elif efa:
        # --- no edit-selection, fallback to active face
        sv = set([v for v in bm.verts if v.select])
        svf = set([v for v in efa.verts])
        
        if svf.issubset(sv):
            return efa
        else:
            return None
    
    else:
        return None

