import bpy
from mathutils import Vector
from bl_math import smoothstep, clamp
from math import pi


def dot(matrix, use_negative=False, loc=None):
    # --- if loc is None, then matrix is a 4x4 matrix
    # --- if loc is not None, then matrix is Quaternion
    context = bpy.context
    region_data = context.region_data
    if loc:
        l = loc
        q = matrix
    else:
        l, q, s = matrix.decompose()
    if region_data.is_perspective:
        vL = region_data.view_matrix.inverted().translation
        vv = vL - l
    else:
        vv = region_data.view_rotation @ Vector((0, 0, 1))
    norm = q @ Vector((0, 0, 1))
    dot = norm.angle(vv)
    nDeg = pi / 2
    if dot > nDeg:
        if use_negative:
            dot =  -(pi - dot)
        else:
            dot =  pi - dot
    return dot


def alpha_gizmo(matrix, props):
    d = clamp(dot(matrix), 0, 1)
    a = clamp(smoothstep(0, props.alpha_gizmo, d), 0, props.alpha_gizmo)

    if a < 0.2:
        hs = True
        if d < 0.1:
            a = 0.0
    else:
        hs = False
    return a, hs


def alpha_arc_gizmo(matrix, props):
    d = dot(matrix)
    nDeg = pi / 2
    srt = 1.3
    d = (d-srt) / (nDeg-srt)
    d = clamp(d, 0, 1)
    a = smoothstep(props.alpha_gizmo, 0,  d)
    a = clamp(a, 0.0, props.alpha_gizmo)
    if a < 0.2:
        hs = True
    else:
        hs = False
    return a, hs


def alpha_gizmo_vr(matrix, props):
    d = dot(matrix, use_negative=True)
    scale_factor = 1 if d >= 0 else clamp(-d, 0.3, 1)
    d = clamp(abs(d), 0, 1)
    a = clamp(smoothstep(0, props.alpha_gizmo, d), 0, props.alpha_gizmo)
    hs = a < 0.2 and d < 0.1
    return (0, hs, scale_factor) if hs else (a, False, scale_factor)