import bpy
from bl_math import clamp
from .. import get_prefs



def pro_center_orient_reset(context):
    from .. import var
    props = get_prefs()
    if props.tdc_temporary:
        context.scene.transform_orientation_slots[0].type = var.PCO_STORE_TOS
        context.scene.tool_settings.transform_pivot_point = var.PCO_STORE_TPP


def convert_units(context, value=2.0):
    system = context.scene.unit_settings.system
    length_unit = context.scene.unit_settings.length_unit

    unit = 'm'
    if system == 'METRIC':
        if length_unit == 'KILOMETERS':
            nValue = value / 1000
            unit = 'km'

        elif length_unit == 'METERS':
            nValue = value
            unit = 'm'

        elif length_unit == 'CENTIMETERS':
            nValue = value * 100
            unit = 'cm'

        elif length_unit == 'MILLIMETERS':
            nValue = value * 1000
            unit = 'mm'

        elif length_unit == 'MICROMETERS':
            nValue = value * 1000000
            unit = 'μm'
        else:
            nValue = value
            unit = 'm'

    elif system == 'IMPERIAL':
        if length_unit == 'MILES':
            nValue = value / 1609
            unit = 'mi'

        elif length_unit == 'FEET':
            nValue = value * 3.281
            unit = "\'"

        elif length_unit == 'INCHES':
            nValue = value * 39.37
            unit = "\'"

        elif length_unit == 'THOU':
            nValue = value * 39370
            unit = 'thou'

        else:
            nValue = value * 3.281
            unit = "\'"

    else:
        nValue = value
        unit = 'm'

    return nValue, unit


# --- UI
def rotate_method_ui(context, layout): # --- Rotate Method
    prefs = context.preferences.inputs
    if prefs.view_rotate_method == 'TURNTABLE':
        layout.prop_enum( prefs, 'view_rotate_method', 'TRACKBALL', icon = 'ORIENTATION_GIMBAL' )
    else:
        layout.prop_enum( prefs, 'view_rotate_method', 'TURNTABLE', icon = 'FORCE_MAGNETIC' ) # 'FORCE_MAGNETIC' text = '',


def action_center_ui(context, layout): # --- Action Center
    settings = context.scene.GP_scene_set

    row = layout.row( align = True )
    
    row.prop( settings, 'orientation', text='', icon_only=True )

    row.separator( factor = 0.2 )

    row.prop( settings, 'position', text='',  icon_only=True )

    row.separator( factor = 0.2 )


def increment_rotate_ui(context, layout): # --- Increment Rotate
    settings = context.scene.GP_scene_set
    row = layout.row( align = True )
    row.prop( settings, 'use_ia', text = '', icon='DRIVER_ROTATIONAL_DIFFERENCE' )
    row.popover( panel='GP_PT_angle_ia', text=str(settings.ia)+'°' )


def viewport_points(context):
    use_tool_header = context.space_data.show_region_tool_header

    # context.area.regions[4].width - Tool Bar Width
    # context.area.regions[5].width - N-Panel Width

    if use_tool_header:
        header_height = context.area.regions[0].height * 2
    else:
        header_height = context.area.regions[0].height

    wl = context.area.regions[4].width
    ht = context.area.height - header_height
    hb = 0
    wr = context.area.width - context.area.regions[5].width

    wc = (wl + wr) / 2
    hc = (hb + ht) / 2

    return wl, wr, hb, ht, wc, hc

# --- QW
def transform_orientation(context):
    settings = context.scene.GP_scene_set
    to = context.scene.transform_orientation_slots[0].type

    text = 'Default'
    index = 0
    icon_id = 1
    icon_standard = 'NONE'


    if settings.orientation != 'NONE':
        if settings.orientation == 'GLOBAL':
            text = 'Global(!)'
            icon_id = 81
            icon_standard = 'ORIENTATION_GLOBAL'
        elif settings.orientation == 'LOCAL':
            text = 'Local(!)'
            icon_id = 82
            icon_standard = 'ORIENTATION_LOCAL'
        elif settings.orientation == 'NORMAL':
            text = 'Normal(!)'
            icon_id = 83
            icon_standard = 'ORIENTATION_NORMAL'
        elif settings.orientation == 'GIMBAL':
            text = 'Gimbal(!)'
            icon_id = 84
            icon_standard = 'ORIENTATION_GIMBAL'
        elif settings.orientation == 'VIEW':
            text = 'View(!)'
            icon_id = 85
            icon_standard = 'ORIENTATION_VIEW'
        elif settings.orientation == 'CURSOR':
            text = '3D Cursor(!)'
            icon_id = 86
            icon_standard = 'ORIENTATION_CURSOR'
        elif settings.orientation == 'PARENT':
            text = 'Parent(!)'
            icon_id = 1 # TODO
            icon_standard = 'FILE_PARENT'
        
    else:
        if to == 'GLOBAL':
            text = 'Global'
            icon_id = 81
            icon_standard = 'ORIENTATION_GLOBAL'
        elif to == 'LOCAL':
            text = 'Local'
            icon_id = 82
            icon_standard = 'ORIENTATION_LOCAL'
        elif to == 'NORMAL':
            text = 'Normal'
            icon_id = 83
            icon_standard = 'ORIENTATION_NORMAL'
        elif to == 'GIMBAL':
            text = 'Gimbal'
            icon_id = 84
            icon_standard = 'ORIENTATION_GIMBAL'
        elif to == 'VIEW':
            text = 'View'
            icon_id = 85
            icon_standard = 'ORIENTATION_VIEW'
        elif to == 'CURSOR':
            text = '3D Cursor'
            icon_id = 86
            icon_standard = 'ORIENTATION_CURSOR'
        elif to == 'PARENT':
            text = 'Parent'
            icon_id = 1 # TODO
            icon_standard = 'FILE_PARENT'
        else:
            text = to
            index = 1
            icon_id = 8
            icon_standard = 'OBJECT_ORIGIN'

    return text, index, icon_id, icon_standard


def transform_pivot_point(context):
    settings = context.scene.GP_scene_set
    tpp = context.scene.tool_settings.transform_pivot_point
    
    text = 'Default'
    index = 0
    icon_id = 1
    icon_standard = 'NONE'

    if settings.position == 'OBJECT':
        text = 'Object'
        index = 1
        icon_id = 76
        icon_standard = 'FILE_3D'
    elif settings.position == 'SEL_BOR':
        text = 'Selection Border'
        index = 1
        icon_id = 77
        icon_standard = 'SELECT_SET'
    elif settings.position == 'BOR_MESH':
        text = 'Boundary Mesh'
        index = 1
        icon_id = 78
        icon_standard = 'SPHERECURVE'
    elif settings.position == 'BOUNDING_BOX_CENTER':
        text = 'Bounding Box'
        icon_id = 75
        icon_standard = 'PIVOT_BOUNDBOX'
    elif settings.position == 'CURSOR':
        text = '3D Cursor'
        icon_id = 74
        icon_standard = 'PIVOT_CURSOR'
    elif settings.position == 'INDIVIDUAL_ORIGINS':
        text = 'Individual Origins'
        icon_id = 73
        icon_standard = 'PIVOT_INDIVIDUAL'
    elif settings.position == 'MEDIAN_POINT':
        text = 'Median Point'
        icon_id = 72
        icon_standard = 'PIVOT_MEDIAN'
    elif settings.position == 'ACTIVE_ELEMENT':
        text = 'Active Element'
        icon_id = 71
        icon_standard = 'PIVOT_ACTIVE'
    elif settings.position == 'VIEW':
        text = 'View'
        icon_id = 85 # TODO
        icon_standard = 'RESTRICT_VIEW_ON'
    elif settings.position == 'WORLD':
        text = 'World'
        icon_id = 85 # TODO
        icon_standard = 'WORLD'
    else:
        if tpp == 'BOUNDING_BOX_CENTER':
            text = 'Bounding Box'
            icon_id = 75
            icon_standard = 'PIVOT_BOUNDBOX'
        elif tpp == 'CURSOR':
            text = '3D Cursor'
            icon_id = 74
            icon_standard = 'PIVOT_CURSOR'
        elif tpp == 'INDIVIDUAL_ORIGINS':
            text = 'Individual Origins'
            icon_id = 73
            icon_standard = 'PIVOT_INDIVIDUAL'
        elif tpp == 'MEDIAN_POINT':
            text = 'Median Point'
            icon_id = 72
            icon_standard = 'PIVOT_MEDIAN'
        elif tpp == 'ACTIVE_ELEMENT':
            text = 'Active Element'
            icon_id = 71
            icon_standard = 'PIVOT_ACTIVE'
    
    return text, index, icon_id, icon_standard


def transform_snap_to(context):
    se = context.scene.tool_settings.snap_elements

    text = 'Default'
    index = 0
    icon_id = 1

    if se == {'INCREMENT'}:
        text = 'Increment'
        icon_id = 91
    elif se == {'VERTEX'}:
        text = 'Vertex'
        icon_id = 92
    elif se == {'EDGE'}:
        text = 'Edge'
        icon_id = 93
    elif se == {'FACE'}:
        text = 'Face Project'
        icon_id = 94
    elif se == {'VOLUME'}:
        text = 'Volume'
        icon_id = 96
    elif se == {'EDGE_MIDPOINT'}:
        text = 'Edge Center'
        icon_id = 97
    elif se == {'EDGE_PERPENDICULAR'}:
        text = 'Edge Perpendicular'
        icon_id = 98
    elif se == {'FACE_NEAREST'}:
        text = 'Face Nearest'
        icon_id = 95
    else:
        text = 'Mix'
        index = 1
        icon_id = 99

    return text, index, icon_id


def transform_proportional(context):
    pe = context.scene.tool_settings.proportional_edit_falloff

    text = 'Default'
    index = 0
    icon_id = 1

    if pe == 'SMOOTH':
        text = 'Smooth'
        icon_id = 61
    elif pe == 'SPHERE':
        text = 'Sphere'
        icon_id = 62
    elif pe == 'ROOT':
        text = 'Root'
        icon_id = 63
    elif pe == 'INVERSE_SQUARE':
        text = 'Inverse Square'
        icon_id = 64
    elif pe == 'SHARP':
        text = 'Sharp'
        icon_id = 65
    elif pe == 'LINEAR':
        text = 'Linear'
        icon_id = 66
    elif pe == 'CONSTANT':
        text = 'Constant'
        icon_id = 67
    elif pe == 'RANDOM':
        text = 'Random'
        icon_id = 68

    else: # ??
        text = 'Curve'
        index = 1
        icon_id = 'INVERSE_SQUARE'

    return text, index, icon_id


# --- Paint
def get_tool(self, context):
    if context.mode == 'PAINT_TEXTURE':
        return context.tool_settings.image_paint
    elif context.mode == 'PAINT_VERTEX':
        return context.tool_settings.vertex_paint
    elif context.mode == 'PAINT_GPENCIL':
        return context.tool_settings.gpencil_paint
    elif context.mode == 'VERTEX_GPENCIL':
        return context.tool_settings.gpencil_vertex_paint
    else:
        return None


def get_brush_size(self, context):
    ups = context.tool_settings.unified_paint_settings
    if ups and ups.use_unified_size:
        return ups.size
    else:
        ts = get_tool( self, context )
        brush = ts.brush
        return brush.size


def set_brush_size(self, context, value):
    value = int ( value )
    if value < 1:
        value = 1
    #value = int ( clamp(value, 1, 500) )
    ups = context.tool_settings.unified_paint_settings
    if ups and ups.use_unified_size:
        ups.size = value
    else:
        ts = get_tool( self, context )
        brush = ts.brush
        brush.size = value


def get_brush_strength(self, context):
    ups = context.tool_settings.unified_paint_settings
    ts = get_tool( self, context )
    brush = ts.brush
    if context.active_object.type == 'GPENCIL':
        return brush.gpencil_settings.pen_strength
    elif ups and ups.use_unified_strength:
        return ups.strength
    else:
        return brush.strength


def set_brush_strength(self, context, value):
    value = clamp(value, 0.0, 1.0)
    ups = context.tool_settings.unified_paint_settings
    ts = get_tool( self, context )
    brush = ts.brush
    if context.active_object.type == 'GPENCIL':
        brush.gpencil_settings.pen_strength = value
    elif ups and ups.use_unified_strength:
        ups.strength = value
    else:
        brush.strength = value


def get_brush_color(self, context):
    ups = context.tool_settings.unified_paint_settings
    if ups and ups.use_unified_color:
        return ups.color
    else:
        ts = get_tool( self, context )
        brush = ts.brush
        return brush.color


def set_brush_color(self, context, value):
    ups = context.tool_settings.unified_paint_settings
    if ups and ups.use_unified_color:
        ups.color = value
    else:
        ts = get_tool( self, context )
        brush = ts.brush
        brush.color = value