


# --- Test
def get_vis_accel(self, verts, edges, faces):
    self.accel_vis_accel = Accel2D( accel_vis_verts, accel_vis_edges, accel_vis_faces, get_point2D )
    accel_vis_verts = { bmv for bmv in verts if bmv.is_valid } if self.accel_vis_verts is not None else None
    accel_vis_edges = { bme for bme in edges if bme.is_valid } if self.accel_vis_edges is not None else None
    accel_vis_faces = { bmf for bmf in faces if bmf.is_valid } if self.accel_vis_faces is not None else None

    return accel_vis_verts, accel_vis_edges, accel_vis_faces


class Point2D(Vector, Entity2D):
    @stats_wrapper
    def __init__(self, *args, **kwargs):
        Vector.__init__(*args, **kwargs)

    def __str__(self):
        return '<Point2D (%0.4f, %0.4f)>' % (self.x, self.y)

    def __repr__(self):
        return self.__str__()

    def __iter__(self):
        return iter([self.x, self.y])

    def __add__(self, other):
        t = type(other)
        if t is Direction2D:
            return Point2D((self.x + other.x, self.y + other.y))
        if t is Vector or t is Vec2D:
            return Point2D((self.x + other.x, self.y + other.y))
        if t is RelPoint2D:
            return Point2D((self.x + other.x, self.y + other.y))
        assert False, "unhandled type of other: %s (%s)" % (str(other), str(t))

    def __radd__(self, other):
        return self.__add__(other)

    def __sub__(self, other):
        t = type(other)
        if t is Vector or t is Vec2D:
            return Point2D((self.x - other.x, self.y - other.y))
        elif t is Point2D:
            return Vec2D((self.x - other.x, self.y - other.y))
        elif t is RelPoint2D:
            return Point2D((self.x - other.x, self.y - other.y))
        assert False, "unhandled type of other: %s (%s)" % (str(other), str(t))

    def distance_squared_to(self, other) -> float:
        return (self.x - other.x)**2 + (self.y - other.y)**2

    def distance_to(self, other) -> float:
        return sqrt((self.x - other.x)**2 + (self.y - other.y)**2)

    def as_vector(self):
        return Vector(self)

    def from_vector(self, v):
        self.x, self.y = v

    @staticmethod
    def average(points):
        x, y, c = 0, 0, 0
        for p in points:
            x += p.x
            y += p.y
            c += 1
        if c == 0:
            return Point2D((0, 0))
        return Point2D((x / c, y / c))

    @staticmethod
    def weighted_average(weight_points):
        x, y, c = 0, 0, 0
        for w, p in weight_points:
            x += p.x * w
            y += p.y * w
            c += w
        if c == 0:
            return Point2D((0, 0))
        return Point2D((x / c, y / c))


        
class Accel2D:
    bin_cols = 20
    bin_rows = 20

    class SimpleVert:
        def __init__(self, co):
            self.co = co
            self.is_valid = True

    class SimpleEdge:
        def __init__(self, verts):
            self.verts = verts
            self.p0 = verts[0].co
            self.p1 = verts[1].co
            self.v01 = self.p1 - self.p0
            self.l = self.v01.length
            self.d01 = self.v01 / max(self.l, zero_threshold)
            self.is_valid = True
        def closest(self, p):
            v0p = p - self.p0
            d = self.d01.dot(v0p)
            return self.p0 + self.d01 * mid(d, 0, self.l)

    @staticmethod
    def simple_verts(verts, Point_to_Point2D):
        verts = [Accel2D.SimpleVert(v) for v in verts]
        return Accel2D(verts, [], [], Point_to_Point2D)

    @staticmethod
    def simple_edges(edges, Point_to_Point2D):
        edges = [Accel2D.SimpleEdge((Accel2D.SimpleVert(v0), Accel2D.SimpleVert(v1))) for (v0, v1) in edges]
        verts = [v for e in edges for v in e.verts]
        return Accel2D(verts, edges, [], Point_to_Point2D)

    # @profiler.function
    def __init__(self, verts, edges, faces, Point_to_Point2D):
        self.verts = list(verts) if verts else []
        self.edges = list(edges) if edges else []
        self.faces = list(faces) if faces else []
        self.Point_to_Point2D = Point_to_Point2D
        self.vert_type = type(self.verts[0]) if self.verts else None
        self.edge_type = type(self.edges[0]) if self.edges else None
        self.face_type = type(self.faces[0]) if self.faces else None
        self.bins = {}

        v2Ds = [Point_to_Point2D(v.co) for v in verts]
        self.map_v_v2D = {v: v2d for (v, v2d) in zip(verts, v2Ds)}
        v2Ds = [p for p in v2Ds if p]
        if v2Ds:
            self.min = Point2D((
                min(x - 0.001 for (x, _) in v2Ds),
                min(y - 0.001 for (_, y) in v2Ds)
            ))
            self.max = Point2D((
                max(x + 0.001 for (x, _) in v2Ds),
                max(y + 0.001 for (_, y) in v2Ds)
            ))
        else:
            self.min = Point2D((0, 0))
            self.max = Point2D((1, 1))
        self.size = self.max - self.min

        self.bin_cols = ceil(sqrt(len(v2Ds)))
        self.bin_rows = ceil(sqrt(len(v2Ds)))

        # inserting verts
        for (v, v2d) in zip(verts, v2Ds):
            if not v2d: continue
            i, j = self.compute_ij(v2d)
            self._put(i, j, v)

        # inserting edges
        for e in edges:
            v0, v1 = self.map_v_v2D[e.verts[0]], self.map_v_v2D[e.verts[1]]
            if not v0 or not v1: continue
            ij0, ij1 = self.compute_ij(v0), self.compute_ij(v1)
            mini, minj = min(ij0[0], ij1[0]), min(ij0[1], ij1[1])
            maxi, maxj = max(ij0[0], ij1[0]), max(ij0[1], ij1[1])
            for i in range(mini, maxi + 1):
                for j in range(minj, maxj + 1):
                    self._put(i, j, e)
            # v0,v1 = e.verts
            # self._put_edge(e, self.map_v_v2D[v0], self.map_v_v2D[v1])

        # inserting faces
        for f in faces:
            v2ds = [self.map_v_v2D[v] for v in f.verts]
            if not all(v2ds): continue
            ijs = list(map(self.compute_ij, v2ds))
            mini, minj = min(i for (i, j) in ijs), min(j for (i, j) in ijs)
            maxi, maxj = max(i for (i, j) in ijs), max(j for (i, j) in ijs)
            for i in range(mini, maxi + 1):
                for j in range(minj, maxj + 1):
                    self._put(i, j, f)
            # v0 = v2ds[0]
            # for v1,v2 in zip(v2ds[1:-1],v2ds[2:]):
            #    self._put_face(f, v0, v1, v2)

    # @profiler.function
    def compute_ij(self, v2d):
        n = v2d - self.min
        i = int(self.bin_cols * n.x / self.size.x)
        j = int(self.bin_rows * n.y / self.size.y)
        i = max(0, min(self.bin_cols - 1, i))
        j = max(0, min(self.bin_rows - 1, j))
        return (i, j)

    def _put(self, i, j, o):
        t = (i, j)
        if t not in self.bins: self.bins[t] = set()
        self.bins[t].add(o)

    def _get(self, i, j):
        t = (i, j)
        return self.bins.get(t, set())

    # @profiler.function
    def clean_invalid(self):
        self.bins = {
            t: {o for o in objs if o.is_valid}
            for (t, objs) in self.bins.items()
        }

    def _put_edge(self, e, v0, v1, depth=0):
        i0, j0 = self.compute_ij(v0)
        i1, j1 = self.compute_ij(v1)
        if i0 == i1 and j0 == j1:
            self._put(i0, j0, e)
        elif i0 == i1:
            i = i0
            for j in range(min(j0, j1), max(j0, j1) + 1):
                self._put(i, j, e)
        elif j0 == j1:
            j = j0
            for i in range(min(i0, i1), max(i0, i1) + 1):
                self._put(i, j, e)
        elif depth == 6:
            self._put(i0, j0, e)
            self._put(i1, j1, e)
        else:
            vm = v0 + (v1 - v0) / 2
            self._put_edge(e, v0, vm, depth=depth + 1)
            self._put_edge(e, vm, v1, depth=depth + 1)

    def _put_face(self, f, v0, v1, v2, depth=0):
        i0, j0 = self.compute_ij(v0)
        i1, j1 = self.compute_ij(v1)
        i2, j2 = self.compute_ij(v2)
        if i0 == i1 and i0 == i2 and j0 == j1 and j0 == j2:
            self._put(i0, j0, f)
        elif i0 == i1 and j0 == j1:
            self._put_edge(f, v0, v2, depth=depth)
        elif i0 == i2 and j0 == j2:
            self._put_edge(f, v0, v1, depth=depth)
        elif i1 == i2 and j1 == j2:
            self._put_edge(f, v1, v2, depth=depth)
        elif depth == 6:
            self._put(i0, j0, f)
            self._put(i1, j1, f)
            self._put(i2, j2, f)
        else:
            v01 = v0 + (v1 - v0) / 2
            v12 = v1 + (v2 - v1) / 2
            v20 = v2 + (v0 - v2) / 2
            self._put_face(f, v0, v01, v20, depth=depth + 1)
            self._put_face(f, v1, v12, v01, depth=depth + 1)
            self._put_face(f, v2, v20, v12, depth=depth + 1)

    # @profiler.function
    def get(self, v2d, within):
        delta = Vec2D((within, within))
        p0, p1 = v2d - delta, v2d + delta
        if isinf(p0.x) or isinf(p0.y) or isinf(p1.x) or isinf(p1.y): return set()
        if isnan(p0.x) or isnan(p0.y) or isnan(p1.x) or isnan(p1.y): return set()
        i0, j0 = self.compute_ij(p0)
        i1, j1 = self.compute_ij(p1)
        l = set()
        for i in range(i0, i1 + 1):
            for j in range(j0, j1 + 1):
                l |= self._get(i, j)
        return {v for v in l if v.is_valid}

    # @profiler.function
    def get_verts(self, v2d, within):
        vert_type = self.vert_type
        return {g for g in self.get(v2d, within) if type(g) is vert_type}

    # @profiler.function
    def get_edges(self, v2d, within):
        edge_type = self.edge_type
        return {g for g in self.get(v2d, within) if type(g) is edge_type}

    # @profiler.function
    def get_faces(self, v2d, within):
        face_type = self.face_type
        return {g for g in self.get(v2d, within) if type(g) is face_type}

    def nearest_vert(self, v2d):
        Point_to_Point2D = self.Point_to_Point2D
        vert_type = self.vert_type
        x,y = v2d
        i, j = self.compute_ij(v2d)
        working = {(i,j)}
        touched = set()
        bv,bd = None,0
        while working:
            binij = working.pop()
            if binij in touched: continue
            touched.add(binij)
            i,j = binij
            if i < 0 or j < 0 or i >= self.bin_cols or j >= self.bin_rows: continue
            mx,my = self.min + Vec2D((self.size.x * i / self.bin_cols, self.size.y * j / self.bin_rows))
            Mx,My = self.min + Vec2D((self.size.x * (i+1) / self.bin_cols, self.size.y * (j+1) / self.bin_rows))
            closest = Point2D((mid(x, mx, Mx), mid(y, my, My)))
            d = (v2d - closest).length
            if bv and d > bd:
                # we have seen a vert that is closer than anything in this bin
                continue
            for v in self._get(i, j):
                if type(v) is not vert_type: continue
                d = (Point_to_Point2D(v.co) - v2d).length
                if bv and d > bd: continue
                bv,bd = v,d
            working |= {(i-1,j-1), (i,j-1), (i+1,j-1), (i-1,j), (i+1,j), (i-1,j+1), (i,j+1), (i+1,j+1)}
        return Point_to_Point2D(bv.co)

    # @profiler.function
    def nearest_face(self, v2d):
        ########################################
        # XXXX: ONLY FINDING FACE UNDER V2D!!! #
        ########################################

        # @profiler.function
        def intersect_face(bmf):
            pts = [Point_to_Point2D(bmv.co) for bmv in bmf.verts]
            pts = [pt for pt in pts if pt]
            pt0 = pts[0]
            for pt1, pt2 in zip(pts[1:-1], pts[2:]):
                if intersect_point_tri(v2d, pt0, pt1, pt2):
                    return True
            return False

        Point_to_Point2D = self.Point_to_Point2D
        face_type = self.face_type
        i, j = self.compute_ij(v2d)
        faces = [bmf for bmf in self._get(i, j) if type(bmf) is face_type]
        for bmf in faces:
            if not bmf.is_valid:
                continue
            if intersect_face(bmf):
                return bmf
        return None