import bpy
from bpy.types import Operator
from bpy.props import EnumProperty, FloatVectorProperty
from mathutils import Quaternion
from math import radians
from ..func import pro_center_orient_reset
from ... import get_prefs



class GP_OT_arc_operator(Operator): 
    bl_idname = 'gp.arc_operator'
    bl_label = 'Rotate'
    bl_description = 'Rotate'
    #bl_options = {'UNDO'}


    axis: EnumProperty(
        name='Axis',
        description='Axis',
        items=[
            ('X', 'X', '', '', 0),
            ('Y', 'Y', '', '', 1),
            ('Z', 'Z', '', '', 2),
            ('VIEW', 'View', '', '', 3),
            ],
        default='VIEW',
        )

    matrix_basis: FloatVectorProperty(size=(4, 4), subtype='MATRIX')
    pp = None
    axis_rot = (True, True, True)
    axis_tr = (True, True, True)
    orient_axis_ortho = 'Z'
    

    @staticmethod
    def get_orient_matrix(self):
        mat = self.matrix_basis.copy().to_3x3()
        if self.axis == 'X':
            mat = mat @ Quaternion((0.0, 1.0, 0.0), radians(90)).to_matrix().to_3x3()
        elif self.axis == 'Y':
            mat = mat @ Quaternion((1.0, 0.0, 0.0), radians(90)).to_matrix().to_3x3()
        return mat


    def execute(self, context):
        matrix_orient = self.get_orient_matrix(self)
        tpp = context.scene.tool_settings.transform_pivot_point
        settings = context.scene.GP_scene_set
        if settings.use_ia:
            if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                bpy.ops.gp.increment_rotate(
                    'INVOKE_DEFAULT',
                    axis = self.axis,
                    matrix_basis = self.matrix_basis,
                    constraint_axis = self.axis_rot,
                    use_pp = False,
                    )
            else:
                bpy.ops.gp.increment_rotate(
                    'INVOKE_DEFAULT',
                    axis = self.axis,
                    matrix_basis = self.matrix_basis,
                    constraint_axis = self.axis_rot,
                    )
            return {'FINISHED'}
        else:
            if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                bpy.ops.transform.rotate(
                    'INVOKE_DEFAULT',
                    orient_matrix = matrix_orient,
                    constraint_axis = self.axis_rot,
                    release_confirm = True,
                )
            else:
                bpy.ops.transform.rotate(
                    'INVOKE_DEFAULT',
                    orient_matrix = matrix_orient,
                    constraint_axis = self.axis_rot,
                    release_confirm = True,
                    center_override = self.pp,
                )

        pro_center_orient_reset(context) # --- Temporary
        return {'FINISHED'}


    def invoke(self, context, event):
        props = get_prefs()
        settings = context.scene.GP_scene_set
        tpp = context.scene.tool_settings.transform_pivot_point


        if self.pp is None:
            self.pp = self.matrix_basis.translation


        if self.axis == 'X':
            self.axis_rot = (True, False, False)
            self.axis_tr = (False, True, True)
            self.orient_axis_ortho = 'Z'

        elif self.axis == 'Y':
            self.axis_rot = (False, True, False)
            self.axis_tr = (True, False, True)
            self.orient_axis_ortho = 'X'

        elif self.axis == 'Z':
            self.axis_rot = (False, False, True)
            self.axis_tr = (True, True, False)
            self.orient_axis_ortho = 'Y'
            
        else:
            self.axis_rot = (True, True, True)
            self.axis_tr = (True, True, True)
            self.orient_axis_ortho = 'Z'

        # --- Transform From Value
        if event.type == 'MIDDLEMOUSE' and event.value == 'CLICK':
            bpy.ops.gp.transform_from_value(
                'INVOKE_DEFAULT',
                axis = self.axis_rot,
                operator = 'ROTATION',
                pp = self.pp,
                matrix_basis = self.get_orient_matrix(self),
                )
            return {'FINISHED'}

        # --- PT
        if event.type == 'MIDDLEMOUSE':
            bpy.ops.gp.pivot_transform(
                'INVOKE_DEFAULT',
                TRANSFORM_OT_transform = {
                    'mode': 'ROTATION',
                    'orient_matrix': self.get_orient_matrix(self),
                    'constraint_axis': self.axis_rot,
                    'release_confirm': True,
                    'center_override': self.pp,
                    },
                    )
            

        elif self.axis == 'VIEW':
            if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                bpy.ops.transform.rotate(
                    'INVOKE_DEFAULT',
                    release_confirm = True,
                    )
            else:
                bpy.ops.transform.rotate(
                    'INVOKE_DEFAULT',
                    release_confirm = True,
                    center_override = self.pp,
                    )


        else:
            if context.mode == 'OBJECT':
                if (event.ctrl and props.obArc_ctrl == 'DUPLICATE') or (event.shift and props.obArc_shift == 'DUPLICATE'):
                    bpy.ops.gp.duplicate_rotate_object(
                        'INVOKE_DEFAULT',
                        TRANSFORM_OT_rotate = {
                            'orient_matrix': self.get_orient_matrix(self),
                            'constraint_axis': self.axis_rot,
                            'release_confirm': True,
                            'center_override': self.pp,
                            },
                            )
            
                elif (event.ctrl and props.obArc_ctrl == 'MOVE_TWO') or (event.shift and props.obArc_shift == 'MOVE_TWO'):
                    bpy.ops.transform.transform(
                        'INVOKE_DEFAULT',
                        mode = 'TRANSLATION',
                        orient_matrix = self.get_orient_matrix(self),
                        constraint_axis = self.axis_tr,
                        release_confirm = True,
                        center_override = self.pp,
                        )
                    return {'FINISHED'}

                elif (event.ctrl and props.obArc_ctrl == 'INCROT') or (event.shift and props.obArc_shift == 'INCROT'):
                    if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                        bpy.ops.gp.increment_rotate(
                            'INVOKE_DEFAULT',
                            axis = self.axis,
                            matrix_basis = self.matrix_basis,
                            constraint_axis = self.axis_rot,
                            use_pp = False,
                            )
                    else:
                        bpy.ops.gp.increment_rotate(
                            'INVOKE_DEFAULT',
                            axis = self.axis,
                            matrix_basis = self.matrix_basis,
                            constraint_axis = self.axis_rot,
                            )
                    return {'FINISHED'}

                else:
                    return self.execute(context)
                    

            elif context.mode == 'EDIT_MESH':
                if (event.ctrl and props.meArc_ctrl == 'DUPLICATE') or (event.shift and props.meArc_shift == 'DUPLICATE'):
                    bpy.ops.gp.duplicate_rotate_mesh(
                        'INVOKE_DEFAULT',
                        TRANSFORM_OT_rotate = {
                            'orient_matrix': self.get_orient_matrix(self),
                            'constraint_axis': self.axis_rot,
                            'release_confirm': True,
                            'center_override': self.pp,
                            },
                            )

                elif (event.ctrl and props.meArc_ctrl == 'MOVE_TWO') or (event.shift and props.meArc_shift == 'MOVE_TWO'):
                    bpy.ops.transform.transform(
                        'INVOKE_DEFAULT', 
                        mode = 'TRANSLATION',
                        orient_matrix = self.get_orient_matrix(self),
                        constraint_axis = self.axis_tr,
                        release_confirm = True,
                        center_override = self.pp,
                        )
                    return {'FINISHED'}
                

                elif (event.ctrl and props.meArc_ctrl == 'INCROT') or (event.shift and props.meArc_shift == 'INCROT'):
                    if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                        bpy.ops.gp.increment_rotate(
                            'INVOKE_DEFAULT',
                            axis = self.axis,
                            matrix_basis = self.matrix_basis,
                            constraint_axis = self.axis_rot,
                            use_pp = False,
                            )
                    else:
                        bpy.ops.gp.increment_rotate(
                            'INVOKE_DEFAULT',
                            axis = self.axis,
                            matrix_basis = self.matrix_basis,
                            constraint_axis = self.axis_rot,
                            )
                    return {'FINISHED'}
                
                else:
                    return self.execute(context)

            # elif context.mode in {'EDIT_CURVE', 'EDIT_SURFACE'}:
            # elif context.mode in {'EDIT_ARMATURE', 'POSE'}
            else:
                return self.execute(context)

        return {'FINISHED'}


    
classes = [
    GP_OT_arc_operator,
    ]   


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)  