import bpy
from bpy.types import Operator
from bpy.props import EnumProperty, FloatVectorProperty, BoolVectorProperty
from mathutils import Quaternion
from math import radians
from ..func import pro_center_orient_reset
from ... import get_prefs



class GP_OT_blank_operator(Operator): 
    bl_idname = 'gp.blank_operator'
    bl_label = 'Blank'
    bl_description = 'Blank'
    #bl_options = {'REGISTER', 'UNDO'}

    axis: EnumProperty(
        name='Axis',
        description='Axis',
        items=[
            ('X', 'X', '', '', 0),
            ('Y', 'Y', '', '', 1),
            ('Z', 'Z', '', '', 2),
            ('VIEW', 'View', '', '', 3),
            ],
        default='VIEW',
        )
    #axis_set: BoolVectorProperty( default = (True, True, True), size = 3 )
    matrix_basis: FloatVectorProperty( size = (4, 4), subtype = 'MATRIX' )
    pp = None


    @staticmethod
    def get_orient_matrix(self):
        mat = self.matrix_basis.copy().to_3x3()
        if self.axis == 'X':
            mat = mat @ Quaternion( (0.0, 1.0, 0.0), radians(90) ).to_matrix().to_3x3()
        elif self.axis == 'Y':
            mat = mat @ Quaternion( (1.0, 0.0, 0.0), radians(90) ).to_matrix().to_3x3()
        return mat


    @staticmethod
    def translate(self, context, axis_set, matrix_orient):
        tpp = context.scene.tool_settings.transform_pivot_point
        #os = context.window.scene.transform_orientation_slots[0].type
        settings = context.scene.GP_scene_set

        if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
            bpy.ops.transform.transform(
                'INVOKE_DEFAULT',
                mode = 'TRANSLATION',
                #orient_axis = self.axis, # ??
                orient_matrix = matrix_orient,
                constraint_axis = axis_set,
                release_confirm = True,
                )
        else:
            bpy.ops.transform.transform(
                'INVOKE_DEFAULT',
                mode = 'TRANSLATION',
                orient_matrix = matrix_orient,
                constraint_axis = axis_set,
                release_confirm = True,
                center_override = self.pp,
                )

        pro_center_orient_reset(context) # --- Temporary


    def modal(self, context, event):
        if event.value == 'RELEASE' or event.value_prev == 'RELEASE' or event.type_prev in {'RET', 'LEFTMOUSE'} or event.type in {'RET', 'LEFTMOUSE'}:
            return {'FINISHED'}
        elif event.value in {'ESC', 'RIGHTMOUSE'} or event.value_prev in {'ESC', 'RIGHTMOUSE'}:
            return {'CANCELLED'}
        return {'RUNNING_MODAL'}


    def invoke(self, context, event):
        props = get_prefs()

        if self.pp is None:
            self.pp = self.matrix_basis.translation

        if self.axis == 'X':
            axis_set = (False, True, True)
        elif self.axis == 'Y':
            axis_set = (True, False, True)
        elif self.axis == 'Z':
            axis_set = (True, True, False)
        else:
            axis_set = (True, True, True)


        # --- Transform From Value
        if event.type == 'MIDDLEMOUSE' and event.value == 'CLICK':
            bpy.ops.gp.transform_from_value(
                'INVOKE_DEFAULT',
                axis = axis_set,
                operator = 'TRANSLATION_PLANE',
                pp = self.pp,
                matrix_basis = self.get_orient_matrix(self),
                )
            return {'FINISHED'}


        # --- PT
        if event.type == 'MIDDLEMOUSE': # (event.ctrl and event.shift) or 
            bpy.ops.gp.pivot_transform(
                'INVOKE_DEFAULT', 
                TRANSFORM_OT_transform = {
                    'mode': 'TRANSLATION',
                    'orient_matrix': self.get_orient_matrix(self),
                    'constraint_axis': axis_set,
                    'release_confirm': True,
                    },
                    )

        elif context.mode == 'OBJECT':
            if (event.ctrl and props.obCone_ctrl == 'DUPLICATE') or (event.shift and props.obCone_shift == 'DUPLICATE'):
                bpy.ops.object.duplicate_move(
                    'INVOKE_DEFAULT',
                    TRANSFORM_OT_translate = {
                        'orient_matrix': self.get_orient_matrix(self),
                        'constraint_axis': axis_set,
                        'release_confirm': True,
                        },
                    )
                    
            elif (event.ctrl and props.obCone_ctrl == 'DUPLICATE_LINKED') or (event.shift and props.obCone_shift == 'DUPLICATE_LINKED'):
                bpy.ops.object.duplicate_move_linked(
                    'INVOKE_DEFAULT',
                    OBJECT_OT_duplicate = {
                        'linked': True,
                        'mode': 'TRANSLATION',
                        },
                    TRANSFORM_OT_translate = {
                        'orient_matrix': self.get_orient_matrix(self),
                        'constraint_axis': axis_set,
                        'release_confirm': True,
                        },
                    )

            else:
                self.translate(self, context, axis_set, self.get_orient_matrix(self))

        elif context.mode == 'EDIT_MESH':
            if (event.ctrl and props.meCone_ctrl == 'INSET') or (event.shift and props.meCone_shift == 'INSET'):
                bpy.ops.mesh.inset('INVOKE_DEFAULT', release_confirm = True)

            elif (event.ctrl and props.meCone_ctrl == 'EXTRUDE') or (event.shift and props.meCone_shift == 'EXTRUDE'):
                bpy.ops.mesh.extrude_context_move(
                    'INVOKE_DEFAULT',
                    MESH_OT_extrude_context = {
                        'use_normal_flip': False,
                        'mirror': False,
                        },
                    TRANSFORM_OT_translate = {
                        'orient_matrix': self.get_orient_matrix(self),
                        'constraint_axis': axis_set,
                        'release_confirm':True,
                        },
                    )
                    
            elif (event.ctrl and props.meCone_ctrl == 'EXTRUDE_MANIFLOD') or (event.shift and props.meCone_shift == 'EXTRUDE_MANIFLOD'):
                bpy.ops.mesh.extrude_manifold(
                    'INVOKE_DEFAULT', 
                    MESH_OT_extrude_region = {
                        'use_normal_flip': False,
                        'use_dissolve_ortho_edges': True,
                        'mirror': False,
                        },
                    TRANSFORM_OT_translate = {
                        'orient_matrix': self.get_orient_matrix(self),
                        'constraint_axis': axis_set,
                        'mirror': False, 
                        'release_confirm': True,
                        },
                    )

            else:
                self.translate(self, context, axis_set, self.get_orient_matrix(self))
        

        # elif context.mode in {'EDIT_CURVE', 'EDIT_SURFACE'}:
        # elif context.mode in {'EDIT_ARMATURE', 'POSE'}
        else:
            self.translate(self, context, axis_set, self.get_orient_matrix(self))

        return {'RUNNING_MODAL'}



classes = [
    GP_OT_blank_operator,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)