import bpy
from bpy.types import Operator
from bpy.props import EnumProperty, FloatVectorProperty
from mathutils import Quaternion
from math import radians
from ..func import pro_center_orient_reset
from ... import get_prefs



class GP_OT_box_operator(Operator): 
    bl_idname = 'gp.box_operator'
    bl_label = 'Scale'
    bl_description = 'Scale'
    #bl_options = {'REGISTER', 'UNDO'}


    axis: EnumProperty(
        name='Axis',
        description='Axis',
        items=[
            ('X', 'X', '', '', 0),
            ('Y', 'Y', '', '', 1),
            ('Z', 'Z', '', '', 2),
            ('VIEW', 'View', '', '', 3),
            ],
        default='VIEW',
        )

    matrix_basis: FloatVectorProperty(size=(4, 4), subtype='MATRIX')
    pp = None


    @staticmethod
    def get_orient_matrix(self):
        mat = self.matrix_basis.copy().to_3x3()
        if self.axis == 'X':
            mat = mat @ Quaternion( (0.0, 1.0, 0.0), radians(90) ).to_matrix().to_3x3()
        elif self.axis == 'Y':
            mat = mat @ Quaternion( (1.0, 0.0, 0.0), radians(90) ).to_matrix().to_3x3()
        return mat
        

    @staticmethod
    def resize(self, context, axis_set, matrix_orient):
        tool_settings = context.scene.tool_settings
        tpp = tool_settings.transform_pivot_point
        settings = context.scene.GP_scene_set

        # --- если включен VR то включаем снап при скейле
        use_snap_translate = tool_settings.use_snap_translate
        use_snap_rotate = tool_settings.use_snap_rotate
        use_snap_scale = tool_settings.use_snap_scale
        snap_target = tool_settings.snap_target
        if settings.gzm_type == 'VR':
            tool_settings.use_snap_translate = True
            tool_settings.use_snap_rotate = True
            tool_settings.use_snap_scale = True
            tool_settings.snap_target = 'CLOSEST'

        
        if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
            bpy.ops.transform.resize(
                'INVOKE_DEFAULT',
                constraint_axis = axis_set,
                release_confirm = True,
                )
        else:
            bpy.ops.transform.resize(
                'INVOKE_DEFAULT',
                orient_matrix = matrix_orient,
                constraint_axis = axis_set,
                release_confirm = True,
                center_override = self.pp,
                )

        if settings.gzm_type == 'VR': # --- если включен VR то включаем снап при скейле
            tool_settings.use_snap_translate = use_snap_translate
            tool_settings.use_snap_rotate = use_snap_rotate
            tool_settings.use_snap_scale = use_snap_scale
            tool_settings.snap_target = snap_target
            
        pro_center_orient_reset(context) # --- Temporary


    def modal(self, context, event):
        if event.value == 'RELEASE' or event.value_prev == 'RELEASE' or event.type_prev in {'RET', 'LEFTMOUSE'} or event.type in {'RET', 'LEFTMOUSE'}:
            return {'FINISHED'}
        elif event.value in {'ESC', 'RIGHTMOUSE'} or event.value_prev in {'ESC', 'RIGHTMOUSE'}:
            return {'CANCELLED'}
        return {'RUNNING_MODAL'}


    def invoke(self, context, event):
        props = get_prefs()
        #settings = context.scene.GP_scene_set

        if self.pp is None:
            self.pp = self.matrix_basis.translation

        #utppa = context.scene.tool_settings.use_transform_pivot_point_align
        
        if self.axis == 'X':
            axis_set = (True, False, False)
            scale = (0.0, 1.0, 1.0)
            axis_sca = (False, True, True)

        elif self.axis == 'Y':
            axis_set = (False, True, False)
            scale = (1.0, 0.0, 1.0)
            axis_sca = (True, False, True)

        elif self.axis == 'Z':
            axis_set = (False, False, True)
            scale = (1.0, 1.0, 0.0)
            axis_sca = (True, True, False)

        else:
            axis_set = (True, True, True)
            scale = (1.0, 1.0, 1.0)
            axis_sca = (False, False, False)

        
        # --- Transform From Value
        if event.type == 'MIDDLEMOUSE' and event.value == 'CLICK':
            bpy.ops.gp.transform_from_value(
                'INVOKE_DEFAULT',
                axis = axis_set,
                operator = 'RESIZE',
                pp = self.pp,
                matrix_basis = self.get_orient_matrix(self),
                )
            return {'FINISHED'}
        

        # --- PT
        if event.type == 'MIDDLEMOUSE': # (event.ctrl and event.shift) or 
            bpy.ops.gp.pivot_transform(
                'INVOKE_DEFAULT',
                TRANSFORM_OT_transform = {
                    'mode': 'RESIZE',
                    'orient_matrix': self.get_orient_matrix(self),
                    'constraint_axis': axis_set,
                    'release_confirm': True,
                    'center_override': self.pp,
                    },
                    )
            return {'FINISHED'}


        elif self.axis == 'VIEW':
            tpp = context.scene.tool_settings.transform_pivot_point
            if tpp == 'INDIVIDUAL_ORIGINS': 
                bpy.ops.transform.resize(
                    'INVOKE_DEFAULT',
                    release_confirm = True,
                    )
            else:
                bpy.ops.transform.resize(
                    'INVOKE_DEFAULT',
                    release_confirm = True,
                    center_override = self.pp,
                    )
            return {'FINISHED'}


        else:
            if context.mode == 'OBJECT':
                if (event.ctrl and props.obBox_ctrl == 'SCALE_TWO') or (event.shift and props.obBox_shift == 'SCALE_TWO'): 
                    bpy.ops.transform.resize(
                        'INVOKE_DEFAULT',
                        orient_matrix = self.get_orient_matrix(self),
                        constraint_axis = axis_sca,
                        release_confirm = True,
                        center_override = self.pp,
                        ) #FIXME

                elif (event.ctrl and props.obBox_ctrl == 'ALIGN') or (event.shift and props.obBox_shift == 'ALIGN'):  
                    bpy.ops.gp.align(
                        WM_OT_context_set_boolean = {
                            'value': True,
                            },  
                        TRANSFORM_OT_resize = {
                            'value': scale,
                            'orient_matrix': self.get_orient_matrix(self),
                            'constraint_axis': axis_set,
                            'release_confirm': True,
                            'center_override': self.pp,
                            },
                            )
                    return {'FINISHED'}

                else:
                    self.resize(self, context, axis_set, self.get_orient_matrix(self))


            elif context.mode == 'EDIT_MESH':
                if (event.ctrl and props.meBox_ctrl == 'INSET') or (event.shift and props.meBox_shift == 'INSET'): # --- TODO DEL
                    bpy.ops.mesh.inset(
                        'INVOKE_DEFAULT',
                        release_confirm = True,
                        )

                elif (event.ctrl and props.meBox_ctrl == 'BEVEL') or (event.shift and props.meBox_shift == 'BEVEL'):
                    bpy.ops.gp.bevel(
                        'INVOKE_DEFAULT',
                        )

                elif (event.ctrl and props.meBox_ctrl == 'ALIGN') or (event.shift and props.meBox_shift == 'ALIGN'):
                    bpy.ops.gp.align(
                        WM_OT_context_set_boolean = {
                            'value': False,
                            },
                        TRANSFORM_OT_resize = {
                            'value': scale,
                            'orient_matrix': self.get_orient_matrix(self),
                            'constraint_axis': axis_set,
                            'release_confirm': True,
                            'center_override': self.pp,
                            },
                            )
                    return {'FINISHED'}
                
                elif (event.ctrl and props.meBox_ctrl == 'SCALE_TWO') or (event.shift and props.meBox_shift == 'SCALE_TWO'):
                    bpy.ops.transform.resize(
                        'INVOKE_DEFAULT',
                        orient_matrix = self.get_orient_matrix(self),
                        constraint_axis = axis_sca,
                        release_confirm = True,
                        center_override = self.pp,
                        )
                
                else:
                    self.resize(self, context, axis_set, self.get_orient_matrix(self))


            # elif context.mode in {'EDIT_CURVE', 'EDIT_SURFACE'}:
            # elif context.mode in {'EDIT_ARMATURE', 'POSE'}
            else:
                self.resize(self, context, axis_set, self.get_orient_matrix(self))
        
    
        return {'RUNNING_MODAL'}  



    


classes = [
    GP_OT_box_operator,
    ]   


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)  