import bpy
from bpy.types import Operator
from bpy.props import FloatVectorProperty
from ..func import pro_center_orient_reset
from ... import var,get_prefs



class GP_OT_dot_operator(Operator): 
    bl_idname = 'gp.dot_operator'
    bl_label = 'Translate'
    bl_description = 'Translate'
    #bl_options = {'REGISTER', 'UNDO'}


    matrix_basis: FloatVectorProperty( size = (4, 4), subtype = 'MATRIX' )
    pp = None


    @staticmethod
    def get_orient_matrix(self):
        mat = self.matrix_basis.copy().to_3x3()
        return mat

    
    @staticmethod
    def translate(self, context, matrix_orient, axis_set=None):
        tpp = context.scene.tool_settings.transform_pivot_point
        #os = context.window.scene.transform_orientation_slots[0].type
        settings = context.scene.GP_scene_set

        if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
            if axis_set:
                bpy.ops.transform.transform(
                    'INVOKE_DEFAULT',
                    mode = 'TRANSLATION',
                    orient_matrix = matrix_orient,
                    constraint_axis = axis_set,
                    release_confirm = True,
                    )
            else:
                bpy.ops.transform.transform(
                    'INVOKE_DEFAULT',
                    mode = 'TRANSLATION',
                    release_confirm = True,
                    )
        else:
            if axis_set:
                bpy.ops.transform.transform(
                    'INVOKE_DEFAULT',
                    mode = 'TRANSLATION',
                    orient_matrix = matrix_orient,
                    constraint_axis = axis_set,
                    center_override = self.pp,
                    release_confirm = True,
                    )
            else:
                bpy.ops.transform.transform(
                    'INVOKE_DEFAULT',
                    mode = 'TRANSLATION',
                    center_override = self.pp,
                    release_confirm = True,
                    )

        pro_center_orient_reset(context) # --- Temporary


    def modal(self, context, event):
        if event.value == 'RELEASE' or event.value_prev == 'RELEASE' or event.type_prev in {'RET', 'LEFTMOUSE'} or event.type in {'RET', 'LEFTMOUSE'}:
            return {'FINISHED'}
        elif event.value in {'ESC', 'RIGHTMOUSE'} or event.value_prev in {'ESC', 'RIGHTMOUSE'}:
            return {'CANCELLED'}
        return {'RUNNING_MODAL'}


    def invoke(self, context, event):
        props = get_prefs()
        settings = context.scene.GP_scene_set

        if self.pp is None:
            self.pp = self.matrix_basis.translation

        # --- PCO
        if props.use_pco_actions:
            if event.type == 'MIDDLEMOUSE' and event.value == 'CLICK':
                if event.shift and event.ctrl:
                    if context.scene.transform_orientation_slots[0].type == 'CURSOR' or context.scene.tool_settings.transform_pivot_point == 'CURSOR':
                        context.scene.transform_orientation_slots[0].type = var.PCO_STORE_TOS
                        context.scene.tool_settings.transform_pivot_point = var.PCO_STORE_TPP
                    else:
                        context.scene.transform_orientation_slots[0].type = 'CURSOR'
                        context.scene.tool_settings.transform_pivot_point = 'CURSOR'

                elif event.shift:
                    if context.scene.tool_settings.transform_pivot_point == 'CURSOR':
                        context.scene.tool_settings.transform_pivot_point = var.PCO_STORE_TPP
                    else:
                        context.scene.tool_settings.transform_pivot_point = 'CURSOR'
                    
                elif event.ctrl:
                    if context.scene.transform_orientation_slots[0].type == 'CURSOR':
                        context.scene.transform_orientation_slots[0].type = var.PCO_STORE_TOS
                    else:
                        context.scene.transform_orientation_slots[0].type = 'CURSOR'

                return {'FINISHED'}

            elif event.type == 'MIDDLEMOUSE' and event.ctrl and event.shift:
                bpy.ops.gp.pro_center_ensure('INVOKE_DEFAULT', action_ex='ALL')
                return {'FINISHED'}

            elif event.type == 'MIDDLEMOUSE' and event.ctrl:
                bpy.ops.gp.pro_center_ensure('INVOKE_DEFAULT', action_ex='ORIENTATION')
                return {'FINISHED'}

            elif event.type == 'MIDDLEMOUSE' and event.shift:
                bpy.ops.gp.pro_center_ensure('INVOKE_DEFAULT', action_ex='POSITION')
                return {'FINISHED'}
       

        # --- Transform From Value        
        if event.type == 'MIDDLEMOUSE' and event.value == 'CLICK':
            bpy.ops.gp.transform_from_value(
                'INVOKE_DEFAULT',
                operator = 'DOT',
                pp = self.pp,
                matrix_basis = self.get_orient_matrix(self),
                )
            return {'FINISHED'}
        
# --- PT
        elif event.type == 'MIDDLEMOUSE': # (event.ctrl and event.shift) or 
            bpy.ops.gp.pivot_transform(
                'INVOKE_DEFAULT', 
                TRANSFORM_OT_transform = {
                    'mode': 'TRANSLATION', 
                    'orient_matrix': self.get_orient_matrix(self),
                    'release_confirm': True,
                    'center_override': self.pp,
                    },
                    )
            return {'FINISHED'}

        else:
            if props.dot_action == 'ADVANCE':
                if event.shift:
                    self.translate( self, context, self.get_orient_matrix(self), axis_set = (False, False, True) )
                
                    """ elif event.ctrl:
                    self.translate( self, context, self.get_orient_matrix(self) )
                    bpy.ops.gp.snap( 'INVOKE_DEFAULT' ) """
                
                else:
                    self.translate( self, context, self.get_orient_matrix(self), axis_set = (True, True, False) )

            else:
                if event.shift:
                    if context.mode == 'OBJECT':
                        self.translate( self, context, self.get_orient_matrix(self), axis_set = (True, True, False) )
                        #bpy.ops.gp.qmesh('INVOKE_DEFAULT')
                        return {'RUNNING_MODAL'}

                    elif context.mode == 'EDIT_MESH':
                        bpy.ops.transform.vert_slide('INVOKE_DEFAULT')

                        #bpy.ops.view3d.edit_mesh_extrude_move_normal( 'INVOKE_DEFAULT' )
                    else:
                        self.translate(self, context, self.get_orient_matrix(self))

                else:
                    self.translate(self, context, self.get_orient_matrix(self))

                

        return {'RUNNING_MODAL'}



classes = [
    GP_OT_dot_operator,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)