import bpy
from bpy.types import Operator
from bpy.props import FloatVectorProperty
from ..func import pro_center_orient_reset



class GP_OT_trackball_operator(Operator): 
    bl_idname = 'gp.trackball_operator'
    bl_label = 'Trackball'
    bl_description = 'Trackball'
    #bl_options = {'REGISTER', 'UNDO'}


    matrix_basis: FloatVectorProperty(size=(4, 4), subtype='MATRIX')
    pp = None


    @staticmethod
    def trackball(self, context):
        tpp = context.scene.tool_settings.transform_pivot_point
        settings = context.scene.GP_scene_set

        if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
            bpy.ops.transform.trackball(
                'INVOKE_DEFAULT',
                release_confirm = True,
                )
        else:
            bpy.ops.transform.trackball(
                'INVOKE_DEFAULT',
                center_override = self.pp,
                release_confirm = True,
                )

        pro_center_orient_reset(context) # --- Temporary


    def modal(self, context, event):
        if event.value == 'RELEASE' or event.value_prev == 'RELEASE' or event.type_prev in {'RET', 'LEFTMOUSE'} or event.type in {'RET', 'LEFTMOUSE'}:
            return {'FINISHED'}
        elif event.value in {'ESC', 'RIGHTMOUSE'} or event.value_prev in {'ESC', 'RIGHTMOUSE'}:
            return {'CANCELLED'}
        return {'RUNNING_MODAL'}


    def invoke(self, context, event):
        settings = context.scene.GP_scene_set
        tpp = context.scene.tool_settings.transform_pivot_point
        if self.pp is None:
            self.pp = self.matrix_basis.translation

        

        # --- PT
        if event.type == 'MIDDLEMOUSE': # (event.ctrl and event.shift) or 
            bpy.ops.gp.pivot_transform(
                'INVOKE_DEFAULT',
                TRANSFORM_OT_transform = {
                    'mode': 'TRACKBALL',
                    'release_confirm': True,
                    'center_override': self.pp,
                    },
                    )
            return {'FINISHED'}


        else:
            if event.shift:
                if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                    bpy.ops.transform.rotate(
                        'INVOKE_DEFAULT',
                        release_confirm = True,
                        )
                else:
                    bpy.ops.transform.rotate(
                        'INVOKE_DEFAULT',
                        release_confirm = True,
                        center_override = self.pp,
                        )

            elif event.ctrl:
                if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                    bpy.ops.gp.increment_rotate(
                        'INVOKE_DEFAULT',
                        axis = 'VIEW',
                        matrix_basis = self.matrix_basis,
                        use_pp = False,
                        )
                else:
                    bpy.ops.gp.increment_rotate(
                        'INVOKE_DEFAULT',
                        matrix_basis = self.matrix_basis,
                        axis = 'VIEW',
                        )
            else:
                self.trackball(self, context)


        return {'RUNNING_MODAL'}



classes = [
    GP_OT_trackball_operator,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)