import bpy
from bpy.types import Operator, GizmoGroup
from mathutils import Vector
from ...matrix.transpose_matrix import transpose_matrix
from ... import var
from ..poll import poll_transpose
from bpy.props import EnumProperty



class GP_OT_transpose_axis_op(Operator): 
    bl_idname = 'gp.transpose_axis_op'
    bl_label = 'Axis Operator'
    bl_description = ''
    bl_options = {'UNDO'}


    axis: EnumProperty(
        name = 'Axis',
        description = '',
        items=[
            ('+X', '+X', ''),
            ('-X', '-X', ''),
            ('+Y', '+Y', ''),
            ('-Y', '-Y', ''),
            ('+Z', '+Z', ''),
            ('-Z', '-Z', ''),
            ],
        default = '+X',
        )


    def execute(self, context):
        settings = context.scene.GP_scene_set

        mX, mY, mZ, mX_d, mY_d, mZ_d = transpose_matrix()
        rot = mZ.decompose()[1]


        dim = Vector(settings.tr_p1) - Vector(settings.tr_p2) 
        length = dim.length

        if self.axis == '+X':
            new_pos = mZ @ Vector((0+length, 0, 0)) #( Vector(self.settings.tr_p1) + Vector((0+len, 0, 0)) )
        elif self.axis == '-X':
            new_pos = mZ @ Vector((0-length, 0, 0))

        elif self.axis == '+Y':
            new_pos = mZ @ Vector((0, 0+length, 0))
        elif self.axis == '-Y':
            new_pos = mZ @ Vector((0, 0-length, 0))

        elif self.axis == '+Z':
            new_pos = mZ @ Vector((0, 0, 0+length))
        elif self.axis == '-Z':
            new_pos = mZ @ Vector((0, 0, 0-length))

    
        settings.tr_p2 = new_pos
        settings.mes_rot_quat = rot


        target =  Vector(settings.tr_p1) - Vector(settings.tr_p2)
        settings.transpose_direction = target

        settings.use_align = False
        settings.use_global_mat = False

        #if context.area:
        context.area.tag_redraw()

        return {'FINISHED'}



class GP_OT_transpose_transform_op(Operator): 
    bl_idname = 'gp.transpose_transform_op'
    bl_label = 'Transform Operator'
    bl_description = ''
    bl_options = {'UNDO'}


    operator: EnumProperty(
        name = 'Axis',
        description = '',
        items = [
            ('TRANSLATION', 'TRANSLATION', ''),
            ('ROTATION', 'ROTATION', ''),
            ('RESIZE', 'RESIZE', ''),
            ],
        default = 'TRANSLATION',
        )
    use_shift = None
    pp = None
    mat_orient = None


    def execute(self, context):
        tool_settings = context.scene.tool_settings
        use_snap_translate = tool_settings.use_snap_translate
        use_snap_rotate = tool_settings.use_snap_rotate
        use_snap_scale = tool_settings.use_snap_scale
        snap_target = tool_settings.snap_target
        tool_settings.use_snap_translate = True
        tool_settings.use_snap_rotate = True
        tool_settings.use_snap_scale = True
        tool_settings.snap_target = 'CLOSEST'

        if self.operator == 'TRANSLATION':
            if self.use_shift:
                bpy.ops.transform.transform(
                    'INVOKE_DEFAULT',
                    mode='TRANSLATION',
                    constraint_axis = (True, True, False),
                    orient_matrix = self.mat_orient,
                    center_override = self.pp,
                    release_confirm = True)
            else:
                bpy.ops.transform.transform(
                    'INVOKE_DEFAULT',
                    mode='TRANSLATION',
                    constraint_axis = (False, False, True),
                    orient_matrix = self.mat_orient,
                    center_override = self.pp,
                    release_confirm = True)
            

        elif self.operator == 'ROTATION':
            if self.use_shift:
                bpy.ops.transform.transform(
                    'INVOKE_DEFAULT',
                    mode='ROTATION',
                    constraint_axis = (True, True, False),
                    orient_matrix = self.mat_orient,
                    center_override = self.pp,
                    release_confirm = True)
            else:
                bpy.ops.transform.transform(
                    'INVOKE_DEFAULT',
                    mode='ROTATION',
                    constraint_axis = (False, False, True),
                    orient_matrix = self.mat_orient,
                    center_override = self.pp,
                    release_confirm = True)
            

        elif self.operator == 'RESIZE':
            if self.use_shift:
                bpy.ops.transform.resize(
                    'INVOKE_DEFAULT',
                    constraint_axis = (True, True, False),
                    orient_matrix = self.mat_orient,
                    center_override = self.pp,
                    release_confirm = True)
            else:
                bpy.ops.transform.transform(
                    'INVOKE_DEFAULT',
                    mode = 'RESIZE',
                    orient_axis = 'Z',
                    constraint_axis = (False, False, True),
                    orient_matrix = self.mat_orient,
                    center_override = self.pp,
                    release_confirm = True)
                """ bpy.ops.transform.resize(
                    'INVOKE_DEFAULT',
                    constraint_axis = (False, False, True),
                    orient_matrix = mat_orient,
                    center_override = settings.tr_p1,
                    release_confirm = True) """
        
        tool_settings.use_snap_translate = use_snap_translate
        tool_settings.use_snap_rotate = use_snap_rotate
        tool_settings.use_snap_scale = use_snap_scale
        tool_settings.snap_target = snap_target
        return {'FINISHED'}


    def invoke(self, context, event):
        settings = context.scene.GP_scene_set

        settings.transpose_direction = Vector(settings.tr_p1) - Vector(settings.tr_p2)
        target =  Vector(settings.transpose_direction).normalized()
        self.mat_orient = target.to_track_quat('Z', 'X').to_matrix().to_3x3()

        self.pp = settings.tr_p1
        self.use_shift = event.shift
            
        return self.execute(context)



class GP_GGT_set_transpose(GizmoGroup):
    
    bl_idname = 'GP_GGT_set_transpose'
    bl_label = 'Set Transpose'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'3D', 'SHOW_MODAL_ALL', 'VR_REDRAWS'} # , 'TOOL_INIT', 'SELECT', 'SCALE', 'DEPTH_3D', 'PERSISTENT', 


    @classmethod
    def poll(cls, context):
        return poll_transpose(context)


    def setup(self, context):
        self.ray = self.gizmos.new('GIZMO_GT_snap_3d')
        self.ray.snap_elements_force = {'EDGE_PERPENDICULAR', 'VERTEX', 'EDGE_MIDPOINT', 'EDGE', 'FACE'} # {'EDGE_PERPENDICULAR', 'VERTEX', 'EDGE_MIDPOINT', 'EDGE', 'FACE'}


    def draw_prepare(self, context):
        var.TRANSPOSE_LOC = self.ray.location
        var.TRANSPOSE_NORM = self.ray.normal



class GP_OT_transpose_ensure(Operator):
    bl_idname = 'gp.transpose_ensure'
    bl_label = 'Set Transpose'
    bl_description = ' '
    bl_options = {'UNDO'} #'REGISTER', 


    type: EnumProperty(
        name='Type',
        description='Type',
        items=[
            ('STRT_LOC', 'Start', '', 0),
            ('MID_LOC', 'Middle', '', 1),
            ('END_LOC', 'End', '', 2),
            ],
        default='STRT_LOC',
        )


    def __init__(self):
        self.snap_state = False


    @staticmethod
    def set_settings(self, context):
        settings = context.scene.GP_scene_set
        if self.type == 'STRT_LOC':
            settings.tr_p1 = var.TRANSPOSE_LOC
        elif self.type == 'END_LOC':
            settings.tr_p2 = var.TRANSPOSE_LOC

        target =  Vector(settings.tr_p1) - Vector(settings.tr_p2)
        settings.transpose_direction = target #var.TRANSPOSE_NORM

        settings.use_align = True
        settings.use_global_mat = False
        


    def modal(self, context, event):
        settings = context.scene.GP_scene_set

        if event.value == 'RELEASE':
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_set_transpose')
            context.scene.tool_settings.use_snap = self.snap_state

            self.set_settings(self, context)
            var.REFRESH_MAT = True
            return {'FINISHED'}
            
        elif event.type in {'ESC'}:
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_set_transpose')
            context.scene.tool_settings.use_snap = self.snap_state
            return {'CANCELLED'}

        else:
            self.set_settings(self, context)
            var.REFRESH_MAT = True
            
        return {'PASS_THROUGH'}


    def invoke(self, context, event):
        self.snap_state = context.scene.tool_settings.use_snap
        context.scene.tool_settings.use_snap = True

        context.window_manager.gizmo_group_type_ensure('GP_GGT_set_transpose')
        var.REFRESH_MAT = True
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}



classes = [
    GP_OT_transpose_axis_op,
    GP_OT_transpose_transform_op,
    GP_GGT_set_transpose,
    GP_OT_transpose_ensure,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)