import bpy
from bpy.types import Macro, Operator
import bmesh
from mathutils import Vector
from bpy.props import BoolProperty



class GP_OT_bevel(Operator):
    bl_idname = 'gp.bevel'
    bl_label = 'Bevel Fast'
    bl_description = ' '
    bl_options = {'REGISTER', 'UNDO', 'BLOCKING'}


    loop_slide: BoolProperty( 
        name = 'Loop Slide',
        description = 'Prefer sliding along edges to even widths',
        default = False,
        )


    @classmethod
    def poll(cls, context):
        return context.mode == 'EDIT_MESH'


    def __init__(self):
        self._init_mouse = []
        self._value = None
        #self._profile = None
        self._segments = None
        self._bm = None
        self._bm_orig = None
        self._geom = None
        self._mesh = None


        self.__first = True
        self.__use_support_edge = False
        self.__store_profile = 0.5
        
    """ def __del__(self):
        print("End") """

    """ def execute(self, context):

        bmesh.ops.bevel(
                self._bm,
                geom = geom,
                offset = self._value, # value set,
                segments = self._segments, #
                affect = 'EDGES',
                profile = self._profile, # 0.25 or 0.5,
                loop_slide = self.loop_slide,
                )
            
            #print( 'data', data )
            
            
        bpy.ops.object.mode_set(mode='OBJECT')
        
        self._bm.to_mesh( context.object.data )
        context.object.data.update()

        bpy.ops.object.mode_set(mode='EDIT')


        return {'FINISHED'} """
   

    def modal(self, context, event):
        mouse = Vector( (event.mouse_region_x, event.mouse_region_y) )
        cross = mouse.cross( self._init_mouse )
        if self.__first:
            length = cross * -0.0000005
        else:
            length = cross * -0.00001

        


        if event.type == 'MOUSEMOVE':
            if self.__first:
                self._value = abs( length )
                self.__store_profile = 0.5 if length > 0 else 0.25
                self._segments = 2 if length > 0 else 1
            else:
                self._segments = round( abs(length)  )
            

            #print( 'segments', segments )
            self._bm = self._bm_orig.copy()
            self._geom = [ e for e in self._bm.edges if e.select ]

            bmesh.ops.bevel(
                self._bm,
                geom = self._geom,
                offset = self._value, # value set,
                segments = self._segments, #
                affect = 'EDGES',
                profile = self.__store_profile if self.__use_support_edge is False else 1.0,
                loop_slide = self.loop_slide,
                )
            
            #print( 'data', data )
            
            
            bpy.ops.object.mode_set(mode='OBJECT')
            
            self._bm.to_mesh( context.object.data )
            context.object.data.update()

            bpy.ops.object.mode_set(mode='EDIT')
            #bmesh.update_edit_mesh( self._mesh, loop_triangles = True )


        

        elif event.value == 'RELEASE' and  event.type in {'ESC', 'RIGHTMOUSE'}:             # --- CANCELLED
            bpy.ops.object.mode_set(mode='OBJECT')
            self._bm_orig.to_mesh( context.object.data )
            context.object.data.update()
            bpy.ops.object.mode_set(mode='EDIT')
            return {'CANCELLED'}

        elif event.value == 'RELEASE' and event.type in {'RET', 'LEFTMOUSE'}:               # --- FINISHED
            #print('FINISHED')
            self._bm_orig.free()
            self._bm.free()
            return {'FINISHED'}

        elif event.shift and event.value in {'CLICK', 'PRESS'}:                             # --- использовать Support Edge Profile
            if self.__use_support_edge:
                self.__use_support_edge = False
            else:
                self.__use_support_edge = True

        elif self.__first and event.value == 'RELEASE' and event.shift is False:            # --- переключение на второстепенную настройку
            self.__first = False

        #return {'RUNNING_MODAL'}
        return {'PASS_THROUGH'}
        


    def invoke(self, context, event):
        self.__first = True
        self._init_mouse = Vector( (event.mouse_region_x, event.mouse_region_y) )
        
        
        obj = context.object
        self._mesh = obj.data
        """ self._bm = bmesh.new()
        self._bm.from_mesh( self._mesh ) """
        self._bm_orig = bmesh.from_edit_mesh( obj.data ).copy()
        self._bm_orig.edges.ensure_lookup_table()
        

        #uniques = context.selected_objects
        #for obj in uniques:
      
        
        #self._geom = [ e for e in self._bm.edges if e.select ]
        

        context.window_manager.modal_handler_add( self )
        return {'RUNNING_MODAL'}  

    def draw(self, context):
        layout = self.layout


        layout.prop(self, 'loop_slide')



classes = [
    GP_OT_bevel,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

    
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls) 