import bpy
from bpy.types import Operator
import bmesh
from mathutils import Vector


class GP_OT_slide_edge(Operator): # TODO FIXME
    bl_idname = 'gp.slide_edge'
    bl_label = 'Edge Slide'
    bl_options = {'REGISTER', 'UNDO'}

    angle: bpy.props.FloatProperty(name="Rotation Angle", default=0.0)

    @classmethod
    def poll(cls, context):
        return context.object is not None and context.object.type == 'MESH' and context.mode == 'EDIT_MESH'

    def execute(self, context):
        obj = context.object
        bm = bmesh.from_edit_mesh(obj.data)
        selected_edges = [e for e in bm.edges if e.select]
        
        if not selected_edges:
            self.report({'ERROR'}, "No edges selected!")
            return {'CANCELLED'}

        # Вычисляем центр вращения
        center = sum([v.co for e in selected_edges for v in e.verts], Vector()) / (2 * len(selected_edges))
        
        # Вращаем и применяем Edge Slide
        for e in selected_edges:
            for v in e.verts:
                direction = (v.co - center).normalized()
                v.co += direction * self.angle

        bmesh.update_edit_mesh(obj.data)
        return {'FINISHED'}






classes = [
    GP_OT_slide_edge,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)