import bpy
from bpy.types import Operator, GizmoGroup
from bpy.props import IntProperty, EnumProperty, BoolVectorProperty, BoolProperty, FloatVectorProperty
import math
from math import degrees, radians, atan2, pi, copysign
from bpy_extras import view3d_utils
from mathutils import Quaternion, Euler, Vector, Matrix
from ...matrix.matrix_ensure import get_orient_from_type
from bpy_extras.view3d_utils import location_3d_to_region_2d , region_2d_to_vector_3d
from ... import var,get_prefs

ANGLE = 0
MATRIX = Matrix()



def get_rotation_angle(cx, cy, mx, my, nx, ny):
    # calculate vectors between center and initial/new cursor position
    vx1, vy1 = mx - cx, my - cy
    vx2, vy2 = nx - cx, ny - cy
    # calculate dot product and determinant of vectors
    dot_product = vx1 * vx2 + vy1 * vy2
    determinant = vx1 * vy2 - vx2 * vy1
    # calculate angle of rotation in radians
    angle = atan2(determinant, dot_product)
    # convert angle to degrees
    angle_degrees = angle * 180 / pi
    return angle_degrees


def sign(self, context, event):
    # Позиция 3D курсора в 2D
    cursor_2d = location_3d_to_region_2d(context.region, context.region_data, Vector(self.pp))

    # Текущая и предыдущая позиции курсора мыши в 2D
    current_mouse_pos = Vector((event.mouse_region_x, event.mouse_region_y))
    previous_mouse_pos = self.prev_mouse

    # Векторы от 3D курсора до текущего и предыдущего положения мыши
    vector_current = current_mouse_pos - cursor_2d
    vector_previous = previous_mouse_pos - cursor_2d

    # Векторное произведение для определения знака (+1 или -1)
    cross = vector_previous.cross(vector_current)

    return -1 if cross < 0 else 1


def dot(self, context, matrix=None):
    region_data = context.region_data
    if matrix is None:
        l, q, s = self.matrix_basis.decompose()
    else:
        l, q, s = matrix.decompose()
    l = Vector(self.pp)
    if region_data.is_perspective:
        vL = region_data.view_matrix.inverted().translation
        vv = vL - l
    else:
        vv = region_data.view_rotation @ Vector((0, 0, -1))
    norm = q @ Vector((0, 0, 1))
    dot = norm.angle(vv)
    nDeg = pi / 2
    if dot > nDeg:
        dot = -(pi - dot)
    return int(copysign(1, dot)) 


def rotation(self, context, event):
    cursor = Vector((event.mouse_region_x, event.mouse_region_y))
    if cursor == self.prev_mouse:
        return Matrix()

    # Новый вектор направления взгляда с позиции курсора
    new_vector = region_2d_to_vector_3d(context.region, context.region_data, cursor).normalized()
    
    angle = (self.prev_mouse_vector.angle(new_vector) * (sign(self, context, event)*dot(self, context))) * 10
    
    self.prev_mouse_vector = new_vector
    self.prev_mouse = cursor

    return int(copysign(1, angle))
    

class GP_OT_increment_rotate(Operator): 
    bl_idname = 'gp.increment_rotate'
    bl_label = 'Increment Rotate'
    bl_options = {'REGISTER', 'UNDO'}



    axis: EnumProperty(
        name='Axis',
        description='Axis',
        items=[
            ('X', 'X', ''),
            ('Y', 'Y', ''),
            ('Z', 'Z', ''),
            ('VIEW', 'VIEW', ''),
            ],
        default='Z',
        )


    matrix_basis: FloatVectorProperty(size=(4, 4), subtype='MATRIX')
    constraint_axis: BoolVectorProperty(size=3, subtype='XYZ', default=(False, False, True))
    use_pp: BoolProperty(default=True)

    


    def __init__(self):
        self.initial_mouse = (0.0, 0.0)
        self.rotLenght = 50
        self.init_angle = 0.0
        self.mouse_pos = None
        self.prev_mouse = Vector((0.0, 0.0))

        self.orient_matrix = None
        self.pp = None

    @staticmethod
    def get_orient_matrix(self):
        mat = self.matrix_basis.copy().to_3x3()
        if self.axis == 'X':
            mat = mat @ Quaternion((0.0, 1.0, 0.0), radians(-90)).to_matrix().to_3x3()
        elif self.axis == 'Y':
            mat = mat @ Quaternion((1.0, 0.0, 0.0), radians(90)).to_matrix().to_3x3()
        return mat


    def modal(self, context, event):
        global ANGLE
        #props = context.preferences.addons['GIZMO_PRO'].preferences
        settings = context.scene.GP_scene_set

        angle = 0.0

        if event.type == 'MOUSEMOVE':
            if self.mouse_pos is None:
                self.mouse_pos = Vector((event.mouse_x, event.mouse_y))
            else:
                rotate = rotation(self, context, event)
                delta = Vector((event.mouse_x, event.mouse_y)) - self.mouse_pos
                if delta.length > self.rotLenght:
                    self.mouse_pos = Vector((event.mouse_x, event.mouse_y))
                    angle = radians(settings.ia) * rotate  # * round(delta.y / self.rotLenght))
                    
                    self.init_angle += angle
                    ANGLE = self.init_angle
                 
                    if self.use_pp:
                        if self.axis == 'VIEW':
                            bpy.ops.transform.rotate(
                                value=angle,
                                center_override=self.pp,
                            )
                        else:
                            bpy.ops.transform.rotate(
                                value=angle,
                                orient_axis=self.axis,
                                constraint_axis=self.constraint_axis,
                                orient_matrix=self.get_orient_matrix(self),
                                center_override=self.pp,
                            )
                    else:
                        if self.axis == 'VIEW':
                            bpy.ops.transform.rotate(
                                value=angle,
                            )
                        else:
                            bpy.ops.transform.rotate(
                                value=angle,
                                orient_axis=self.axis,
                                constraint_axis=self.constraint_axis,
                                orient_matrix=self.get_orient_matrix(self),
                            )
        elif event.type == 'LEFTMOUSE':
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_rotate_circle')
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_value')
            return {'FINISHED'}

        elif event.type in {'RIGHTMOUSE', 'ESC'}:
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_rotate_circle')
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_value')
            if self.axis == 'VIEW':
                bpy.ops.transform.rotate(
                    value=-self.init_angle,
                )
            else:
                bpy.ops.transform.rotate(
                    value=-self.init_angle,
                    orient_axis=self.axis,
                    constraint_axis=self.constraint_axis,
                    orient_matrix=self.get_orient_matrix(self),
                )

            #bpy.ops.transform.rotate(value=-angle, orient_axis=self.axis, constraint_axis=(False, False, True) )
            return {'CANCELLED'}

        return {'RUNNING_MODAL'}


    def invoke(self, context, event):
        if context.active_object is not None:
            global ANGLE, MATRIX
            
            self.pp = self.matrix_basis.translation
            if self.axis == 'VIEW':
                q = context.region_data.view_matrix.inverted().decompose()[1]
                s = Vector((1.0, 1.0, 1.0))
                MATRIX = Matrix.LocRotScale(self.pp, q, s)
            else:
                MATRIX = self.matrix_basis

            self.init_angle, ANGLE = 0, 0

            self.prev_mouse = Vector((event.mouse_region_x, event.mouse_region_y))
            self.prev_mouse_vector = region_2d_to_vector_3d(context.region, context.region_data, self.prev_mouse)

            context.window_manager.gizmo_group_type_ensure('GP_GGT_rotate_circle') # включить гизмо группу
            context.window_manager.gizmo_group_type_ensure('GP_GGT_value')
            context.window_manager.modal_handler_add(self)
            return {'RUNNING_MODAL'}
        else:
            self.report({'WARNING'}, "No active object")
            return {'CANCELLED'}

        
    """ def draw(self, context):
        layout = self.layout
        row = layout.row()
        row.prop(self, "angle")
        row = layout.row()
        row.prop(self, "axis")
        row = layout.row()
        row.prop(self, "matrix", text="Orientation") """

from bpy_extras.view3d_utils import (
    location_3d_to_region_2d,
    region_2d_to_location_3d,
    region_2d_to_vector_3d,
    region_2d_to_origin_3d,
)

class GP_GGT_rotate_circle(GizmoGroup):
    bl_idname = 'GP_GGT_rotate_circle'
    bl_label = 'Gizmo PRO'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW' 
    bl_options = {'3D', 'SHOW_MODAL_ALL'}
    
    def setup(self, context):
        global ANGLE, MATRIX
        settings = context.scene.GP_scene_set
        props = get_prefs()

        self.dial = self.gizmos.new('GIZMO_GT_dial_3d')
        self.dial.draw_options = {'FILL_SELECT', 'ANGLE_VALUE'} #, 'ANGLE_START_Y'
        #self.dial.click_value = 2
        self.dial.arc_inner_factor = 0.6
        self.dial.scale_basis = 1.5
        #self.dial.wrap_angle = False
        self.dial.incremental_angle = settings.ia
        self.dial.use_tooltip = False
        self.dial.line_width = 3.0
        #self.dial.use_grab_cursor = True
        self.dial.use_draw_value = True
        self.dial.hide_select = True
        self.dial.color = props.primary_color

        matrix = MATRIX @ Matrix.Rotation(radians(180), 4, 'X') @ Matrix.Rotation(radians(180), 4, 'Z')
        self.dial.matrix_basis = matrix.normalized()

        pos = location_3d_to_region_2d(context.region, context.space_data.region_3d, self.dial.matrix_basis.translation)
        var.VALUE[1] = pos[0], pos[1]-230

        def get_value():
            var.VALUE[0] = int(round(degrees(ANGLE)))
            return ANGLE
        def set_value(value):
            pass
        self.dial.target_set_handler("offset", get=get_value, set=set_value)
        

        


classes = [
    GP_OT_increment_rotate,
    GP_GGT_rotate_circle,
    ]   


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)  