import bpy
from bpy.types import Operator, Gizmo
from mathutils import Matrix, Vector, Quaternion
from ...matrix.matrix_ensure import get_orient_from_type
from bpy_extras import view3d_utils
from mathutils.geometry import intersect_line_plane
import blf
import bmesh
from bpy_extras.view3d_utils import region_2d_to_vector_3d, region_2d_to_location_3d, region_2d_to_origin_3d
from mathutils import Vector, Matrix
import gpu
from gpu_extras.batch import batch_for_shader
from ..draw_utils.draw import draw_batch
from bpy.props import IntProperty, EnumProperty, BoolVectorProperty, BoolProperty, FloatVectorProperty
from gpu import state




def get_coord_mouse(self, context, event, matrix=None):
    view_normal = bpy.context.region_data.view_rotation @ Vector((0.0, 0.0, -1.0))

    dot_x = abs(view_normal.dot(self.vec_x))
    dot_y = abs(view_normal.dot(self.vec_y))
    dot_z = abs(view_normal.dot(self.vec_z))

    max_dot = max(dot_x, dot_y, dot_z)

    if max_dot == dot_x:
        normal = self.vec_x
    elif max_dot == dot_y:
        normal = self.vec_y
    else:
        normal = self.vec_z

    region = context.region
    rv3d = context.region_data
    coord = event.mouse_region_x, event.mouse_region_y
    view_vector_mouse = view3d_utils.region_2d_to_vector_3d(region, rv3d, coord)
    ray_origin_mouse = view3d_utils.region_2d_to_origin_3d(region, rv3d, coord)
    
    pp = matrix.translation
    loc = intersect_line_plane(ray_origin_mouse, ray_origin_mouse + view_vector_mouse, pp, normal, False)
    return loc







class GP_OT_modo_transform(Operator):
    bl_idname = 'gp.modo_transform'
    bl_label = 'Transform'
    bl_description = ' '
    bl_options = {'REGISTER', 'UNDO'} # , 'INTERNAL'


    

    axis: EnumProperty(
        name='Axis',
        description='Axis',
        items=[
            ('X', 'x', '', '', 0),
            ('Y', 'y', '', '', 1),
            ('Z', 'z', '', '', 2),
            ],
        default='Z',
        )


    #orient_matrix: FloatVectorProperty(size=(3, 3), subtype='MATRIX')
    #pp: FloatVectorProperty(default=(0,0,0))




    _handle = None
    mouse_path = []
    matrix = None

    vec_x = None
    vec_y = None
    vec_z = None
    pp = None


    @classmethod
    def poll(cls, context):
        return context.area.type == 'VIEW_3D'


    

    def modal(self, context, event):
        context.area.tag_redraw()
        if event.type == 'MOUSEMOVE':
            # Определение направления движения мыши и выбор оси
            #direction = self.get_mouse_direction(context, event).normalized()
            loc = get_coord_mouse(self, context, event, matrix=self.matrix)
            self.mouse_path[1] = loc

            direction = self.mouse_path[0] - loc
            

            mouse_sqrt = (self.mouse_path[0] - loc).length
            #print(mouse_sqrt)

            #print('direction  :', direction.length)
            if mouse_sqrt > 0.05:
                direction = direction.normalized()
                #print('direction  :', direction)
                dot_x = abs(direction.dot(self.vec_x))
                dot_y = abs(direction.dot(self.vec_y))
                dot_z = abs(direction.dot(self.vec_z))

                #print(direction, dot_x, dot_y, dot_z)
                # Определение максимального скалярного произведения
                max_dot = max(dot_x, dot_y, dot_z)

                # Выбор оси, соответствующей наименьшему углу
                if max_dot == dot_x:
                    bpy.ops.transform.translate('INVOKE_DEFAULT', constraint_axis=(False, False, True))
                    #self.mouse_path[1] = self.mouse_path[0] - self.vec_x
                    #bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                    
                    #print('X')
                    return {'FINISHED'}
                elif max_dot == dot_y:
                    #self.mouse_path[1] = self.mouse_path[0]  - self.vec_y
                    #bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                    bpy.ops.transform.translate('INVOKE_DEFAULT', constraint_axis=(False, True, False))
                    #print('Y')
                    return {'FINISHED'}
                else:
                    #self.mouse_path[1] = self.mouse_path[0]  - self.vec_z
                    #bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                    bpy.ops.transform.translate('INVOKE_DEFAULT', constraint_axis=(True, False, False))
                    #print('Z')
                    return {'FINISHED'}

            #self.set_transform_axis(direction)
        elif event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            #bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            # Завершение операции
            #print('FINISHED')
            return {'FINISHED'}

        #return {'RUNNING_MODAL'}
        return {'PASS_THROUGH'}

    def invoke(self, context, event):
        self.matrix = get_orient_from_type()[0]
        #self.pp = self.matrix.to_translation()

        mat = self.matrix.to_3x3()
        self.vec_x = (mat @ Vector((1,0,0))).normalized()
        self.vec_y = (mat @ Vector((0,1,0))).normalized()
        self.vec_z = (mat @ Vector((0,0,1))).normalized()

        #print('invoke')

        loc = get_coord_mouse(self, context, event, matrix=self.matrix)
        self.mouse_path = [loc, loc + Vector((0, 0, 1))] 


        args = (self, context)
        #self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_callback_px, args, 'WINDOW', 'POST_VIEW')
        #self._handle = bpy.types.SpaceView3D.draw_handler_add(self.draw_callback_px, args, 'WINDOW', 'PRE_VIEW')
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

    def get_mouse_direction(self, context, event):
        # Определение направления движения мыши
        region = context.region
        rv3d = context.region_data
        coord = event.mouse_region_x, event.mouse_region_y
        vec = region_2d_to_vector_3d(region, rv3d, coord)
        loc = region_2d_to_location_3d(region, rv3d, coord, vec)
        origin = region_2d_to_origin_3d(region, rv3d, coord)
        direction = (loc - origin).normalized()
        #self.mouse_path[1] = origin - loc# Обновление пути мыши
        return direction


    #def set_transform_axis(self, direction):
    #print('direction  :', direction)
    #print(self.mouse_path)
    # Установка оси трансформации
    # Здесь добавьте логику для выбора оси на основе направления мыши и матрицы ориентации
    #pass

def draw_callback_px(self, context):
    #print('direction  :', direction)
    #draw_batch(self.mouse_path, (1.0, 0.0, 0.0, 1.0), type='LINES', custom_shader=None)
    #print(self.mouse_path)
    # 50% alpha, 2 pixel width line
    shader = gpu.shader.from_builtin('UNIFORM_COLOR')
    gpu.state.blend_set('ALPHA')
    gpu.state.line_width_set(10.0)
    batch = batch_for_shader(shader, 'LINES', {"pos": self.mouse_path})
    shader.uniform_float("color", (1.0, 0.0, 0.0, 1.0))
    batch.draw(shader)

    # restore opengl defaults
    gpu.state.line_width_set(1.0)
    gpu.state.blend_set('NONE')



































classes = [
    GP_OT_modo_transform,

    #GP_OT_transform_modo_macro,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)





def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls) 