import bpy
from bpy.types import Macro, Operator
from ... import get_prefs

EDM = False
PTS = False
OGL = False # Luna: Orient To The Camera

class GP_OT_pivot_transform(Macro):
    bl_idname = 'gp.pivot_transform'
    bl_label = 'Pivot Transform'
    bl_options = {'UNDO'}


class GP_OT_pt_store_set(Operator):
    bl_idname = 'gp.pt_store_set'
    bl_label = 'Pivot Transform'
    bl_options = {'INTERNAL'}

    def execute(self, context):
        global EDM, PTS, OGL
        props = get_prefs()
        EDM = True if context.mode == 'EDIT_MESH' else False
        PTS = context.scene.tool_settings.use_transform_data_origin
        OGL = props.use_orientation_to_camera
        props.use_orientation_to_camera = False
        return {'FINISHED'}


class GP_OT_pt_store_get(Operator):
    bl_idname = 'gp.pt_store_get'
    bl_label = 'Pivot Transform'
    bl_options = {'INTERNAL'}

    def execute(self, context):
        global PTS, EDM, OGL
        props = get_prefs()
        context.scene.tool_settings.use_transform_data_origin = PTS
        if EDM:
            bpy.ops.object.mode_set(mode='EDIT')
        props.use_orientation_to_camera = OGL
        return {'FINISHED'}



classes = [
    GP_OT_pivot_transform,
    GP_OT_pt_store_set,
    GP_OT_pt_store_get,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

    
    GP_OT_pivot_transform.define('GP_OT_pt_store_set')

    op = GP_OT_pivot_transform.define('OBJECT_OT_mode_set')
    op.properties.mode = 'OBJECT'

    op = GP_OT_pivot_transform.define('WM_OT_context_set_boolean')
    op.properties.data_path = 'scene.tool_settings.use_transform_data_origin'
    op.properties.value = True

    GP_OT_pivot_transform.define('TRANSFORM_OT_transform')
    
    GP_OT_pivot_transform.define('GP_OT_pt_store_get')


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls) 