import bpy
from bpy.types import Operator, GizmoGroup
from mathutils import Matrix, Vector, Quaternion
from ... import var,get_prefs
from bpy.props import BoolProperty, FloatVectorProperty, EnumProperty
from ...matrix.orientation.orientation_mesh import normal_qat_from_two_point



class GP_GGT_pco_ensure(GizmoGroup):
    
    bl_idname = 'GP_GGT_pco_ensure'
    bl_label = 'Set Pro Cursor'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'3D', 'SHOW_MODAL_ALL', 'VR_REDRAWS'} # , 'TOOL_INIT', 'SELECT', 'SCALE', 'DEPTH_3D', 'PERSISTENT', 


    __slots__ = (
        'ray',
    )

    """ @classmethod
    def poll(cls, context):
        return context.scene.GP_scene_set.pro_center """
        
    
    def setup(self, context):
        self.ray = self.gizmos.new('GIZMO_GT_snap_3d')
        #self.ray.snap_elements_force = {'EDGE_PERPENDICULAR', 'VERTEX', 'EDGE_MIDPOINT', 'EDGE', 'FACE'}


    def draw_prepare(self, context):
        props = get_prefs()
        settings = context.scene.GP_scene_set
        
        mat = context.scene.cursor.matrix.copy() # TODO FIXME
        sl, sq, s = mat.decompose()

        if var.PCO_ALIGN:
            l = sl
            if props.tdc_advance_align:
                q = normal_qat_from_two_point(sl, self.ray.location, var.PCO_NORMAL_STORE, self.ray.normal)
            else:
                n = sl - self.ray.location 
                q = n.to_track_quat('-Z', 'X')
        else:
            if var.PCO_USE_POS:
                l = self.ray.location
            else:
                l = sl

            if var.PCO_USE_ORIENT:
                n = self.ray.normal
                q = n.to_track_quat('Z', 'X')
            else:
                q = sq

            var.PCO_NORMAL_STORE = self.ray.normal # --- Store

        var.PCO_MATRIX = Matrix.LocRotScale(l, q, s) # idx = self.ray.snap_elem_index[:]



class GP_OT_pro_center_ensure(Operator):
    bl_idname = 'gp.pro_center_ensure'
    bl_label = 'Pro Cursor Ensure'
    bl_description = 'Set Cursor'
    bl_options = {'UNDO', 'INTERNAL'} # 'REGISTER',
    
    action_ex: EnumProperty(
        name = 'Action',
        items = (
            ('POSITION', 'Position', ''),
            ('ORIENTATION', 'Orientation', ''),
            ('ALL', 'All', ''),
        ),
        default = 'ALL',
    )

    def __init__(self):
        self._snap_state_store = False
        self._pro_center_store = False
        self._pro_orient_store = False
        self._mw_store = Matrix()


    def modal(self, context, event):
        settings = context.scene.GP_scene_set
        

        if event.value == 'RELEASE' and event.type in {'ESC', 'RIGHTMOUSE'}:
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_pco_ensure')

            context.scene.tool_settings.use_snap = self._snap_state_store
            settings.position = self._pro_center_store
            settings.orientation = self._pro_orient_store

            context.scene.cursor.matrix = self._mw_store

            context.scene.transform_orientation_slots[0].type = var.PCO_STORE_TOS
            context.scene.tool_settings.transform_pivot_point = var.PCO_STORE_TPP
            
            var.PCO_MATRIX = Matrix()
            var.PCO_ALIGN = False
            var.REFRESH_MAT = True
            return {'CANCELLED'}


        elif event.value == 'RELEASE' and event.type in {'RET', 'MIDDLEMOUSE'}:
            context.window_manager.gizmo_group_type_unlink_delayed('GP_GGT_pco_ensure')

            context.scene.tool_settings.use_snap = self._snap_state_store
            settings.position = self._pro_center_store
            settings.orientation = self._pro_orient_store

            context.scene.cursor.matrix = var.PCO_MATRIX

            if self.action_ex == 'POSITION':
                context.scene.transform_orientation_slots[0].type = var.PCO_STORE_TOS
            elif self.action_ex == 'ORIENTATION':
                context.scene.tool_settings.transform_pivot_point = var.PCO_STORE_TPP

            var.PCO_MATRIX = Matrix()
            var.PCO_ALIGN = False
            var.REFRESH_MAT = True
            #print('FINISHED')
            return {'FINISHED'}
        

        if event.value == 'RELEASE' and event.ctrl is False:
            context.scene.tool_settings.use_snap = True
            var.PCO_ALIGN = True


        # --- Apply Matrix
        context.scene.cursor.matrix = var.PCO_MATRIX

        var.REFRESH_MAT = True
            
        return {'PASS_THROUGH'}


    def invoke(self, context, event):
        settings = context.scene.GP_scene_set
        
        self._pro_center_store = settings.position
        self._pro_orient_store = settings.orientation
        settings.position = 'NONE'
        settings.orientation = 'NONE'

        self._mw_store = context.scene.cursor.matrix

        self._snap_state_store = context.scene.tool_settings.use_snap
        if event.ctrl is False:
            context.scene.tool_settings.use_snap = True
        else:
            context.scene.tool_settings.use_snap = False


        if context.scene.transform_orientation_slots[0].type != 'CURSOR':
            var.PCO_STORE_TOS = context.scene.transform_orientation_slots[0].type
        if context.scene.tool_settings.transform_pivot_point != 'CURSOR':
            var.PCO_STORE_TPP = context.scene.tool_settings.transform_pivot_point
        context.scene.transform_orientation_slots[0].type = 'CURSOR'
        context.scene.tool_settings.transform_pivot_point = 'CURSOR'

        
        context.window_manager.gizmo_group_type_ensure('GP_GGT_pco_ensure')
        var.REFRESH_MAT = True
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}



    




class GP_OT_pco_transform(Operator):
    bl_idname = 'gp.pco_transform'
    bl_label = 'Transform Pro Cursor'
    bl_description = 'Transform Cursor'
    #bl_options = {'REGISTER', 'UNDO'}

    type_transform: EnumProperty(
        name='Type Transform',
        items=(
            ('LOC', 'Location', '', '', 0),
            ('ROT', 'Rotation', '', '', 1),
        ),
        default='LOC',
    )
    first_pos: FloatVectorProperty(name='First Position', default=(0.0, 0.0, 0.0))
    first_orient: FloatVectorProperty(name='First Orientation', default=(0.0, 0.0, 0.0))


    def execute(self, context):
        #settings = context.scene.GP_scene_set
        from ...matrix.matrix_ensure import get_position
        if self.type_transform == 'LOC':
            pos_new = get_position(context, position='MEDIAN_POINT')
            print(Vector(self.first_pos), pos_new, Vector(self.first_pos)==pos_new)
            

        return {'FINISHED'}







classes = [
    GP_GGT_pco_ensure,
    GP_OT_pro_center_ensure,
    GP_OT_pco_transform,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)