import bpy
from bpy.types import Operator
import bmesh


class GP_OT_qmesh(Operator):
    bl_idname = 'gp.qmesh'
    bl_label = 'Q-Mesh'
    bl_description = ' '
    bl_options = {'REGISTER', 'UNDO', 'INTERNAL'}


    @classmethod
    def poll(cls, context):
        return context.mode == 'EDIT_MESH'


    def __init__(self):
        self.vert = []
        self.active_element = None
        self.store_snap_mode = {}
        self.store_use_snap = False
        self.store_use_snap_self = False
        self.store_automerge = False


    def modal(self, context, event):
        if event.type in {'RIGHTMOUSE', 'ESC'}:
            return {'CANCELLED'}

        if event.value == 'RELEASE' or event.type in {'RET'}:
            self.exit(context)

        if len(self.vert) == 1:
            bpy.ops.mesh.merge(type='CENTER', uvs=True)
            #bpy.ops.transform.translate('INVOKE_DEFAULT', release_confirm = True)
            #return {'PASS_THROUGH'}

        else:
            if context.scene.tool_settings.transform_pivot_point == 'ACTIVE_ELEMENT':
                if isinstance(self.active_element, bmesh.types.BMVert):
                    bpy.ops.mesh.merge(type='LAST', uvs=True)
                else:
                    bpy.ops.mesh.merge(type='CENTER', uvs=True)
            else:
                bpy.ops.mesh.merge(type='CENTER', uvs=True)

            
            self.exit(context)




        if event.value == 'RELEASE' or event.type in {'RET'}:
            self.exit(context)

        if event.type in {'RIGHTMOUSE', 'ESC'}:
            return {'CANCELLED'}
 
        #return {'PASS_THROUGH'}
        return {'FINISHED'}

    def invoke(self, context, event):
        
        # --- Vertex Count
        uniques = context.selected_objects
        for obj in uniques:
            bm = bmesh.from_edit_mesh(obj.data)
            for v in bm.verts:
                if v.select:
                    self.vert.append(v.index)

        # --- Active Element
        obj = context.active_object
        bm = bmesh.from_edit_mesh(obj.data)
        self.active_element = bm.select_history.active

        # --- Set Parametr
        tool_settings = context.scene.tool_settings

        self.store_snap_mode = tool_settings.snap_elements
        tool_settings.snap_elements = {'VERTEX'}

        self.store_use_snap = tool_settings.use_snap
        tool_settings.use_snap = True

        self.store_use_snap_self = tool_settings.use_snap_self
        tool_settings.use_snap_self = True

        self.store_automerge = tool_settings.use_mesh_automerge
        tool_settings.use_mesh_automerge = True



        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}  


    def exit(self, context):
        tool_settings = context.scene.tool_settings
        tool_settings.snap_elements = self.store_snap_mode
        tool_settings.use_snap = self.store_use_snap
        tool_settings.use_snap_self = self.store_use_snap_self
        tool_settings.use_mesh_automerge = self.store_automerge
        return {'FINISHED'}


classes = [
    GP_OT_qmesh,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls) 