import bpy
from bpy.types import Operator
from bpy_extras import view3d_utils

from ...matrix.matrix_ensure import get_orient_from_type
from ..func import convert_units

import blf
from ... import get_prefs



class GP_OT_resize_empty(Operator):
    bl_idname = 'gp.resize_empty'
    bl_label = 'Resize Empty'


    def __init__(self):
        self.props = get_prefs()

        self.start_size = {}
        self.pos_ob = {}

        self.mouse_start = None
        self._handle_text = None

    
    def draw_text(self, context, event):
        font_info = {
                'font_id': 0,
                'handler': None,
                }
        
        for obj in self.pos_ob:
            if obj.type == 'EMPTY':
                font_id= font_info['font_id']
                pos = self.pos_ob[obj]
                pos_x = pos[0] + 25
                pos_y = pos[1] - 150 * self.props.size_gizmo_pro
                blf.position(font_id, pos_x, pos_y, 0)
                blf.size(font_id, 16, 72)
                blf.color(font_id, 0.9, 0.9, 0.9, 1.0)
                blf.shadow(font_id, 3, 0.0, 0.0, 0.0, 1.0)

                size = obj.empty_display_size
                value, unit = convert_units( context, size )
                rVal = round( value, 3 )
             
                blf.draw(font_id, str(rVal) + unit)


    def modal(self, context, event):   
        coef = 100
        value = (self.mouse_start[1] - event.mouse_region_y) / coef

        if event.ctrl:
            value = round(value)
        if event.shift:
            value /= 10.0



        for obj in context.selected_objects:
            if obj.type == 'EMPTY':
                obj.empty_display_size = self.start_size[obj] + value 



        if event.type in {'RIGHTMOUSE', 'ESC'}:
            for obj in context.selected_objects:
                if obj.type == 'EMPTY':
                    obj.empty_display_size = self.start_size[obj]

            bpy.types.SpaceView3D.draw_handler_remove(self._handle_text, 'WINDOW')
            return {'FINISHED'}
            
        

        if event.value == 'RELEASE':
            bpy.types.SpaceView3D.draw_handler_remove(self._handle_text, 'WINDOW')
            return {'FINISHED'}


        return {'PASS_THROUGH'}


    def invoke(self, context, event):
        self.mouse_start = (event.mouse_region_x, event.mouse_region_y)

        region = context.region
        rv3d = context.region_data

        for obj in context.selected_objects:
            if obj.type == 'EMPTY':
                self.start_size[obj] = obj.empty_display_size
                self.pos_ob[obj] = view3d_utils.location_3d_to_region_2d(region, rv3d, obj.location) 

        args = (self, context)
        self._handle_text = bpy.types.SpaceView3D.draw_handler_add(self.draw_text, args, 'WINDOW', 'POST_PIXEL')

        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}  



        



classes = [
    GP_OT_resize_empty,
    ]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls) 