import bpy, bmesh
from bpy.types import Operator, Macro
from bpy.props import (
    EnumProperty, 
    StringProperty, 
    FloatProperty, 
    BoolVectorProperty, 
    BoolProperty, 
    FloatVectorProperty,
)
from math import radians
from ...matrix.matrix_ensure import get_position
from ..poll import poll_gizmopro
from ..func import pro_center_orient_reset
from mathutils import Vector, Matrix
from ... import get_prefs



class GP_OT_transform_from_value(Operator):
    bl_idname = 'gp.transform_from_value'
    bl_label = 'Transform From Value'
    #bl_description = ' '
    bl_options = {'REGISTER', 'UNDO', 'INTERNAL'}
    

    def update_rotate(self, context):
        if self.negative:
            self.value_angle = radians( -float(self.angle) )
        else:
            self.value_angle = radians( float(self.angle) )


    def update_scale(self, context):
        if self.setZ:
            self.value_scale = 0.0
            self.setZ = False
        if self.setO:
            self.value_scale = 1.0
            self.setO = False



    pp: FloatVectorProperty()
    matrix_basis: FloatVectorProperty( size=(3, 3), subtype='MATRIX' )
    axis: BoolVectorProperty( default=(True, True, True), size=3 )
    value_transform: FloatProperty(name='Value', default=0.0, subtype='DISTANCE')

    val_x: FloatProperty( name='X', default=0.0, subtype='DISTANCE' )
    val_y: FloatProperty( name='Y', default=0.0, subtype='DISTANCE' )
    val_z: FloatProperty( name='Z', default=0.0, subtype='DISTANCE' )

    value_angle: FloatProperty( name='Value', default=0.0, unit='ROTATION', subtype='ANGLE' )
    value_scale: FloatProperty( name='Value', default=1.0 )
    operator: StringProperty( default='DOT' )
    negative: BoolProperty( name='-', default=False, update=update_rotate )
    setZ: BoolProperty( name='0', default=False, update=update_scale )
    setO: BoolProperty( name='1', default=False, update=update_scale )
    angle: EnumProperty(
        name='Angle',
        description='Angle Preset',
        items=[
            ('0', '0°', '', '', 0),
            ('45', '45°', '', '', 1),
            ('90', '90°', '', '', 2),
            ('135', '135°', '', '', 3),
            ('180', '180°', '', '', 4),
            ],
        default='0',
        update=update_rotate,
        )


    def execute(self, context):
        tpp = context.scene.tool_settings.transform_pivot_point
        #os = context.window.scene.transform_orientation_slots[0].type
        settings = context.scene.GP_scene_set

        if self.operator in {'DOT', 'TRANSLATION_PLANE', 'TRANSLATION'}:
            if self.operator == 'DOT' or self.operator == 'TRANSLATION_PLANE':
                val_final = [0.0, 0.0, 0.0, 0.0]
                if self.axis[0]:
                    val_final[0] = self.val_x
                if self.axis[1]:
                    val_final[1] = self.val_y
                if self.axis[2]:
                    val_final[2] = self.val_z


            elif self.operator == 'TRANSLATION':
                value = self.value_transform

                val_final = [0.0, 0.0, 0.0, 0.0]
                if self.axis[0]:
                    val_final[0] = value
                if self.axis[1]:
                    val_final[1] = value
                if self.axis[2]:
                    val_final[2] = value
                

            if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                bpy.ops.transform.transform(
                    mode = 'TRANSLATION',
                    value = val_final,
                    )
            else:
                bpy.ops.transform.transform(
                    mode = 'TRANSLATION',
                    value = val_final,
                    orient_matrix = self.matrix_basis,
                    center_override = self.pp,
                    )


        elif self.operator == 'ROTATION':
            if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                if self.axis[:] == (True, True, True): # --- VIEW
                    bpy.ops.transform.rotate(
                        value = self.value_angle,
                        orient_type = 'VIEW',
                        orient_matrix_type = 'VIEW',
                        )
                else:
                    bpy.ops.transform.rotate(
                        value = self.value_angle,
                        orient_matrix = self.matrix_basis,
                        constraint_axis = self.axis[:],
                        )
            else:
                if self.axis[:] == (True, True, True): # --- VIEW
                    bpy.ops.transform.rotate(
                        value = self.value_angle,
                        orient_type = 'VIEW',
                        orient_matrix_type = 'VIEW',
                        center_override = self.pp,
                        )
                else:
                    bpy.ops.transform.rotate(
                        value = self.value_angle,
                        orient_matrix = self.matrix_basis,
                        constraint_axis = self.axis[:],
                        center_override = self.pp,
                        )


        elif self.operator == 'RESIZE':
            if self.axis[:] == (True, True, True):  # --- VIEW
                val_final = (self.value_scale, self.value_scale, self.value_scale, 0)
            else:
                if self.axis[0]:
                    val_final = (self.value_scale, 1.0, 1.0, 0.0)
                elif self.axis[1]:
                    val_final = (1.0, self.value_scale, 1.0, 0.0)
                elif self.axis[2]:
                    val_final = (1.0, 1.0, self.value_scale, 0.0)


            if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                bpy.ops.transform.transform(
                    mode = 'RESIZE',
                    value = val_final,
                    orient_matrix = self.matrix_basis,
                    constraint_axis = self.axis[:],
                    )
            else:
                bpy.ops.transform.transform(
                    mode = 'RESIZE',
                    value = val_final,
                    orient_matrix = self.matrix_basis,
                    constraint_axis = self.axis[:],
                    center_override = self.pp,
                    )
        
        pro_center_orient_reset(context) # --- Temporary

        return {'FINISHED'}
     


    def invoke(self, context, event):
        #self._init_position = self.pp
        #print('1')
        wm = context.window_manager
        #return wm.popover(self.draw,  ui_units_x=10)
        #return wm.invoke_popup(self, width=100)
        return wm.invoke_props_popup(self, event)
        #return wm.invoke_props_dialog( self, width = 100 )
        #return wm.invoke_confirm(self, event)
        #return wm.invoke_search_popup(self)
        #return wm.invoke_popup(self, width=200)
        

    def draw(self, context):
        layout = self.layout
        if self.operator in { 'DOT', 'TRANSLATION_PLANE' }:
            l = self.layout
            l.activate_init = True
            l.enabled = True

            row = l.row()
            if self.axis[0]:
                row.prop(self, 'val_x')
            if self.axis[1]:
                row.prop(self, 'val_y')
            if self.axis[2]:
                row.prop(self, 'val_z')


        elif self.operator == 'TRANSLATION':
            row = layout.row(align=True)
            #row.activate_init = True
            row.operator('gp.reset_transform_position', text='', icon='SNAP_PEEL_OBJECT').axis = self.axis
            row.separator(factor=0.3)
            row.prop(self, 'value_transform')
            row.separator(factor=0.3)
            row.operator('gp.invert_transform_position', text='', icon='ARROW_LEFTRIGHT').axis = self.axis
            
            
        elif self.operator == 'ROTATION':
            col = layout.column()
            row_a = col.row()
            row_a.scale_x = 2
            row_a.prop(self, 'value_angle')
            
            row_b = col.row(align=True)
            row_b.prop(self, 'negative', toggle=True)
            row_b.prop(self, 'angle', expand=True)


        else:
            row_all = layout.row(align=True)

            row_a = row_all.row()
            row_a.scale_x = 2
            row_a.prop(self, 'value_scale')

            row_b = row_all.row(align=True)
            row_b.prop(self, 'setZ', toggle=True)
            row_b.prop(self, 'setO', toggle=True)








class GP_OT_transform(Operator):
    bl_idname = 'gp.transform'
    bl_label = 'Transform'
    bl_options = {'GRAB_CURSOR', 'INTERNAL'}
    

    operator: EnumProperty(
        name = 'Angle',
        description = 'Angle Preset',
        items=[
            ('TRANSLATION', 'Move', '', '', 0),
            ('RESIZE', 'Resize', '', '', 1),
            ('ROTATION', 'Rotation', '', '', 2),
            ('TRANSLATION_PLANE', 'Move Plane', '', '', 2),
            ],
        default = 'TRANSLATION',
        )


    @classmethod
    def description(self, context, properties):
        props = get_prefs()
        if props.operator == 'TRANSLATION':
            return 'Move'
        elif props.operator == 'RESIZE':
            return 'Resize'
        elif props.operator == 'ROTATION':
            return 'Rotate'
        elif props.operator == 'TRANSLATION_PLANE':
            return 'Move Plane'


    @classmethod
    def poll(cls, context):
        return poll_gizmopro(context)


    def execute(self, context):
        pp = get_position(context)
        tpp = context.scene.tool_settings.transform_pivot_point
        settings = context.scene.GP_scene_set

        if self.operator == 'RESIZE':
            if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                bpy.ops.transform.resize( 'INVOKE_DEFAULT' ) # release_confirm = True, 
            else:
                bpy.ops.transform.resize( 'INVOKE_DEFAULT', center_override = pp )

        elif self.operator == 'TRANSLATION_PLANE':
            if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                bpy.ops.transform.transform( 'INVOKE_DEFAULT', mode = 'TRANSLATION' )
            else:
                bpy.ops.transform.transform( 'INVOKE_DEFAULT', mode = 'TRANSLATION', center_override = pp )

        else:
            if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
                bpy.ops.transform.transform( 'INVOKE_DEFAULT', mode = self.operator )
            else:
                bpy.ops.transform.transform( 'INVOKE_DEFAULT', mode = self.operator, center_override = pp )

        return {'FINISHED'}
     


class GP_OT_invert_transform_position(Operator): # TODO FIXME GP_MODE сделать под разные context.mode
    bl_idname = 'gp.invert_transform_position'
    bl_label = 'Invert'
    bl_description = 'Invert Position Value'
    bl_options = {'UNDO', 'INTERNAL'}

    axis: BoolVectorProperty(
        default = (False, False, False),
        size = 3,
        )

    def execute(self, context):
        if context.mode == 'OBJECT':
            obSel = context.selected_objects
            for ob in obSel:
                for i in range(3):
                    if self.axis[i]:
                        ob.location[i] *= -1

        elif context.mode == 'EDIT_MESH':
            edObj = context.objects_in_mode_unique_data
            for ob in edObj:
                me = ob.data
                bm = bmesh.from_edit_mesh(me)
                mw = ob.matrix_world
                imw = mw.inverted()
                for v in bm.verts:
                    if v.select:
                        world_pos = mw @ v.co
                        for i in range(3):
                            if self.axis[i]:
                                world_pos[i] *= -1
                        v.co = imw @ world_pos
                # Обновление нормалей
                bmesh.ops.recalc_face_normals(bm, faces=bm.faces)
                bmesh.update_edit_mesh(me, loop_triangles=True)
                me.update(calc_edges=True, calc_edges_loose=True)
        context.view_layer.update()
        return {'FINISHED'}


class GP_OT_reset_transform_position(Operator): # TODO FIXME GP_MODE сделать под разные context.mode
    bl_idname = 'gp.reset_transform_position'
    bl_label = 'Reset'
    bl_description = 'Reset Position Value'
    bl_options = {'UNDO', 'INTERNAL'}

    axis: BoolVectorProperty(
        default = (False, False, False),
        size = 3,
        )

    def execute(self, context):
        if context.mode == 'OBJECT':
            obSel = context.selected_objects
            for ob in obSel:
                for i in range(3):
                    if self.axis[i]:
                        ob.location[i] = 0

        elif context.mode == 'EDIT_MESH':
            edObj = context.objects_in_mode_unique_data
            for ob in edObj:
                me = ob.data
                bm = bmesh.from_edit_mesh(me)
                mw = ob.matrix_world
                imw = mw.inverted()
                for v in bm.verts:
                    if v.select:
                        world_pos = mw @ v.co
                        for i in range(3):
                            if self.axis[i]:
                                world_pos[i] = 0
                        v.co = imw @ world_pos
                # Обновление нормалей
                bmesh.ops.recalc_face_normals(bm, faces=bm.faces)
                bmesh.update_edit_mesh(me, loop_triangles=True)
                me.update(calc_edges=True, calc_edges_loose=True)
        context.view_layer.update()
        return {'FINISHED'}



classes = [
    GP_OT_transform_from_value,
    GP_OT_transform,
    GP_OT_invert_transform_position,
    GP_OT_reset_transform_position,
    ]

def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
