import bpy
from bpy.types import Operator
import configparser
from pathlib import Path
import ast
import re
from bpy_extras.io_utils import ImportHelper, ExportHelper
from .. import var
from bpy.props import EnumProperty, BoolProperty, StringProperty, FloatVectorProperty
from mathutils import Matrix, Quaternion, Vector, Euler, Color
from math import radians
from ..utils.func import (
    transform_orientation,
    transform_pivot_point,
    transform_snap_to,
    transform_proportional,
    pro_center_orient_reset,
)
from .. import get_prefs


class GP_OT_documentation(Operator): # --- Documentation
    bl_idname = 'gp.documentation'
    bl_label = 'Gizmo PRO Documentation'
    bl_description = 'Open The Documentation On Web Browser'

    def execute(self, context):
        bpy.ops.wm.url_open(url = 'https://max-derksen.gitbook.io/gizmo-pro/')
        return {'FINISHED'} 
    
    def invoke(self, context, event):
        return context.window_manager.invoke_confirm(self, event)


class GP_OT_switch_gizmo(Operator): # --- Switch Gizmo
    bl_idname = 'gp.switch_gizmo'
    bl_label = 'Switch Gizmo'
    bl_options = {'INTERNAL'}
    

    type: EnumProperty(
        name='Type',
        description='Type',
        items=[
            ('TRANSLATE', 'Translate', '', 0),
            ('ROTATE', 'Rotate', '', 1),
            ('SCALE', 'Scale', '', 2),
            ],
        default='TRANSLATE',
        )


    @classmethod
    def description(self, context, properties):
        if properties.type == 'TRANSLATION':
            return 'Switch Gizmo Translate'
        elif properties.type == 'RESIZE':
            return 'Switch Gizmo Resize'
        elif properties.type == 'ROTATION':
            return 'Switch Gizmo Rotation'


    @classmethod
    def poll(cls, context):
        return context.area.type == 'VIEW_3D'


    def execute(self, context):
        #props = context.preferences.addons['GIZMO_PRO'].preferences
        settings = context.scene.GP_scene_set


        if self.type == 'TRANSLATE':
            if settings.gzm_type=='GIZMOPRO':
                if settings.tra_gizmo and settings.rot_gizmo == False and settings.sca_gizmo == False:
                    settings.tra_gizmo = var.TYPE_GIZMO[0] #True
                    settings.rot_gizmo = var.TYPE_GIZMO[1] #True
                    settings.sca_gizmo = var.TYPE_GIZMO[2] #True
                else:
                    var.TYPE_GIZMO = ( settings.tra_gizmo, settings.rot_gizmo, settings.sca_gizmo )
                    settings.tra_gizmo = True
                    settings.rot_gizmo = False
                    settings.sca_gizmo = False

            elif settings.gzm_type=='LUNA':
                if settings.tra_gizmo and settings.rot_gizmo == False and settings.sca_gizmo == False:
                    settings.tra_gizmo = var.TYPE_LUNA[0]
                    settings.rot_gizmo = var.TYPE_LUNA[1]
                    settings.sca_gizmo = var.TYPE_LUNA[2]
                else:
                    var.TYPE_LUNA = ( settings.tra_gizmo, settings.rot_gizmo, settings.sca_gizmo )
                    settings.tra_gizmo = True
                    settings.rot_gizmo = False
                    settings.sca_gizmo = False

        elif self.type == 'ROTATE':
            if settings.gzm_type=='GIZMOPRO':
                if settings.rot_gizmo and settings.tra_gizmo == False and settings.sca_gizmo == False:
                    settings.tra_gizmo = var.TYPE_GIZMO[0]
                    settings.rot_gizmo = var.TYPE_GIZMO[1]
                    settings.sca_gizmo = var.TYPE_GIZMO[2]
                else:
                    var.TYPE_GIZMO = (settings.tra_gizmo, settings.rot_gizmo, settings.sca_gizmo)
                    settings.tra_gizmo = False
                    settings.rot_gizmo = True
                    settings.sca_gizmo = False
                    
            elif settings.gzm_type=='LUNA':
                if settings.rot_gizmo and settings.tra_gizmo == False and settings.sca_gizmo == False:
                    settings.tra_gizmo = var.TYPE_LUNA[0]
                    settings.rot_gizmo = var.TYPE_LUNA[1]
                    settings.sca_gizmo = var.TYPE_LUNA[2]
                else:
                    var.TYPE_LUNA = ( settings.tra_gizmo, settings.rot_gizmo, settings.sca_gizmo )
                    settings.tra_gizmo = False
                    settings.rot_gizmo = True
                    settings.sca_gizmo = False


        elif self.type == 'SCALE':
            if settings.gzm_type=='GIZMOPRO':
                if settings.sca_gizmo and settings.tra_gizmo == False and settings.rot_gizmo == False:
                    settings.tra_gizmo = var.TYPE_GIZMO[0]
                    settings.rot_gizmo = var.TYPE_GIZMO[1]
                    settings.sca_gizmo = var.TYPE_GIZMO[2]
                else:
                    var.TYPE_GIZMO = (settings.tra_gizmo, settings.rot_gizmo, settings.sca_gizmo)
                    settings.tra_gizmo = False
                    settings.rot_gizmo = False
                    settings.sca_gizmo = True

            elif settings.gzm_type=='LUNA':
                if settings.sca_gizmo and settings.tra_gizmo == False and settings.rot_gizmo == False:
                    settings.tra_gizmo = var.TYPE_LUNA[0]
                    settings.rot_gizmo = var.TYPE_LUNA[1]
                    settings.sca_gizmo = var.TYPE_LUNA[2]
                else:
                    var.TYPE_LUNA = ( settings.tra_gizmo, settings.rot_gizmo, settings.sca_gizmo )
                    settings.tra_gizmo = False
                    settings.rot_gizmo = False
                    settings.sca_gizmo = True
            
                    

        if context.area:
            context.area.tag_redraw()
        return {'FINISHED'}


class GP_OT_panel_settings(Operator): # --- Open Panel Settings
    bl_idname = 'gp.panel_settings'
    bl_label = 'Gizmo PRO Settings'
    bl_description = ''
    bl_options = {'INTERNAL'}
    

    def execute(self, context):
        bpy.ops.screen.userpref_show()
        bpy.context.preferences.active_section = 'ADDONS'
        bpy.data.window_managers["WinMan"].addon_search = 'Gizmo PRO'
        return {'FINISHED'}


class GP_OT_save_preferences(Operator, ExportHelper, ImportHelper): # --- Save Preferences to File
    bl_idname = 'gp.save_preferences'
    bl_label = 'Save Preferences'
    bl_description = ''
    bl_options = {'REGISTER', 'UNDO', 'INTERNAL'}

    filename_ext = '.ini'

    type: EnumProperty(
        name='Type',
        description='',
        items=[
            ('SAVE', 'Save', '', '', 0),
            ('LOAD', 'Load', '', '', 1),
        ],
        default='SAVE',
    )

    def execute(self, context):
        addon_prefs = get_prefs()
        config = configparser.ConfigParser()

        not_used = [
            'check_updates', 'update', 'version', 'test',
            'tab', 'type_key', 'type_gizmo',
        ]

        if self.type == 'SAVE':
            # Save addon preferences to file
            addon_name = __name__.split('.')[0]
            addon_prefs_dict = {}
            for key, value in addon_prefs.__annotations__.items():
                if key not in not_used:
                    addon_prefs_dict[key] = getattr(addon_prefs, key)
            config[addon_name] = addon_prefs_dict
            filepath = Path(self.filepath)
            with open(filepath, 'w') as f:
                config.write(f)
        elif self.type == 'LOAD':
            # Load addon preferences from file
            config.read(self.filepath)
            if __name__.split('.')[0] in config:
                addon_prefs_dict = config[__name__.split('.')[0]]
                for key, value in addon_prefs_dict.items():
                    if hasattr(addon_prefs, key):
                        if key not in not_used:
                            # Convert string values to their corresponding types
                            if isinstance(getattr(addon_prefs, key), bool):
                                value = ast.literal_eval(value)
                            elif isinstance(getattr(addon_prefs, key), float):
                                value = ast.literal_eval(value)
                            elif isinstance(getattr(addon_prefs, key), int):
                                value = ast.literal_eval(value)
                            elif isinstance(getattr(addon_prefs, key), str):
                                value = str(value)
                            elif isinstance(getattr(addon_prefs, key), Vector):
                                value = re.findall(r'\((.*?)\)', value)[0]
                                value = [float(x) for x in value.split(',')]
                                value = Vector((value[0], value[1], value[2]))
                            elif isinstance(getattr(addon_prefs, key), Color):
                                value = re.findall(r'[-+]?\d*\.\d+|\d+', value)
                                value = [float(x) for x in value]
                                value = (value[0], value[1], value[2])
                            else:
                                print(key, value, type(getattr(addon_prefs, key)))
                                pass
                            setattr(addon_prefs, key, value)
        return {'FINISHED'}
 

class GP_OT_preset_modifiers(Operator): # --- Preset Modifiers (set None all or Default)
    bl_idname = 'gp.preset_modifiers'
    bl_label = 'Load Preset'
    bl_description = ''
    bl_options = {'INTERNAL'}

    type: EnumProperty(
        name='Type',
        description='',
        items=[
            ('NONE', 'Turn Off All', '', '', 0),
            ('DEFAULT', 'Default', '', '', 1),
        ],
        default='NONE',
    )

    @classmethod
    def description(self, context, properties):
        if properties.type == 'NONE':
            return 'Turn Off All'
        elif properties.type == 'DEFAULT':
            return 'Default'

    def execute(self, context):
        props = get_prefs()
        if self.type == 'NONE':
            props.obCone_shift = 'NONE'
            props.obCone_ctrl = 'NONE'
            props.meCone_shift = 'NONE'
            props.meCone_ctrl = 'NONE'

            props.obArc_shift = 'NONE'
            props.obArc_ctrl = 'NONE'
            props.meArc_shift = 'NONE'
            props.meArc_ctrl = 'NONE'

            props.obBox_shift = 'NONE'
            props.obBox_ctrl = 'NONE'
            props.meBox_shift = 'NONE'
            props.meBox_ctrl = 'NONE'

        elif self.type == 'DEFAULT':
            props.obCone_shift = 'DUPLICATE'
            props.obCone_ctrl = 'NONE'
            props.meCone_shift = 'EXTRUDE'
            props.meCone_ctrl = 'INSET'

            props.obArc_shift = 'DUPLICATE'
            props.obArc_ctrl = 'INCROT'
            props.meArc_shift = 'DUPLICATE'
            props.meArc_ctrl = 'INCROT'

            props.obBox_shift = 'SCALE_TWO'
            props.obBox_ctrl = 'ALIGN'
            props.meBox_shift = 'BEVEL'
            props.meBox_ctrl = 'ALIGN'
        return {'FINISHED'}




# --- 3D CURSOR
class GP_OT_cursor_to_active(Operator):
    bl_idname = 'gp.cursor_to_active'
    bl_label = '3D Cursor To Active'
    bl_description = 'To Selected'
    bl_options = {'UNDO', 'INTERNAL'}

    
    position: BoolProperty(name='Position', default=True)
    rotation: BoolProperty(name='Rotation', default=True)
    to_pivot: BoolProperty(name='Snap To Pivot', default=True)

    _init_orient = None
    _init_mat = None


    @staticmethod
    def cursor_orient(self, context):
        scene = context.window.scene
        cursor = scene.cursor
        cursor_pos = cursor.location.copy()
        name = 'GizmoPRO-3D_Cursor'
        scene.transform_orientation_slots[0].type = 'NORMAL'
        bpy.ops.transform.create_orientation(name=name, use=True, overwrite=True)
        user_matrix = scene.transform_orientation_slots[0].custom_orientation.matrix.to_4x4()
        cursor.matrix = Matrix.Translation(cursor_pos) @ user_matrix
        scene.transform_orientation_slots[0].type = name
        bpy.ops.transform.delete_orientation('INVOKE_DEFAULT')
        scene.transform_orientation_slots[0].type = self._init_orient


    def execute(self, context):
        scene = context.window.scene
        cursor = scene.cursor
        cursor.matrix = self._init_mat


        if self.position:
            objs = context.selected_objects
            if context.mode=='EDIT_MESH' and len(objs)>0:
                s = context.scene.statistics(context.view_layer)
                v = s.split('Verts:')[1].split(' ')[0]
                if v[0] == '0':
                    loc = [o.location for o in objs]
                    mid = sum(loc, Vector()) / len(loc)
                    cursor.location = mid
                else:
                    bpy.ops.view3d.snap_cursor_to_selected()
            else:
                bpy.ops.view3d.snap_cursor_to_selected()
         
        if self.rotation:
            objs = context.selected_objects
            if context.mode=='EDIT_MESH' and len(objs)>0:
                s = context.scene.statistics(context.view_layer)
                v = s.split('Verts:')[1].split(' ')[0]
                if v[0] == '0':
                    l = cursor.location.copy()
                    q = context.active_object.matrix_world.decompose()[1]
                    s = Vector((1,1,1))
                    cursor.matrix = Matrix.LocRotScale(l, q, s)
                else:
                    self.cursor_orient(self, context)
            else:
                self.cursor_orient(self, context)
        
        var.REFRESH_MAT = True
        return {'FINISHED'}
    

    def invoke(self, context, event):
        scene = bpy.context.window.scene
        self._init_mat = scene.cursor.matrix
        self._init_orient = scene.transform_orientation_slots[0].type

        if event.ctrl and event.shift:
            self.position = True
            self.rotation = True
        elif event.ctrl:
            self.position = False
            self.rotation = True
        elif event.shift:
            self.position = True
            self.rotation = False
        else:
            self.position = True
            self.rotation = True
        return self.execute(context)


class GP_OT_align_from_view(Operator):
    bl_idname = 'gp.align_from_view'
    bl_label = '3D Cursor Align From View'
    bl_description = 'Align From View'
    bl_options = {'REGISTER', 'UNDO', 'INTERNAL'}

    def execute(self, context):
        cursor = context.scene.cursor
        rotation_mode = cursor.rotation_mode
        cursor.rotation_mode = 'QUATERNION'
        cursor.rotation_quaternion =  context.region_data.view_rotation
        cursor.rotation_mode = rotation_mode

        var.REFRESH_MAT = True
        return {'FINISHED'}


class GP_OT_reset_cursor(Operator):
    bl_idname = 'gp.reset_cursor'
    bl_label = 'Reset 3D Cursor'
    bl_description = 'Ctrl+LMB: Selected Objects To Cursor'
    bl_options = {'UNDO'} # 'REGISTER', 

    loc: BoolProperty(name='Location', default=True)
    rot: BoolProperty(name='Position', default=True)


    @classmethod
    def description(self, context, properties):
        if properties.loc and properties.rot:
            return 'Location & Rotation (Ctrl+LMB: Selected Objects To Cursor)' 
        elif properties.loc:
            return 'Location (Ctrl+LMB: Selected Objects To Cursor)'
        elif properties.rot:
            return 'Rotation (Ctrl+LMB: Selected Objects To Cursor)'
    

    def invoke(self, context, event):
        if self.loc:
            if event.ctrl:
                cursorLoc = context.scene.cursor.location
                objs = context.selected_objects
                for ob in objs:
                    ob.location = cursorLoc
            else:
                context.scene.cursor.location = Vector()
            
        if self.rot:
            cursor = context.scene.cursor
            cursorMode = cursor.rotation_mode
            cursor.rotation_mode = 'XYZ'

            if event.ctrl:
                objs = context.selected_objects
                for ob in objs:
                    obMode = ob.rotation_mode
                    ob.rotation_mode = 'XYZ'
                    ob.rotation_euler = cursor.rotation_euler
                    ob.rotation_mode = obMode
            else:
                cursor.rotation_euler = Euler()

            cursor.rotation_mode = cursorMode

        var.REFRESH_MAT = True
        return {'FINISHED'}



# --- 2D Navigation
# Core Operator 'Rotate Canvas'
# https://github.com/Pullusb/rotate_canvas/blob/master/__init__.py
class GP_OT_view_roll(bpy.types.Operator):
    bl_idname = 'gp.view_roll'
    bl_label = 'View Roll'
    bl_options = {"REGISTER", "UNDO"}
    bl_description = (  '• Click: Reset Rotation\n'
                        '• Drag: Roll Viewport\n'
                        '• Ctrl+Drag: Roll Viewport With Snap 10°'
    )

    position: bpy.props.FloatVectorProperty(name='Position')
    

    @staticmethod
    def round_to_nearest_increment(angle, increment):
        increment_radians = radians(increment)
        return round(angle / increment_radians) * increment_radians


    def execute(self, context):
        if self.in_cam:
            self.cam.rotation_mode = self.org_rotation_mode
        return {'FINISHED'}


    def modal(self, context, event):
        if event.type in {'MOUSEMOVE','INBETWEEN_MOUSEMOVE'}:
            # Get current mouse coordination (region)
            self.pos_current = Vector((event.mouse_region_x, event.mouse_region_y))
            # Get current vector
            self.vector_current = (self.pos_current - self.center).normalized()
            # Calculates the angle between initial and current vectors
            self.angle = self.vector_initial.angle_signed(self.vector_current)#radian
            
            if event.ctrl:
                self.angle = self.round_to_nearest_increment(self.angle, 10)
            

            if self.in_cam:
                self.cam.matrix_world = self.cam_matrix
                self.cam.rotation_euler.rotate_axis("Z", self.angle)
            else:
                context.space_data.region_3d.view_rotation = self._rotation
                rot = context.space_data.region_3d.view_rotation
                rot = rot.to_euler()
                rot.rotate_axis("Z", self.angle)
                context.space_data.region_3d.view_rotation = rot.to_quaternion()
        
        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            self.execute(context)
            return {'FINISHED'}
        
        if event.type in {'ESC', 'RIGHTMOUSE'}:#Cancel
            self.execute(context)
            if self.in_cam:
                self.cam.matrix_world = self.cam_matrix
            else:
                context.space_data.region_3d.view_rotation = self._rotation
            return {'CANCELLED'}


        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        global USED
        USED = True
        self.angle = 0.0
        self.in_cam = context.region_data.view_perspective == 'CAMERA'
        self.center = Vector((self.position[0], self.position[1]))
        if self.in_cam:
            self.cam = bpy.context.scene.camera
            if self.cam.lock_rotation[:] != (False, False, False):
                self.report({'WARNING'}, 'Camera rotation is locked') 
                return {'CANCELLED'}
            self.org_rotation_mode = self.cam.rotation_mode
            self.cam.rotation_mode = 'XYZ'
            self.cam_matrix = self.cam.matrix_world.copy()
        else:
            self._rotation = context.space_data.region_3d.view_rotation.copy()

        # Get current mouse coordination
        self.pos_current = Vector((event.mouse_region_x, event.mouse_region_y))
        
        self.initial_pos = self.pos_current# for draw debug, else no need
        # Calculate inital vector
        self.vector_initial = self.pos_current - self.center
        self.vector_initial.normalize()
        
        # Initializes the current vector with the same initial vector.
        self.vector_current = self.vector_initial.copy()
        
        if event.value == 'CLICK':
            aim = context.space_data.region_3d.view_rotation @ Vector((0.0, 0.0, 1.0))
            z_up_quat = aim.to_track_quat('Z','Y')
            if self.in_cam:
                if self.cam.parent:
                    cam_quat = self.cam.parent.matrix_world.inverted().to_quaternion() @ z_up_quat
                else:
                    cam_quat = z_up_quat
                self.cam.rotation_euler = cam_quat.to_euler('XYZ')
            else:
                context.space_data.region_3d.view_rotation = z_up_quat
            if self.in_cam:
                self.cam.rotation_mode = self.org_rotation_mode
            
            USED = False
            return {'FINISHED'}

        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}


class GP_OT_view_pan(bpy.types.Operator):
    bl_idname = 'gp.view_pan'
    bl_label = 'View Pan'
    bl_description = (  '• Click: Toggle Perspective\n'
                        '• Drag: View Pan\n'
                        '• Ctrl+Drag: Zoom View'
                    )

    def invoke(self, context, event):
        if event.value == 'CLICK':
            bpy.ops.view3d.view_persportho()
        elif event.ctrl:
            bpy.ops.view3d.zoom('INVOKE_DEFAULT', use_cursor_init=False)
        else:
            bpy.ops.view3d.move('INVOKE_DEFAULT', use_cursor_init=False)
        return {'FINISHED'}




# --- Call Menu/Panel For Quick Widget
def bool_op_swich(self, context):
    toolSet = context.scene.tool_settings
    props = get_prefs()
    settings = context.scene.GP_scene_set

    if self.bool_op == 'SNAP':
        if toolSet.use_snap:
            toolSet.use_snap = False
        else:
            toolSet.use_snap = True

    elif self.bool_op == 'PROPORT_EDIT':
        if context.mode == 'OBJECT':
            if toolSet.use_proportional_edit_objects:
                toolSet.use_proportional_edit_objects = False
            else:
                toolSet.use_proportional_edit_objects = True
        else:
            if toolSet.use_proportional_edit:
                toolSet.use_proportional_edit = False
            else:
                toolSet.use_proportional_edit = True
                
    elif self.bool_op == 'PIVOT':
        if toolSet.use_transform_data_origin:
            toolSet.use_transform_data_origin = False
        else:
            toolSet.use_transform_data_origin = True

    elif self.bool_op == 'GIZMO_CURSOR':
        if settings.cursor_gizmo:
            settings.cursor_gizmo = False
        else:
            settings.cursor_gizmo = True
            
    if context.area:  
        context.area.tag_redraw()


class GP_OT_call_menu_pie(Operator):
    bl_idname = 'gp.call_menu_pie'
    bl_label = 'Call Menu Pie'
    bl_description = ' '

    name: StringProperty()
    bool_op: StringProperty(default='NONE')

    def invoke(self, context, event):
        if event.type == 'RIGHTMOUSE' and self.bool_op != 'NONE':
            bool_op_swich(self, context)
        else:
            if event.type != 'RIGHTMOUSE':
                bpy.ops.wm.call_menu_pie(name=self.name)
        return {'FINISHED'}


class GP_OT_call_panel(Operator):
    bl_idname = 'gp.call_panel'
    bl_label = 'Call Panel'
    bl_description = ' '

    name: StringProperty()
    keep_open: BoolProperty( default = True )
    bool_op: StringProperty( default = 'NONE' )


    @classmethod
    def description(self, context, properties):
        
        if properties.name == 'VIEW3D_PT_proportional_edit':
            tp = transform_proportional(context)
            return tp[0] #'Proportional Edit'
        elif properties.name == 'VIEW3D_PT_snapping':
            tst = transform_snap_to(context)
            return tst[0] #'Snapping'
        elif properties.name == 'GP_PT_transform_positions':
            tpp = transform_pivot_point(context)
            return tpp[0] #'Transform Positions'
        elif properties.name == 'GP_PT_transform_orientations':
            to = transform_orientation(context)
            return to[0] #'Transform Orientations'
        elif properties.name == 'GP_PT_3d_cursor':
            return '3d Cursor'
        elif properties.name == 'OBJECT_PT_display':
            return 'Viewport Display'
        elif properties.name == 'VIEW3D_PT_tools_object_options':
            return 'Tools Object Options'
        elif properties.name == 'GP_PT_pivot_transform' or properties.name == 'PT_PT_transform_panel':
            return 'Pivot Transform'


    def invoke(self, context, event):
        if event.type == 'RIGHTMOUSE' and self.bool_op != 'NONE':
            bool_op_swich(self, context)
        else:
            if event.type != 'RIGHTMOUSE':
                bpy.ops.wm.call_panel( name = self.name, keep_open = self.keep_open )
        return {'FINISHED'}


class GP_OT_scale_from_bbox(Operator):
    bl_idname = 'gp.scale_from_bbox'
    bl_label = 'Scale From BBox'

    axis: EnumProperty(
        name='Axis',
        description='Axis',
        items=[
            ('X', 'X', '', '', 0),
            ('Y', 'Y', '', '', 1),
            ('Z', 'Z', '', '', 2),
            ],
        default='Z',
        )

    matrix_basis: FloatVectorProperty(size=(4, 4), subtype='MATRIX')
    pp: FloatVectorProperty(default=(0.0, 0.0, 0.0))
    axis_set = (False, False, False)


    def get_orient_matrix(self):
        mat = self.matrix_basis.copy().to_3x3()
        if self.axis == 'X':
            mat = mat @ Quaternion( (0.0, 1.0, 0.0), radians(90) ).to_matrix().to_3x3()
        elif self.axis == 'Y':
            mat = mat @ Quaternion( (1.0, 0.0, 0.0), radians(90) ).to_matrix().to_3x3()
        return mat
    

    def resize(self, context, matrix_orient):
        tool_settings = context.scene.tool_settings
        tpp = tool_settings.transform_pivot_point
        settings = context.scene.GP_scene_set

        # --- cохраняем настройки
        use_snap_scale = tool_settings.use_snap_scale
        snap_target = tool_settings.snap_target
        
        # --- временно включаем некоторые опции
        tool_settings.use_snap_scale = True
        tool_settings.snap_target = 'CLOSEST'


        if (tpp or settings.position) == 'INDIVIDUAL_ORIGINS':
            bpy.ops.transform.resize(
                'INVOKE_DEFAULT',
                constraint_axis = self.axis_set,
                release_confirm = True,
                )
        else:
            bpy.ops.transform.resize(
                'INVOKE_DEFAULT',
                orient_matrix = matrix_orient,
                constraint_axis = self.axis_set,
                release_confirm = True,
                center_override = self.pp,
                )

        # --- возвращаем настройки
        tool_settings.use_snap_scale = use_snap_scale
        tool_settings.snap_target = snap_target
            
        pro_center_orient_reset(context)


    def invoke(self, context, event):
        if self.axis == 'X':
            self.axis_set = (True, False, False)

        elif self.axis == 'Y':
            self.axis_set = (False, True, False)

        elif self.axis == 'Z':
            self.axis_set = (False, False, True)


        if event.shift:
            return self.execute(context)
        else:
            self.pp = self.matrix_basis.translation
            return self.execute(context)
       

    def execute(self, context):
        self.resize(context, self.get_orient_matrix())
        return {'FINISHED'}





classes = [
    GP_OT_documentation,
    GP_OT_switch_gizmo,
    GP_OT_panel_settings,
    GP_OT_save_preferences,
    GP_OT_preset_modifiers,

    GP_OT_cursor_to_active,
    GP_OT_align_from_view,
    GP_OT_reset_cursor,

    GP_OT_view_roll,
    GP_OT_view_pan,

    GP_OT_call_menu_pie,
    GP_OT_call_panel,

    GP_OT_scale_from_bbox,
    ]

def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)