import bpy
from bl_ui.space_toolsystem_common import activate_by_id as activate_tool
from bl_ui.space_toolsystem_toolbar import VIEW3D_PT_tools_active as view3d_tools
from .. import get_prefs

def active_tool():
    return view3d_tools.tool_active_from_context(bpy.context)


def check_data_select(context):
    obj = context.object
    if obj is not None:
        edObj = context.objects_in_mode_unique_data

        if context.mode == 'OBJECT':
            obSel = context.selected_objects
            return len(obSel)>0

        elif context.mode in {'EDIT_MESH', 'EDIT_CURVE', 'EDIT_SURFACE'} and len(edObj) > 0:
            s = context.scene.statistics(context.view_layer)
            v = s.split('Verts:')[1].split(' ')[0]
            return v[0] != '0'

        elif context.mode in {'EDIT_ARMATURE', 'POSE'} and len(edObj) > 0:
            s = context.scene.statistics(context.view_layer)
            v = s.split('Joints:')[1].split(' ')[0] if context.mode == 'EDIT_ARMATURE' else s.split('Bones:')[1].split(' ')[0]
            return v[0] != '0'

        """ elif context.mode == 'SCULPT':
            return obj.mode == 'SCULPT' """


def use_with_standard_gizmos(context):
    props = get_prefs()
    if props.use_with_standard_gizmos is False:
        tool = ['builtin.move', 'builtin.rotate', 'builtin.scale', 'builtin.transform']
        sd = context.space_data
        if active_tool().idname in tool or sd.show_gizmo_object_translate or sd.show_gizmo_object_rotate or sd.show_gizmo_object_scale:
            return False
    return True


def poll_gizmopro(context):
    if context.space_data.show_gizmo and use_with_standard_gizmos(context):
        settings = context.scene.GP_scene_set
        if settings.gzm_type=='GIZMOPRO' and settings.show_gizmo:
            return check_data_select(context)


def poll_luna(context):
    if context.space_data.show_gizmo and use_with_standard_gizmos(context):
        settings = context.scene.GP_scene_set
        if settings.gzm_type=='LUNA' and settings.show_gizmo:
            return check_data_select(context)


def poll_malibu(context):
    if context.space_data.show_gizmo and use_with_standard_gizmos(context):
        settings = context.scene.GP_scene_set
        if settings.gzm_type=='MALIBU' and settings.show_gizmo:
            return check_data_select(context)


def poll_transpose(context):
    if context.space_data.show_gizmo and use_with_standard_gizmos(context):
        settings = context.scene.GP_scene_set
        if settings.show_gizmo:
            if active_tool().idname=='tool.transpose_pro':
                obSel = context.selected_objects
                return len(obSel)>0
            elif settings.gzm_type=='TRANSPOSE':
                return check_data_select(context)


def poll_vr(context):
    if context.space_data.show_gizmo and use_with_standard_gizmos(context):
        settings = context.scene.GP_scene_set
        if settings.gzm_type=='VR' and settings.show_gizmo:
            return check_data_select(context)


def poll_scale(context):
    if context.space_data.show_gizmo and use_with_standard_gizmos(context):
        settings = context.scene.GP_scene_set
        if settings.scale_gizmo and settings.show_gizmo:
            return check_data_select(context)


def poll_curve(context):
    if context.mode == 'EDIT_CURVE':
        if context.space_data.show_gizmo and use_with_standard_gizmos(context):
            settings = context.scene.GP_scene_set
            if settings.curve_gizmo and settings.show_gizmo:
                if context.object is not None:
                    if len(context.objects_in_mode_unique_data) > 0:
                        s = context.scene.statistics(context.view_layer)
                        v = s.split('Verts:')[1].split(' ')[0]
                        return v[0] != '0'


def poll_placement(context):
    if context.space_data.show_gizmo and use_with_standard_gizmos(context):
        settings = context.scene.GP_scene_set
        if settings.gzm_type=='PLACEMENT' and settings.show_gizmo:
            return check_data_select(context)


def poll_mirror(context):
    if context.space_data.show_gizmo and use_with_standard_gizmos(context):
        settings = context.scene.GP_scene_set
        if settings.gzm_type=='MIRROR' and settings.show_gizmo:
            return check_data_select(context)


def poll_shear(context):
    if context.space_data.show_gizmo and use_with_standard_gizmos(context):
        settings = context.scene.GP_scene_set
        if settings.gzm_type=='SHEAR' and settings.show_gizmo:
            return check_data_select(context)


def poll_cursor(context):
    if context.space_data.show_gizmo:
        settings = context.scene.GP_scene_set
        return settings.cursor_gizmo


def poll_cursor_navigator(context):
    if context.space_data.show_gizmo:
        settings = context.scene.GP_scene_set
        return settings.cursor_navigator


def poll_cam_gizmo(context):
    if context.space_data.show_gizmo:
        settings = context.scene.GP_scene_set
        if settings.camera_gizmo:
            ob = context.active_object
            if ob:
                if ob.select_get():
                    return ob.type=='CAMERA'


def poll_preselect(context):
    if context.space_data.show_gizmo:
        settings = context.scene.GP_scene_set
        if context.mode=='EDIT_MESH':
            return settings.preselect_gizmo


def poll_searcher(context):
    if context.space_data.show_gizmo:
        settings = context.scene.GP_scene_set
        if settings.gzm_type in {'GIZMOPRO', 'LUNA', 'MALIBU'} and settings.show_gizmo:
            if settings.searcher_gizmo:
                return check_data_select(context)


def poll_quick_widget(context):
    if context.space_data.show_gizmo and use_with_standard_gizmos(context):
        settings = context.scene.GP_scene_set
        if settings.gzm_type in {'GIZMOPRO', 'LUNA', 'MALIBU', 'SHEAR', 'MIRROR'}:
            if settings.qw:
                return check_data_select(context)