import bpy, gpu
from bpy.types import Gizmo, ShaderFx
from gpu_extras.batch import batch_for_shader
from .shaders import (
    GPU_OUTLINE_SHADER_UNIFORM_COLOR,
    GPU_LIGHT_SHADER_UNIFORM_COLOR,
    GPU_SHADER_DIAG_STRIPES_UNIFORM_COLOR,
    GPU_SHADER_3D_POINT_UNIFORM_SIZE_OUTLINE,
)
from gpu import state
from .. import get_prefs



class GizmoPRO(Gizmo):
    __slots__ = ()


    @staticmethod
    def new_custom_shape(
                        type = 'TRIS', 
                        verts = [], 
                        normal = None,
                        ):

        props = get_prefs()


        if props.use_shader == 'OUTLINE' and normal:
            shader = GPU_OUTLINE_SHADER_UNIFORM_COLOR()
            batch = batch_for_shader( shader, type, { 'pos': verts, 'norm': normal } )
            batch_out = batch_for_shader( shader, type, { 'pos': verts, 'norm': normal } )
            """ shader = GPU_OUTLINE_SHADER_3D()
            batch = batch_for_shader( shader, type, { 'pos': verts } ) """
            return batch, shader, batch_out

        elif props.use_shader == 'BASIC_LIGHT' and normal:
            shader = GPU_LIGHT_SHADER_UNIFORM_COLOR()
            batch = batch_for_shader( shader, type, { 'pos': verts, 'norm': normal } )
            return batch, shader, None

        elif props.use_shader == 'DIAG_STRIPES':
            shader = GPU_SHADER_DIAG_STRIPES_UNIFORM_COLOR()
            batch = batch_for_shader( shader, type, { 'pos': verts } )
            return batch, shader, None
        
        else:
            shader = gpu.shader.from_builtin('UNIFORM_COLOR')
            batch = batch_for_shader(shader, type, {'pos': verts})
            return batch, shader, None


    def draw_custom_shape( 
                        self,
                        shape,
                        *,
                        matrix = None,
                        select_id = None,
                        normal = None,
                        use_blend = True,
                        ):

        props = get_prefs()
        
         
        if matrix is None:
            matrix = self.matrix_world
        
        batch, shader, batch_out = shape
        shader.bind()



        


        color = (*self.color, self.alpha)
        color_out = (0.0, 0.0, 0.0, self.alpha)
        if select_id is not None:
            gpu.select.load_id(select_id)
        else:
            if self.is_highlight:
                color = (*self.color_highlight, self.alpha_highlight)
                color_out = (0.0, 0.0, 0.0, self.alpha_highlight)
            else:
                color = (*self.color, self.alpha)
                color_out = (0.0, 0.0, 0.0, self.alpha)


       
        if use_blend:
            state.blend_set('ALPHA')
            if props.use_shader == 'BASIC_LIGHT' and normal:
                state.face_culling_set('BACK')

        with gpu.matrix.push_pop():
            gpu.matrix.multiply_matrix(matrix)

            if props.use_shader == 'OUTLINE' and normal:
                shader.uniform_float( 'lineTh', (props.gizmo_thickness + props.outline_thickness) )
                shader.uniform_float( 'color', color_out )
                batch_out.draw(shader)
                shader.uniform_float( 'lineTh', props.gizmo_thickness )

                """ shader.uniform_float( 'outline_color', color_out )
                shader.uniform_float( 'base_color', color )
                shader.uniform_float( 'outline_thickness', (props.gizmo_thickness + props.outline_thickness) ) """

            elif props.use_shader == 'BASIC_LIGHT' and normal:
                #camera_pos = bpy.context.region_data.view_matrix.inverted().translation'

                # Получаем текущую область 3D View
                for area in bpy.context.screen.areas:
                    if area.type == 'VIEW_3D':
                        space_data = area.spaces.active

                # Получаем позицию камеры 3D View
                view_matrix = space_data.region_3d.view_matrix
                camera_pos = view_matrix.inverted().translation

                # Переводим позицию в кортеж и передаем в шейдер
                #camera_pos = (camera_position.x, camera_position.y, camera_position.z)

                shader.uniform_float('light', camera_pos)
                shader.uniform_float('ambientIntensity', 0.8)

            elif props.use_shader == 'DIAG_STRIPES':
                second_color = (0.0, 0.0, 0.0) #props.elements_color
                shader.uniform_float( 'color2', (*second_color, self.alpha/2) ) # (color[0]/3, color[1]/3, color[2]/3, self.alpha/3)
                shader.uniform_int( 'size1', int(8*props.size_gizmo_pro) )
                shader.uniform_int( 'size2', int(8*props.size_gizmo_pro) )

            

                


            shader.uniform_float('color', color)
            batch.draw(shader)
        

        if use_blend:
            if props.use_shader == 'BASIC_LIGHT':
                state.face_culling_set('NONE')
            state.blend_set('NONE')






class Gizmo_Malibu(Gizmo):
    __slots__ = ()

    @staticmethod
    def new_custom_shape(type='LINES', verts = []):
        if type == 'POINTS':
            shader = GPU_SHADER_3D_POINT_UNIFORM_SIZE_OUTLINE()
            batch = batch_for_shader(shader, 'POINTS', {'pos': verts})
        else:
            shader = gpu.shader.from_builtin('UNIFORM_COLOR')
            batch = batch_for_shader(shader, type, {'pos': verts})
        return batch, shader

    def draw_custom_shape(self, shape, *, matrix=None, select_id=None):
        if matrix is None:
            matrix = self.matrix_world

        batch, shader = shape


        if select_id is not None:
            gpu.select.load_id(select_id)
            #use_blend = False
        else:
            if self.is_highlight:
                color = (*self.color_highlight, self.alpha_highlight)
            else:
                color = (*self.color, self.alpha)

            if self.align_view:
                shader.uniform_float("outlineColor", (0.0, 0.0, 0.0, self.alpha))
                shader.uniform_float("size", 300.0)
                shader.uniform_float("outlineWidth", 30.0 * self.line_width)
                
            shader.uniform_float("color", color)
            

        #if use_blend:
        gpu.state.blend_set('ALPHA')

        with gpu.matrix.push_pop():
            gpu.matrix.multiply_matrix(matrix)
            
            
            

            batch.draw(shader)
        #if use_blend:
        gpu.state.blend_set('NONE')

    








    