import bpy
from bpy.types import Macro, Operator
from .raycast import ray_cast
from bpy.props import EnumProperty, BoolProperty
from .. import var,get_prefs



SXR = False



class GP_OT_select_box(Macro):
    bl_idname = 'gp.select_box'
    bl_label = 'Select Box'
    bl_options = {'MACRO', 'UNDO'}



class GP_OT_select_circle(Macro):
    bl_idname = 'gp.select_circle'
    bl_label = 'Select Circle'
    bl_options = {'MACRO', 'UNDO'}



class GP_OT_select_lasso(Macro):
    bl_idname = 'gp.select_lasso'
    bl_label = 'Select Lasso'
    bl_options = {'MACRO', 'UNDO'}




""" class GP_OT_select_box(Operator):
    bl_idname = 'gp.select_box'
    bl_label = 'Select Box'
    bl_options = {'UNDO', 'INTERNAL'}
    

    def __init__(self):
        self.show_xray = False
        self.mode = None


    def modal(self, context, event):
        
        print(1)
        bpy.ops.view3d.select_box( 'INVOKE_DEFAULT', wait_for_input = True, mode = self.mode )

        if event.value == 'RELEASE' or event.value_prev == 'RELEASE' or event.type_prev in {'RET'} or event.type in {'RET'}:
            print(2)
            context.space_data.shading.show_xray = self.show_xray
            return {'FINISHED'}
        elif event.value in {'ESC'} or event.value_prev in {'ESC'}:
            print(3)
            context.space_data.shading.show_xray = self.show_xray
            return {'CANCELLED'}

        #return {'RUNNING_MODAL'}
        return {'PASS_THROUGH'}


    def invoke(self, context, event):
        props = context.preferences.addons['GIZMO_PRO'].preferences
        print(2)
        if props.sel_xray:
            self.show_xray = context.space_data.shading.show_xray
            context.space_data.shading.show_xray = True

        
        if event.shift and event.ctrl:
            self.mode = 'AND'
        elif event.shift:
            self.mode = 'ADD'
        elif event.ctrl:
            self.mode = 'SUB'
        else:
            tool = context.workspace.tools.from_space_view3d_mode( context.mode, create = False )
            self.mode = tool.operator_properties( 'view3d.select_box' ).mode

        print(5)
        return {'RUNNING_MODAL'}
 """


class GP_OT_st_store(Operator):
    bl_idname = 'gp.st_store'
    bl_label = 'Select Tool'
    bl_options = {'MACRO', 'INTERNAL'}

    def execute(self, context):
        props = get_prefs()
        show_xray = context.space_data.shading.show_xray
        if props.sel_xray and show_xray == False:
            global SXR
            SXR = True
            context.space_data.shading.show_xray = True
        return {'FINISHED'}



class GP_OT_st_restore(Operator):
    bl_idname = 'gp.st_restore'
    bl_label = 'Select Tool'
    bl_options = {'MACRO', 'INTERNAL'}

    def execute(self, context):
        props = get_prefs()
        global SXR
        if props.sel_xray and SXR:
            context.space_data.shading.show_xray = False
            SXR = False
        return {'FINISHED'}



# --- Base
def space_drag_op(self, context, event):
    tool = context.workspace.tools.from_space_view3d_mode(context.mode, create=False)
    props = get_prefs()
    settings = context.scene.GP_scene_set
    
    if event.type == 'LEFTMOUSE':
        sd = props.space_drag_lmb
    elif event.type == 'RIGHTMOUSE':
        sd = props.space_drag_rmb
    else:
        sd = props.space_drag_lmb


    # --- Pro Navigate
    if sd == 'ORBIT':
        if settings.lock_orbit:
            bpy.ops.view3d.move('INVOKE_DEFAULT', use_cursor_init=True)
        else:
            bpy.ops.view3d.rotate('INVOKE_DEFAULT', use_cursor_init=True)

    elif sd == 'PAN':   
        bpy.ops.view3d.move('INVOKE_DEFAULT', use_cursor_init=True)

    elif sd == 'DOLLY':   
        bpy.ops.view3d.zoom('INVOKE_DEFAULT', use_cursor_init=True) 

    # --- Transform
    elif sd == 'TRANSFORM':
        bpy.ops.gp.modo_transform('INVOKE_DEFAULT')

    elif sd == 'TRANSLATE':
        bpy.ops.transform.transform('INVOKE_DEFAULT', mode='TRANSLATION')

    elif sd == 'ROTATE':
        bpy.ops.transform.transform('INVOKE_DEFAULT', mode='ROTATION') 

    elif sd == 'RESIZE':
        bpy.ops.transform.transform('INVOKE_DEFAULT', mode='RESIZE') 
 
    # --- Tweak
    elif sd == 'TWEAK':
        bpy.ops.view3d.select('INVOKE_DEFAULT', deselect_all=True)
        bpy.ops.transform.translate('INVOKE_DEFAULT')

    # --- Select
    elif sd == 'BOX_SEL':
        if event.shift and event.ctrl:
            mode = 'AND'
        elif event.shift:
            mode = 'ADD'
        elif event.ctrl:
            mode = 'SUB'
        else:
            BST = tool.operator_properties('view3d.select_box')
            mode = BST.mode
        bpy.ops.gp.select_box('INVOKE_DEFAULT', VIEW3D_OT_select_box={'mode': mode})
      
    elif sd == 'CIRCLE_SEL':
        CST = tool.operator_properties('view3d.select_circle')
        if (event.shift and event.ctrl) or event.shift:
            mode = 'ADD'
        elif event.ctrl:
            mode = 'SUB'
        else:
            mode = CST.mode
        bpy.ops.gp.select_circle('INVOKE_DEFAULT', VIEW3D_OT_select_circle={'mode': mode, 'radius': CST.radius})


    elif sd == 'LASSO_SEL':
        if event.shift and event.ctrl:
            mode = 'AND'
        elif event.shift:
            mode = 'ADD'
        elif event.ctrl:
            mode = 'SUB'
        else:
            LST = tool.operator_properties('view3d.select_lasso')
            mode = LST.mode
        bpy.ops.gp.select_lasso('INVOKE_DEFAULT', VIEW3D_OT_select_lasso={'mode': mode})


    elif sd == 'PAINT_SEL':
        if context.mode == 'EDIT_MESH':
            bpy.ops.gp.paint_select('INVOKE_DEFAULT')
        else:
            return {'CANCELLED'}

    else:
        return {'CANCELLED'}



class GP_OT_space_drag_lmb(Operator):
    bl_idname = 'gp.space_drag_lmb'
    bl_label = 'Space Drag LMB'


    @classmethod
    def poll(cls, context):
        props = get_prefs()
        return props.space_drag_lmb != 'NONE' 

    
    def invoke(self, context, event):
        space_drag_op(self, context, event)
        return {'FINISHED'}



class GP_OT_space_drag_rmb(Operator):
    bl_idname = 'gp.space_drag_rmb'
    bl_label = 'Space Drag RMB'


    @classmethod
    def poll(cls, context):
        props = get_prefs()
        return props.space_drag_rmb != 'NONE' 

    
    def invoke(self, context, event):
        space_drag_op(self, context, event)
        return {'FINISHED'}





class GP_OT_tweak(Operator):
    bl_idname = 'gp.tweak'
    bl_label = 'Tweak'

    deselect_all: BoolProperty( default = True )
    raycast: BoolProperty( default = False )

    @classmethod
    def poll(cls, context):
        if context.area:
            return context.area.type == 'VIEW_3D'
               
    
    def invoke(self, context, event):
        if self.raycast:
            ray = ray_cast(context, (event.mouse_region_x, event.mouse_region_y))[0]
            if ray:
                bpy.ops.view3d.select( 'INVOKE_DEFAULT', deselect_all = self.deselect_all )
                bpy.ops.transform.translate( 'INVOKE_DEFAULT' )
            else:
                space_drag_op(self, context, event)
                return {'FINISHED'}
                #return {'CANCELLED'}
        else:
            bpy.ops.view3d.select( 'INVOKE_DEFAULT', deselect_all = self.deselect_all )
            bpy.ops.transform.translate( 'INVOKE_DEFAULT' )
        return {'FINISHED'}



classes = [
    GP_OT_select_box,
    GP_OT_select_circle,
    GP_OT_select_lasso,

    GP_OT_st_store,
    GP_OT_st_restore,

    GP_OT_space_drag_lmb,
    GP_OT_space_drag_rmb,

    GP_OT_tweak,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


    # --- Select  Box
    GP_OT_select_box.define('GP_OT_st_store')
    op = GP_OT_select_box.define('VIEW3D_OT_select_box')
    op.properties.wait_for_input = True
    GP_OT_select_box.define('GP_OT_st_restore')


    # --- Select Circle
    GP_OT_select_circle.define('GP_OT_st_store')
    op = GP_OT_select_circle.define('VIEW3D_OT_select_circle')
    op.properties.wait_for_input = True
    GP_OT_select_circle.define('GP_OT_st_restore')


    # --- Select Lasso
    GP_OT_select_lasso.define('GP_OT_st_store')
    GP_OT_select_lasso.define('VIEW3D_OT_select_lasso')
    GP_OT_select_lasso.define('GP_OT_st_restore')


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)