from gpu.types import GPUShader
#gpu_shader_builtin.c






def GPU_FLAT_SHADER_UNIFORM_COLOR_WITH_OUTLINE():
    vert = '''
        uniform mat4 ModelViewProjectionMatrix;
        uniform mat4 ModelMatrix; // Теперь мы передаем ModelMatrix
        uniform mat3 NormalMatrix;
        uniform float OutlineScale;

        in vec3 pos;
        in vec3 normal;
        in vec4 color;

        out vec4 finalColor;
        out float vertexDepth; // Для передачи глубины вершины

        void main() {
            vec3 scaledPos = pos;
            if (!gl_FrontFacing) {
                // Увеличиваем позицию вершины для создания обводки
                scaledPos += normal * OutlineScale;
                finalColor = vec4(0, 0, 0, 1); // Цвет обводки, например, черный
            } else {
                finalColor = color; // Основной цвет
            }
            vec4 worldPosition = ModelMatrix * vec4(scaledPos, 1.0);
            vertexDepth = -(ModelViewProjectionMatrix * worldPosition).z; // Сохраняем глубину вершины
            gl_Position = ModelViewProjectionMatrix * vec4(scaledPos, 1.0);
        }
    '''

    frag = '''
        uniform vec4 BaseColor; // Основной цвет
        uniform float OutlineDepthThreshold; // Пороговое значение глубины для обводки

        in vec4 finalColor;
        in float vertexDepth;

        out vec4 fragColor;

        void main() {
            if (vertexDepth > OutlineDepthThreshold) {
                fragColor = vec4(0, 0, 0, 1); // Цвет обводки, если глубина вершины превышает порог
            } else {
                fragColor = BaseColor; // Иначе используем основной цвет
            }
            fragColor = blender_srgb_to_framebuffer_space(fragColor);
        }
    '''


    return GPUShader(vert, frag)




def GPU_OUTLINE_SHADER_2D_UNIFORM_COLOR(): # FIXME

    vert = '''
        #ifndef USE_GPU_SHADER_CREATE_INFO
        uniform mat4 ModelViewProjectionMatrix;
        uniform float size;
        uniform float outlineWidth;

        in vec2 pos;
        out vec4 radii;
        #endif

        void main()
        {
        gl_Position = ModelViewProjectionMatrix * vec4(pos, 0.0, 1.0);
        gl_PointSize = size;

        /* calculate concentric radii in pixels */
        float radius = 0.5 * size;

        /* start at the outside and progress toward the center */
        radii[0] = radius;
        radii[1] = radius - 1.0;
        radii[2] = radius - outlineWidth;
        radii[3] = radius - outlineWidth - 1.0;

        /* convert to PointCoord units */
        radii /= size;
        }
    '''

    frag = '''
        uniform vec4 color;
        //uniform vec4 color_out;

        //in float oDot;
        out vec4 fragColor;

        void main()
        {   
            //fragColor = vec4(color.xyz * (0.8 + 0.4*oDot), color.w);
            //fragColor = color * oDot + color_out * (2-oDot);
            fragColor = color;
            fragColor = blender_srgb_to_framebuffer_space(fragColor);
        }
    '''

    return GPUShader(vert, frag)


def GPU_OUTLINE_SHADER_UNIFORM_COLOR(): # работает, но можно лучше

    vert = '''
        uniform mat4 ModelViewProjectionMatrix;
        #ifdef USE_WORLD_CLIP_PLANES
        uniform mat4 ModelMatrix;
        
        #endif
        
        uniform float lineTh;

        vec4 outPos;
        vec4 posGen;
    
        in vec3 pos;
        in vec3 norm;
        
        //out float oDot;
        void main()
        {
            outPos = ModelViewProjectionMatrix * vec4(pos+(norm*lineTh), 1.0);
            //outPos = ModelViewProjectionMatrix * vec4(pos, 1.0);
            //posGen = ModelViewProjectionMatrix * vec4(pos, 1.0);
            gl_Position = outPos;
            
        #ifdef USE_WORLD_CLIP_PLANES
            world_clip_planes_calc_clip_distance((ModelMatrix * vec4(pos, 1.0)).xyz);
        #endif
        }
    '''

    frag = '''
        uniform vec4 color;
        //uniform vec4 color_out;

        //in float oDot;
        out vec4 fragColor;

        void main()
        {   
            //fragColor = vec4(color.xyz * (0.8 + 0.4*oDot), color.w);
            //fragColor = color * oDot + color_out * (2-oDot);
            fragColor = color;
            fragColor = blender_srgb_to_framebuffer_space(fragColor);
        }
    '''

    return GPUShader(vert, frag)


def GPU_FLAT_SHADER_UNIFORM_COLOR(): # --- плоский шейдинг
    vert = '''
        uniform mat4 ModelViewProjectionMatrix;

        #ifdef USE_WORLD_CLIP_PLANES
        uniform mat4 ModelMatrix;
        #endif

        in vec3 pos;
        in vec4 color;

        out vec4 finalColor;

        void main()
        {
            gl_Position = ModelViewProjectionMatrix * vec4(pos, 1.0);
            finalColor = color;

            #ifdef USE_WORLD_CLIP_PLANES
            world_clip_planes_calc_clip_distance((ModelMatrix * vec4(pos, 1.0)).xyz);
            #endif
        }
    '''

    frag = '''
        uniform vec4 color;
        out vec4 fragColor;

        void main()
        {
            fragColor = color;
            fragColor = blender_srgb_to_framebuffer_space(fragColor);
        }
    '''

    return GPUShader(vert, frag)


def GPU_SHADER_2D_LINE_DASHED_UNIFORM_COLOR():
    vert = '''
        uniform mat4 ModelViewProjectionMatrix;

        uniform vec4 color;
        uniform vec2 viewport_size;

        in vec2 pos;

        flat out vec4 color_vert;

        /* We leverage hardware interpolation to compute distance along the line. */
        noperspective out vec2 stipple_pos; /* In screen space */
        flat out vec2 stipple_start;        /* In screen space */

        void main()
        {
        gl_Position = ModelViewProjectionMatrix * vec4(pos, 0.0, 1.0);
        stipple_start = stipple_pos = viewport_size * 0.5 * (gl_Position.xy / gl_Position.w);
        color_vert = color;
        }
    '''

    frag = ''' 
        
        uniform float dash_width;
        uniform float dash_factor;

        uniform int colors_len;
        uniform vec4 colors[32];

        flat in vec4 color_vert;

        noperspective in vec2 stipple_pos;
        flat in vec2 stipple_start;

        out vec4 fragColor;

        void main()
        {
        float distance_along_line = distance(stipple_pos, stipple_start);

        if (colors_len > 0) {
            
            if (colors_len == 1) {
            fragColor = colors[0];
            }
            
            else {
            float normalized_distance = fract(distance_along_line / dash_width);
            fragColor = colors[int(normalized_distance * colors_len)];
            }
        }
        
        else {
            
            if (dash_factor >= 1.0f) {
            fragColor = color_vert;
            }
            
            else {
            float normalized_distance = fract(distance_along_line / dash_width);
            if (normalized_distance <= dash_factor) {
                fragColor = color_vert;
            }
            else {
                discard;
            }
            }
        }
        }
  
    '''
    #viewport_size = gpu.state.viewport_get()
    #shader.uniform_float('viewport_size', (viewport_size[2], viewport_size[3]))
    #shader.uniform_int('colors_len', 0)
    #shader.uniform_float('dash_width', 10.0)
    #shader.uniform_float('dash_factor', 0.5)
    #batch_y = batch_for_shader(shader, 'LINE_STRIP', {'pos': coords_y})
    #shader.uniform_float('color', (*props.y_axis_color, 0.9))

    return GPUShader(vert, frag)


def GPU_SHADER_3D_LINE_DASHED_UNIFORM_COLOR():
    vert = '''
        uniform mat4 ModelViewProjectionMatrix;

        #ifdef USE_WORLD_CLIP_PLANES
        uniform mat4 ModelMatrix;
        #endif

        uniform vec4 color;
        uniform vec2 viewport_size;

        in vec3 pos;

        flat out vec4 color_vert;

        /* We leverage hardware interpolation to compute distance along the line. */
        noperspective out vec2 stipple_pos; /* In screen space */
        flat out vec2 stipple_start;        /* In screen space */

        void main()
        {
            vec4 pos_4d = vec4(pos, 1.0);
            gl_Position = ModelViewProjectionMatrix * pos_4d;
            stipple_start = stipple_pos = viewport_size * 0.5 * (gl_Position.xy / gl_Position.w);
            color_vert = color;
            #ifdef USE_WORLD_CLIP_PLANES
            world_clip_planes_calc_clip_distance((ModelMatrix * pos_4d).xyz);
            #endif
        }
    '''

    frag = ''' 
        
        uniform float dash_width;
        uniform float dash_factor;

        uniform int colors_len;
        uniform vec4 colors[32];

        flat in vec4 color_vert;

        noperspective in vec2 stipple_pos;
        flat in vec2 stipple_start;

        out vec4 fragColor;

        void main()
        {
        float distance_along_line = distance(stipple_pos, stipple_start);

        if (colors_len > 0) {
            
            if (colors_len == 1) {
            fragColor = colors[0];
            }
            
            else {
            float normalized_distance = fract(distance_along_line / dash_width);
            fragColor = colors[int(normalized_distance * colors_len)];
            }
        }
        
        else {
            
            if (dash_factor >= 1.0f) {
            fragColor = color_vert;
            }
            
            else {
            float normalized_distance = fract(distance_along_line / dash_width);
            if (normalized_distance <= dash_factor) {
                fragColor = color_vert;
            }
            else {
                discard;
            }
            }
        }
        }
  
    '''
    #viewport_size = gpu.state.viewport_get()
    #shader.uniform_float('viewport_size', (viewport_size[2], viewport_size[3]))
    #shader.uniform_int('colors_len', 0)
    #shader.uniform_float('dash_width', 10.0)
    #shader.uniform_float('dash_factor', 0.5)
    #batch_y = batch_for_shader(shader, 'LINE_STRIP', {'pos': coords_y})
    #shader.uniform_float('color', (*props.y_axis_color, 0.9))

    return GPUShader(vert, frag)


def GPU_SHADER_CHECKER_UNIFORM_COLOR():
    vert = '''
        uniform mat4 ModelViewProjectionMatrix;

        #ifdef USE_WORLD_CLIP_PLANES
        uniform mat4 ModelMatrix;
        #endif

        in vec3 pos;

        void main()
        {
          gl_Position = ModelViewProjectionMatrix * vec4(pos, 1.0);

        #ifdef USE_WORLD_CLIP_PLANES
          world_clip_planes_calc_clip_distance((ModelMatrix * vec4(pos, 1.0)).xyz);
        #endif
        }

    '''

    frag = ''' 
        
        uniform vec4 color1;
        uniform vec4 color2;
        uniform int size;

        out vec4 fragColor;

        void main()
        {
          vec2 phase = mod(gl_FragCoord.xy, (size * 2));

          if ((phase.x > size && phase.y < size) || (phase.x < size && phase.y > size)) {
            fragColor = color1;
          }
          else {
            fragColor = color2;
          }
        }
  
    '''
    #shader.uniform_float("color1", (0.2, 0.2, 0.9, 1.0))
    #shader.uniform_float("color2", (1.0, 0.2, 0.2, 0.5))
    #shader.uniform_int("size", 4)

    return GPUShader(vert, frag)






def GPU_SHADER_DIAG_STRIPES_UNIFORM_COLOR():
    vert = '''
        uniform mat4 ModelViewProjectionMatrix;

        #ifdef USE_WORLD_CLIP_PLANES
        uniform mat4 ModelMatrix;
        #endif

        in vec3 pos;

        void main()
        {
          gl_Position = ModelViewProjectionMatrix * vec4(pos, 1.0);

        #ifdef USE_WORLD_CLIP_PLANES
          world_clip_planes_calc_clip_distance((ModelMatrix * vec4(pos, 1.0)).xyz);
        #endif
        }

    '''

    frag = '''
        #ifndef USE_GPU_SHADER_CREATE_INFO
        uniform vec4 color;
        uniform vec4 color2;
        uniform int size1;
        uniform int size2;

        out vec4 fragColor;
        #endif

        void main()
        {
          float phase = mod((gl_FragCoord.x + gl_FragCoord.y), (size1 + size2));

          if (phase < size1) {
            fragColor = color;
          }
          else {
            fragColor = color2;
          }
        }
  
    '''
    #shader.uniform_float("color", (0.2, 0.2, 0.9, 1.0))
    #shader.uniform_float("color2", (1.0, 0.2, 0.2, 0.5))
    #shader.uniform_int("size1", 10)
    #shader.uniform_int("size2", 10)

    return GPUShader(vert, frag)


def GPU_SHADER_2D_POINT_UNIFORM_SIZE_OUTLINE(): # --- круглые вертексы с обводкой
    vert = '''
        
        #ifndef USE_GPU_SHADER_CREATE_INFO
        uniform mat4 ModelViewProjectionMatrix;
        uniform float size;
        uniform float outlineWidth;

        in vec2 pos;
        out vec4 radii;
        #endif

        void main()
        {
        gl_Position = ModelViewProjectionMatrix * vec4(pos, 0.0, 1.0);
        gl_PointSize = size;

        /* calculate concentric radii in pixels */
        float radius = 0.5 * size;

        /* start at the outside and progress toward the center */
        radii[0] = radius;
        radii[1] = radius - 1.0;
        radii[2] = radius - outlineWidth;
        radii[3] = radius - outlineWidth - 1.0;

        /* convert to PointCoord units */
        radii /= size;
        }

    '''

    frag = ''' 
        
        #ifndef USE_GPU_SHADER_CREATE_INFO
        uniform vec4 color;
        uniform vec4 outlineColor;

        in vec4 radii;
        out vec4 fragColor;
        #endif

        void main()
        {
          float dist = length(gl_PointCoord - vec2(0.5));

          /* transparent outside of point
           * --- 0 ---
           * smooth transition
           * --- 1 ---
           * pure outline color
           * --- 2 ---
           * smooth transition
           * --- 3 ---
           * pure point color
           * ...
           * dist = 0 at center of point */

          float midStroke = 0.5 * (radii[1] + radii[2]);

          if (dist > midStroke) {
            fragColor.rgb = outlineColor.rgb;
            fragColor.a = mix(outlineColor.a, 0.0, smoothstep(radii[1], radii[0], dist));
          }
          else {
            fragColor = mix(color, outlineColor, smoothstep(radii[3], radii[2], dist));
          }

          if (fragColor.a == 0.0) {
            discard;
          }
        }
  
    '''
    #shader.uniform_float("color", (0.2, 0.2, 0.9, 1.0))
    #shader.uniform_float("outlineColor", (0.0, 0.0, 0.0, 0.5))
    #shader.uniform_float("size", 400.0)
    #shader.uniform_float("outlineWidth", 30.0)

    return GPUShader(vert, frag)


def GPU_SHADER_3D_POINT_UNIFORM_SIZE_OUTLINE(): # --- круглые вертексы с обводкой
    vert = '''
        
        #ifndef USE_GPU_SHADER_CREATE_INFO
        uniform mat4 ModelViewProjectionMatrix;
        uniform float size;
        uniform float outlineWidth;
        uniform vec4 color;
        in vec3 pos;
        out vec4 radii;
        #endif

        void main()
        {
          gl_Position = ModelViewProjectionMatrix * vec4(pos, 1.0);
          gl_PointSize = size;

          /* calculate concentric radii in pixels */
          float radius = 0.5 * size;

          /* start at the outside and progress toward the center */
          radii[0] = radius;
          radii[1] = radius - 1.0;
          radii[2] = radius - outlineWidth;
          radii[3] = radius - outlineWidth - 1.0;

          /* convert to PointCoord units */
          radii /= size;
          
          
          #ifdef USE_WORLD_CLIP_PLANES
            world_clip_planes_calc_clip_distance((ModelMatrix * vec4(pos, 1.0)).xyz);
        #endif
        }

    '''

    frag = ''' 
        
        #ifndef USE_GPU_SHADER_CREATE_INFO
        uniform vec4 color;
        uniform vec4 outlineColor;

        in vec4 radii;
        out vec4 fragColor;
        #endif

        void main()
        {
            float dist = length(gl_PointCoord - vec2(0.5));

            /* transparent outside of point
            * --- 0 ---
            * smooth transition
            * --- 1 ---
            * pure outline color
            * --- 2 ---
            * smooth transition
            * --- 3 ---
            * pure point color
            * ...
            * dist = 0 at center of point */

            float midStroke = 0.5 * (radii[1] + radii[2]);

            if (dist > midStroke) {
                fragColor.rgb = outlineColor.rgb;
                fragColor.a = mix(outlineColor.a, 0.0, smoothstep(radii[1], radii[0], dist));

            }
            else {
                fragColor = mix(color, outlineColor, smoothstep(radii[3], radii[2], dist));
            }

            if (fragColor.a == 0.0) {
                discard;
            }
            fragColor = blender_srgb_to_framebuffer_space(fragColor);
        }
  
    '''
    #shader.uniform_float("color", (0.2, 0.2, 0.9, 1.0))
    #shader.uniform_float("outlineColor", (0.0, 0.0, 0.0, 0.5))
    #shader.uniform_float("size", 400.0)
    #shader.uniform_float("outlineWidth", 30.0)

    return GPUShader(vert, frag)


def GPU_SHADER_3D_POINT_UNIFORM_SIZE(): # --- круглые вертексы с обводкой
    vert = '''
        uniform mat4 ModelViewProjectionMatrix;
        #ifdef USE_WORLD_CLIP_PLANES
        uniform mat4 ModelMatrix;
        #endif
        uniform float size;

        in vec3 pos;
        out vec2 radii;

        void main()
        {
            vec4 pos_4d = vec4(pos, 1.0);
            gl_Position = ModelViewProjectionMatrix * pos_4d;
            gl_PointSize = size;

            // calculate concentric radii in pixels
            float radius = 0.5 * size;

            // start at the outside and progress toward the center
            radii[0] = radius;
            radii[1] = radius - 1.0;

            // convert to PointCoord units
            radii /= size;

            #ifdef USE_WORLD_CLIP_PLANES
            world_clip_planes_calc_clip_distance((ModelMatrix * pos_4d).xyz);
            #endif
        }
    '''

    frag = ''' 
        
        
        uniform vec4 color;

        in vec2 radii;
        out vec4 fragColor;

        void main()
        {
        float dist = length(gl_PointCoord - vec2(0.5));

        // transparent outside of point
        // --- 0 ---
        //  smooth transition
        // --- 1 ---
        // pure point color
        // ...
        // dist = 0 at center of point

        fragColor.rgb = color.rgb;
        fragColor.a = mix(color.a, 0.0, smoothstep(radii[1], radii[0], dist));

        if (fragColor.a == 0.0) {
            discard;
        }
        }
  
    '''
    #shader.uniform_float("color", (0.2, 0.2, 0.9, 1.0))
    #shader.uniform_float("size", 20.0)

    return GPUShader(vert, frag)


def GPU_SHADER_3D_POINT_BLUR(): # --- круглые вертексы с размытием
    vert = '''
        uniform mat4 ModelViewProjectionMatrix;

        in vec3 pos;
        out vec2 lineCoord;

        void main()
        {
            gl_Position = ModelViewProjectionMatrix * vec4(pos, 1.0);
            lineCoord = pos.xy; // Пространственные координаты для вычисления радиуса в фрагментном шейдере
        }
    '''

    frag = '''
        uniform float lineWidth;
        uniform vec4 color;
        in vec2 lineCoord;
        out vec4 fragColor;

        void main()
        {
            float radius = lineWidth / 2.0;
            float dist = length(gl_PointCoord - vec2(0.5, 0.5));
            if (dist > radius) {
                discard; // Убираем фрагменты за пределами радиуса
            }

            // Плавный переход цвета для создания скруглённых краёв
            fragColor = color;
            fragColor.a *= 1.0 - smoothstep(radius - 1.0, radius, dist);
        }
    '''


    #shader.uniform_float("color1", (0.2, 0.2, 0.9, 1.0))
    #shader.uniform_float("color2", (1.0, 0.2, 0.2, 0.5))
    #shader.uniform_int("size", 4)

    return GPUShader(vert, frag)


def GPU_SHADER_3D_POINT(): # --- круглые вертексы
    vert = '''
        uniform mat4 ModelViewProjectionMatrix;
        uniform float size;

        in vec3 pos;

        void main()
        {
            vec4 pos_4d = vec4(pos, 1.0);
            gl_Position = ModelViewProjectionMatrix * pos_4d;
            gl_PointSize = size;
        }
    '''

    frag = ''' 
        uniform vec4 color;
        out vec4 fragColor;

        void main()
        {
            float dist = length(gl_PointCoord - vec2(0.5));

            if (dist <= 0.5) {
                fragColor = color;
            } else {
                discard;
            }
        }
    '''

    return GPUShader(vert, frag)



def GPU_LIGHT_SHADER_UNIFORM_COLOR(): # --- сглаженый шейдинг
    vert = '''
        uniform mat4 ModelViewProjectionMatrix;
        uniform mat3 NormalMatrix;

        in vec3 pos;
        in vec3 norm;
        out vec3 normal;

        void main()
        {
        normal = normalize(NormalMatrix * norm);
        gl_Position = ModelViewProjectionMatrix * vec4(pos, 1.0);
        }
    '''

    frag = '''
        #ifndef USE_INSTANCE_COLOR
        uniform vec4 color;
        #endif
        uniform vec3 light;
        uniform float ambientIntensity; // Добавляем интенсивность ambient света

        in vec3 normal;
        #ifdef USE_INSTANCE_COLOR
        flat in vec4 finalColor;
        #  define color finalColor
        #endif
        out vec4 fragColor;

        void main() {
            vec3 normalizedNormal = normalize(normal);
            float diff = max(dot(normalizedNormal, light), 0.0); // Диффузное освещение
            diff = diff * (1.0 - ambientIntensity) + ambientIntensity; // Смешиваем с ambient светом
            fragColor = color * diff; // Применяем освещение к цвету
        }
    '''
    #shader.uniform_float('light', (1.0,1.0,1.0) )
    #shader.uniform_float("color", (0.2, 0.2, 0.9, 1.0))

    return GPUShader(vert, frag)














